/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
 /**
 * \~english
 * @file sphApp.h
 * Sphere application Class header File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file sphApp.h
 * Sphereアプリケーションクラスヘッダーファイル
 * @author     hira
 * @date       2009/10/01
 */
#ifndef _SPHAPP_H_
#define _SPHAPP_H_

#include <signal.h>

#include "base/sphBase.h"
#include "app/sphSphere.h"
#include "solverbase/sphSolverFactory.h"
#include "solverbase/sphCouplerFactory.h"
#include "solverbase/sphFlowFactory.h"
#include "config/sphConfigController.h"
#include "skl_config/SklSolverConfig.h"

class sphApp: public sphBase
{
private:
  sphSphere* m_sphere;          ///< Sphere World Class
  bool m_usageFlag;             ///< usage flag
  bool m_p4Flag;                ///< p4 flag
  std::string m_config_fname;        ///< configuration fime name
  std::string m_runningPath;         ///< running path

protected:
  bool initialize();
  bool initializeParallel(int argc, char** argv);
  static void terminateSignalHandler(int signal_no);
  bool finalize();

public:
  sphApp();
  virtual ~sphApp();
  bool run(int argc, char** argv);
  void setUsageFlag(bool flag);
  void setP4Flag(bool flag);
  void setConfigFile(const char* config_fname);
};

#endif /* _SPHAPP_H_ */
