/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphExternalFunction.h
 * External Function Header File
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file sphExternalFunction.h
 * 外部関数ヘッダーファイル
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPHEXTERNALFUNCTION_H_
#define _SPHEXTERNALFUNCTION_H_

#include "solverbase/sphSolverBase.h"
#include "data/sphDataManager.h"
#include "parallel/sphParaManager.h"

extern "C" {

sphSolverBase* sph_getSolverObject(int solvID);
sphParaController* sph_getParaController(int solvID);
const sphDataManager* sph_getDataManager(int solvID);

void sph_isParallel_(int* ret);
void sph_getNumberOfGroup_(int* solvID, int* num, int* err);
void sph_getNumberOfNode_(int* solvID, int* num, int* err);
void sph_getWholeVoxelSize_(
                    int* solvID,
                    int* isz,
                    int* jsz,
                    int* ksz,
                    int* err);
void sph_getVoxelDivInfo_(int* solvID, int* idiv, int* jdiv, int* kdiv, int* err);
void sph_getVoxelHeadIndex_(
                    int* solvID,
                    int* id,
                    int* dim,
                    int* idx,
                    int* err);
void sph_getVoxelTailIndex_(
                    int* solvID,
                    int* id,
                    int* dim,
                    int* idx,
                    int* err);
void sph_getVoxelSize_(
                    int* solvID,
                    int* isz,
                    int* jsz,
                    int* ksz,
                    int* err);
void sph_getCommIDNum_(int* solvID, int* i, int* j, int* k, int* id, int* err);
void sph_getCommIDs_(int* solvID, int* i, int* j, int* k, int* id_list, int* err);
void sph_getCommIDEv_(int* solvID, int* i, int* j, int* k, int* id, int* err);
void sph_isCommID_(int* solvID, int* i, int* j, int* k, int* num, int* err);
void sph_getMyID_(int* solvID, int* id, int* err);
void sph_broadcast_(int* solvID, void* data, int* c, int* dtype, int* src, int* err);
void sph_send_(int* solvID, void* data, int* c, int* dtype, int* dst, int* err);
void sph_recv_(int* solvID, void* data, int* c, int* dtype, int* src, int* err);
void sph_iSend_(
                    int* solvID,
                    void* data,
                    int* c,
                    int* dtype,
                    int* dst,
                    int* key,
                    int* err);
void sph_iRecv_(
                    int* solvID,
                    void* data,
                    int* c,
                    int* dtype,
                    int* src,
                    int* key,
                    int* err);
void sph_allreduce_(
                    int* solvID,
                    void* sendbuf,
                    void* recvbuf,
                    int* c,
                    int* dtype,
                    int* optype,
                    int* err);
void sph_wait_(int* solvID, int* key, int* err);
void sph_waitAll_(int* solvID, int* key_num, int* key_list, int* err);
void sph_waitRequestAll_(int* solvID, int* err);
void sph_abort_(int* errcode);
void sph_barrier_(int* solvID, int* err);
void sph_commBndCell_(int* solvID, void* data, int* face, int* err);
void sph_commBndCellEv_(int* solvID, void* data, int* face, int* err);
void sph_commPeriodicBndCell_(int* solvID, void* data, int* face_dir, int* axis, int* face, int* err);

//void sph_commBndCell2_(int* solvID, void* data, int* face, int* wait_num, int* req, int* err);
//void sph_commCrdBndCell_(int* solvID, void* data, int* face, int* err);

void sph_printText_(int* solvID, char* file_label, char* text, int lbl_len, int txt_len);
double sph_getTime_();

};
#endif // _SPHEXTERNALFUNCTION_H_
