/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphBase.h
 * Sphere super Class Header File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file sphBase.h
 * Sphere基底クラスヘッダーファイル
 * @author     hira
 * @date       2009/10/01
 */

#ifndef _SPHBASE_H_
#define _SPHBASE_H_

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <iostream>
#include <map>
#include <vector>
#include <list>
#include <assert.h>

#ifdef SUPER_UX
#include <sys/types.h>
#include <sys/socket.h>
#endif // SUPER_UX

#ifndef _WIN32
#include <libgen.h>           // for linux
#include <sys/param.h>
#include <unistd.h> // for debug   // for linux
#include <sys/time.h>
#include "sph_file_util.h"

#else
#include "sph_win32_util.h"   // for win32
#endif

#ifdef _DEBUGLOG_
#include "debugLog.h"
#endif

#include "sphVersion.h"
#include "base/sphDefine.h"
#include "base/sphErrorHandler.h"
#include "spl/spl.h"
#include "spl/spl_errno.h"
#include "spl/splmpi.h"

/**
 * \~english
 * Sphere super Class
 *
 * \~japanese
 * Sphere基底クラス.
 * すべてのSphereの基底クラスである。
 * エラー処理等の基本処理を行う。
 */
class sphBase
{
private:

protected:
  /// エラーハンドラークラス
  sphErrorHandler* m_errHandler;

  /// Sphere Exit Flag for signal
  static bool m_sphereExitFlag;

public:
  sphBase();
  virtual ~sphBase();

  /**
   * sphereの強制終了であるか判断する。
   * @param     void
   * @return    true:強制終了/false:継続
   */
  bool isSphereExit() {return m_sphereExitFlag;};

};

#endif /* _SPHBASE_H_ */

