/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphMessageDefine.h
 * Error/Warning Message Define File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file sphMessageDefine.h
 * エラー/警告メッセージ定義ファイル
 * @author     hira
 * @date       2009/10/01
 */

#ifndef _SPHERRORDEFINE_H_
#define _SPHERRORDEFINE_H_

/// エラーメッセージのスタックサイズ（古いエラーメッセージは削除し、スタックサイズ以上スタックしない。）
#define SPH_ERROR_STACK_LIMIT          128

// エラーメッセージ番号
#define SPH_SUCCESS      0

// null, invalid
#define SPH_ERR_INVALID_PARAMETER  1000
#define SPH_ERR_NULL_POINTER    1001
#define SPH_ERR_CAST_NULL  1002
#define SPH_ERR_DATA_IS_NULL    1003
#define SPH_ERR_RETURN_FAULT    1004
#define SPH_ERR_INVALID_VALUE  1005

// low level
#define SPH_ERR_BUFFER_OVERFLOW    1010
#define SPH_ERR_CONSTRACTOR    1011
#define SPH_ERR_STL    1012
#define SPH_ERR_INITIALIZE    1013
#define SPH_ERR_INVALID_ENVIRONMENT  1014
#define SPH_ERR_NOT_SUPPORT_METHOD  1015
#define SPH_ERR_NOT_FOUND_IN_LIST  1016
#define SPH_ERR_STL_DUPLICATEKEY    1017
#define SPH_ERR_INTERNAL_IMPOSSIBLE  1018
#define SPH_ERR_TERMINATE_SIGNAL     1019

// MPI
#define SPH_ERR_MPI_ERROR    1020
#define SPH_ERR_ALLREDUCE_SUM  1021

// read/write file, memory
#define SPH_ERR_MEMORY_ALLOCATE    1030
#define SPH_ERR_READ_FILE    1031
#define SPH_ERR_WRITE_FILE    1032

// File Header
#define SPH_ERR_INVALID_DTYPE  1040
#define SPH_ERR_INVALID_ARRAY_SIZE  1041
#define SPH_ERR_INVALID_DIMS  1042
#define SPH_ERR_INVALID_VLEN  1043
#define SPH_ERR_INVALID_RLEN  1044
#define SPH_ERR_INVALID_CRDDEF  1045
#define SPH_ERR_INVALID_GC  1046

// XML
#define SPH_ERR_READ_FILE_XML  1050
#define SPH_ERR_WRITE_FILE_XML  1051
#define SPH_ERR_PARSE_XML  1052
#define SPH_ERR_CREATE_ELEMENT  1053
#define SPH_ERR_INVALID_XML  1054
#define SPH_ERR_EVALUATE_XPATH  1055
#define SPH_ERR_NOTFOUND_XPATH  1056
#define SPH_ERR_CIRCULAR_READ_FILE_XML  1057


// config
#define SPH_ERR_NOTFOUND_ELEMENT  1070
#define SPH_ERR_NOTFOUND_ATTRIBUTE  1071
#define SPH_ERR_INVALID_ELEMENT  1072
#define SPH_ERR_INVALID_ATTRIBUTE  1073
#define SPH_ERR_CONFIG_DUPLICATE_ID  1074
#define SPH_ERR_CONFIG_NOTFOUND_ID   1075
#define SPH_ERR_CONFIG_INVALID_ATTR_VALUE   1076


// warning
#define SPH_WARNING_NOT_EXIST_ELEMENT  1100
#define SPH_WARNING_NOT_EXIST_ATTRIBUTE  1101
#define SPH_WARNING_DO_NOTHING  1102
#define SPH_WARNING_NOTDEFINE_LABEL  1103
#define SPH_WARNING_NOT_EXIST_LABEL  1104
#define SPH_WARNING_NOT_EXIST_DATA  1105
#define SPH_WARNING_NULL_POINTER  1106


#define SPH_ERR_LASTCODE        80

// エラーメッセージ
typedef struct {
  int msg_no;
  char* message;
} type_error_list;

const type_error_list shp_error_list[SPH_ERR_LASTCODE-1] = {
    // NULL value
    {SPH_ERR_INVALID_PARAMETER,   "Invalid parameter"},
    {SPH_ERR_NULL_POINTER,        "Pointer is NULL"},
    {SPH_ERR_CAST_NULL ,          "Cast result is null"},
    {SPH_ERR_DATA_IS_NULL ,        "Data pointer is NULL"    },
    {SPH_ERR_RETURN_FAULT,         "Return of fault"},
    {SPH_ERR_INVALID_VALUE,   "Invalid values"},

    // low level
    {SPH_ERR_BUFFER_OVERFLOW ,    "Buffer over flow"      },
    {SPH_ERR_CONSTRACTOR,          "Constractor Error"},
    {SPH_ERR_STL,                  "STL error"},
    {SPH_ERR_INITIALIZE,          "Initialize error"},
    {SPH_ERR_INVALID_ENVIRONMENT,  "Invalid Environment"},
    {SPH_ERR_NOT_SUPPORT_METHOD,  "Not suport method"},
    {SPH_ERR_NOT_FOUND_IN_LIST ,  "Not found in list"},
    {SPH_ERR_STL_DUPLICATEKEY,    "Duplicate map key value."},
    {SPH_ERR_INTERNAL_IMPOSSIBLE, "Internal impossible error."},
    {SPH_ERR_TERMINATE_SIGNAL,    "terminate signal."},

    // MPI
    {SPH_ERR_MPI_ERROR ,          "MPI error"},
    {SPH_ERR_ALLREDUCE_SUM,        "Allreduce sum value failed"},

    // read/write file, memory
    {SPH_ERR_MEMORY_ALLOCATE,     "Memory allocate error"},
    {SPH_ERR_READ_FILE,           "File read error"},
    {SPH_ERR_WRITE_FILE,          "File write error"},

    // File Header
    {SPH_ERR_INVALID_DTYPE, "Invalid data type"},
    {SPH_ERR_INVALID_ARRAY_SIZE ,  "Invalid(or different) Array size"  },
    {SPH_ERR_INVALID_DIMS,        "Invalid dims."},
    {SPH_ERR_INVALID_VLEN,        "Invalid vlen."},
    {SPH_ERR_INVALID_RLEN,        "Invalid rlen."},
    {SPH_ERR_INVALID_CRDDEF,      "Invalid crddef."},
    {SPH_ERR_INVALID_GC,          "Invalid gc."},

    // XML
    {SPH_ERR_READ_FILE_XML,        "Xml File read error"},
    {SPH_ERR_WRITE_FILE_XML,       "XML File write error."},
    {SPH_ERR_PARSE_XML ,          "Xml Parse error"},
    {SPH_ERR_CREATE_ELEMENT,      "can not create xml element."},
    {SPH_ERR_INVALID_XML,          "Invalid Xml data"},
    {SPH_ERR_EVALUATE_XPATH,      "unable to evaluate xpath expression"},
    {SPH_ERR_NOTFOUND_XPATH,      "not found xml node of XPath"},
    {SPH_ERR_CIRCULAR_READ_FILE_XML, "Circular read XML file"},

    // warning
    {SPH_WARNING_NOT_EXIST_ELEMENT,    "Not exist element."},
    {SPH_WARNING_NOT_EXIST_ATTRIBUTE,   "Not exist attribute."},
    {SPH_WARNING_DO_NOTHING,      "do nothing."},
    {SPH_WARNING_NOTDEFINE_LABEL,      "Not Define label."},
    {SPH_WARNING_NOT_EXIST_LABEL,      "Not exist label."},
    {SPH_WARNING_NOT_EXIST_DATA,      "Not exist data object."},
    {SPH_WARNING_NULL_POINTER,      "Pointer is NULL."},

    // config
    {SPH_ERR_NOTFOUND_ELEMENT,       "not found xml element"},
    {SPH_ERR_NOTFOUND_ATTRIBUTE,        "not found xml attribute"},
    {SPH_ERR_INVALID_ELEMENT,           "invalid element."},
    {SPH_ERR_INVALID_ATTRIBUTE,           "invalid attribute."},
    {SPH_ERR_CONFIG_DUPLICATE_ID,      "Duplicate id in configXML."},
    {SPH_ERR_CONFIG_NOTFOUND_ID,      "Not found id in configXML."},
    {SPH_ERR_CONFIG_INVALID_ATTR_VALUE,      "Invalid attribute value in configXML."}

};

#endif /* _SPHERRORDEFINE_H_ */
