/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphConfigController.h
 * Configuration Control Class Header File
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file sphConfigController.h
 * コンフィギュレーション制御クラスヘッダーファイル
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPH_CONFIG_H_
#define _SPH_CONFIG_H_

#include "config/sphCfgBase.h"
#include "config/sphElem.h"
#include "skl_config/SklSolverConfig.h"

/**
 * \~english
 * Configuration Control Class
 *
 * \~japanese
 * コンフィギュレーション制御クラス
 *
 * コンフィギュレーションXMLファイルからの読み込みを行い、XML Documentの作成を行う。<br>
 * XMLファイルへのIOへの基本機能の提供であり、データ取得、検索機能は持たない。<br>
 * データ取得、検索機能は、sphCfgDataクラスが持つ。
 */
class sphConfigController : public sphCfgBase
{
private:

  sphConfigController();
  virtual ~sphConfigController();
  sphConfigController(sphConfigController&);
  void operator = (sphConfigController&);
  static sphConfigController* m_instance;

  /**
   * \~english
   * Read XML File List.
   * Temporary File List to check circular of reading XML File
   *
   * \~japanese
   * 読み込み済みXML ファイルリスト.
   * XMLファイルの循環読み込みのチェックの為の読み込み済みXMLファイルの一時リスト
   */
  std::vector<std::string> m_xmlFileStack;
  bool isCircularReadFile(const char* fname);

protected:
  /// \~english  Configuration Elements Data  \~japanese  コンフィグレーション要素データ
  sphElem* m_rootElem;
  std::string m_filename;
  SklCfg::SklSolverConfig* m_sklconfig;             ///< Vsphere Configuration

  bool parseStream(const char* stream, const size_t sz);
  xmlDocPtr readConfigFile(const char* fname, xmlNodePtr curNode);
  bool writeConfigFile(const char* fname, xmlDocPtr xmlDoc, bool one_file);
  xmlXPathObjectPtr getXPathObject(xmlNodePtr curNode, const char* xpathExpr) const;
  bool isClassicConfig(xmlNodePtr root);
  bool isSphereConfig(xmlNodePtr root);

public:
  static sphConfigController* getInstance();
  static void destruct();
  bool readConfigFile(const char* fname);
  const sphElem* getRootElem() const;
  const SklCfg::SklSolverConfig* getSklSolverConfig() const;
  bool isClassicConfig();
  bool isSphereConfig();

  // for debug
  bool writeConfigFile(const char* fname, bool one_file = false);
};
#endif /* _SPH_CONFIG_H_ */
