/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphConfigDefine.h
 * Define Configuration XML Parameter File
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file sphConfigDefine.h
 * コンフィギュレーションXMLのパラメータ定義ファイル
 *
 * 要素名、属性名の定義ファイル
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPH_CONFIG_DEFINE_H_
#define _SPH_CONFIG_DEFINE_H_

// tag name in Configuration XML
#define SPH_CFG_ELEM_SPHERECONFIG         "SphereConfig"

// XML Element name in Configuration XML
#define SPH_CFG_NAME_XML_CONFIG     "sph_xml_config_file"
#define SPH_CFG_NAME_SOLVER_LIST     "SphSolverList"
#define SPH_CFG_NAME_SOLVER     "SphSolver"
#define SPH_CFG_NAME_COUPLER_LIST     "SphCouplerList"
#define SPH_CFG_NAME_COUPLER     "SphCoupler"
#define SPH_CFG_NAME_FLOW     "SphFlow"
#define SPH_CFG_NAME_NODE_ASSIGN     "SphNodeAssign"
#define SPH_CFG_NAME_STEER     "SphSteer"
#define SPH_CFG_NAME_NODELIST     "SphNodeList"
#define SPH_CFG_NAME_NODE     "SphNode"
#define SPH_CFG_NAME_DIVISION     "SphVoxelDivision"
#define SPH_CFG_NAME_NODEPROCESS     "SphNodeProcess"
#define SPH_CFG_NAME_STEPTIME     "SphStepTime"
#define SPH_CFG_NAME_DOMAININFO     "SphDomainInfo"
#define SPH_CFG_NAME_VOXELORIGIN           "SphVoxelOrigin"
#define SPH_CFG_NAME_VOXELPITCH           "SphVoxelPitch"
#define SPH_CFG_NAME_VOXELSIZE           "SphVoxelSize"
#define SPH_CFG_NAME_STARTINDEX           "SphStartIndex"
#define SPH_CFG_NAME_VOXELWIDTH           "SphVoxelWidth"
#define SPH_CFG_NAME_STARTCONDITION           "ShpStartCondition"
#define SPH_CFG_NAME_BASEDIR           "SphDir"
#define SPH_CFG_NAME_SIZE           "SphSize"
#define SPH_CFG_NAME_DATALIST           "SphDataList"
#define SPH_CFG_NAME_DATAOBJ           "SphDataObj"
#define SPH_CFG_NAME_FILELIST           "SphFileList"
#define SPH_CFG_NAME_FILE           "SphFile"
#define SPH_CFG_NAME_FILEOPTION           "SphFileOption"
#define SPH_CFG_NAME_MULTIFILE           "SphMultiFile"
#define SPH_CFG_NAME_DATACOPY           "SphDataCopy"
#define SPH_CFG_NAME_REFOBJ           "SphRefObj"
#define SPH_CFG_NAME_UDTOBJ           "SphUdtObj"
#define SPH_CFG_NAME_FILEFORMAT           "SphFileNameFormat"
#define SPH_CFG_NAME_USERDEFINE           "SphUserDefine"
#define SPH_CFG_NAME_PARALLELTEMPLATE           "SphParallelTemplate"
#define SPH_CFG_NAME_MULTIPLYPARTITIONING           "SphMultiplyConnectedPartitioning"
#define SPH_CFG_NAME_EQUALPARTITIONING           "SphEqualPartitioning"


// attribute name
#define SPH_CFG_ATTR_VER      "version"
#define SPH_CFG_ATTR_FILENAME      "file_name"
#define SPH_CFG_ATTR_LABEL      "label"
#define SPH_CFG_ATTR_ID      "id"
#define SPH_CFG_ATTR_VALUE      "value"
#define SPH_CFG_ATTR_COMMENT      "comment"
#define SPH_CFG_ATTR_SOLVERID      "solverID"
#define SPH_CFG_ATTR_COUPLERID      "couplerID"
#define SPH_CFG_ATTR_SIZE      "size"
#define SPH_CFG_ATTR_NODEIDS      "nodeIDs"
#define SPH_CFG_ATTR_MODE      "mode"
#define SPH_CFG_ATTR_ENABLE      "enable"
#define SPH_CFG_ATTR_DIMS      "dims"
#define SPH_CFG_ATTR_CLASS      "class"
// for SphStartIndex
#define SPH_CFG_ATTR_I      "i"
#define SPH_CFG_ATTR_J      "j"
#define SPH_CFG_ATTR_K      "k"
#define SPH_CFG_ATTR_ASSIGN      "assign"
// for SphVoxelSize, SphSize
#define SPH_CFG_ATTR_IX      "ix"
#define SPH_CFG_ATTR_JX      "jx"
#define SPH_CFG_ATTR_KX      "kx"
// for SphVoxelOrigin
#define SPH_CFG_ATTR_OX      "ox"
#define SPH_CFG_ATTR_OY      "oy"
#define SPH_CFG_ATTR_OZ      "oz"
// for SphVoxelPitch, SphVoxelWidth
#define SPH_CFG_ATTR_DX      "dx"
#define SPH_CFG_ATTR_DY      "dy"
#define SPH_CFG_ATTR_DZ      "dz"
// for SphNode
#define SPH_CFG_ATTR_NODEID      "nodeID"
#define SPH_CFG_ATTR_GROUPID      "group"
// for SphDataObj
#define SPH_CFG_ATTR_FORMAT      "format"
#define SPH_CFG_ATTR_DATATYPE      "data_type"
#define SPH_CFG_ATTR_DOMAIN      "domain"
#define SPH_CFG_ATTR_CRDDEF      "crddef"
#define SPH_CFG_ATTR_DATANUM      "data_num"
#define SPH_CFG_ATTR_GC      "gc"
#define SPH_CFG_ATTR_INSTANCE      "instance"
// for SphFile
#define SPH_CFG_ATTR_TYPE      "type"
#define SPH_CFG_ATTR_IO_NODEID      "io_nodeID"
#define SPH_CFG_ATTR_PREFIX      "prefix"
// for SphFileOption
#define SPH_CFG_ATTR_BLOCKSIZE      "block_size"
#define SPH_CFG_ATTR_EXTRECORD      "extrec_flag"
#define SPH_CFG_ATTR_RLEN      "rlen"
#define SPH_CFG_ATTR_SBXVAILDNUM      "sbx_validnum"
#define SPH_CFG_ATTR_SBXVAILDFLAG      "sbx_validflag"
#define SPH_CFG_ATTR_INTERVAL      "interval"
#define SPH_CFG_ATTR_GCFLAG      "guide_out"
#define SPH_CFG_ATTR_BITSHANDLING      "sbx_bitshandling"
#define SPH_CFG_ATTR_GCINSPH      "gc_insph"
// for SphDir
#define SPH_CFG_ATTR_INPUTDIR      "input"
#define SPH_CFG_ATTR_OUTPUTDIR      "output"
// for SphDataCopy
#define SPH_CFG_ATTR_EXTGC      "ext_gc"
// for SphStepTime
#define SPH_CFG_ATTR_MAXSTEP      "max_step"
#define SPH_CFG_ATTR_BASESTEP      "base_step"
#define SPH_CFG_ATTR_BASETIME      "base_time"
#define SPH_CFG_ATTR_DELTATIME      "delta_time"
// for SphParallelTemplate/SphMultiplyConnectedPartitioning/SphEqualPartitioning
#define SPH_CFG_ATTR_MBXGROUPING      "mbx_grouping"
#define SPH_CFG_ATTR_NUMBEROFSUBBLOCK      "numberof_subblock"

// default value
#define SPH_CFG_DEF_MAX_STEPS      1000
#define SPH_CFG_DEF_DELTA_TIME      0.1

// attribute value
#define SPH_CFG_LABEL_SIMPLEVOXEL      "sph_simple_voxel"
#define SPH_CFG_SOLVER_CHECKMODE      "check"
#define SPH_CFG_SBX_MASKUPPER      "maskupper"
#define SPH_CFG_SBX_SHIFTLOW      "shiftlow"
#define SPH_CFG_FILEIO_MULTI      "block"
#define SPH_CFG_FILEIO_WHOLE      "whole"
#define SPH_CFG_FILEIO_PROCESS      "process"
#define SPH_CFG_ASSIGN_BLOCK      "block"
#define SPH_CFG_ASSIGN_WHOLE      "whole"
#define SPH_CFG_ASSIGN_PROCESS      "process"


/**
 * ファイル入出力モード(mode属性)
 */
typedef enum {
  CFG_FILE_UNKNOWN = -1,       ///< 不明
  CFG_FILE_DEFAULT = 0,       ///< mode属性が存在しない。
  CFG_FILE_MULTI = 1,         ///< 分割(サブブロック)ファイル入出力
  CFG_FILE_PROCESS = 2,       ///< プロセスファイル入出力
  CFG_FILE_WHOLE = 3         ///< 一括ファイル入出力
} SphCfgFileIOMode;


/**
 * ファイル始点インデックスタイプ(assign属性)
 */
typedef enum {
  CFG_ASSIGN_UNKNOWN = 0,       ///< 不明
  CFG_ASSIGN_BLOCK = 1,       ///< サブブロックの始点インデックス
  CFG_ASSIGN_PROCESS = 2,         ///< プロセスの始点インデックス
  CFG_ASSIGN_WHOLE = 3,       ///< ソルバ全体の計算領域の始点インデックス
} SphCfgAssignType;

#endif /* _SPH_CONFIG_DEFINE_H_ */
