/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphDataManager.h
 * Data Manager Class Header File
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file sphDataManager.h
 * データマネージャクラスヘッダーファイル
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPHDATAMANAGER_H_
#define _SPHDATAMANAGER_H_

#include "base/sphBase.h"
#include "data/sphData.h"
#include "data/sphArray.h"
#include "data/sphVoxArray.h"
#include "data/sphUnstData.h"
#include "data/sphVoxContainer.h"
#include "config/sphElem.h"

/**
 * \~english
 * Data Manager Class
 *
 * \~japanese
 * データマネージャクラス
 */
class sphDataManager : public sphBase
{
  protected:
    /// \~english  Registered Data Map  \~japanese  登録済みデータマップ
    std::map<std::string, sphData*> m_mapDataList;

  public:
    sphDataManager();
    virtual ~sphDataManager();

    sphData* createVoxelArray(
                          const sphElem* data_elem,
                          const sphParaController *paraCtrlr);

    sphData* createVoxelArray(
                          SphDCType dcType,
                          SPL_Datatype dataType,
                          const char* label,
                          const sphParaController *paraCtrlr,
                          const size_t gc = 0,
                          const size_t data_num = 1,
                          SphCrdDef crddef = CRDDEF_REGULAR);
    sphData* createVoxelArray(
                          SphDCType dcType,
                          SPL_Datatype dataType,
                          const sphParaController *paraCtrlr,
                          const size_t gc = 0,
                          const size_t data_num = 1,
                          SphCrdDef crddef = CRDDEF_REGULAR);

    sphData* createVoxelArray(
                          SphDCType dcType,
                          SPL_Datatype dataType,
                          const char* label,
                          const size_t array_size[3],
                          const size_t gc = 0,
                          const size_t data_num = 1,
                          SphCrdDef crddef = CRDDEF_REGULAR);

    sphData* createVoxelArray(
                          SphDCType dcType,
                          SPL_Datatype dataType,
                          const size_t array_size[3],
                          const size_t gc = 0,
                          const size_t data_num = 1,
                          SphCrdDef crddef = CRDDEF_REGULAR);

    sphData* createUnstData(
                          const sphElem* data_elem,
                          const sphParaController *paraCtrlr);

    sphData* createUnstData(
                          SphDCType dcType,
                          SPL_Datatype dtType,
                          const char* label,
                          const sphParaController *paraCtrlr,
                          const size_t vlen = 1);

    sphData* createUnstData(
                          SphDCType dcType,
                          SPL_Datatype dtType,
                          const sphParaController *paraCtrlr,
                          const size_t vlen = 1);

    sphData* createVoxelContainer(
                          const sphElem* data_elem,
                          const sphParaController *paraCtrlr);

    sphData* createVoxelContainer(
                          SphDCType dcType,
                          SPL_Datatype dataType,
                          const char* label,
                          const sphParaController *paraCtrlr,
                          const size_t gc,
                          const size_t data_num,
                          SphCrdDef crddef);

    sphData* createVoxelContainer(
                                  SphDCType dcType,
                                  SPL_Datatype dataType,
                                  const sphParaController *paraCtrlr,
                                  const size_t gc,
                                  const size_t data_num,
                                  SphCrdDef crddef = CRDDEF_REGULAR);

    sphData* getDataObj(const char* label) const;
    sphData* getDataObj(void* ptr) const;
    bool deleteDataObj(const char* label);
    bool deleteDataObj(sphData* data);
    bool registDataObj(const char* label, sphData* data);

};
#endif /* _SPHDATAMANAGER_H_ */
