/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphDataUtils.h
 * Data Utilities Class Header File
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file sphDataUtils.h
 * データユーティリティクラスヘッダーファイル
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPHDATAUTILS_H_
#define _SPHDATAUTILS_H_

#include "base/sphBase.h"
#include "data/sphDataDefs.h"
#include "data/sphVoxArray.h"


/**
 * \~english
 * Data Utilities Class
 *
 * \~japanese
 * データユーティリティクラス
 */
class sphDataUtils: public sphBase
{
protected:
//-----------------------------------------------------------
//  Calc index method
//-----------------------------------------------------------
  // for 2D
  static inline void calcIndex(size_t dst_ilen,  // input
                                size_t dst_jlen,  // input
                                size_t dst_gc,    // input
                                size_t src_gc,    // input
                                size_t& dst_ix,   // output
                                size_t& dst_jx,   // output
                                int& diff,          // output
                                size_t& sta);     // output
  // for 3D
  static inline void calcIndex(size_t dst_ilen,  // input
                                size_t dst_jlen,  // input
                                size_t dst_klen,  // input
                                size_t dst_gc,    // input
                                size_t src_gc,    // input
                                size_t& dst_ix,   // output
                                size_t& dst_jx,   // output
                                size_t& dst_kx,   // output
                                int& diff,          // output
                                size_t& sta);     // output

public:
  sphDataUtils();
  virtual ~sphDataUtils();
  static int getDimsOfSphDCType(SphDCType dctype);

  static unsigned char shiftLowerBits(unsigned char dt, unsigned int bit) {
    return (dt >> bit);
  };
  static unsigned char maskUpperBits(unsigned char dt, unsigned int bit) {
    return (dt & (0xff >> bit));
  };

  //-----------------------------------------------------------
  //  Index calc utility
  //-----------------------------------------------------------
  // size_t imax  sz[0]
  // size_t jmax  sz[1]
  // size_t kmax  sz[2]
  template <typename T_sz>
  static size_t getCindexS2D  (const T_sz* sz, size_t gc,
                                 size_t i, size_t j);
  template <typename T_sz>
  static size_t getCindexS3D  (const T_sz* sz, size_t gc,
                                 size_t i, size_t j, size_t k);
  template <typename T_sz>
  static size_t getCindexV2D  (const T_sz* sz, size_t gc,
                                 size_t i, size_t j, size_t l);
  template <typename T_sz>
  static size_t getCindexV2DEx(const T_sz* sz, size_t gc,
                                 size_t l, size_t i, size_t j);
  template <typename T_sz>
  static size_t getCindexV3D  (const T_sz* sz, size_t gc,
                                 size_t i, size_t j, size_t k, size_t l);
  template <typename T_sz>
  static size_t getCindexV3DEx(const T_sz* sz, size_t gc,
                                 size_t l, size_t i, size_t j, size_t k);
  template <typename T_sz>
  static size_t getFindexS2D  (const T_sz* sz, size_t gc,
                                 size_t i, size_t j);
  template <typename T_sz>
  static inline size_t getFindexS3D(const T_sz* sz, size_t gc,
                                size_t i, size_t j, size_t k);
  template <typename T_sz>
  static size_t getFindexV2D  (const T_sz* sz, size_t gc,
                                 size_t i, size_t j, size_t l);
  template <typename T_sz>
  static size_t getFindexV2DEx(const T_sz* sz, size_t gc,
                                 size_t l, size_t i, size_t j);
  template <typename T_sz>
  static size_t getFindexV3D  (const T_sz* sz, size_t gc,
                                 size_t i, size_t j, size_t k, size_t l);
  template <typename T_sz>
  static size_t getFindexV3DEx(const T_sz* sz, size_t gc,
                                 size_t l, size_t i, size_t j, size_t k);

  //-----------------------------------------------------------
  //  Data convert utility
  //-----------------------------------------------------------
  // sphData
  static bool copyArray(sphData* dst, const sphData* src);
  static bool copyArray(sphData* dst,
                          const sphData* src,
                          const size_t copy_size[3],
                          const size_t dst_start_idx[3],
                          const size_t src_start_idx[3]);

  // 2D
  static bool avrS2D        (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src);
  static bool avrV2D        (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src);
  static bool scaleAvrS2D   (sphVoxArray<float>* src,
                             unsigned stepAvr = 1,
                             bool mode = false);

  static bool avrS2D        (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src);
  static bool avrV2D        (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src);
  static bool scaleAvrS2D   (sphVoxArray<double>* src,
                             unsigned stepAvr = 1,
                             bool mode = false);

  template <typename T_dt>
  static bool cpyS2D        (sphVoxArray<T_dt>* dst,
                             const sphVoxArray<T_dt>* src);
  static bool shiftVin2D    (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src,
                             float v00[2],
                             unsigned stepAvr = 1);
  static bool shiftVout2D   (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src,
                             float v00[2],
                             unsigned stepAvr = 1);
  static bool scaleAvrV2D   (sphVoxArray<float>* vec,
                             unsigned stepAvr = 1,
                             bool mode = false);

  static bool shiftVin2D    (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src,
                             double v00[2],
                             unsigned stepAvr = 1);
  static bool shiftVout2D   (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src,
                             double v00[2],
                             unsigned stepAvr = 1);
  static bool scaleAvrV2D   (sphVoxArray<double>* vec,
                             unsigned stepAvr = 1,
                             bool mode = false);

  template <typename T_dt>
  static bool cpyS2DCenter  (sphVoxArray<T_dt>* dst,
                             const sphVoxArray<T_dt>* src);

  // 3D
  static bool avrS3D        (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src);
  static bool avrV3D        (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src);
  static bool scaleAvrS3D   (sphVoxArray<float>* src,
                             unsigned stepAvr = 1,
                             bool mode = false);

  static bool avrS3D        (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src);
  static bool avrV3D        (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src);
  static bool scaleAvrS3D   (sphVoxArray<double>* src,
                             unsigned stepAvr = 1,
                             bool mode = false);

  template <typename T_dt>
  static bool cpyS3D        (sphVoxArray<T_dt>* dst,
                             const sphVoxArray<T_dt>* src);

  static bool shiftVin3D    (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src,
                             float v00[3],
                             unsigned stepAvr = 1);
  static bool shiftVout3D   (sphVoxArray<float>* dst,
                             const sphVoxArray<float>* src,
                             float v00[3],
                             unsigned stepAvr = 1);
  static bool scaleAvrV3D   (sphVoxArray<float>* vec,
                             unsigned stepAvr = 1,
                             bool mode = false);

  static bool shiftVin3D    (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src,
                             double v00[3],
                             unsigned stepAvr = 1);
  static bool shiftVout3D   (sphVoxArray<double>* dst,
                             const sphVoxArray<double>* src,
                             double v00[3],
                             unsigned stepAvr = 1);
  static bool scaleAvrV3D   (sphVoxArray<double>* vec,
                             unsigned stepAvr = 1,
                             bool mode = false);

  template <typename T_dt>
  static bool cpyS3DCenter  (sphVoxArray<T_dt>* dst,
                             const sphVoxArray<T_dt>* src);

  // 4D
  template <typename T_dt>
  static bool cpyS4D        (sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src);
  template <typename T_dt>
  static bool cpyS4D        (sphVoxArray<T_dt>* dst, size_t dst_pos,
                             const sphVoxArray<T_dt>* src, size_t src_pos);
  template <typename T_dt>
  static bool cpyS4D        (sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src,
                              size_t pos);

  template <typename T_dt>
  static bool cpyS4DEx      (sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src);
  template <typename T_dt>
  static bool cpyS4DEx      (sphVoxArray<T_dt>* dst, size_t dst_pos,
                             const sphVoxArray<T_dt>* src, size_t src_pos);
  template <typename T_dt>
  static bool cpyS4DEx      (sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src,
                              size_t pos);
  template <typename T_dt>
  static bool convS4D       (sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src);
};


/**
 * ２つの2Dデータをコピーする時のコピー先サイズ、オフセット値を算出する.
 * @param[in]    dst_ilen        I方向サイズ（ガイドセルを含む）
 * @param[in]    dst_ilen        J方向サイズ（ガイドセルを含む）
 * @param[in]    dst_gc          コピー先ガイドセル
 * @param[in]    src_gc          コピー元ガイドセル
 * @param[out]    dst_ix          コピー先I方向サイズ（ガイドセルを含む）
 * @param[out]    dst_jx          コピー先J方向サイズ（ガイドセルを含む）
 * @param[out]    diff            ガイドセルの差
 * @param[out]    sta             コピー先始点位置
 * @return    void
 */
inline void
sphDataUtils::calcIndex(size_t dst_ilen,  // dimension size /w guide cell for dst
                        size_t dst_jlen,
                        size_t dst_gc,    // size of guide cell
                        size_t src_gc,
                        size_t& dst_ix,   // return
                        size_t& dst_jx,
                        int& diff,
                        size_t& sta) {
  diff = src_gc - dst_gc;
  if( src_gc >= dst_gc ){
    /// コピー先ガイドセルが小さいのですべてコピーする。
    sta = 0;
    dst_ix = dst_ilen; dst_jx = dst_jlen;
  }
  else {
    /// コピー先ガイドセルが大きいのでガイドセルの差分を引く。
    sta = abs(diff);
    dst_ix = dst_ilen+diff; dst_jx = dst_jlen+diff;
  }
}


/**
 * ２つの3Dデータをコピーする時のコピー先サイズ、オフセット値を算出する。
 * @param[in]    dst_ilen        I方向サイズ（ガイドセルを含む）
 * @param[in]    dst_jlen        J方向サイズ（ガイドセルを含む）
 * @param[in]    dst_klen        K方向サイズ（ガイドセルを含む）
 * @param[in]    dst_gc          コピー先ガイドセル
 * @param[in]    src_gc          コピー元ガイドセル
 * @param[out]    dst_ix          コピー先I方向サイズ（ガイドセルを含む）
 * @param[out]    dst_jx          コピー先J方向サイズ（ガイドセルを含む）
 * @param[out]    dst_kx          コピー先J方向サイズ（ガイドセルを含む）
 * @param[out]    diff            ガイドセルの差
 * @param[out]    sta             コピー先始点位置
 * @return    void
 */
inline void
sphDataUtils::calcIndex(size_t dst_ilen,  // dimension size /w guide cell for dst
                        size_t dst_jlen,
                        size_t dst_klen,
                        size_t dst_gc,    // size of guide cell
                        size_t src_gc,
                        size_t& dst_ix,   // return
                        size_t& dst_jx,
                        size_t& dst_kx,
                        int& diff,
                        size_t& sta) {
  diff = src_gc - dst_gc;
  if( src_gc >= dst_gc ){
    /// コピー先ガイドセルが小さいのですべてコピーする。
    sta = 0;
    dst_ix = dst_ilen; dst_jx = dst_jlen; dst_kx = dst_klen;
  }
  else {
    /// コピー先ガイドセルが大きいのでガイドセルの差分を引く。
    sta = abs(diff);
    dst_ix = dst_ilen+diff; dst_jx = dst_jlen+diff; dst_kx = dst_klen+diff;
  }
}


/**
 * ガイドセルを含むI,Jインデックスからの領域のインデックスを算出する。(Scalar2D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含む）
 * @param[in]    j              J方向インデックス（ガイドセルを含む）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getCindexS2D(const T_sz* sz, size_t gc,
                      size_t i, size_t j) {
  return ( (sz[0]+gc*2)*j + i );
}


/**
 * ガイドセルを含むI,J,Kインデックスからの領域のインデックスを算出する。(Scalar3D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含む）
 * @param[in]    j              J方向インデックス（ガイドセルを含む）
 * @param[in]    k              K方向インデックス（ガイドセルを含む）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getCindexS3D(const T_sz* sz, size_t gc,
                      size_t i, size_t j, size_t k) {
  return ( (sz[0]+gc*2)*(sz[1]+gc*2)*k + (sz[0]+gc*2)*j + i );
}


/**
 * ガイドセルを含むI,Jインデックスからの領域のインデックスを算出する。(Vector2D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含む）
 * @param[in]    j              J方向インデックス（ガイドセルを含む）
 * @param[in]    l              データ数インデックス
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getCindexV2D(const T_sz* sz, size_t gc,
                      size_t i, size_t j, size_t l) {
  return ( (sz[0]+gc*2)*(sz[1]+gc*2)*l + (sz[0]+gc*2)*j + i );
}


/**
 * ガイドセルを含むI,Jインデックスからの領域のインデックスを算出する。(Vector2DEx)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    l              データ数インデックス
 * @param[in]    i              I方向インデックス（ガイドセルを含む）
 * @param[in]    j              J方向インデックス（ガイドセルを含む）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getCindexV2DEx(const T_sz* sz, size_t gc,
                        size_t l, size_t i, size_t j) {
  return ( 2*(sz[0]+gc*2)*j + 2*i + l );
}


/**
 * ガイドセルを含むI,J,Kインデックスからの領域のインデックスを算出する。(Vector3D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含む）
 * @param[in]    j              J方向インデックス（ガイドセルを含む）
 * @param[in]    k              J方向インデックス（ガイドセルを含む）
 * @param[in]    l              データ数インデックス
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getCindexV3D(const T_sz* sz, size_t gc,
                      size_t i, size_t j, size_t k, size_t l) {
  return ( (sz[0]+gc*2)*(sz[1]+gc*2)*(sz[2]+gc*2)*l
         + (sz[0]+gc*2)*(sz[1]+gc*2)*k
         + (sz[0]+gc*2)*j + i );
}


/**
 * ガイドセルを含むI,J,Kインデックスからの領域のインデックスを算出する。(Vector3DEx)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    l              データ数インデックス
 * @param[in]    i              I方向インデックス（ガイドセルを含む）
 * @param[in]    j              J方向インデックス（ガイドセルを含む）
 * @param[in]    k              J方向インデックス（ガイドセルを含む）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getCindexV3DEx(const T_sz* sz, size_t gc,
                        size_t l, size_t i, size_t j, size_t k) {
  return ( 3*(sz[0]+gc*2)*(sz[1]+gc*2)*k + 3*(sz[0]+gc*2)*j + 3*i + l );
}


/**
 * ガイドセルを含まないI,Jインデックスからの領域のインデックスを算出する。(Scalar2D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含まない）
 * @param[in]    j              J方向インデックス（ガイドセルを含まない）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getFindexS2D(const T_sz* sz, size_t gc,
                      size_t i, size_t j) {
  return ( (sz[0]+gc*2)*(j+gc-1) + i+gc-1 );
}


/**
 * ガイドセルを含まないI,J,Kインデックスからの領域のインデックスを算出する。(Scalar3D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含まない）
 * @param[in]    j              J方向インデックス（ガイドセルを含まない）
 * @param[in]    k              J方向インデックス（ガイドセルを含まない）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getFindexS3D(const T_sz* sz, size_t gc,
                      size_t i, size_t j, size_t k) {
  return ( (sz[0]+gc*2)*(sz[1]+gc*2)*(k+gc-1)
         + (sz[0]+gc*2)*(j+gc-1) + i+gc-1 );
}


/**
 * ガイドセルを含まないI,Jインデックスからの領域のインデックスを算出する。(Vector2D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含まない）
 * @param[in]    j              J方向インデックス（ガイドセルを含まない）
 * @param[in]    l              データ数インデックス
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getFindexV2D(const T_sz* sz, size_t gc,
                      size_t i, size_t j, size_t l) {
  return ( (sz[0]+gc*2)*(sz[1]+gc*2)*l
         + (sz[0]+gc*2)*(j+gc-1) + i+gc-1 );
}


/**
 * ガイドセルを含まないI,Jインデックスからの領域のインデックスを算出する。(Vector2DEx)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    l              データ数インデックス
 * @param[in]    i              I方向インデックス（ガイドセルを含まない）
 * @param[in]    j              J方向インデックス（ガイドセルを含まない）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getFindexV2DEx(const T_sz* sz, size_t gc,
                        size_t l, size_t i, size_t j) {
  return ( 2*(sz[0]+gc*2)*(j+gc-1) + 2*(i+gc-1) + l );
}


/**
 * ガイドセルを含まないI,J,Kインデックスからの領域のインデックスを算出する。(Vector3D)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    i              I方向インデックス（ガイドセルを含まない）
 * @param[in]    j              J方向インデックス（ガイドセルを含まない）
 * @param[in]    k              K方向インデックス（ガイドセルを含まない）
 * @param[in]    l              データ数インデックス
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getFindexV3D(const T_sz* sz, size_t gc,
                      size_t i, size_t j, size_t k, size_t l) {
  return ( (sz[0]+gc*2)*(sz[1]+gc*2)*(sz[2]+gc*2)*l
         + (sz[0]+gc*2)*(sz[1]+gc*2)*(k+gc-1)
         + (sz[0]+gc*2)*(j+gc-1) + i+gc-1 );
}

/**
 * ガイドセルを含まないI,J,Kインデックスからの領域のインデックスを算出する。(Vector3DEx)
 * @param[in]    sz              サイズ（ガイドセルを含まない）
 * @param[in]    gc              ガイドセル
 * @param[in]    l              データ数インデックス
 * @param[in]    i              I方向インデックス（ガイドセルを含まない）
 * @param[in]    j              J方向インデックス（ガイドセルを含まない）
 * @param[in]    k              K方向インデックス（ガイドセルを含まない）
 * @return       １次配列インデックス
 */
template <typename T_sz>
inline size_t
sphDataUtils::getFindexV3DEx(const T_sz* sz, size_t gc,
                        size_t l, size_t i, size_t j, size_t k) {
  return ( 3*(sz[0]+gc*2)*(sz[1]+gc*2)*(k+gc-1)
         + 3*(sz[0]+gc*2)*(j+gc-1) + 3*(i+gc-1) + l );
}



/**
 * Scalar2Dの２つのデータをコピーする。
 * @param[out]    dst        コピー先Scalar2Dデータ
 * @param[in]    src        コピー元Scalar2Dデータ
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS2D(sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src) {
  if( !dst || !src ) return false;
  return dst->copyArray(src);
}

/**
 * Scalar3Dの２つのデータをコピーする。
 * @param[out]    dst        コピー先Scalar3Dデータ
 * @param[in]    src        コピー元Scalar3Dデータ
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS3D(sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src) {
  if( !dst || !src ) return false;
  return dst->copyArray(src);
}


/**
 * Scalar2Dのデータを中央部分にコピーする。
 * @param[out]    dst       コピー先Scalar2Dデータ
 * @param[in]    src        コピー元Scalar2Dデータ
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS2DCenter(sphVoxArray<T_dt>* dst,
                      const sphVoxArray<T_dt>* src) {
  if( !dst || !src ) return false;
  return dst->copyCenter(src);
}



/**
 * Scalar3Dのデータを中央部分にコピーする。
 * @param[out]    dst       コピー先Scalar3Dデータ
 * @param[in]    src        コピー元Scalar3Dデータ
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS3DCenter(sphVoxArray<T_dt>* dst,
                      const sphVoxArray<T_dt>* src) {
  if( !dst || !src ) return false;
  return dst->copyCenter(src);
}

// --------------------- Scalar4D<T_dt> copy methods  ---------------------
/**
 * Scalar4Dの２つのデータをコピーする。
 * @param[out]    dst        コピー先Scalar4Dデータ
 * @param[in]    src        コピー元Scalar4Dデータ
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS4D(sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src) {
  if( !dst || !src ) return false;
  return dst->copyArray(src);
}


/**
 * Scalar4Dのベクトルデータの１つの成分をコピーする。
 * @param[out]    dst        コピー先Scalar4Dデータ
 * @param[in]    dst_pos     コピー先成分インデックス(0〜ベクトル数-1)
 * @param[in]    src         コピー元Scalar4Dデータ
 * @param[in]    src_pos     コピー元成分インデックス(0〜ベクトル数-1)
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS4D(sphVoxArray<T_dt>* dst, size_t dst_pos,
                     const sphVoxArray<T_dt>* src, size_t src_pos) {
  if( !dst || !src ) return false;
  return dst->copyComponent(src, dst_pos, src_pos);
}


/**
 * Scalar3Dデータ又Scalar4Dデータ成分をScalar3Dデータ又はScalar4Dデータ成分にコピーする。
 * @param[out]    dst        コピー先Scalar4Dデータ
 * @param[in]    src         コピー元Scalar3Dデータ
 * @param[in]    pos         コピー成分インデックス(0〜ベクトル数-1)
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS4D( sphVoxArray<T_dt>* dst,
                      const sphVoxArray<T_dt>* src,
                      size_t pos) {

  if( !dst || !src ) return false;
  size_t dst_pos = 0;
  size_t src_pos = 0;
  if ( dst->getNumOfDtElem() > 1) {
    dst_pos = pos;
  }
  else if (src->getNumOfDtElem() > 1) {
    src_pos = pos;
  }
  return dst->copyComponent(src, dst_pos, src_pos);
}


/**
 * Scalar4DExの２つのデータをコピーする。
 * @param[out]    dst        コピー先Scalar4Dデータ
 * @param[in]    src        コピー元Scalar4Dデータ
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS4DEx(sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src) {
  if( !dst || !src ) return false;
  return dst->copyArray(src);
}


/**
 * Scalar4DExのベクトルデータの１つの成分をコピーする。
 * @param[out]    dst        コピー先Scalar4Dデータ
 * @param[in]    dst_pos     コピー先成分インデックス(0〜ベクトル数-1)
 * @param[in]    src         コピー元Scalar4Dデータ(0〜ベクトル数-1)
 * @param[in]    src_pos     コピー元成分インデックス(0〜ベクトル数-1)
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS4DEx(sphVoxArray<T_dt>* dst, size_t dst_pos,
                     const sphVoxArray<T_dt>* src, size_t src_pos) {
  if( !dst || !src ) return false;
  return dst->copyComponent(src, dst_pos, src_pos);
}


/**
 * Scalar3Dデータ又Scalar4DExデータ成分をScalar3Dデータ又はScalar4DExデータ成分にコピーする。
 * @param[out]    dst        コピー先Scalar4Dデータ
 * @param[in]    src         コピー元Scalar3Dデータ
 * @param[in]    pos         コピー成分インデックス(0〜ベクトル数-1)
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::cpyS4DEx( sphVoxArray<T_dt>* dst,
                      const sphVoxArray<T_dt>* src,
                      size_t pos) {

  if( !dst || !src ) return false;
  size_t dst_pos = 0;
  size_t src_pos = 0;
  if ( dst->getNumOfDtElem() > 1) {
    dst_pos = pos;
  }
  else if (src->getNumOfDtElem() > 1) {
    src_pos = pos;
  }
  return dst->copyComponent(src, dst_pos, src_pos);
}


/**
 * Scalar4DデータとScalar4DExデータを相互に変換する。
 * @param[out]    dst        コピー先Scalar4D/Scalar4DExデータ
 * @param[in]    src         コピー元Scalar4D/Scalar4DExデータ
 * @return    success(=true)/fault(=false)
 */
template <typename T_dt>
bool
sphDataUtils::convS4D(sphVoxArray<T_dt>* dst, const sphVoxArray<T_dt>* src) {
  if( !dst || !src ) return false;
  return dst->copyReverse(src);
}



#endif /* _SPHDATAUTILS_H_ */
