#ifndef DEBUGLOG_H_
#define DEBUGLOG_H_

#define _DEBUGLOG_ 1

#include <stdio.h>
#include <iostream>
#include <time.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "spl/util_endian.h"
#include "spl/splmpi.h"


inline void debugLog(const char* msg, const char* file = NULL, int line = 0, const char* func = NULL) {

  struct timeval tv;
  struct tm *today;

  if (access("logs", 0) != 0) {
    return;
  }
  if (gettimeofday(&tv, 0) < 0) {
    return;
  }
  today = localtime(&tv.tv_sec);
  char buf[100];
  sprintf(buf, "%4d/%02d/%02d %02d:%02d:%02d", today->tm_year + 1900, today->tm_mon+1, today->tm_mday, today->tm_hour, today->tm_min,  today->tm_sec);

  char name[100];
  sprintf(name, "logs/pid_%d.log", getpid());
  FILE* fp = fopen(name, "a");
  if (file != NULL) {
    fprintf(fp,"%s[PID=%d] %s:%d [%s] : %s\n", buf, getpid(), file, line, func, msg);
  }
  else {
    fprintf(fp,"%s[PID=%d] %s\n", buf, getpid(), msg);
  }

  fclose(fp);
};
inline void debugLogHex(const void *_p, int n, const char* file, int line, const char* func)  /* メモリ内容を16進でプリント */
{

  struct timeval tv;
  struct tm *today;

  if (access("logs", 0) != 0) {
    return;
  }
  if (gettimeofday(&tv, 0) < 0) {
    return;
  }
  today = localtime(&tv.tv_sec);
  char tm_buf[100];
  sprintf(tm_buf, "%4d/%02d/%02d %02d:%02d:%02d", today->tm_year + 1900, today->tm_mon+1, today->tm_mday, today->tm_hour, today->tm_min,  today->tm_sec);

  const unsigned char *p = (unsigned char *) _p;  /* 受け取ったパラメータの型を統一している */
  int  i;

  char msg[4092];
  char buf[32];
  memset(msg, NULL, 4092);
  for(i = 0; i < n; ++i, ++p)
  {
    memset(buf, NULL, 32);
      sprintf(buf, "%02x ", *p);
    strcat(msg, buf);
  }

  char name[100];
  sprintf(name, "logs/pid_%d.log", getpid());
  FILE* fp = fopen(name, "a");
  fprintf(fp,"%s[PID=%d] %s:%d [%s] : %s\n", tm_buf, getpid(), file, line, func, msg);
  fclose(fp);
}


inline void debugLogDump(const void *_p, const size_t size[3], size_t dtype)  /* メモリ内容を16進でプリント */
{

  struct timeval tv;
  struct tm *today;

  if (access("logs", 0) != 0) {
    return;
  }
  if (gettimeofday(&tv, 0) < 0) {
    return;
  }
  today = localtime(&tv.tv_sec);
  char tm_buf[100];
  sprintf(tm_buf, "%4d/%02d/%02d %02d:%02d:%02d", today->tm_year + 1900, today->tm_mon+1, today->tm_mday, today->tm_hour, today->tm_min,  today->tm_sec);

  const unsigned char *p = (unsigned char *) _p;  /* 受け取ったパラメータの型を統一している */
  size_t  i, j, k;

  char msg[4092];
  char buf[32];
  memset(msg, NULL, 4092);

  char name[100];
  sprintf(name, "logs/pid_%d.log", getpid());
  FILE* fp = fopen(name, "a");

  for (k=0; k<size[2];k++) {
    fprintf(fp,"k=%ld\n", k);
    fprintf(fp,"j/i   0 - %ld\n", size[0]-1);
    for (j=0; j<size[1]; j++) {
      fprintf(fp, "%05ld:", j);
      for (i=0; i<size[0]; i++) {
        char hex[64] = "";
        size_t idx = size[0]*size[1]*k + size[0]*j + i;
//          idx *= len;
        if (dtype == 1) {
          sprintf(hex, "%02X ", ((unsigned char*)_p)[idx]);
        }
        else if (dtype == 2) {
          unsigned short value = ((unsigned short*)_p)[idx];
          BSwap16(&value);
          sprintf(hex, "%04X ", value);
        }
        else if (dtype == 4) {
          unsigned int value = ((unsigned int*)_p)[idx];
          BSwap32(&value);
          sprintf(hex, "%08X ", value);
        }
        else if (dtype == 8) {
          unsigned long long value = ((unsigned long long*)_p)[idx];
          BSwap64(&value);
          unsigned int hex_value;
          hex_value = (unsigned int)(value >> 32);
          char hexU[64] = "";
          sprintf(hexU, "%08X", hex_value);
          hex_value = (unsigned int)(value & 0x00000000FFFFFFFF);
          sprintf(hex, "%s%08X ", hexU, hex_value);
        }
        fprintf(fp,hex);
      }
      fprintf(fp,"\n");
    }
    fprintf(fp,"\n");
  }
  fprintf(fp,"\n");

  fclose(fp);
}

inline void debugLogDumpVector(const void *_p, const size_t size[3], size_t dtype, size_t vlen, bool rev_flag)
{

  struct timeval tv;
  struct tm *today;

  if (access("logs", 0) != 0) {
    return;
  }
  if (gettimeofday(&tv, 0) < 0) {
    return;
  }
  today = localtime(&tv.tv_sec);
  char tm_buf[100];
  sprintf(tm_buf, "%4d/%02d/%02d %02d:%02d:%02d", today->tm_year + 1900, today->tm_mon+1, today->tm_mday, today->tm_hour, today->tm_min,  today->tm_sec);

  const unsigned char *p = (unsigned char *) _p;  /* 受け取ったパラメータの型を統一している */
  size_t  i, j, k, l, n;

  char msg[4092];
  char buf[32];
  memset(msg, NULL, 4092);

  char name[100];
  sprintf(name, "logs/pid_%d.log", getpid());
  FILE* fp = fopen(name, "a");


  for (l=0; l<vlen; l++) {
    if (!rev_flag) {
      fprintf(fp,"vlen=%ld\n", l);
    }
    else {
      if (l==1) break;
    }
    for (k=0; k<size[2];k++) {
      fprintf(fp,"k=%ld\n", k);
      if (!rev_flag) {
        fprintf(fp,"j/i   0 - %ld\n", size[0]-1);
      }
      else {
        fprintf(fp,"j/i   0 - %ld [vlen=%ld]\n", size[0]-1, vlen);
      }
      for (j=0; j<size[1]; j++) {
        fprintf(fp, "%05ld:", j);
        for (i=0; i<size[0]; i++) {
          for (n=0; n<vlen; n++) {
            if (!rev_flag) {
              if (n==1) break;
            }
            char hex[1024] = "";
            size_t idx;
            if (!rev_flag) {
                idx = size[0]*size[1]*size[2]*l + size[0]*size[1]*k + size[0]*j + i;
            }
            else {
              idx = size[0]*size[1]*vlen*k + size[0]*vlen*j + vlen*i + n;
            }
            if (dtype == 1) {
              sprintf(hex, "%02X ", ((unsigned char*)_p)[idx]);
            }
            else if (dtype == 2) {
              unsigned short value = ((unsigned short*)_p)[idx];
              BSwap16(&value);
              sprintf(hex, "%04X ", value);
            }
            else if (dtype == 4) {
              unsigned int value = ((unsigned int*)_p)[idx];
              BSwap32(&value);
              sprintf(hex, "%08X ", value);
            }
            else if (dtype == 8) {
              unsigned long long value = ((unsigned long long*)_p)[idx];
              BSwap64(&value);
              unsigned int hex_value;
              hex_value = (unsigned int)(value >> 32);
              char hexU[64] = "";
              sprintf(hexU, "%08X", hex_value);
              hex_value = (unsigned int)(value & 0x00000000FFFFFFFF);
              sprintf(hex, "%s%08X ", hexU, hex_value);
            }
            fprintf(fp,hex);
          }
        }
        fprintf(fp,"\n");
      }
      fprintf(fp,"\n");
    }
    fprintf(fp,"\n");
  }
  fprintf(fp,"\n");

  fclose(fp);

}



inline void debugDump(const void *_p, size_t size[3], size_t dtype, char* fname = NULL)
{
  register size_t i, j, k;
  size_t total_size = 1;
  for (i=0; i< 3; i++) total_size *= size[i];
  if (total_size <= 0) {
    printf("error : total size is zero, size[i,j,k]=%ld, %ld, %ld.\n", size[0], size[1], size[2]);
    return;
  }

  if (fname == NULL) {
    for (k=0; k<size[2];k++) {
      printf("k=%ld\n", k);
      printf("j/i   0 - %ld\n", size[0]-1);
      for (j=0; j<size[1]; j++) {
        printf("%05ld:", j);
        for (i=0; i<size[0]; i++) {
          char hex[64] = "";
          size_t idx = size[0]*size[1]*k + size[0]*j + i;
//          idx *= len;
          if (dtype == 1) {
            sprintf(hex, "%02X ", ((unsigned char*)_p)[idx]);
          }
          else if (dtype == 2) {
            unsigned short value = ((unsigned short*)_p)[idx];
            BSwap16(&value);
            sprintf(hex, "%04X ", value);
          }
          else if (dtype == 4) {
            unsigned int value = ((unsigned int*)_p)[idx];
            BSwap32(&value);
            sprintf(hex, "%08X ", value);
          }
          else if (dtype == 8) {
            unsigned long long value = ((unsigned long long*)_p)[idx];
            BSwap64(&value);
            unsigned int hex_value;
            hex_value = (unsigned int)(value >> 32);
            char hexU[64] = "";
            sprintf(hexU, "%08X", hex_value);
            hex_value = (unsigned int)(value & 0x00000000FFFFFFFF);
            sprintf(hex, "%s%08X ", hexU, hex_value);
          }
          printf(hex);
        }
        printf("\n");
      }
      printf("\n");
    }
    printf("\n");
  }
  else {
    FILE* fp = fopen(fname, "wb");
    fwrite(_p, total_size, dtype, fp);
    fclose(fp);
  }
}


inline void debugDumpVector(const void *_p, size_t size[3], size_t len, size_t vlen)
{
  register size_t i, j, k, n;
  size_t total_size = 1;
  for (i=0; i< 3; i++) total_size *= size[i];
  if (total_size <= 0) {
    printf("error : total size is zero, size[i,j,k]=%ld, %ld, %ld.\n", size[0], size[1], size[2]);
    return;
  }

  for (k=0; k<size[2];k++) {
    printf("k=%ld\n", k);
    printf("j/i[vlen=%ld]   0 - %ld\n", vlen, size[0]-1);
    for (j=0; j<size[1]; j++) {
      printf("%05ld:", j);
      for (i=0; i<size[0]; i++) {
        for (n=0; n<vlen; n++) {
          char hex[64] = "";
          size_t idx = (size[0]*size[1]*k + size[0]*j + i)*vlen + n;
  //          idx *= len;
          if (len == 1) {
            sprintf(hex, "%02X ", ((unsigned char*)_p)[idx]);
          }
          else if (len == 2) {
            unsigned short value = ((unsigned short*)_p)[idx];
            BSwap16(&value);
            sprintf(hex, "%04X ", value);
          }
          else if (len == 4) {
            unsigned int value = ((unsigned int*)_p)[idx];
            BSwap32(&value);
            sprintf(hex, "%08X ", value);
          }
          else if (len == 8) {
            unsigned long long value = ((unsigned long long*)_p)[idx];
            BSwap64(&value);
            unsigned int hex_value;
            hex_value = (unsigned int)(value >> 32);
            char hexU[64] = "";
            sprintf(hexU, "%08X", hex_value);
            hex_value = (unsigned int)(value & 0x00000000FFFFFFFF);
            sprintf(hex, "%s%08X ", hexU, hex_value);
          }
          printf(hex);
        }
      }
      printf("\n");
    }
    printf("\n");
  }
  printf("\n");

}


/**
 * データを数値文字列として出力する。
 * @param      data           出力データ
 * @param      dtype           データ型
 * @param      size           出力サイズ
 * @return    success(=true)/fault(=false)
 */
inline void debugDumpNumber(const void* data, SPL_Datatype dtype, const size_t size[4])
{

  struct timeval tv;
  struct tm *today;

  if (access("logs", 0) != 0) {
    return;
  }
  if (gettimeofday(&tv, 0) < 0) {
    return;
  }
  today = localtime(&tv.tv_sec);
  char tm_buf[100];
  sprintf(tm_buf, "%4d/%02d/%02d %02d:%02d:%02d", today->tm_year + 1900, today->tm_mon+1, today->tm_mday, today->tm_hour, today->tm_min,  today->tm_sec);

  const unsigned char *p = (unsigned char *) data;  /* 受け取ったパラメータの型を統一している */
  size_t  i, j, k, l, n;

  char msg[4092];
  char buf[32];
  memset(msg, NULL, 4092);

  char name[100];
  sprintf(name, "logs/pid_%d.log", getpid());
  FILE* fp = fopen(name, "a");

  for (l=0; l<size[3]; l++) {
    for (k=0; k<size[2]; k++) {
      fprintf(fp,"k=%ld\n", k);
      fprintf(fp,"j/i   0 - %ld\n", size[0]-1);
      for (j=0; j<size[1]; j++) {
        fprintf(fp, "%05ld:", j);
        for (i=0; i<size[0]; i++) {
          char hex[64] = {0};
          size_t idx = size[0]*size[1]*size[2]*l+size[0]*size[1]*k + size[0]*j + i;
          if (dtype == SPL_CHAR) {
            char value = ((char*)data)[idx];
            sprintf(hex, "%3d ", value);
          }
          else if (dtype == SPL_UNSIGNED_CHAR) {
            unsigned char value = ((unsigned char*)data)[idx];
            sprintf(hex, "%3d ", value);
          }
          else if (dtype == SPL_SHORT) {
            short value = ((short*)data)[idx];
            sprintf(hex, "%5d ", value);
          }
          else if (dtype == SPL_UNSIGNED_SHORT) {
            unsigned short value = ((unsigned short*)data)[idx];
            sprintf(hex, "%5d ", value);
          }
          else if (dtype == SPL_INT) {
            int value = ((int*)data)[idx];
            sprintf(hex, "%8d ", value);
          }
          else if (dtype == SPL_UNSIGNED) {
            unsigned int value = ((unsigned int*)data)[idx];
            sprintf(hex, "%8d ", value);
          }
          else if (dtype == SPL_LONG) {
            long value = ((long*)data)[idx];
            sprintf(hex, "%8ld ", value);
          }
          else if (dtype == SPL_UNSIGNED_LONG) {
            unsigned long value = ((unsigned long*)data)[idx];
            sprintf(hex, "%8ld ", value);
          }
          else if (dtype == SPL_LONG_LONG_INT) {
            long long value = ((long long*)data)[idx];
            sprintf(hex, "%8ld ", value);
          }
          else if (dtype == SPL_FLOAT) {
            float value = ((float*)data)[idx];
            sprintf(hex, "%e ", value);
          }
          else if (dtype == SPL_DOUBLE) {
            double value = ((double*)data)[idx];
            sprintf(hex, "%e ", value);
          }
          fprintf(fp, hex);
        }
        fprintf(fp,"\n");
      }
      fprintf(fp,"\n");
    }
    fprintf(fp,"\n");
  }

  fprintf(fp,"\n");

  fclose(fp);
  return;
}


/**
 * ファイルにバイナリ出力する。
 *
 * logsフォルダに出力する。
 * @param _p      書込データポインタ
 * @param size    書込データのバイト長さ
 * @param len     書込データの数
 * @param pref    出力ファイルの接頭文字
 */
inline void debugDump(const void *_p, size_t size, size_t len, char* pref = NULL)
{

  struct timeval tv;
  struct tm *today;

  if (access("logs", 0) != 0) {
    return;
  }
  if (gettimeofday(&tv, 0) < 0) {
    return;
  }
  today = localtime(&tv.tv_sec);
  char tm_buf[100];
  sprintf(tm_buf, "%4d%02d%02d%02d%02d%02d", today->tm_year + 1900, today->tm_mon+1, today->tm_mday, today->tm_hour, today->tm_min,  today->tm_sec);

  char name[100];
  if (pref) {
    sprintf(name, "logs/pid_%d_%s_%s.dump", getpid(), tm_buf, pref);
  }
  else {
    sprintf(name, "logs/pid_%d_%s.dump", getpid(), tm_buf);
  }
  FILE* fp = fopen(name, "wb");
  fwrite(_p, size, len, fp);
  fclose(fp);
}
#endif /*DEBUGLOG_H_*/

