/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphFile.h
 * Sphere File Input/Output Base Class Header File
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphFile.h
 * Sphereのファイル入出力基底クラスヘッダーファイル
 * @author     hira
 * @date       2009/09/07
 */
#ifndef _SPHFILE_H_
#define _SPHFILE_H_


#include <iostream>
#include <string>
#include <vector>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "base/sphBase.h"
#include "data/sphData.h"
#include "data/sphVoxArray.h"
#include "fileio/sphFileHeader.h"
#include "spl/spl.h"
#include "spl/splmpi.h"

/**
 * \~english
 * Shpere File Input/Output Base Class Header File
 *
 * \~japanese
 * Sphereのファイル入出力基底クラスヘッダーファイル
 */
class sphFile : public virtual sphBase
{
private:


protected:

  int m_rankId;          ///< 読込/書込ランク番号
  SPL_Comm m_procGrp;      ///< 読込/書込プロセスグループ

    /// 初期化を行う。
  virtual bool initialize();

public:
  sphFile();
  virtual ~sphFile();

  virtual bool finalize();
  virtual sphData* readFile() = 0;
  virtual bool writeFile() = 0;
  virtual bool getFileInfo(const char* filename, sphFileHeader* header) =0;
  virtual const sphFileHeader* getFileInfo() = 0;
  virtual bool setFileInfo(const sphFileHeader* header) = 0;

  virtual sphData* getData() = 0;
  virtual void setData(sphData* data) = 0;

  /**
   * 入出力制御関数
   */
  /// 読み込みモードを設定する。
  virtual void setReadMode(int mode) = 0;
  /// 書き込みモードを設定する。
  virtual void setWriteMode(int mode) = 0;
  /// 圧縮書込みを行うかのフラグを設定する。(true:圧縮書込みを行う。, false:圧縮書込みを行わない。)
  virtual void setCompressionFlag(bool flag) = 0;
  /// 拡張レコードを書込みかのフラグを設定する。(true:拡張レコードを書き込む。, false:拡張レコードを書き込まない。)
  virtual void setExtRecodeFlag(bool flag) = 0;

  /**
   * データ操作関数
   */
  /// 上位ビットデータをマスクし(0とする）、下位２ビットのみのデータとする。
  virtual sphData* maskUpperBits() = 0;
  /// 下位２ビットをシフトし、上位のみのデータとする。
  virtual sphData* shiftLowBits() = 0;

  /// 読込/書込プロセスグループを設定する
  void setProcGrp(SPL_Comm procGrp);
  /// 読込/書込ランク番号を設定する
  void setRankId(int rankId);

};

#endif /* _SPHFILE_H_ */

