/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphSpxHeader.h
 * SPX File Header Infomation Class Header File
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphSbxFile.h
 * SPXファイルヘッダー情報クラスのヘッダーファイル
 * @author     hira
 * @date       2009/09/07
 */


#ifndef _SPHSPXDATASET_H_
#define _SPHSPXDATASET_H_

#include "fileio/sphSvHeader.h"
#include "spl/splfio_spx.h"

class sphSpxFile;
class sphSphFile;

/**
 * SPXファイルヘッダー情報クラス
 */
class sphSpxHeader: public sphSvHeader
{
  friend class sphSpxFile;
  friend class sphSphFile;

private:
#undef HEADER_FORMAT
#define HEADER_FORMAT             "spx"

private:

  /// header format
  std::string m_foramt;

protected:

  void getSplSpxHeader(splSpx* spx);
  void setSplSpxHeader(const splSpx* spx);

  virtual const size_t* getWholeSize()  const;
  virtual const size_t* getStartIdx()  const;
  virtual unsigned int getDims()  const;
  virtual unsigned int getVlen()  const;
  virtual unsigned int getDtype()  const;
  virtual unsigned int getRlen()  const;
  virtual unsigned int getCellDataLength() const;
  virtual size_t getVoxelDataLength() const;
  virtual size_t getGc() const;
  virtual unsigned int getCrddef() const;
  virtual unsigned int getAux() const;
  virtual const size_t* getSize() const;
  virtual const double* getOrigin() const;
  virtual const double* getPitch() const;
  virtual size_t getBlockSize() const;
  virtual size_t getStep() const;
  virtual double getTime() const;

  virtual void setSize(const size_t size[3]);
  virtual void setBlockSizeBySize();
  virtual void setBlockSize(size_t block_size);
  virtual std::string getFormat();
  virtual void setStartIdx(const size_t start_idx[3]);
  virtual void setGc(size_t gc);
  virtual void setDims(int dims);
  virtual void setWholeSize(const size_t wsize[3]);
  virtual void setVlen(unsigned int vlen);
  virtual void setDtype(unsigned int dtype);
  virtual void setRlen(unsigned int rlen);
  virtual void setStepTime(size_t step, double time);
  virtual void setOrigin(const double org[3]);
  virtual void setPitch(const double pitch[3]);
  virtual void setCrddef(unsigned int crddef);
  virtual void setAux(unsigned int aux);

public:
  sphSpxHeader();
  virtual ~sphSpxHeader();
  sphSpxHeader(const sphSpxHeader& spxHeader);
  sphSpxHeader& operator = (const sphSpxHeader& spxHeader);

  virtual bool initialize();
  virtual bool finalize();
  virtual bool checkHeader() const;
  virtual void printHeaderInfo();

  /**
   * struct _spl_spx_:splSpx Member
   */
  /** whole voxel size for extended record*/
  size_t m_wsize[3];

  /**
   * global start index for node
   *
   * start_idx[0] x-dir start
   * start_idx[1] y-dir start
   * start_idx[2] z-dir start
   */
  size_t m_start_idx[3];

  /** dimension ( 1:1D, 2:2D, 3:3D ) */
  unsigned int m_dims;

  /** vector length */
  unsigned int m_vlen;

  /** data type ( 1:scaler data 3:vector data ) */
  unsigned int m_dtype;

  /** guide cell size */
  size_t m_gc;

  /** real length ( 4:float, 8:double ) */
  unsigned int m_rlen;

  /**
   * origin definision
   *
   *  1:Regular
   *  2:Collocate
   *  3:Staggered(1)
   *  4:Staggered(2)
   */
  unsigned int m_crddef;

  /** extended data */
  unsigned int m_aux;

  /** voxel size */
  size_t m_size[3];

  /** origin */
  double m_origin[3];

  /** pitch */
  double m_pitch[3];

  /** step */
  size_t m_step;

  /** time */
  double m_time;

  /** block size */
  size_t m_block_size;

};

#endif /* _SPHSPXDATASET_H_ */
