/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphSvFile.h
 * Sphere Simple Voxel File Input/Output Class Header File
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphSvFile.h
 * SphereのSimple Voxelファイル入出力クラスヘッダーファイル
 * @author     hira
 * @date       2009/09/07
 */

#ifndef _SPHSVFILE_H_
#define _SPHSVFILE_H_

#include "fileio/sphFile.h"
#include "fileio/sphSvHeader.h"
#include "spl/splfio_sv_file.h"

/**
 * \~english
 * Shpere File Input/Output Class Header File
 *
 * \~japanese
 * Sphereのファイル入出力クラスヘッダーファイル
 */
class sphSvFile: public sphFile
{
private:

protected:
  splSvFileList* m_fileList;    ///< 読込/書込ファイルリスト

    /// 初期化を行う。
  virtual bool initialize();
  bool createFileHeader(sphSvHeader* header);

  /// 読み込みファイル情報を取得する。
  splSvFile* getSplFile(int id);
  /// 読み込みファイル情報を取得する。
  bool getFileName(
            int id,
            char* filename,
            size_t& sidx_i, size_t& sidx_j, size_t& sidx_k);
  /// 読み込みファイル情報を取得する。
  bool getFileName(
            int id,
            char* filename);
  /// 読み込みファイルリストのファイル数を取得する。
  size_t getFileListCount();
  /// 書込前準備を行う。
  void* preWriteFile();

  /// 圧縮書込みを行うかのフラグを取得する。(true:圧縮書込みを行う。, false:圧縮書込みを行わない。)
  virtual bool isCompressionFlag() { return false;};
  /// 読み込みモードを取得する。
  virtual int getReadMode() const = 0;
  /// 書き込みモードを取得する。
  virtual int getWriteMode() const = 0;

public:
  sphSvFile();
  virtual ~sphSvFile();

  virtual bool finalize();
  virtual sphData* readFile() = 0;
  virtual bool writeFile() = 0;
  virtual bool getFileInfo(const char* filename, sphFileHeader* header) =0;
  virtual const sphFileHeader* getFileInfo() const = 0;
  virtual sphFileHeader* getFileInfo() = 0;
  virtual bool setFileInfo(const sphFileHeader* header) = 0;

  virtual sphData* getData() = 0;
  virtual void setData(sphData* data) = 0;

  /**
   * 入出力制御関数
   */
  /// 読み込みモードを設定する。
  virtual void setReadMode(int mode) = 0;
  /// 書き込みモードを設定する。
  virtual void setWriteMode(int mode) = 0;
  /// 圧縮書込みを行うかのフラグを設定する。(true:圧縮書込みを行う。, false:圧縮書込みを行わない。)
  virtual void setCompressionFlag(bool flag) { return;};
  /// 拡張レコードを書込みかのフラグを設定する。(true:拡張レコードを書き込む。, false:拡張レコードを書き込まない。)
  virtual void setExtRecodeFlag(bool flag) { return;};

  /**
   * データ操作関数
   */
  /// 上位ビットデータをマスクし(0とする）、下位２ビットのみのデータとする。
  virtual sphData* maskUpperBits();
  /// 下位２ビットをシフトし、上位のみのデータとする。
  virtual sphData* shiftLowBits();

  /**
   * 読込/書込ファイル関数
   */
  /// 読み込みファイルリストにファイル、始点インデックスを追加する。
  bool addSvFileInfo(const char* filename, const size_t sidx[3]);
  /// 読み込みファイルリストにファイルを追加する。
  bool addSvFileName(const char* filename);
  /// 読み込みファイルリストをクリアする。
  void clearSvFileList();
  /// 読み込みファイルリストを設定する。
  void setSvFileList(splSvFileList* filelist);

};

#endif /* _SPHSVFILE_H_ */
