/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphSvHeader.h
 * Sphere Simple Voxel File Header Information Class Header File
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphSvHeader.h
 * SphereのSimple Voxel ファイルヘッダー情報クラスヘッダーファイル
 * @author     hira
 * @date       2009/09/07
 */

#ifndef _SPHSVHEADER_H_
#define _SPHSVHEADER_H_

#include "fileio/sphFileHeader.h"
#include "spl/splfio_sv_data.h"
#include "data/sphDataDefs.h"

/**
 * SphereのSimple Voxel ファイルヘッダー情報クラス
 */
class sphSvFile;

class sphSvHeader : public sphFileHeader
{
  friend class sphFileController;
  friend class sphSvFile;
  friend class sphSolverSv;

protected:
  virtual std::string getFormat() = 0;
  virtual const size_t* getStartIdx() const;
  virtual unsigned int getDims() const = 0;
  virtual unsigned int getVlen()  const = 0;
  virtual unsigned int getDtype()  const = 0;
  virtual unsigned int getRlen()  const =0;
  virtual unsigned int getCellDataLength() const = 0;
  virtual size_t getVoxelDataLength() const = 0;
  virtual size_t getGc() const = 0;
  virtual unsigned int getCrddef() const;
  virtual unsigned int getAux() const;
  virtual const size_t* getSize() const = 0;
  virtual const double* getOrigin() const = 0;
  virtual const double* getPitch() const = 0 ;
  virtual size_t getBlockSize() const = 0;
  virtual const size_t* getWholeSize()  const;
  virtual size_t getStep() const = 0;
  virtual double getTime() const = 0;

  virtual void setSize(const size_t size[3]) = 0;
  virtual void setStartIdx(const size_t start_idx[3]);
  virtual void setBlockSizeBySize() = 0;
  virtual void setBlockSize(size_t block_size) = 0;
  virtual void setGc(size_t gc) = 0;
  virtual void setDims(int dims) = 0;
  virtual void setWholeSize(const size_t wsize[3]);
  virtual void setVlen(const unsigned int vlen) = 0;
  virtual void setDtype(const unsigned int dtype) = 0;
  virtual void setRlen(const unsigned int dtype) = 0;
  virtual void setStepTime(size_t step, double time) = 0;
  virtual void setOrigin(const double org[3]) = 0;
  virtual void setPitch(const double pitch[3]) = 0;
  virtual void setCrddef(unsigned int crddef);
  virtual void setAux(unsigned int aux);

public:
  sphSvHeader();
  virtual ~sphSvHeader();

  virtual bool initialize() = 0;
  virtual bool finalize() = 0;
  virtual bool checkHeader() const = 0;
  virtual void printHeaderInfo() = 0;

  virtual void getDataExistFlags(unsigned int& bits, unsigned int& dataflag);
  virtual void setDataExistFlags(unsigned int bits, unsigned int dataflag);

};

#endif /* _SPHSVHEADER_H_ */
