/*
 * sphTextFile.h
 *
 *  Created on: 2010/01/04
 *      Author: hira
 */

#ifndef _SPHTEXTFILE_H_
#define _SPHTEXTFILE_H_

#include "base/sphBase.h"
#include "config/sphElem.h"
#include "base/sphStepTime.h"
#include "data/sphData.h"

class sphTextFile: public sphBase
{
protected:

  int m_rankId;          ///< 読込/書込ランク番号
  SPL_Comm m_procGrp;      ///< 読込/書込プロセスグループ
  const sphElem* m_configElem;          ///< コンフィグレーションファイル要素
  FILE *m_fp;                           ///< テキストファイルポインタ
  const sphStepTime *m_steptime;        ///< 現在ステップ・時刻

  /**
   * 書込みモード
   * - ０：全てのノードがファイル出力する
   * - １：識別子rankIdのノードのみファイルを出力する。
   */
  int m_writeMode;
  size_t m_interval;            ///< 書込インターバル
  std::string m_outputPath;     ///< 書込フォルダ名
  std::string m_filename;       ///< 書込ファイル名

public:
  sphTextFile();
  virtual ~sphTextFile();

  bool openFile(bool init = false);
  bool print(const char* fmt, ...) const;
  bool printLine(const char* fmt, ...) const;
  bool printWithConsole(const char* fmt, ...) const;
  bool printLineWithConsole(const char* fmt, ...) const;
  bool printDumpHex(const void* data, SPL_Datatype dtype, const size_t size[3], size_t data_num = 1, bool rev_flag = false) const;
  bool printDump(const void* data, SPL_Datatype dtype, const size_t size[3], size_t data_num = 1, bool rev_flag = false) const;
  bool printDumpHex(const sphData* data) const;
  bool printDump(const sphData* data) const;
  bool print(std::string msg) const;
  bool printWithConsole(std::string msg) const;
  bool close();
  bool flush();

  bool setConfigElem(const sphElem* config_elem);
  bool setStepTime(const sphStepTime* steptime);
  bool setFileName(const char* file_name);
  bool getFileName(char* file_name) const;
  bool setBaseDir(const char* dir);
  bool getBaseDir(char* dir) const;
  bool setInterval(size_t interval);
  size_t getInterval() const;
  void setProcGrp(SPL_Comm procGrp);
  void setRankId(int rankId);
  SPL_Comm getProcGrp() const;
  int getRankId() const;

};

#endif /* _SPHTEXTFILE_H_ */
