/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphParaManager.h
 * Parallel Manager Class Header File
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file sphParaManager.h
 * パラレルマネージャークラスヘッダーファイル
 * @author     hira
 * @date       2009/02/20
 */
#ifndef _SPHPARAMANAGER_H_
#define _SPHPARAMANAGER_H_

#include "base/sphBase.h"
#include "parallel/sphParaController.h"
#include "parallel/sphUnstMesh.h"
#include "config/sphElem.h"

class sphParaManager : public sphBase
{
protected:
  int m_mapKey;               ///< m_mapParaCtrlのキーとなるユニーク番号（１〜）
  std::map<int, sphParaController*> m_mapParaCtrl;

  int addParaController(sphParaController* paraCtrl);

  sphParaController* createUnstParaController(
                  unsigned int proc_num,
                  SPL_Comm comm );

  // SphParallelTemplate要素の取得
  bool validParallelTemplate(const sphElem* solverElem);
  bool existsParallelTemplate(const sphElem* solverElem);
  bool isMultiplyConnectedPartitioning(const sphElem* solverElem);
  int countEqualPartitioning(const sphElem* solverElem);

public:
  sphParaManager();
  virtual ~sphParaManager();

  // 構造格子パラレルコントローラ作成
  sphParaController* createSvParaController(
                  const sphElem* solverElem,
                  SPL_Comm comm);

  sphParaController* createSvParaController(
                  int dims,
                  const size_t wsize[3],
                  const size_t wstart[3],
                  const sphElem* nodelist_elem,
                  const double voxelorigin[3],
                  const double voxelpitch[3],
                  SPL_Comm comm,
                  bool mbx_grouping = false);

  sphParaController* createSvParaController(
                  const size_t voxelsize[3],
                  const size_t startindex[3],
                  const double voxelorigin[3],
                  const double voxelpitch[3],
                  unsigned int division[3],
                  SPL_Comm comm,
                  unsigned int block_count = 0);

  sphParaController* createSvParaController(
                  const size_t voxelsize[3],
                  const size_t startindex[3],
                  const double voxelorigin[3],
                  const double voxelpitch[3],
                  unsigned int proc_num,
                  SPL_Comm comm,
                  unsigned int block_count = 0);

  sphParaController* createSvParaController(
                  unsigned int proc_num,
                  const size_t wsize[3],
                  const size_t wstart[3],
                  const size_t (*voxelsize)[3],
                  const size_t (*startindex)[3],
                  const double voxelorigin[3],
                  const double voxelpitch[3],
                  SPL_Comm comm,
                  const unsigned int *groupid_table = NULL);

  sphParaController* createSvParaControllerForNewGroup(
                  unsigned int proc_num,
                  const size_t wsize[3],
                  const size_t wstart[3],
                  const size_t (*voxelsize)[3],
                  const size_t (*startindex)[3],
                  const double voxelorigin[3],
                  const double voxelpitch[3],
                  SPL_Comm old_comm,
                  int* procid_list,
                  const unsigned int *groupid_table = NULL);

  // 非構造格子パラレルコントローラ作成
  sphParaController* createUnstParaController(
                               const sphElem* solverElem,
                               SPL_Comm comm);
  sphParaController* createUnstParaController(
                               const sphElem* solverElem,
                               SPL_Comm comm,
                               const char* fn_node,
                               const char* fn_elem,
                               const char* fn_grp = NULL,
                               const char* fn_bnd = NULL,
                               sphUnstMesh::MeshDataType dtype = sphUnstMesh::MESH_DOUBLE);
  sphParaController* createUnstParaController(
                               const sphElem* solverElem,
                               SPL_Comm comm,
                               const char* dist_fname,
                               sphUnstMesh::MeshDataType dtype = sphUnstMesh::MESH_DOUBLE);

  // 登録パラレルコントローラ制御
  int getNumberOfParaController() const;
  const sphParaController* getParaControllerByIndex(int id) const;
  const sphParaController* getParaControllerByComm(SPL_Comm comm) const;
  const sphParaController* getParaController(int map_key) const;
  void removeParaControllerByComm(SPL_Comm comm);
  void removeParaController(int map_key);
  int getKeyNumber(SPL_Comm comm) const;

};

#endif /* _SPHPARAMANAGER_H_ */
