/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphUnstMesh.h
 * Unstruct Mesh Information Classes Header File
 * @author     tamaki
 * @date       2010/04/22
 *
 * \~japanese
 * @file sphUnstMesh.h
 * 非構造格子情報クラス郡ヘッダーファイル
 * @author     tamaki
 * @date       2010/04/22
 */

#ifndef _SPHUNSTMESHINFO_H_
#define _SPHUNSTMESHINFO_H_


#include "base/sphBase.h"
#include "spl/splmpi.h"
 
/**
 * \~english
 * UNstruct Mesh Information Class
 *
 * \~japanese
 * 非構造格子情報クラス
 */
class sphUnstMesh : public virtual sphBase {
friend class sphUnstParaController;

public:
  typedef enum { INVALID_MESH_DATA_TYPE=0,
                 MESH_FLOAT, MESH_DOUBLE } MeshDataType;
protected:
// ----- datas -----
  MeshDataType m_meshDtType;

  // SPL_MPI Infomations
  int      m_nPE; // "m_nPE" is "MPI_Comm_size()-1"->("num of proc" - "myproc")
  int      m_myRank; ///< my node id
  SPL_Comm m_comm;

  // SPL_MPI comm buffers
  SPL_Request *m_sRequest;
  SPL_Request *m_rRequest;
  SPL_Status  *m_sStatus;
  SPL_Status  *m_rStatus;
  /* double     **m_sData; // float or double */
  /* double     **m_rData; // float or double */

  // Parallel Mesh Infomations
  int         *m_PElist;
  int         *m_commTable;
  int        **m_commList;

public:
  // Mesh
  int     nNode;
  int     nnNode;
  int    *node;
  /* double *xyz; // float or double */
  int     nInner;
  int     nOverlapped;
  int     nElem;
  int    *elem;
  int    *nop;
  int     nNgrp;
  char  **NgrpName;
  int    *NgrpIndex;
  int    *Ngrp;
  int     nEgrp;
  char  **EgrpName;
  int    *EgrpIndex;
  int    *Egrp;
  int     nSgrp;
  char  **SgrpName;
  int    *SgrpIndex;
  int    *Sgrp;

// ----- methods -----
protected:
  sphUnstMesh();

  virtual bool readNode(const char* fn) = 0; // float or double
  virtual bool readElement(const char* fn);
  virtual bool readGroup(const char* fn);
  virtual bool readBoundary(const char* fn);

public:
  virtual ~sphUnstMesh();
  MeshDataType getMeshDataType() const { return m_meshDtType; };

  int getNPE() const { return m_nPE; };
  int getMyRank() const { return m_myRank; };
  SPL_Comm getComm() const { return m_comm; };

  int*  getPElist() const { return m_PElist; };
  int*  getCommTable() const { return m_commTable; };
  int** getCommList() const { return m_commList; };

  virtual bool getCoord(float  **c) const { *c = NULL; return false; };
  virtual bool getCoord(double **c) const { *c = NULL; return false; };

  virtual bool comm(float  *x) = 0; // float or double
  virtual bool comm(double *x) = 0; // float or double
  virtual bool wait(float  *x) = 0; // float or double
  virtual bool wait(double *x) = 0; // float or double

  virtual bool readMesh(const char* fn_node,
                        const char* fn_elem,
                        const char* fn_grp,
                        const char* fn_bnd  );
  //virtual bool readDistMesh(const char* file_prefix, int peNo = -1) = 0;
  virtual bool readDistMesh(const char* filename) = 0;

};

#endif /* _SPHUNSTMESHINFO_H_ */
