/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphUnstParaController.h
 * Unstructured Mesh Type Parallel Controller Header File
 * @author     hira
 * @date       2010/04/01
 *
 * \~japanese
 * @file sphUnstParaController.h
 * 非構造格子パラレルコントローラーヘッダーファイル
 * @author     hira
 * @date       2010/04/01
 */


#ifndef _SPHUNSTPARACONTROLLER_H_
#define _SPHUNSTPARACONTROLLER_H_


#include "parallel/sphParaManager.h"
#include "parallel/sphParaController.h"
#include "parallel/sphUnstMesh.h"

/**
 * \~english
 * Unstructured Mesh Type Parallel Controller
 *
 * \~japanese
 * 非格子構造パラレルコントローラー
 */
class sphUnstParaController : public sphParaController {
friend class sphParaManager;

protected:
  sphUnstMesh *m_mesh;

  bool setMesh(sphUnstMesh* mesh);
  bool createMesh(sphUnstMesh::MeshDataType dtype,
                  const char* fn_node,
                  const char* fn_elem,
                  const char* fn_grp,
                  const char* fn_bnd);
  bool createMesh(sphUnstMesh::MeshDataType dtype,
                  const char* filename);

public:
  sphUnstParaController(unsigned int paraKey = 0);
  virtual ~sphUnstParaController();

  // node info method
  virtual bool getVoxelOrigin(double org[3]) const;
  virtual bool getNodeOrigin(double org[3]) const;

  const sphUnstMesh* getMesh() const;
        sphUnstMesh* getMesh();

  virtual bool isEv() const;
  virtual bool isMb() const;
  virtual bool isUnst() const;
  virtual bool isParallel() const;

};
#endif /* _SPHUNSTPARACONTROLLER_H_ */
