/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgDatas.h
 * コンフィグレーションデータ定義ファイル
 * @author     riken
 * @date       2010/10/01
 */

#ifndef _SKL_CFG_DATA_
#define _SKL_CFG_DATA_

namespace SklCfg {

// for solver start type
/// ソルバーの開始条件タイプ
enum StartType { Initial=0, Restart };

// for node "Param" data type
/// Param要素データ型属性タイプ
enum DataType { DTYPE_UNKNOWN=0, _INT, _REAL, _STR };

// for  node "Param" data value
/// Param要素value属性値
union PrmVal {
  char* sval;
  int ival;
  double fval;
};

// for reserve node of sphere configuration 
/// コンフィグレーション要素タイプ
enum SKLNODE { NODE_UNKNOWN = 0,
               DOMAININFO,
               STEER,
               PARAMETER,
               INNERBND,
               OUTERBND,
               MDMTBL,
               MBOXTBL,
               USERDEF  };

// for Boundary condition data
enum BCData     {BC_UX=1, BC_UY, BC_UZ, BC_PRS, BC_TMP};
enum BCType     {BCT_UNKNOWN=0, BCT_FIXED, BCT_DIFFERENTIAL, BCT_COMPOUND};
enum BCSpcType  {ST_UNKNOWN=0, ST_VAL=1, ST_FUNC, ST_UNN, ST_CONVECT};
enum BCDifType  {DFT_UNKNOWN=0, DFT_T, DFT_N, DFT_X, DFT_Y, DFT_Z,
                 DFT_0, DFT_1, DFT_2, DFT_I, DFT_J, DFT_K};

// for SPHERE File Type
/// ファイル種別
enum SklFileType { SKL_FILE_TYPE_UNKOWN = 0,
                   SKL_SPH,     ///< SPHERE Data File
                   SKL_SVX,     ///< SPHERE Model File
                   SKL_OTV,     ///< SPHERE OcTree Data File
                   SKL_OVX,     ///< SPHERE OcTree Model File
                   SKL_FDV,     ///< 4DVis Structured File
                   SKL_NBF,     ///< Legacy Data File
                   SKL_NVX,     ///< Legacy Model File
                   SKL_VCAR     ///< VCAD File
                 };

} // end of namespace SklCfg

#endif // _SKL_CFG_DATA_

