/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgInFile.h
 * コンフィグレーションInFile要素クラスヘッダーファイル
 * @author     riken
 * @date       2010/10/01
 */

#ifndef _SKL_CFG_STEER_INFILE_
#define _SKL_CFG_STEER_INFILE_

#include "SklCfgNodeBase.h"

namespace SklCfg {

/**
 * コンフィグレーションInFile要素クラス
 */
class SklCfgInFile : public SklCfgNodeBase {
protected:
  char* m_attr;           ///< ファイル属性
  char* m_format;         ///< ファイルフォーマット
  char* m_fname;          ///< ファイル名
  bool m_multiIn;         ///< 分散入力フラグ
  int m_coordID;          ///< coordID

  virtual bool CheckNodeName(const char* nname);
  virtual bool ParseSklDefNode(xmlNodePtr node);
  virtual bool Verification();
  bool ParseAttr(xmlNodePtr node);
  bool ParseFormat(xmlNodePtr node);
  bool ParseFileName(xmlNodePtr node);
  bool ParseMultiInput(xmlNodePtr node);
  bool ParseCoordID(xmlNodePtr node);

public:
  SklCfgInFile();
  virtual ~SklCfgInFile();
  virtual bool ParseNode(xmlNodePtr node);
  bool CreateFileName(const char* basedir);

  // Setting Methods
  void SetMultiInput();
  void UnsetMultiInput();

  // Getting Methods
  const char* GetAttr() const;
  const char* GetFormat() const;
  SklFileType GetFileType() const;
  const char* GetFileName() const;
  bool IsMultiInput() const;
  int GetCoordID() const;
  bool GetData(const char** attr,const char** format,const char** fname) const;
  bool GetData(const char** attr,SklFileType* format,const char** fname) const;

#ifdef DEBUG
  virtual void Print();
#endif //DEBUG
};

} // end of namespace SklCfg

#endif // _SKL_CFG_STEER_INFILE_

