/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */

/**
 * @file SklCfgNodeBase.h
 * コンフィグレーション要素基底クラスヘッダーファイル
 * @author     riken
 * @date       2010/10/01
 */

#ifndef _SKL_CFG_NODE_BASE_
#define _SKL_CFG_NODE_BASE_

#include <libxml/tree.h>
#include "SklCfgBase.h"
#include "SklCfgUsrDefNode.h"

namespace SklCfg {

/**
 * コンフィグレーション要素基底クラス
 */
class SklCfgNodeBase : public SklCfgBase {
protected:
  CfgElem m_root;     ///< 子要素
  virtual bool Parse(xmlNodePtr node);

  /**
   * 要素名が一致しているかチェックする。
   * @param nname     要素名
   * @return          true=一致
   */
  virtual bool CheckNodeName(const char* nname) = 0;

  /**
   * XML要素の既定要素をコンフィグレーション要素にパースする。
   * @param node      要素ポインタ
   * @return      true=success, false=fault
   */
  virtual bool ParseSklDefNode(xmlNodePtr node) = 0;

  /**
   * 要素の検証を行う。
   * @return      true=success, false=fault
   */
  virtual bool Verification() = 0;

public:
  SklCfgNodeBase();
  virtual ~SklCfgNodeBase();
  virtual bool ParseNode(xmlNodePtr node);

  bool AddParam(CfgParam* param);
  bool AddElem (CfgElem* elem );
  void ClearRoot();

  // Getting Methods
  const CfgElem* GetRootElem() const;
  unsigned int GetSize() const;
  unsigned int GetElemSize() const;
  unsigned int GetParamSize() const;
  const CfgElem* GetElemFirst(const char* name = NULL) const;
  const CfgElem* GetElemFirst(const char* name, int id,
                              const char* cmt = NULL) const;
  const CfgElem* GetElemNext(const CfgElem* elem,
                             const char* name = NULL) const;
  const CfgElem* GetElemNext(const CfgElem* elem, const char* name,
                             int id, const char* cmt = NULL) const;
  const CfgParam* GetParamFirst(const char* name = NULL) const;
  const CfgParam* GetParamFirst(const char* name, int id,
                                const char* cmt = NULL) const;
  const CfgParam* GetParamNext(const CfgParam* param,
                               const char* name = NULL) const;
  const CfgParam* GetParamNext(const CfgParam* param, const char* name,
                               int id, const char* cmt = NULL) const;
};

} // end of namespace SklCfg

#endif // _SKL_CFG_NODE_BASE_

