/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgOutFile.h
 * コンフィグレーションOutFile要素クラスヘッダーファイル
 * @author     riken
 * @date       2010/10/01
 */

#ifndef _SKL_CFG_STEER_OUTFILE_
#define _SKL_CFG_STEER_OUTFILE_

#include "SklCfgNodeBase.h"
#include "config/sphConfigDefine.h"

namespace SklCfg {

/**
 * コンフィグレーションOutFile要素クラス
 */
class SklCfgOutFile : public SklCfgNodeBase {
protected:
  unsigned int m_interval;      ///< 出力インターバル
  char* m_attr;                 ///< ファイル属性
  char* m_format;               ///< ファイルフォーマット
  char* m_baseName;             ///< ファイルベース名

// Sphere用 delete start at 2010/12/07 by @hira
//  bool m_multiOut;              ///< 分散出力フラグ(bool)
  virtual bool CheckNodeName(const char* nname);
  virtual bool ParseSklDefNode(xmlNodePtr node);
  virtual bool Verification();
  bool ParseInterval(xmlNodePtr node);
  bool ParseAttr(xmlNodePtr node);
  bool ParseFormat(xmlNodePtr node);
  bool ParseBaseName(xmlNodePtr node);
  bool ParseMultiOutput(xmlNodePtr node);

  // Sphere用 add at 2010/12/07 by @hira
  SphCfgFileIOMode m_multimode;                ///< 分散出力フラグ

public:
  SklCfgOutFile();
  virtual ~SklCfgOutFile();
  virtual bool ParseNode(xmlNodePtr node);
  bool CreateFileName(const char* basedir);

  // Setting Methods
  void SetMultiOutput();
  void UnsetMultiOutput();
  void SetInterval(unsigned val);

  // Getting Methods
  unsigned int GetInterval() const;
  const char* GetAttr() const;
  const char* GetFormat() const;
  const char* GetBaseName() const;
  bool IsMultiOutput() const;

  // Sphere用 add at 2010/12/07 by @hira
  SphCfgFileIOMode GetMultiMode() const;
  void SetMultiMode(SphCfgFileIOMode multi);

#ifdef DEBUG
  virtual void Print();
#endif //DEBUG
};

} // end of namespace SklCfg

#endif // _SKL_CFG_STEER_OUTFILE_

