/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgSteer.h
 * コンフィグレーションSteer要素クラスヘッダーファイル
 * @author     riken
 * @date       2010/10/01
 */

#ifndef _SKL_CFG_STEER_
#define _SKL_CFG_STEER_

#include "SklCfgDatas.h"
#include "SklCfgNodeBase.h"
#include "SklCfgSteerInputData.h"
#include "SklCfgSteerOutputData.h"
#include "SklCfgInFile.h"

namespace SklCfg {

/**
 * コンフィグレーションSteer要素クラス
 */
class SklCfgSteer : public SklCfgNodeBase {
protected:
  /// 計算ステップ数
  unsigned int m_calcSteps; // default is 1000 steps.
  bool m_calcStepsFlg;
  /// 開始条件
  StartType m_strType;      // default is Initial.
  bool m_strTypeFlg;
  /// ソルバー起動条件
  bool m_chkModeFlg;        // solver check mode
  char* m_chkModeFile;      // output file_name for solver check mode
  /// コンフィグレーションInputData要素クラス
  SklCfgSteerInputData* m_inputData;
  /// コンフィグレーションOutputData要素クラス
  SklCfgSteerOutputData* m_outputData;

  virtual bool CheckNodeName(const char* nname);
  virtual bool ParseSklDefNode(xmlNodePtr node);
  virtual bool Verification();

  bool ParseCalcSteps(xmlNodePtr node);
  bool ParseStartCondition(xmlNodePtr node);
  bool ParseCheckMode(xmlNodePtr node);
  bool ParseInputData(xmlNodePtr node);
  bool ParseOutputData(xmlNodePtr node);

public:
  SklCfgSteer();
  virtual ~SklCfgSteer();

  // Getting Methods
  unsigned int GetCalcSteps() const;
  StartType GetStartType() const;
  bool IsCheckMode() const;
  const char* GetCheckModeFile() const;
  unsigned int GetInFileSize() const;
  const SklCfgInFile* GetInFileFirst() const;
  const SklCfgInFile* GetInFileNext(const SklCfgInFile* file) const;
  unsigned int GetMonitorInterval() const;
  unsigned int GetOutFileSize() const;
  const SklCfgOutFile* GetOutFileFirst() const;
  const SklCfgOutFile* GetOutFileNext(const SklCfgOutFile* file) const;
  unsigned int GetHistorySize() const;
  const SklCfgHistory* GetHistoryFirst() const;
  const SklCfgHistory* GetHistoryNext(const SklCfgHistory* history) const;
  const char* getInputBaseDir() const;
  const char* getOutputBaseDir() const;

#ifdef DEBUG
  virtual void Print();
#endif //DEBUG
};

} // end of namespace SklCfg

#endif // _SKL_CFG_STEER_

