/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgSteerOutputData.h
 * コンフィグレーションOutputData要素クラスヘッダーファイル
 * @author     riken
 * @date       2010/10/01
 */

#ifndef _SKL_CFG_STEER_OUTPUT_DATA_
#define _SKL_CFG_STEER_OUTPUT_DATA_

#include "SklCfgNodeBase.h"
#include "SklCfgHistory.h"
#include "SklCfgOutFile.h"

namespace SklCfg {
/**
 * コンフィグレーションOutputData要素クラス
 */
class SklCfgSteerOutputData : public SklCfgNodeBase {
protected:
  char* m_baseDir;      ///< ディレクトリパス
  list<SklCfgOutFile*> m_outFileList;   ///< OutFile要素リスト
  list<SklCfgHistory*> m_historyList;   ///< History要素リスト
  /// 出力インターバル
  unsigned int m_monitorInterval; // default is 1.
  bool m_monIntFlg;
  virtual bool CheckNodeName(const char* nname);
  virtual bool ParseSklDefNode(xmlNodePtr node);
  virtual bool Verification();
  bool ParseMonitor(xmlNodePtr node);

public:
  SklCfgSteerOutputData();
  virtual ~SklCfgSteerOutputData();
  virtual bool ParseNode(xmlNodePtr node);

  // Getting Methods
  const char* GetBaseDir() const;
  unsigned int GetMonitorInterval() const;
  unsigned int GetOutFileSize() const;
  const SklCfgOutFile* GetOutFileFirst() const;
  const SklCfgOutFile* GetOutFileNext(const SklCfgOutFile* file) const;
  unsigned int GetHistorySize() const;
  const SklCfgHistory* GetHistoryFirst() const;
  const SklCfgHistory* GetHistoryNext(const SklCfgHistory* history) const;

#ifdef DEBUG
  virtual void Print();
#endif //DEBUG
};

} // end of namespace SklCfg

#endif // _SKL_CFG_STEER_OUTPUT_DATA_

