
/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */

#ifndef _SKLPARSEUTILS_H_
#define _SKLPARSEUTILS_H_


#include "SklSolverConfig.h"
#include "app/sphSphere.h"

using namespace SklCfg;

//--------------------------------------------------------------------------
//  Macros
//--------------------------------------------------------------------------
#define stmpd_printf printf("%s (%d):  ",__FILE__, __LINE__); printf

//--------------------------------------------------------------------------
//  Parse Class ( Parser Utility base class )
//--------------------------------------------------------------------------
class Parse {
protected:
  const SklSolverConfig* Root;  // for XML parsing
  const CfgElem *elemTop;
  const CfgParam* param;
  const char *keyword;
public:
  Parse(const SklSolverConfig* Croot = NULL) : Root(Croot) {
    elemTop = NULL;
    param = NULL;
    keyword = NULL;
  };
  virtual ~Parse() {};
// ---------------------------------------------------------------------------
  const char* getParam(const char* str) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return keyword;
    }
    if( !(param = elemTop->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
    }
    else if ( !(param->GetData(&keyword)) ) {
      fprintf(stderr, "\tParsing error : Invalid string for %s\n", str);
    }
    return keyword;
  };
// ---------------------------------------------------------------------------
//  const char* getParam(const char* str) {
//    if( !str ) {
//      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
//      return NULL;
//    }
//    if( !(param = elemTop->GetParamFirst(str)) ) {
//      fprintf(stderr,
//              "\tParsing error : Missing the Param of %s definition\n", str);
//      return NULL;
//    }
//    if ( !(param->GetData(&keyword)) ) {
//      fprintf(stderr, "\tParsing error : Invalid string for %s\n", str);
//      return NULL;
//    }
//    return keyword;
//  };
// ---------------------------------------------------------------------------

  bool getParam(const char* str, size_t var_size, char* var) {
    if( !str || !var ) {
      fprintf(stderr, "\tInvalid parameter. str or var is NULL.\n");
      return false;
    }
    if( !(param = elemTop->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
      return false;
    }
    char* word = new char[var_size];
    if( !word ) {
      fprintf(stderr, "\tParsing error : Temporaly string allocate failed.\n");
      return false;
    }
    memset(word, 0, sizeof(char)*var_size);
    if ( !(param->GetData((const char**)&word)) ) {
      fprintf(stderr, "\tParsing error : Invalid char value for %s\n", str);
      delete [] word;
      return false;
    }
    strncpy(var, word, var_size);
    delete [] word;
    return true;
  };
  bool getParam(const char* str, int &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid Parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elemTop->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
      return false;
    }
    if ( !(param->GetData(&var)) ) {
      fprintf(stderr, "\tParsing error : Invalid int value for %s\n", str);
      return false;
    }
    return true;
  };
  bool getParam(const char* str, unsigned &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL\n");
      return false;
    }
    if( !(param = elemTop->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
      return false;
    }
    int var_tmp;
    if ( !(param->GetData(&var_tmp)) ) {
      fprintf(stderr, "\tParsing error : Invalid int value for %s\n", str);
      return false;
    }
    if( var_tmp < 0 ) {
      fprintf(stderr, "\tWarning : value is less than zero.(%d)\n", var_tmp);
    }
    var = (unsigned)var_tmp;
    return true;
  };
  bool getParam(const char* str, float &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elemTop->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
      return false;
    }
    if ( !(param->GetData(&var)) ) {
      fprintf(stderr, "\tParsing error : Invalid float value for %s\n", str);
      return false;
    }
    return true;
  };
  bool getParam(const char* str, double &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid paremeter. str is NULL.\n");
      return false;
    }
    if( !(param = elemTop->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
      return false;
    }
    float var_tmp;
    if ( !(param->GetData(&var_tmp)) ) {
      fprintf(stderr, "\tParsing error : Invalid double value for %s\n", str);
      return false;
    }
    var = (double)var_tmp;
    return true;
  };

};

//--------------------------------------------------------------------------
//  ParseSteer Class
//--------------------------------------------------------------------------
class ParseSteer : public Parse {
  const CfgElem *elmL1;
public:
  ParseSteer(const SklSolverConfig* Croot = NULL) : Parse(Croot) {
    elmL1 = NULL;
    elemTop = Root->GetTop(STEER);
  };
  ~ParseSteer() {};
  bool IsSetElem(const char* str) {
    if( !(elmL1 = elemTop->GetElemFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Elem of %s\n", str);
      return false;
    }
    return true;
  };
  bool getEParam(const char* str, size_t var_size, char* var) {
    if( !str || !var ) {
      fprintf(stderr, "\tInvalid parameter. str or var is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
      return false;
    }
    char* word = new char[var_size];
    if( !word ) {
      fprintf(stderr, "\tParsing error : Temporaly string allocate failed.\n");
      return false;
    }
    memset(word, 0, sizeof(char)*var_size);
    if ( !(param->GetData((const char**)&word)) ) {
      fprintf(stderr, "\tParsing error : Invalid char value for %s\n", str);
      delete [] word;
      return false;
    }
    strncpy(var, word, var_size);
    delete [] word;
    return true;
  };
  bool getEParam(const char* str, int &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    if ( !(param->GetData(&var)) ) {
      fprintf(stderr, "\tParsing error : Invalid int value for %s\n", str);
      return false;
    }
    return true;
  };
  bool getEParam(const char* str, unsigned &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    int var_tmp;
    if ( !(param->GetData(&var_tmp)) ) {
      fprintf(stderr, "\tParsing error : Invalid int value for %s\n", str);
      return false;
    }
    if( var_tmp < 0 ) {
      fprintf(stderr, "Warning : value is less than zero.(%d)\n", var_tmp);
    }
    var = (unsigned)var_tmp;
    return true;
  };
  bool getEParam(const char* str, float &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    if ( !(param->GetData(&var)) ) {
      fprintf(stderr, "\tParsing error : Invalid float value for %s\n", str);
      return false;
    }
    return true;
  };
  bool getEParam(const char* str, double &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    float var_tmp;
    if ( !(param->GetData(&var_tmp)) ) {
      fprintf(stderr, "\tParsing error : Invalid double value for %s\n", str);
      return false;
    }
    var = (double)var_tmp;
    return true;
  };
};

//--------------------------------------------------------------------------
//  ParsePara Class
//--------------------------------------------------------------------------
class ParsePara : public Parse {
protected:
  const CfgElem *elmL1;
public:
  ParsePara(const SklSolverConfig* Croot = NULL) : Parse(Croot) {
    elmL1 = NULL;
    elemTop = Root->GetTop(PARAMETER);
  };
  ~ParsePara() {};
  bool IsSetElem(const char* str) {
    if( !(elmL1 = elemTop->GetElemFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Elem of %s\n", str);
      return false;
    }
    return true;
  };
  bool getEParam(const char* str, size_t var_size, char* var) {
    if( !str || !var ) {
      fprintf(stderr, "\tInvalid parameter. str or var is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr,
              "\tParsing error : Missing the Param of %s definition\n", str);
      return false;
    }
    char* word = new char[var_size];
    if( !word ) {
      fprintf(stderr, "\tParsing error : Temporaly string allocate failed.\n");
      return false;
    }
    memset(word, 0, sizeof(char)*var_size);
    if ( !(param->GetData((const char**)&word)) ) {
      fprintf(stderr, "\tParsing error : Invalid char value for %s\n", str);
      delete [] word;
      return false;
    }
    strncpy(var, word, var_size);
    delete [] word;
    return true;
  };
  bool getEParam(const char* str, int &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    if ( !(param->GetData(&var)) ) {
      fprintf(stderr, "\tParsing error : Invalid int value for %s\n", str);
      return false;
    }
    return true;
  };
  bool getEParam(const char* str, unsigned &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    int var_tmp;
    if ( !(param->GetData(&var_tmp)) ) {
      fprintf(stderr, "\tParsing error : Invalid int value for %s\n", str);
      return false;
    }
    if( var_tmp < 0 ) {
      fprintf(stderr, "Warning : value is less than zero.(%d)\n", var_tmp);
    }
    var = (unsigned)var_tmp;
    return true;
  };
  bool getEParam(const char* str, float &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    if ( !(param->GetData(&var)) ) {
      fprintf(stderr, "\tParsing error : Invalid float value for %s\n", str);
      return false;
    }
    return true;
  };
  bool getEParam(const char* str, double &var) {
    if( !str ) {
      fprintf(stderr, "\tInvalid parameter. str is NULL.\n");
      return false;
    }
    if( !(param = elmL1->GetParamFirst(str)) ) {
      fprintf(stderr, "\tParsing error : Missing the Param of %s\n", str);
      return false;
    }
    float var_tmp;
    if ( !(param->GetData(&var_tmp)) ) {
      fprintf(stderr, "\tParsing error : Invalid double value for %s\n", str);
      return false;
    }
    var = (double)var_tmp;
    return true;
  };
};


//--------------------------------------------------------------------------
//  SklUtil Class
//--------------------------------------------------------------------------
class SklParseUtils {
public:
  SklParseUtils(){};
  virtual ~SklParseUtils(){};

//-----------------------------------------------------------
//  Configratuion utility
//-----------------------------------------------------------
  static int getParamID(const CfgParam* p);

  static bool getVecParams(const CfgElem* elmL,
                           const char* key1,
                           const char* key2,
                           const char* key3,
                           int* values);

  static bool getVecParams(const CfgElem* elmL,
                           const char* key1,
                           const char* key2,
                           const char* key3,
                           unsigned* values);

  static bool getVecParams(const CfgElem* elmL,
                           const char* key1,
                           const char* key2,
                           const char* key3,
                           float* vec);

  static bool getVecParams(const CfgElem* elmL,
                           const char* key1,
                           const char* key2,
                           const char* key3,
                           double* vec);

  static bool chkXMLTopTag(int label, const char* name = NULL);
  static bool getGridInfo(unsigned Dims,
                          unsigned* size,
                          float* origin = NULL,
                          float* pitch  = NULL,
                          float* width  = NULL);
  static bool getGridInfo(unsigned Dims,
                          unsigned* size,
                          double* origin = NULL,
                          double* pitch  = NULL,
                          double* width  = NULL);
  static bool getCellInfo(unsigned Dims,
                          unsigned* size,
                          float* origin = NULL,
                          float* pitch  = NULL,
                          float* width  = NULL);
  static bool getCellInfo(unsigned Dims,
                          unsigned* size,
                          double* origin = NULL,
                          double* pitch  = NULL,
                          double* width  = NULL);

  static char* searchTextFname(const char* key);
};

#endif // _SKLPARSEUTILS_H_


