/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */

/**
 * @file SklSolverConfig.h
 * V-Sphereソルバーコンフィグレーション情報格納クラスヘッダーファイル
 * @author     RIKEN
 * @date       2010/10/01
 */
#ifndef _SKL_SOLVER_CONFIG_H_
#define _SKL_SOLVER_CONFIG_H_

#include "SklCfgDatas.h"
#include "SklCfgBase.h"
#include "SklCfgDomainInfo.h"
#include "SklCfgSteer.h"
#include "SklCfgParameter.h"
#include "SklCfgInnerBnd.h"
#include "SklCfgOuterBnd.h"
#include "SklCfgMBoxTable.h"
#include "SklCfgMdmTable.h"
#include "SklCfgUsrDefTopNode.h"

#include "SklCfgInFile.h"
#include "SklCfgOutFile.h"
#include "SklCfgHistory.h"
#include "SklCfgUsrDefNode.h"

#include "SklIBCConfig.h"
#include "SklOBCConfig.h"
#include "SklMBoxConfig.h"
#include "SklMDMConfig.h"

/**
 * SklCfg::コンフィグレーション関連クラス群ネームスペース
 */
namespace SklCfg {

/**
 * V-Sphereソルバーコンフィグレーション情報格納クラス
 */
class SklSolverConfig : public SklCfgBase {
protected:
  char* m_solverType;
  SklCfgDomainInfo* m_domainInfo;     ///< DomainInfo要素クラス
  SklCfgSteer* m_steer;               ///< Steer要素クラス
  SklCfgParameter* m_parameter;       ///< Parameter要素クラス
  SklCfgInnerBnd* m_innerBnd;         ///< InnerBoundary要素クラス
  SklCfgOuterBnd* m_outerBnd;         ///< OuterBoundary要素クラス
  SklCfgMdmTable* m_mdmTable;         ///< Medium_Table要素クラス
  SklCfgUsrDefTopNode* m_usrDefNode;  ///< ユーザ定義要素クラス
  SklCfgMBoxTable* m_mbxTable;        ///< MultiBoxDivTable要素クラス

  void Initialize();
  bool SklParseConfigFile(const char* fname);
  bool SklParseConfigFileForParallel(const char* fname);
  bool ParseConfigStream(const size_t sz, const char* stream);
  bool SklParse(xmlNodePtr node);
  virtual bool Parse(xmlNodePtr node);
  SklCfgNodeBase* GetNode(SKLNODE node) const;

  bool ParseBCTable(xmlNodePtr node);
  bool ReadBCFileForInner();
  bool ReadBCFileForOuter();
  bool ReadMDMTableFile();
  bool ReadMBoxTableFile();

public:
  /**
   * コンストラクタ
   */
  SklSolverConfig();
  /**
   * デストラクタ
   */
  virtual ~SklSolverConfig();
  /**
   * コンフィグレーションファイルを読み込み、内部にコンフィグレーションDOMツリーを構築する．
   * @param[in]	fname	コンフィグレーションファイル名
   * @return	success(=true)/fault(=false)
   */
  bool SklParseConfig(const char* fname);

  bool SklParseConfig(xmlNodePtr root_node);

  // Getting Methods
  // for node "Elem" and "Param"
  const CfgElem* GetTop(SKLNODE node, const char* name = NULL) const;

  unsigned int GetSize(SKLNODE node, const char* name = NULL) const;
  unsigned int GetElemSize(SKLNODE node, const char* name = NULL) const;
  unsigned int GetParamSize(SKLNODE node, const char* name = NULL) const;

  const CfgElem* GetElemFirst(SKLNODE node, const char* name = NULL) const;
  const CfgElem* GetElemFirst(SKLNODE node, const char* name,
                              int id, const char* cmt = NULL) const;
  const CfgElem* GetElemNext(SKLNODE node, const CfgElem* elem,
                             const char* name = NULL) const;
  const CfgElem* GetElemNext(SKLNODE node,
                             const CfgElem* elem, const char* name,
                             int id, const char* cmt = NULL) const;
  const CfgParam* GetParamFirst(SKLNODE node, const char* name = NULL) const;
  const CfgParam* GetParamFirst(SKLNODE node, const char* name,
                                int id, const char* cmt = NULL) const;
  const CfgParam* GetParamNext(SKLNODE node, const CfgParam* param,
                               const char* name = NULL) const;
  const CfgParam* GetParamNext(SKLNODE node,
                               const CfgParam* param, const char* name,
                               int id, const char* cmt = NULL) const;
  // for node "DomainInfo"
  bool GetVoxelOrigin(float& x, float& y, float& z) const;
  bool GetVoxelSize(unsigned int& i, unsigned int& j, unsigned int& k) const;
  bool GetVoxelWidth(float& x, float& y, float& z) const;
  bool GetVoxelPitch(float& x, float& y, float& z) const;

  // for node "Steer"
  unsigned int GetCalcSteps() const;
  StartType GetStartType() const;
  bool IsCheckMode() const;
  const char* GetCheckModeFile() const;
  unsigned int GetInFileSize() const;
  const SklCfgInFile* GetInFileFirst() const;
  const SklCfgInFile* GetInFileNext(const SklCfgInFile* file) const;
  unsigned int GetMonitorInterval() const;
  unsigned int GetOutFileSize() const;
  const SklCfgOutFile* GetOutFileFirst() const;
  const SklCfgOutFile* GetOutFileNext(const SklCfgOutFile* file) const;
  unsigned int GetHistorySize() const;
  const SklCfgHistory* GetHistoryFirst() const;
  const SklCfgHistory* GetHistoryNext(const SklCfgHistory* history) const;
  bool IsChkCommPerform(unsigned int& vc, size_t& loop_max) const;
  bool IsExitAfterChkCommPerform() const;
  const char* getSolverType() const;
  const char* getInputBaseDir() const;
  const char* getOutputBaseDir() const;

  // for node "Parameter"
  // for node "InnerBoundary"
  // for node "OuterBoundary"
  // for node "Medium_Table"


#ifdef DEBUG
  virtual void Print();
#endif //DEBUG
};

} // end of namespace SklCfg

#endif // _SKL_SOLVER_CONFIG_H_

