/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphCouplerSv2Sv.h
 * Sphere Simple Voxel to Simple Voxel Coupler super Class Header File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file sphCouplerSv2Sv.h
 * Sphere構造格子To構造格子カプラー基底クラスヘッダーファイル
 * @author     hira
 * @date       2009/10/01
 */

#ifndef _SPHCOUPLERSV2SV_H_
#define _SPHCOUPLERSV2SV_H_

#include "sphCouplerBase.h"
#include "data/sphData.h"

/**
 * Sphere構造格子To構造格子カプラー基底クラス
 */
class sphCouplerSv2Sv: public sphCouplerBase
{
private:

  SPL_Datatype createCommType(
                      SphDCType dcType,
                      SPL_Datatype dtType,
                      const size_t max[3],
                      const size_t sta[3],
                      const size_t end[3],
                      const size_t data_num);

protected:
  size_t m_extGc;                       ///< 拡張ガイドセル値

  bool initialize();
  virtual bool communicateData();
  virtual bool interpolate();

  bool createCommNodeTable(
                          const sphData* refDataObj,
                          const sphData* udtDataObj,
                          size_t ext_gc);

  bool createSendTable(
                        const sphData* refDataObj,
                        int src_nodeid,
                        const double src_start_coords[3],
                        const double src_width[3]);

  bool createReceiveTable(
                        const sphData* udtDataObj,
                        int recv_nodeid,
                        int crddef,
                        const size_t recv_size[3],
                        const double recv_start_coords[3],
                        const double recv_pitch[3],
                        const int faceFlag[3][2]);

  double getRealValue(
                      sphData* dataObj,
                      size_t i_idx, size_t j_idx, size_t k_idx, size_t l_idx);

  virtual bool setData(
                      double& value,
                      size_t i_idx, size_t j_idx, size_t k_idx, size_t l_idx);

  double interpolateLinear(
                          int dims,
                          const double dst_coors[3],
                          const double src_coors[2][2][2][3],
                          const double src_value[2][2][2]);

  double linearValue(double ix0, double dx0,
                              double ix1, double dx1,
                              double ix);

  bool commBndCell();

public:
  sphCouplerSv2Sv();
  sphCouplerSv2Sv(const char* label);
  sphCouplerSv2Sv(const char* label, int id);
  virtual ~sphCouplerSv2Sv();

  virtual int initializeCoupler(const char* ref_reg_label = NULL, const char* upd_reg_label = NULL);
  virtual int executeCoupler();
  virtual int postCoupler();

};

#endif /* _SPHCOUPLERSV2SV_H_ */
