/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphSolverSv.h
 * Sphere Simple Voxel Solver super Class Header File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file sphSolverSv.h
 * Sphere構造格子ソルバー基底クラスヘッダーファイル
 * @author     hira
 * @date       2009/10/01
 */

#ifndef _SPHSOLVERSV_H_
#define _SPHSOLVERSV_H_

#include "solverbase/sphSolverBase.h"
#include "parallel/sphSvParaController.h"
#include "data/sphData.h"
#include "fileio/sphFileController.h"

/**
 * Sphere構造格子ソルバー基底クラス
 */
class sphSolverSv: public sphSolverBase
{
private:

  bool initialize();
  bool setVoxelDivision(const unsigned int div[3]);
  int  initializeVoxelSklConfig();

protected:
  sphSvParaController* m_svParaCtrl;    ///< SVパラレルコントローラ（sphSolverBase::m_paraCtrlと同一ポインタ）

  virtual int setupSolver();
  virtual int setupDataList();

  virtual int initializeVoxel();
  int initializeVoxel(
                      const sphElem* nodelist_elem,
                      const size_t voxel_size[3] = NULL,
                      const size_t voxel_start[3] = NULL,
                      const double voxel_origin[3] = NULL,
                      const double voxel_pitch[3] = NULL);
  int initializeVoxel(
                      const size_t voxelsize[3],
                      const unsigned int division[3],
                      const double voxel_origin[3] = NULL,
                      const double voxel_pitch[3] = NULL);
  int initializeVoxel(
                      const size_t voxelsize[3],
                      int proc_num = 0,
                      const double voxel_origin[3] = NULL,
                      const double voxel_pitch[3] = NULL);
  int initializeVoxel(
                      unsigned int node_num,
                      const size_t wsize[3],
                      const size_t wstart[3],
                      const size_t (*voxelsize)[3],
                      const size_t (*voxelstart)[3],
                      const double voxelorigin[3] = NULL,
                      const double voxelpitch[3] = NULL,
                      const unsigned int *groupid_table = NULL);
  int initializeVoxel(
                      const size_t voxel_size[3],
                      const double voxel_origin[3],
                      const double voxel_pitch[3],
                      const size_t voxel_startidx[3] = NULL);

  virtual bool setParaController(sphParaController* paraCtrl);

  sphSolverSv();
  bool setVoxelCoords(const double org[3], const double pitch[3]);

  bool existsConfigParallelTemplate();
  bool existsConfigEqualPartitioning();
  int  getConfigNumberOfSubblock();
  bool existsConfigMultiplyPartitioning();
  bool isConfigMbxGrouping();
  bool getConfigDomainInfo(size_t sz[3], double org[3], double pitch[3]);
  bool getConfigVoxelDivision(unsigned int voxel_div[3]);

public:
  sphSolverSv(const char* solv_label);
  sphSolverSv(const char* solv_label, int solv_id);
  virtual ~sphSolverSv();

  // -1 : error
  //  0 : forced terminated
  //  1 : Normality
  virtual int initializeSolver(sphStepTime* steptime);
  virtual int loopSolver(sphStepTime* steptime) = 0;
  virtual int postSolver(sphStepTime* steptime) = 0;
  virtual int printSolverUsage(const char* cmd) = 0;


};

#endif /* _SPHSOLVERSV_H_ */
