/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * @file sph_mem_util.h
 * メモリ操作ユーティリティファイル
 * @author     hira
 * @date       2010/12/25
 */


#ifndef _SPH_MEM_UTIL_H
#define _SPH_MEM_UTIL_H

#include <string.h>

/**
 * メモリブロックのビット和(OR)を求める
 *
 * @param[out]      b      ビット和先(結果)のメモリブロック
 * @param[in]       a      ビット和元のメモリブロック
 * @param[in]       n      メモリブロックバイト数
 * @return          ビット和(結果)のメモリポインタ
 */
//void* memadd(void *restrict b, const void *restrict a, size_t n)
inline void* memadd(void * b, const void *a, size_t n)
{
//    char *__restrict__ s1 = (char*) b;
//    const char *__restrict__ s2 = (const char*)a;
  char *s1 = (char*) b;
  const char *s2 = (const char*)a;
  for(; 0<n; --n)*s1++ |= *s2++;      // 論理和を行う
  return b;
}

/**
 * メモリブロックの0x00以外のデータのコピーを行う。
 *
 * コピー元のデータが0x00以外の時のみ、コピーを行う。
 * データが0x00であるかのチェックはデータバイト数にて行う。
 * @param[out]      b      コピー先のメモリブロック
 * @param[in]       a      コピー元のメモリブロック
 * @param[in]       n      メモリブロックバイト数
 * @param[in]       data_size      データバイト数
 * @return          コピー先のメモリポインタ
 */
//void* memadd(void *restrict b, const void *restrict a, size_t n)
inline void* memcpyex(void * b, const void *a, size_t n, size_t data_size)
{
//    char *__restrict__ s1 = (char*) b;
//    const char *__restrict__ s2 = (const char*)a;
  char *s1 = (char*) b;
  const char *s2 = (const char*)a;

  register int cp_flag;
  register int i;
  for(; 0<n; n-=data_size) {
    cp_flag = 0;
    for (i=0; i<data_size; i++) {
      if (s2[i]) {
        cp_flag = 1;
        break;
      }
    }

    if (cp_flag) {
      for (i=0; i<data_size; i++) {
        *s1++ = *s2++;      // コピーを行う
      }
    }
    else {
      s1+=data_size; s2+=data_size;       // コピーは行わない。
    }
  }
  return b;
}

#endif /* _SPH_MEM_UTIL_H */
