/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file spl.h
 * Sphere Low Level Library Header File
 *
 * User include file of Sphere Low Level Library
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file spl.h
 * Sphere低レベル関数ヘッダーファイル
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPL_H_
#define _SPL_H_

#include <string.h>
#include <stdlib.h>
#include <float.h>

#include "spl_errno.h"


#ifndef __FILE__
#define __FILE__ "NONE"
#endif
#ifndef __LINE__
#define __LINE__       0
#endif


#if !defined(__func__) && ((__GNUC__ >= 2) || (_MSC_VER >= 1300))
#define __func__ __FUNCTION__
#else
#define __func__ "NONE"
#endif

#ifndef __LONG_LONG_MAX__
#define __LONG_LONG_MAX__ 9223372036854775807LL
#endif

#if defined(__cplusplus)
extern "C" {
#endif

/* boolian */
typedef int SplBool;
#define SplTrue  1
#define SplFalse 0

/* string max size */
#define SPL_MAX_STR_SIZE 256

/* error code */
//extern int spl_err_code;
void spl_err(const char* s);

/* message */
void spl_msg(const char* format, ...);
void spl_errmsg(const char* format, ...);
void spl_wrnmsg(const char* format, ...);

/* time */
double splGetTime(void);

/* debug message */
void spl_set_errmsg(int err_code, const char* format, ...);
void spl_set_errinfo(int err_code, const char* filename, int lineno, const char* funcname, const char* format, ...);

void spl_set_error_file(const char* file_name, int lineno, const char* func);
void spl_set_errinfo_fmt(int err_code, const char* format, ...);

void spl_clear_errcode();
void spl_set_errcode(int err_code);
int spl_get_errcode();
const char* spl_get_errinfo();
const char* spl_get_define_errmsg(int err_no);
void spl_print_error_message();

/**
 * \~english
 * Macro:set error message number and detail information
 *
 * \~japanese
 * エラーメッセージ番号と詳細情報設定マクロ.
 * ファイル名、行番号、関数名を自動付加する。
 */
/*
#define SPL_ERROR_INFO(msg_no, ...) \
    spl_set_errinfo(msg_no, __FILE__, __LINE__, __func__, __VA_ARGS__)
*/

#define SPL_ERROR_INFO  (spl_set_error_file(__FILE__, __LINE__, __func__), spl_set_errinfo_fmt)


#define SPL_ERROR_CODE(msg_no) \
    spl_set_errinfo(msg_no, __FILE__, __LINE__, __func__, NULL)

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_H_ */

