/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splfio_sph.h
 * Declaration of SPH Level Library
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_sph.h
 * SPHファイル入出力関数ヘッダーファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPL_FIO_SPH_H_
#define _SPL_FIO_SPH_H_

/*
#include "spl.h"
#include "splfio_sv_data.h"
#include "splfio_sv_file.h"
#include "splmpi.h"
*/
#include "splfio_spx.h"

#if defined(__cplusplus)
extern "C" {
#endif

/*
 * Getting SPH header infomation(without data-record).
 *
 * mode : SPH read mode
 *  0 : All nodes open the file and it reads data.
 *  1 : Only the node of rank 0 opens the file and data is broadcast
 *      to other nodes. Rank 0 also sets the data read from the file
 *      in the splSpx object.
 *  2 : Only the node of rank 0 opens the file and data is broadcast
 *      to other nodes. However, rank 0 only reads data from the file,
 *      and doesn't set data in the splSpx object.
 */
extern SplBool splGetSphInfo(const char* fname, splSpx* obj, int mode,
                             int readRank, SPL_Comm grp);

/*
 * Before the call of splReadSph,
 * splSpx object must be set the following values.
 *  wsize, start_idx, dims, size, dtype,
 *  rlen, data, and vlen(=1:default value).
 *
 * mode : SPH read mode
 *  0 : All nodes open the file and it reads data.
 *  1 : Only the node of rank 0 opens the file and data is broadcast
 *      to other nodes. Rank 0 also sets the data read from the file
 *      in the splSpx object.
 *  2 : Only the node of rank 0 opens the file and data is broadcast
 *      to other nodes. However, rank 0 only reads data from the file,
 *      and doesn't set data in the splSpx object.
 */
extern SplBool splReadSph(splSvFileList* file_list, int mode,
                          splSpx* obj, int readRank, SPL_Comm grp);

/*
 * mode : SPH write mode
 *  0 : All nodes output data to each file.
 *  1 : The data of each node is gathered in rank 0, and rank 0 writes
 *      the data to one file.
 */
extern SplBool splWriteSph(const char* fname, int mode,
                           size_t block_size, const splSpx* obj,
                           int writeRank, SPL_Comm grp);

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_FIO_SPH_H_ */

