/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splfio_sv_data.h
 * Declaration of struct splData for Sphere Low Level Library
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_sv_data.h
 * splData共用体定義ファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPL_FIO_SV_DATA_H_
#define _SPL_FIO_SV_DATA_H_

#if defined(__cplusplus)
extern "C" {
#endif

/**
 * \~english
 * union splData
 *
 * splData is struct to store data.<br>
 * splData has pointer of (unsigned)char(1byte),<br>
 * short(2byte), int(4byte) data and float, double.
 *
 * \~japanese
 * splData共用体
 *
 * SBX,SPXのデータポインタを設定する。<br>
 * SBXのデータ型:char, short, int<br>
 * SPXのデータ型:float, double
 */
typedef union _spl_data_ {
  unsigned char* c;            ///< SBXのデータ型:char
  unsigned short* s;          ///< SBXのデータ型:short
  unsigned int* i;            ///< SBXのデータ型:int
  float* f;                    ///< SPXのデータ型:float
  double* d;                  ///< SPXのデータ型:double
} splData;

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_FIO_SV_DATA_H_ */

