/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splfio_sv_file.h
 * Declaration of struct splSvFile and splSvFileList
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_sv_file.h
 * ファイル入出力用構造体定義ファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPL_FIO_SV_FILE_H_
#define _SPL_FIO_SV_FILE_H_

#include <sys/types.h>

#if defined(__cplusplus)
extern "C" {
#endif

/**
 * \~english
 * splSvFile is a struct to store SBX or SPX file infomation.
 *
 *   - int set_flag
 *        0:not set, 1:set
 *   - size_t start_idx[3]
 *       start index in groval voxel size.
 *   - char* fname
 *       SBX or SPX file name.
 *       fname is set from user.
 *   - splSvFile* next
 *       next splSvFile pointer.
 *
 * \~japanese
 * SBX,SPXファイル情報の格納用構造体
 *
 *   - int set_flag
 *        0:not set, 1:set
 *   - size_t start_idx[3]
 *       start index in groval voxel size.
 *   - char* fname
 *       SBX or SPX file name.
 *       fname is set from user.
 *   - splSvFile* next
 *       next splSvFile pointer.
 */
typedef struct _spl_sv_file_ {
  int set_flag;                           ///< 0:not set, 1:set
  size_t start_idx[3];                    ///< start index in groval voxel size.
  const char fname[SPL_MAX_STR_SIZE];      ///< SBX or SPX file name.
  struct _spl_sv_file_* next;              ///< next splSvFile pointer.
} splSvFile;


/**
 * \~english
 * splSvFileList is a struct to store SBX or SPX file list.
 *
 *   - size_t list_size
 *       splSvFile list size
 *   - splSvFile* top
 *       Top of list
 *
 * \~japanese
 * SBX,SPXファイルリストの格納用構造体
 *
 *   - size_t list_size<br>
 *       splSvFile list size
 *   - splSvFile* top<br>
 *       Top of list
 */
typedef struct _spl_sv_file_list_ {
  size_t list_size;            ///< splSvFile list size
  splSvFile* top;            ///< Top of list
}splSvFileList;


#if defined(__cplusplus)
}
#endif /* extern "C" */

#ifdef SPL_SV_FILE_INLINE
 #undef SPL_SV_FILE_INLINE
#endif /* SPL_SV_FILE_INLINE */

#ifndef NO_INLINE
// #define SPL_SV_FILE_INLINE inline
 #define SPL_SV_FILE_INLINE static inline
#else
 #define SPL_SV_FILE_INLINE
#endif /* NO_INLINE */

#include "splfio_sv_file_inline.h"

#undef SPL_SV_FILE_INLINE

#endif /* _SPL_FIO_SV_FILE_H_ */

