/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splmpi_dmy.h
 * Dummy MPI wrapper for Sphere Low Level Library
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splmpi_dmy.h
 * ダミーMPIラッパー定義ファイル
 *
 * 逐次処理用のMPI関数、定義のダミー関数、定義
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPL_MPI_DMY_H_
#define _SPL_MPI_DMY_H_

/* Keep C++ compilers from getting confused */
#if defined(__cplusplus)
extern "C" {
#endif

/*  */
typedef int SPL_Errhandler;

/**
 * Dummy MPI_Request
 *
 * MPI request opjects
 */
typedef union {
  int idmy;
  float fdmy;
} SPL_Request;
static SPL_Request dmy_request_null = {0};
#define SPL_STATUS_SIZE 4

/**
 * Dummy MPI_Status
 *
 * MPI status opject
 */
typedef struct {
    int count;
    int MPI_SOURCE;
    int MPI_TAG;
    int MPI_ERROR;
#if (SPL_STATUS_SIZE > 4)
    int extra[SPL_STATUS_SIZE - 4];
#endif
} SPL_Status;

/* Data types */
typedef int SPL_Datatype;
#define SPL_CHAR    ((SPL_Datatype)1)
#define SPL_UNSIGNED_CHAR  ((SPL_Datatype)2)
#define SPL_BYTE    ((SPL_Datatype)3)
#define SPL_SHORT    ((SPL_Datatype)4)
#define SPL_UNSIGNED_SHORT  ((SPL_Datatype)5)
#define SPL_INT      ((SPL_Datatype)6)
#define SPL_UNSIGNED    ((SPL_Datatype)7)
#define SPL_LONG    ((SPL_Datatype)8)
#define SPL_UNSIGNED_LONG  ((SPL_Datatype)9)
#define SPL_FLOAT    ((SPL_Datatype)10)
#define SPL_DOUBLE    ((SPL_Datatype)11)
#define SPL_LONG_DOUBLE    ((SPL_Datatype)12)
#define SPL_LONG_LONG_INT  ((SPL_Datatype)13)
#define SPL_LONG_LONG    ((SPL_Datatype)13)
#define SPL_PACKED    ((SPL_Datatype)14)
#define SPL_LB      ((SPL_Datatype)15)
#define SPL_UB      ((SPL_Datatype)16)
#define SPL_FLOAT_INT    ((SPL_Datatype)17)
#define SPL_DOUBLE_INT    ((SPL_Datatype)18)
#define SPL_LONG_INT    ((SPL_Datatype)19)
#define SPL_SHORT_INT    ((SPL_Datatype)20)
#define SPL_2INT    ((SPL_Datatype)21)
#define SPL_LONG_DOUBLE_INT  ((SPL_Datatype)22)
/* Fortran data types */
#define SPL_COMPLEX    ((SPL_Datatype)23)
#define SPL_DOUBLE_COMPLEX  ((SPL_Datatype)24)
#define SPL_LOGICAL    ((SPL_Datatype)25)
#define SPL_REAL    ((SPL_Datatype)26)
#define SPL_DOUBLE_PRECISION  ((SPL_Datatype)27)
#define SPL_INTEGER    ((SPL_Datatype)28)
#define SPL_2INTEGER    ((SPL_Datatype)29)
#define SPL_2COMPLEX    ((SPL_Datatype)30)
#define SPL_2DOUBLE_COMPLEX  ((SPL_Datatype)31)
#define SPL_2REAL    ((SPL_Datatype)32)
#define SPL_2DOUBLE_PRECISION  ((SPL_Datatype)33)
#define SPL_CHARACTER    ((SPL_Datatype)1)

/* Communicators */
typedef int SPL_Comm;
#define SPL_COMM_WORLD    91
#define SPL_COMM_SELF    92

/* Groups */
typedef int SPL_Group;
#define SPL_GROUP_EMPTY    90

/* Collective operations */
typedef int SPL_Op;
#define SPL_MAX    (SPL_Op)(100)
#define SPL_MIN    (SPL_Op)(101)
#define SPL_SUM    (SPL_Op)(102)
#define SPL_PROD  (SPL_Op)(103)
#define SPL_LAND  (SPL_Op)(104)
#define SPL_BAND  (SPL_Op)(105)
#define SPL_LOR    (SPL_Op)(106)
#define SPL_BOR    (SPL_Op)(107)
#define SPL_LXOR  (SPL_Op)(108)
#define SPL_BXOR  (SPL_Op)(109)
#define SPL_MINLOC  (SPL_Op)(110)
#define SPL_MAXLOC  (SPL_Op)(111)

/* Permanent key values */
/* C Versions (return pointer to value) */
#define SPL_TAG_UB    81
#define SPL_HOST    83
#define SPL_IO      85
#define SPL_WTIME_IS_GLOBAL  87
/* Fortran Versions (return integer value) */
#define SPLR_TAG_UB    80
#define SPLR_HOST    82
#define SPLR_IO      84
#define SPLR_WTIME_IS_GLOBAL  86

/* Define some null objects */
#define SPL_COMM_NULL    ((SPL_Comm)0)
#define SPL_OP_NULL    ((SPL_Op)0)
#define SPL_GROUP_NULL    ((SPL_Group)0)
#define SPL_DATATYPE_NULL  ((SPL_Datatype)0)
#define SPL_REQUEST_NULL  ((SPL_Request)dmy_request_null)
#define SPL_ERRHANDLER_NULL  ((SPL_Errhandler )0)

/* These are only guesses; make sure you change them in mpif.h as well */
#define SPL_MAX_PROCESSOR_NAME  256
#define SPL_MAX_ERROR_STRING  512
#define SPL_MAX_NAME_STRING  63

/* Pre-defined constants */
#define SPL_UNDEFINED    (-32766)
#define SPL_UNDEFINED_RANK  SPL_UNDEFINED
#define SPL_KEYVAL_INVALID  0

/* Upper bound on the overhead in bsend for each message buffer */
#define SPL_BSEND_OVERHEAD  512

/* Topology types */
#define SPL_GRAPH    1
#define SPL_CART    2

#define SPL_BOTTOM    (void *)0

#define SPL_PROC_NULL    (-1)
#define SPL_ANY_SOURCE    (-2)
#define SPL_ROOT    (-3)
#define SPL_ANY_TAG    (-1)


/* User combination function */
typedef void (SPL_User_function) ( void *, void *, int *, SPL_Datatype * );

/* MPI Attribute copy and delete functions */
typedef int (SPL_Copy_function) ( SPL_Comm, int, void *, void *, void *, int * );
typedef int (SPL_Delete_function) ( SPL_Comm, int, void *, void * );

/* Include Fortran definitions */
typedef int SPL_Fint;
typedef long SPL_Aint;

/* MPI Error handlers */
typedef void (SPL_Handler_function) ();

/* SPL_MPI's error classes */
#include "splmpi_dmy_errno.h"

int spl_get_datasize(SPL_Datatype datatype);

/* MPI Procedures */
#include "splmpi_dmy_inline.h"

/* Sphere special subroutine */
void spl_pmsg(SPL_Comm grp, const char* format, ...);
void spl_perrmsg(SPL_Comm grp, const char* format, ...);
void spl_pwrnmsg(SPL_Comm grp, const char* format, ...);

SplBool
splCreateCommType1DA(
                     size_t ilen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType1D(size_t imin, size_t imax,
                    size_t ista, size_t iend,
                    SPL_Datatype oldType, SPL_Datatype* newType);

SplBool
splCreateCommType2DA(size_t imin, size_t imax,
                     size_t ilen, size_t jlen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType2D(size_t imin, size_t imax, size_t jmin,
                    size_t ista, size_t iend, size_t jsta, size_t jend,
                    SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType3DA(size_t imin, size_t imax,
                     size_t jmin, size_t jmax,
                     size_t ilen, size_t jlen, size_t klen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType3D(size_t imin, size_t imax,
                    size_t jmin, size_t jmax,
                    size_t kmin,
                    size_t ista, size_t iend,
                    size_t jsta, size_t jend,
                    size_t ksta, size_t kend,
                    SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType4DA(size_t imin, size_t imax,
                     size_t jmin, size_t jmax,
                     size_t kmin, size_t kmax,
                     size_t ilen, size_t jlen, size_t klen,
                     size_t llen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType4D(size_t imin, size_t imax,
                    size_t jmin, size_t jmax,
                    size_t kmin, size_t kmax,
                    size_t lmin,
                    size_t ista, size_t iend,
                    size_t jsta, size_t jend,
                    size_t ksta, size_t kend,
                    size_t lsta, size_t lend,
                    SPL_Datatype oldType, SPL_Datatype* newType);

SplBool
splCreateCommTypeVector(
                    int dims,
                    size_t min[3], size_t max[3],
                    size_t sta[3][3], size_t end[3][3],
                    SPL_Datatype oldType, SPL_Datatype* newType);

SplBool
splCreateCommTypeVectorRv(
                    int dims,
                    size_t min[3], size_t max[3],
                    size_t sta[3][3], size_t end[3][3],
                    SPL_Datatype oldType, SPL_Datatype* newType);

//int spl_get_datasize(SPL_Datatype datatype);
SPL_Datatype spl_get_datatype(int dtsize);
int spl_get_dataindex_bydatatype(SPL_Datatype datatype);
SPL_Datatype spl_get_datatype_byindex(int dataindex);

SplBool
splIsAllEquals(void *sendbuf, int count, SPL_Datatype datatype, SPL_Comm comm);


SplBool
splBcastSvFileList( splSvFileList* file_list,
                  int readRank,
                  SPL_Comm grp );

SplBool
splCreateCommGroup( int idNum,
                    int* idList,
                    SPL_Comm *newComm,
                    SPL_Group *newGrp );

SplBool
splCreateCommGroupUnion(
                    SPL_Group oldGroup1,
                    SPL_Group oldGroup2,
                    SPL_Comm *newComm,
                    SPL_Group *newGrp );

/**
 * \~english
 * check parallel execute
 * @return    parallel(=SplTrue)/single(=SplFalse)
 *
 * \~japanese
 * 並列実行されているかチェックする。
 *
 * MPIライブラリを使用できるかのチェックであり、コンパイル時の_HAS_MPICH_定義に従う。<br>
 * @return    並列実行(=SplTrue)/逐次実行(=SplFalse)
 */
SPL_MPI_INLINE SplBool splIsParallel() {return SplFalse;}

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_MPI_DMY_H_ */

