/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splmpi_dmy_errno.h
 * Dummy MPI wrapper for Sphere Low Level Library
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splmpi_dmy_errno.h
 * ダミーMPIエラー定義ファイル
 *
 * 逐次処理用のMPI関数、定義のダミー関数、定義<br>
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPL_MPI_DMY_ERRORNO_H_
#define _SPL_MPI_DMY_ERRORNO_H_

/* error return classes */
#define SPL_MPI_SUCCESS          0      /**< Successful return code */
/* Communication argument parameters */
#define SPL_MPI_ERR_BUFFER       1      /**< Invalid buffer pointer */
#define SPL_MPI_ERR_COUNT        2      /**< Invalid count argument */
#define SPL_MPI_ERR_TYPE         3      /**< Invalid datatype argument */
#define SPL_MPI_ERR_TAG          4      /**< Invalid tag argument */
#define SPL_MPI_ERR_COMM         5      /**< Invalid communicator */
#define SPL_MPI_ERR_RANK         6      /**< Invalid rank */
#define SPL_MPI_ERR_ROOT         7      /**< Invalid root */
#define SPL_MPI_ERR_TRUNCATE    14      /**< Message truncated on receive */

/* MPI Objects (other than COMM) */
#define SPL_MPI_ERR_GROUP        8      /**< Invalid group */
#define SPL_MPI_ERR_OP           9      /**< Invalid operation */
#define SPL_MPI_ERR_REQUEST     19      /**< Invalid mpi_request handle */

/* Special topology argument parameters */
#define SPL_MPI_ERR_TOPOLOGY    10      /**< Invalid topology */
#define SPL_MPI_ERR_DIMS        11      /**< Invalid dimension argument */

/* All other arguments.  This is a class with many kinds */
#define SPL_MPI_ERR_ARG         12      /**< Invalid argument */

/* Other errors that are not simply an invalid argument */
#define SPL_MPI_ERR_OTHER       15      /**< Other error; use Error_string */

#define SPL_MPI_ERR_UNKNOWN     13      /**< Unknown error */
#define SPL_MPI_ERR_INTERN      16      /**< Internal error code    */

/* Multiple completion has two special error classes */
#define SPL_MPI_ERR_IN_STATUS   17      /**< Look in status for error value */
#define SPL_MPI_ERR_PENDING     18      /**< Pending request */

/* New MPI-2 Error classes */
#define SPL_MPI_ERR_FILE        27      /* */
#define SPL_MPI_ERR_ACCESS      20      /* */
#define SPL_MPI_ERR_AMODE       21      /* */
#define SPL_MPI_ERR_BAD_FILE    22      /* */
#define SPL_MPI_ERR_FILE_EXISTS 25      /* */
#define SPL_MPI_ERR_FILE_IN_USE 26      /* */
#define SPL_MPI_ERR_NO_SPACE    36      /* */
#define SPL_MPI_ERR_NO_SUCH_FILE 37     /* */
#define SPL_MPI_ERR_IO          32      /* */
#define SPL_MPI_ERR_READ_ONLY   40      /* */
#define SPL_MPI_ERR_CONVERSION  23      /* */
#define SPL_MPI_ERR_DUP_DATAREP 24      /* */
#define SPL_MPI_ERR_UNSUPPORTED_DATAREP   43  /* */

/* MPI_ERR_INFO is NOT defined in the MPI-2 standard.  I believe that
   this is an oversight */
#define SPL_MPI_ERR_INFO        28      /* */
#define SPL_MPI_ERR_INFO_KEY    29      /* */
#define SPL_MPI_ERR_INFO_VALUE  30      /* */
#define SPL_MPI_ERR_INFO_NOKEY  31      /* */

#define SPL_MPI_ERR_NAME        33      /* */
#define SPL_MPI_ERR_NO_MEM      34      /* */
#define SPL_MPI_ERR_NOT_SAME    35      /* */
#define SPL_MPI_ERR_PORT        38      /* */
#define SPL_MPI_ERR_QUOTA       39      /* */
#define SPL_MPI_ERR_SERVICE     41      /* */
#define SPL_MPI_ERR_SPAWN       42      /* */
#define SPL_MPI_ERR_UNSUPPORTED_OPERATION 44 /* */
#define SPL_MPI_ERR_WIN         45      /* */

#define SPL_MPI_ERR_LASTCODE    0x3FFFFFFF      /**< Last error code*/

#endif /* _SPL_MPI_DMY_ERRORNO_H_ */




