/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splmpi_mpi.h
 * MPI wrapper defines for Sphere Low Level Library
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splmpi_mpi.h
 * MPIラッパー定義ファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPL_MPI_MPI_H_
#define _SPL_MPI_MPI_H_

#include <sys/types.h>
#include <mpi.h>

/* Keep C++ compilers from getting confused */
#if defined(__cplusplus)
extern "C" {
#endif

/* Data types */
typedef MPI_Datatype SPL_Datatype;
#define SPL_CHAR    MPI_CHAR
#define SPL_UNSIGNED_CHAR  MPI_UNSIGNED_CHAR
#define SPL_BYTE    MPI_BYTE
#define SPL_SHORT    MPI_SHORT
#define SPL_UNSIGNED_SHORT  MPI_UNSIGNED_SHORT
#define SPL_INT      MPI_INT
#define SPL_UNSIGNED    MPI_UNSIGNED
#define SPL_LONG    MPI_LONG
#define SPL_UNSIGNED_LONG  MPI_UNSIGNED_LONG
#define SPL_FLOAT    MPI_FLOAT
#define SPL_DOUBLE    MPI_DOUBLE
#define SPL_LONG_DOUBLE    MPI_LONG_DOUBLE
#define SPL_LONG_LONG_INT  MPI_LONG_LONG_INT
#define SPL_LONG_LONG    MPI_LONG_LONG
#define SPL_PACKED    MPI_PACKED
#define SPL_LB      MPI_LB
#define SPL_UB      MPI_UB
#define SPL_FLOAT_INT    MPI_FLOAT_INT
#define SPL_DOUBLE_INT    MPI_DOUBLE_INT
#define SPL_LONG_INT    MPI_LONG_INT
#define SPL_SHORT_INT    MPI_SHORT_INT
#define SPL_2INT    MPI_2INT
#define SPL_LONG_DOUBLE_INT  MPI_LONG_DOUBLE_INT
/* Fortran data types */
#define SPL_COMPLEX    MPI_COMPLEX
#define SPL_DOUBLE_COMPLEX  MPI_DOUBLE_COMPLEX
#define SPL_LOGICAL    MPI_LOGICAL
#define SPL_REAL    MPI_REAL
#define SPL_DOUBLE_PRECISION  MPI_DOUBLE_PRECISION
#define SPL_INTEGER    MPI_INTEGER
#define SPL_2INTEGER    MPI_2INTEGER
#define SPL_2COMPLEX    MPI_2COMPLEX
#define SPL_2DOUBLE_COMPLEX  MPI_2DOUBLE_COMPLEX
#define SPL_2REAL    MPI_2REAL
#define SPL_2DOUBLE_PRECISION  MPI_2DOUBLE_PRECISION
#define SPL_CHARACTER    MPI_CHARACTER

/* Communicators */
typedef MPI_Comm SPL_Comm;
#define SPL_COMM_WORLD    MPI_COMM_WORLD
#define SPL_COMM_SELF    MPI_COMM_SELF

/* Groups */
typedef MPI_Group SPL_Group;
#define SPL_GROUP_EMPTY    MPI_GROUP_EMPTY

/* Collective operations */
typedef MPI_Op SPL_Op;
#define SPL_MAX    MPI_MAX
#define SPL_MIN    MPI_MIN
#define SPL_SUM    MPI_SUM
#define SPL_PROD  MPI_PROD
#define SPL_LAND  MPI_LAND
#define SPL_BAND  MPI_BAND
#define SPL_LOR    MPI_LOR
#define SPL_BOR    MPI_BOR
#define SPL_LXOR  MPI_LXOR
#define SPL_BXOR  MPI_BXOR
#define SPL_MINLOC  MPI_MINLOC
#define SPL_MAXLOC  MPI_MAXLOC

/* Permanent key values */
/* C Versions (return pointer to value) */
#define SPL_TAG_UB    MPI_TAG_UB
#define SPL_HOST    MPI_HOST
#define SPL_IO      MPI_IO
#define SPL_WTIME_IS_GLOBAL  MPI_WTIME_IS_GLOBAL
/* Fortran Versions (return integer value) */
#define SPLR_TAG_UB    MPIR_TAG_UB
#define SPLR_HOST    MPIR_HOST
#define SPLR_IO      MPIR_IO
#define SPLR_WTIME_IS_GLOBAL  MPIR_WTIME_IS_GLOBAL

/* Define some null objects */
#define SPL_COMM_NULL    MPI_COMM_NULL
#define SPL_OP_NULL    MPI_OP_NULL
#define SPL_GROUP_NULL    MPI_GROUP_NULL
#define SPL_DATATYPE_NULL  MPI_DATATYPE_NULL
#define SPL_REQUEST_NULL  MPI_REQUEST_NULL
#define SPL_ERRHANDLER_NULL  MPI_ERRHANDLER_NULL

/* These are only guesses; make sure you change them in mpif.h as well */
#define SPL_MAX_PROCESSOR_NAME  MPI_MAX_PROCESSOR_NAME
#define SPL_MAX_ERROR_STRING  MPI_MAX_ERROR_STRING
#define SPL_MAX_NAME_STRING  MPI_MAX_NAME_STRING

/* Pre-defined constants */
#define SPL_UNDEFINED    MPI_UNDEFINED
#define SPL_UNDEFINED_RANK  MPI_UNDEFINED_RANK
#define SPL_KEYVAL_INVALID  MPI_KEYVAL_INVALID

/* Upper bound on the overhead in bsend for each message buffer */
#define SPL_BSEND_OVERHEAD  MPI_BSEND_OVERHEAD

/* Topology types */
#define SPL_GRAPH    MPI_GRAPH
#define SPL_CART    MPI_CART

#define SPL_BOTTOM    MPI_BOTTOM

#define SPL_PROC_NULL    MPI_PROC_NULL
#define SPL_ANY_SOURCE    MPI_ANY_SOURCE
#define SPL_ROOT    MPI_ROOT
#define SPL_ANY_TAG    MPI_ANY_TAG

/*  */
typedef MPI_Errhandler SPL_Errhandler;

/* MPI request opjects */
typedef MPI_Request SPL_Request;

/* MPI status opject */
typedef MPI_Status SPL_Status;

/* User combination function */
//typedef void (MPI_User_function) ( void *, void *, int *, MPI_Datatype * );
typedef MPI_User_function SPL_User_function;

/* MPI Attribute copy and delete functions */
typedef MPI_Copy_function SPL_Copy_function;
typedef MPI_Delete_function SPL_Delete_function;

/* Include Fortran definitions */
typedef MPI_Fint SPL_Fint;
typedef MPI_Aint SPL_Aint;

/* MPI Error handlers */
typedef MPI_Handler_function SPL_Handler_function;

/* SPL_MPI's error classes */
#include "splmpi_mpi_errno.h"

/* MPI Procedures */
#include "splmpi_mpi_inline.h"

/* Sphere special subroutine */
void spl_pmsg(SPL_Comm grp, const char* format, ...);
void spl_perrmsg(SPL_Comm grp, const char* format, ...);
void spl_pwrnmsg(SPL_Comm grp, const char* format, ...);

SplBool
splCreateCommType1DA(
                     size_t ilen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType1D(size_t imin, size_t imax,
                    size_t ista, size_t iend,
                    SPL_Datatype oldType, SPL_Datatype* newType);

SplBool
splCreateCommType2DA(size_t imin, size_t imax,
                     size_t ilen, size_t jlen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType2D(size_t imin, size_t imax, size_t jmin,
                    size_t ista, size_t iend, size_t jsta, size_t jend,
                    SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType3DA(size_t imin, size_t imax,
                     size_t jmin, size_t jmax,
                     size_t ilen, size_t jlen, size_t klen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType3D(size_t imin, size_t imax,
                    size_t jmin, size_t jmax,
                    size_t kmin,
                    size_t ista, size_t iend,
                    size_t jsta, size_t jend,
                    size_t ksta, size_t kend,
                    SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType4DA(size_t imin, size_t imax,
                     size_t jmin, size_t jmax,
                     size_t kmin, size_t kmax,
                     size_t ilen, size_t jlen, size_t klen,
                     size_t llen,
                     SPL_Datatype oldType, SPL_Datatype* newType);
SplBool
splCreateCommType4D(size_t imin, size_t imax,
                    size_t jmin, size_t jmax,
                    size_t kmin, size_t kmax,
                    size_t lmin,
                    size_t ista, size_t iend,
                    size_t jsta, size_t jend,
                    size_t ksta, size_t kend,
                    size_t lsta, size_t lend,
                    SPL_Datatype oldType, SPL_Datatype* newType);

SplBool
splCreateCommTypeVector(
                    int dims,
                    size_t min[3], size_t max[3],
                    size_t sta[3][3], size_t end[3][3],
                    SPL_Datatype oldType, SPL_Datatype* newType);

SplBool
splCreateCommTypeVectorRv(
                    int dims,
                    size_t min[3], size_t max[3],
                    size_t sta[3][3], size_t end[3][3],
                    SPL_Datatype oldType, SPL_Datatype* newType);

int spl_get_datasize(SPL_Datatype datatype);
SPL_Datatype spl_get_datatype(int dtsize);
int spl_get_dataindex_bydatatype(SPL_Datatype datatype);
SPL_Datatype spl_get_datatype_byindex(int dataindex);

SplBool
splIsAllEquals(void *sendbuf, int count, SPL_Datatype datatype, SPL_Comm comm);


SplBool
splBcastSvFileList( splSvFileList* file_list,
                  int readRank,
                  SPL_Comm grp );

SplBool
splCreateCommGroup( int idNum,
                    int* idList,
                    SPL_Comm *newComm,
                    SPL_Group *newGrp );
SplBool
splCreateCommGroupUnion(
                    SPL_Group oldGroup1,
                    SPL_Group oldGroup2,
                    SPL_Comm *newComm,
                    SPL_Group *newGrp );

/**
 * \~english
 * check parallel execute
 * @return    parallel(=SplTrue)/single(=SplFalse)
 *
 * \~japanese
 * 並列実行されているかチェックする。
 *
 * MPIライブラリを使用できるかのチェックであり、コンパイル時の_HAS_MPICH_定義に従う。<br>
 * @return    並列実行(=SplTrue)/逐次実行(=SplFalse)
 */
SPL_MPI_INLINE SplBool splIsParallel() {return SplTrue;}

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_MPI_MPI_H_ */

