/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splmpi_mpi_errno.h
 * Declaration of Error Number
 *
 * MPI wrapper for Sphere Low Level Library
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splmpi_mpi_errno.h
 * MPIエラー定義ファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPL_MPI_MPI_ERRORNO_H_
#define _SPL_MPI_MPI_ERRORNO_H_

#include "mpi.h"

/* error return classes */
#define SPL_MPI_SUCCESS    MPI_SUCCESS    /**< Successful return code */
/* Communication argument parameters */
#define SPL_MPI_ERR_BUFFER  MPI_ERR_BUFFER    /**< Invalid buffer pointer */
#define SPL_MPI_ERR_COUNT  MPI_ERR_COUNT    /**< Invalid count argument */
#define SPL_MPI_ERR_TYPE  MPI_ERR_TYPE    /**< Invalid datatype argument */
#define SPL_MPI_ERR_TAG    MPI_ERR_TAG    /**< Invalid tag argument */
#define SPL_MPI_ERR_COMM  MPI_ERR_COMM    /**< Invalid communicator */
#define SPL_MPI_ERR_RANK  MPI_ERR_RANK    /**< Invalid rank */
#define SPL_MPI_ERR_ROOT  MPI_ERR_ROOT    /**< Invalid root */
#define SPL_MPI_ERR_TRUNCATE  MPI_ERR_TRUNCATE  /**< Message truncated on receive */

/* MPI Objects (other than COMM) */
#define SPL_MPI_ERR_GROUP  MPI_ERR_GROUP    /**< Invalid group */
#define SPL_MPI_ERR_OP    MPI_ERR_OP    /**< Invalid operation */
#define SPL_MPI_ERR_REQUEST  MPI_ERR_REQUEST    /**< Invalid mpi_request handle */

/* Special topology argument parameters */
#define SPL_MPI_ERR_TOPOLOGY  MPI_ERR_TOPOLOGY  /**< Invalid topology */
#define SPL_MPI_ERR_DIMS  MPI_ERR_DIMS    /**< Invalid dimension argument */

/* All other arguments.  This is a class with many kinds */
#define SPL_MPI_ERR_ARG    MPI_ERR_ARG    /**< Invalid argument */

/* Other errors that are not simply an invalid argument */
#define SPL_MPI_ERR_OTHER  MPI_ERR_OTHER    /**< Other error; use Error_string */

#define SPL_MPI_ERR_UNKNOWN  MPI_ERR_UNKNOWN    /**< Unknown error */
#define SPL_MPI_ERR_INTERN  MPI_ERR_INTERN    /**< Internal error code    */

/* Multiple completion has two special error classes */
#define SPL_MPI_ERR_IN_STATUS  MPI_ERR_IN_STATUS  /**< Look in status for error value */
#define SPL_MPI_ERR_PENDING  MPI_ERR_PENDING    /**< Pending request */

/* New MPI-2 Error classes */
#define SPL_MPI_ERR_FILE    MPI_ERR_FILE
#define SPL_MPI_ERR_ACCESS    MPI_ERR_ACCESS
#define SPL_MPI_ERR_AMODE    MPI_ERR_AMODE
#define SPL_MPI_ERR_BAD_FILE    MPI_ERR_BAD_FILE
#define SPL_MPI_ERR_FILE_EXISTS    MPI_ERR_FILE_EXISTS
#define SPL_MPI_ERR_FILE_IN_USE    MPI_ERR_FILE_IN_USE
#define SPL_MPI_ERR_NO_SPACE    MPI_ERR_NO_SPACE
#define SPL_MPI_ERR_NO_SUCH_FILE  MPI_ERR_NO_SUCH_FILE
#define SPL_MPI_ERR_IO      MPI_ERR_IO
#define SPL_MPI_ERR_READ_ONLY    MPI_ERR_READ_ONLY
#define SPL_MPI_ERR_CONVERSION    MPI_ERR_CONVERSION
#define SPL_MPI_ERR_DUP_DATAREP    MPI_ERR_DUP_DATAREP
#define SPL_MPI_ERR_UNSUPPORTED_DATAREP  MPI_ERR_UNSUPPORTED_DATAREP

/* MPI_ERR_INFO is NOT defined in the MPI-2 standard.  I believe that
   this is an oversight */
#define SPL_MPI_ERR_INFO  MPI_ERR_INFO
#define SPL_MPI_ERR_INFO_KEY  MPI_ERR_INFO_KEY
#define SPL_MPI_ERR_INFO_VALUE  MPI_ERR_INFO_VALUE
#define SPL_MPI_ERR_INFO_NOKEY  MPI_ERR_INFO_NOKEY

#define SPL_MPI_ERR_NAME  MPI_ERR_NAME
#define SPL_MPI_ERR_NO_MEM  MPI_ERR_NO_MEM
#define SPL_MPI_ERR_NOT_SAME  MPI_ERR_NOT_SAME
#define SPL_MPI_ERR_PORT  MPI_ERR_PORT
#define SPL_MPI_ERR_QUOTA  MPI_ERR_QUOTA
#define SPL_MPI_ERR_SERVICE  MPI_ERR_SERVICE
#define SPL_MPI_ERR_SPAWN  MPI_ERR_SPAWN
#define SPL_MPI_ERR_UNSUPPORTED_OPERATION  MPI_ERR_UNSUPPORTED_OPERATION
#define SPL_MPI_ERR_WIN    MPI_ERR_WIN

#define SPL_MPI_ERR_LASTCODE  MPI_ERR_LASTCODE  /**< Last error code*/

#endif /* _SPL_MPI_MPI_ERRORNO_H_ */
