/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splmpi_ompi.h
 * OpenMPI wrapper defines for Sphere Low Level Library
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     tamaki
 * @date       2009/09/07
 *
 * \~japanese
 * @file splmpi_ompi.h
 * OpenMPIラッパー定義ファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     tamaki
 * @date       2009/09/07
 */

#ifndef _SPL_MPI_OMPI_H_
#define _SPL_MPI_OMPI_H_

#include "mpi.h"

#ifndef OMPI_CONFIG_H
  #ifndef MPI_Fint
    #define MPI_Fint ompi_fortran_integer_t
  #endif
#endif /* #ifndef OMPI_CONFIG_H */

#define SPL_MPI_VERSION MPI_VERSION
#define SPL_MPI_SUBVERSION MPI_SUBVERSION

#if defined(c_plusplus) || defined(__cplusplus)
extern "C" {
#endif
//-----------------------------------------------------------------------------------------------------------
/*
 * User typedefs
 */
  typedef int (MPI_Copy_function)(MPI_Comm, int, void *, void *, void *, int *);
  typedef int (MPI_Delete_function)(MPI_Comm, int, void *, void *);
  typedef int (MPI_Datarep_extent_function)(MPI_Datatype, MPI_Aint *, void *);
  typedef int (MPI_Datarep_conversion_function)(void *, MPI_Datatype,  int, void *, MPI_Offset, void *);
  typedef void (MPI_Comm_errhandler_fn)(MPI_Comm *, int *, ...);
  typedef void (MPI_Win_errhandler_fn)(MPI_Win *, int *, ...);
  typedef void (MPI_Handler_function)(MPI_Comm *, int *, ...);
  typedef void (MPI_User_function)(void *, void *, int *, MPI_Datatype *);
  typedef int (MPI_Comm_copy_attr_function)(MPI_Comm, int, void *, void *, void *, int *);
  typedef int (MPI_Comm_delete_attr_function)(MPI_Comm, int, void *, void *);
  typedef int (MPI_Type_copy_attr_function)(MPI_Datatype, int, void *, void *, void *, int *);
  typedef int (MPI_Type_delete_attr_function)(MPI_Datatype, int, void *, void *);
  typedef int (MPI_Win_copy_attr_function)(MPI_Win, int, void *, void *, void *, int *);
  typedef int (MPI_Win_delete_attr_function)(MPI_Win, int, void *, void *);
  typedef int (MPI_Grequest_query_function)(void *, MPI_Status *);
  typedef int (MPI_Grequest_free_function)(void *);
  typedef int (MPI_Grequest_cancel_function)(void *, int); 
//-----------------------------------------------------------------------------------------------------------

/*
 * Miscellaneous constants
 */
  #define SPL_ANY_SOURCE		MPI_ANY_SOURCE         /* match any source rank */
  #define SPL_PROC_NULL			MPI_PROC_NULL          /* rank of null process */
  #define SPL_ROOT			MPI_ROOT
  #define SPL_ANY_TAG			MPI_ANY_TAG            /* match any message tag */
  #define SPL_MAX_PROCESSOR_NAME	MPI_MAX_PROCESSOR_NAME /* max proc. name length */
  #define SPL_MAX_ERROR_STRING		MPI_MAX_ERROR_STRING   /* max error message length */
  #define SPL_MAX_OBJECT_NAME		MPI_MAX_OBJECT_NAME    /* max object name length */
  #define SPL_UNDEFINED			MPI_UNDEFINED          /* undefined stuff */
  #define SPL_CART			MPI_CART               /* cartesian topology */
  #define SPL_GRAPH			MPI_GRAPH              /* graph topology */
  #define SPL_KEYVAL_INVALID		MPI_KEYVAL_INVALID     /* invalid key value */

/*
 * More constants
 */
  #define SPL_BOTTOM			MPI_BOTTOM               /* base reference address */
  #define SPL_IN_PLACE			MPI_IN_PLACE             /* in place buffer */
  #define SPL_BSEND_OVERHEAD		MPI_BSEND_OVERHEAD       /* size of bsend header + ptr */
  #define SPL_MAX_INFO_KEY		MPI_MAX_INFO_KEY         /* max info key length */
  #define SPL_MAX_INFO_VAL		MPI_MAX_INFO_VAL         /* max info value length */
  #define SPL_ARGV_NULL			MPI_ARGV_NULL            /* NULL argument vector */
  #define SPL_ARGVS_NULL		MPI_ARGVS_NULL           /* NULL argument vectors */
  #define SPL_ERRCODES_IGNORE		MPI_ERRCODES_IGNORE      /* don't return error codes */
  #define SPL_MAX_PORT_NAME		MPI_MAX_PORT_NAME        /* max port name length */
  #define SPL_MAX_NAME_LEN		MPI_MAX_NAME_LEN         /* max port name length */
  #define SPL_ORDER_C			MPI_ORDER_C              /* C row major order */
  #define SPL_ORDER_FORTRAN		MPI_ORDER_FORTRAN        /* Fortran column major order */
  #define SPL_DISTRIBUTE_BLOCK		MPI_DISTRIBUTE_BLOCK     /* block distribution */
  #define SPL_DISTRIBUTE_CYCLIC		MPI_DISTRIBUTE_CYCLIC    /* cyclic distribution */
  #define SPL_DISTRIBUTE_NONE		MPI_DISTRIBUTE_NONE      /* not distributed */
  #define SPL_DISTRIBUTE_DFLT_DARG	MPI_DISTRIBUTE_DFLT_DARG /* default distribution arg */

  #if OMPI_PROVIDE_MPI_FILE_INTERFACE
/*
 * Since these values are arbitrary to Open MPI, we might as well make
 * them the same as ROMIO for ease of mapping.  These values taken
 * from ROMIO's mpio.h file.
 */
    #define SPL_MODE_CREATE		MPI_MODE_CREATE          /* ADIO_CREATE */ 
    #define SPL_MODE_RDONLY		MPI_MODE_RDONLY          /* ADIO_RDONLY */
    #define SPL_MODE_WRONLY		MPI_MODE_WRONLY          /* ADIO_WRONLY  */
    #define SPL_MODE_RDWR		MPI_MODE_RDWR            /* ADIO_RDWR  */
    #define SPL_MODE_DELETE_ON_CLOSE	MPI_MODE_DELETE_ON_CLOSE /* ADIO_DELETE_ON_CLOSE */
    #define SPL_MODE_UNIQUE_OPEN	MPI_MODE_UNIQUE_OPEN     /* ADIO_UNIQUE_OPEN */
    #define SPL_MODE_EXCL		MPI_MODE_EXCL            /* ADIO_EXCL */
    #define SPL_MODE_APPEND		MPI_MODE_APPEND          /* ADIO_APPEND */
    #define SPL_MODE_SEQUENTIAL		MPI_MODE_SEQUENTIAL      /* ADIO_SEQUENTIAL */
    #define SPL_DISPLACEMENT_CURRENT	MPI_DISPLACEMENT_CURRENT
    #define SPL_SEEK_SET		MPI_SEEK_SET
    #define SPL_SEEK_CUR		MPI_SEEK_CUR
    #define SPL_SEEK_END		MPI_SEEK_END
    #define SPL_MAX_DATAREP_STRING	MPI_MAX_DATAREP_STRING
  #endif /* #if OMPI_PROVIDE_MPI_FILE_INTERFACE */

/*
 * MPI-2 One-Sided Communications asserts
 */
  #define SPL_MODE_NOCHECK	MPI_MODE_NOCHECK
  #define SPL_MODE_NOPRECEDE	MPI_MODE_NOPRECEDE
  #define SPL_MODE_NOPUT	MPI_MODE_NOPUT
  #define SPL_MODE_NOSTORE	MPI_MODE_NOSTORE
  #define SPL_MODE_NOSUCCEED	MPI_MODE_NOSUCCEED
  #define SPL_LOCK_EXCLUSIVE	MPI_LOCK_EXCLUSIVE
  #define SPL_LOCK_SHARED	MPI_LOCK_SHARED


/*
 * Predefined attribute keyvals
 *
 * DO NOT CHANGE THE ORDER WITHOUT ALSO CHANGING THE ORDER IN
 * src/attribute/attribute_predefined.c and mpif.h.in.
 */
  enum {
    /* MPI-1 */
    SPL_TAG_UB 		= MPI_TAG_UB,
    SPL_HOST 		= MPI_HOST,
    SPL_IO 		= MPI_IO,
    SPL_WTIME_IS_GLOBAL = MPI_WTIME_IS_GLOBAL,

    /* MPI-2 */
    SPL_APPNUM 		= MPI_APPNUM,
    SPL_LASTUSEDCODE 	= MPI_LASTUSEDCODE,
    SPL_UNIVERSE_SIZE 	= MPI_UNIVERSE_SIZE,
    SPL_WIN_BASE 	= MPI_WIN_BASE,
    SPL_WIN_SIZE 	= MPI_WIN_SIZE,
    SPL_WIN_DISP_UNIT 	= MPI_WIN_DISP_UNIT,

    /* Even though these four are IMPI attributes, they need to be there
       for all MPI jobs */
    ISPL_CLIENT_SIZE 	= IMPI_CLIENT_SIZE,
    ISPL_CLIENT_COLOR 	= IMPI_CLIENT_COLOR,
    ISPL_HOST_SIZE 	= IMPI_HOST_SIZE,
    ISPL_HOST_COLOR 	= IMPI_HOST_COLOR
  };

/*
 * Error classes and codes
 * Do not change the values of these without also modifying mpif.h.in.
 */
  #define SPL_SUCCESS			MPI_SUCCESS
  #define SPL_ERR_BUFFER		MPI_ERR_BUFFER
  #define SPL_ERR_COUNT			MPI_ERR_COUNT
  #define SPL_ERR_TYPE			MPI_ERR_TYPE
  #define SPL_ERR_TAG			MPI_ERR_TAG
  #define SPL_ERR_COMM			MPI_ERR_COMM
  #define SPL_ERR_RANK			MPI_ERR_RANK
  #define SPL_ERR_REQUEST		MPI_ERR_REQUEST
  #define SPL_ERR_ROOT			MPI_ERR_ROOT
  #define SPL_ERR_GROUP			MPI_ERR_GROUP
  #define SPL_ERR_OP			MPI_ERR_OP
  #define SPL_ERR_TOPOLOGY		MPI_ERR_TOPOLOGY
  #define SPL_ERR_DIMS			MPI_ERR_DIMS
  #define SPL_ERR_ARG			MPI_ERR_ARG
  #define SPL_ERR_UNKNOWN		MPI_ERR_UNKNOWN
  #define SPL_ERR_TRUNCATE		MPI_ERR_TRUNCATE
  #define SPL_ERR_OTHER			MPI_ERR_OTHER
  #define SPL_ERR_INTERN		MPI_ERR_INTERN
  #define SPL_ERR_IN_STATUS		MPI_ERR_IN_STATUS
  #define SPL_ERR_PENDING		MPI_ERR_PENDING
  #define SPL_ERR_ACCESS		MPI_ERR_ACCESS
  #define SPL_ERR_AMODE			MPI_ERR_AMODE
  #define SPL_ERR_ASSERT		MPI_ERR_ASSERT
  #define SPL_ERR_BAD_FILE		MPI_ERR_BAD_FILE
  #define SPL_ERR_BASE			MPI_ERR_BASE
  #define SPL_ERR_CONVERSION		MPI_ERR_CONVERSION
  #define SPL_ERR_DISP			MPI_ERR_DISP
  #define SPL_ERR_DUP_DATAREP		MPI_ERR_DUP_DATAREP
  #define SPL_ERR_FILE_EXISTS		MPI_ERR_FILE_EXISTS
  #define SPL_ERR_FILE_IN_USE		MPI_ERR_FILE_IN_USE
  #define SPL_ERR_FILE			MPI_ERR_FILE
  #define SPL_ERR_INFO_KEY		MPI_ERR_INFO_KEY
  #define SPL_ERR_INFO_NOKEY		MPI_ERR_INFO_NOKEY
  #define SPL_ERR_INFO_VALUE		MPI_ERR_INFO_VALUE
  #define SPL_ERR_INFO			MPI_ERR_INFO
  #define SPL_ERR_IO			MPI_ERR_IO
  #define SPL_ERR_KEYVAL		MPI_ERR_KEYVAL
  #define SPL_ERR_LOCKTYPE		MPI_ERR_LOCKTYPE
  #define SPL_ERR_NAME			MPI_ERR_NAME
  #define SPL_ERR_NO_MEM		MPI_ERR_NO_MEM
  #define SPL_ERR_NOT_SAME		MPI_ERR_NOT_SAME
  #define SPL_ERR_NO_SPACE		MPI_ERR_NO_SPACE
  #define SPL_ERR_NO_SUCH_FILE		MPI_ERR_NO_SUCH_FILE
  #define SPL_ERR_PORT			MPI_ERR_PORT
  #define SPL_ERR_QUOTA			MPI_ERR_QUOTA
  #define SPL_ERR_READ_ONLY		MPI_ERR_READ_ONLY
  #define SPL_ERR_RMA_CONFLICT		MPI_ERR_RMA_CONFLICT
  #define SPL_ERR_RMA_SYNC		MPI_ERR_RMA_SYNC
  #define SPL_ERR_SERVICE		MPI_ERR_SERVICE
  #define SPL_ERR_SIZE			MPI_ERR_SIZE
  #define SPL_ERR_SPAWN			MPI_ERR_SPAWN
  #define SPL_ERR_UNSUPPORTED_DATAREP	MPI_ERR_UNSUPPORTED_DATAREP
  #define SPL_ERR_UNSUPPORTED_OPERATION	MPI_ERR_UNSUPPORTED_OPERATION
  #define SPL_ERR_WIN			MPI_ERR_WIN
  #define SPL_ERR_LASTCODE		MPI_ERR_LASTCODE

  #define SPL_ERR_SYSRESOURCE		MPI_ERR_SYSRESOURCE


/*
 * Comparison results.  Don't change the order of these, the group
 * comparison functions rely on it.
 * Do not change the order of these without also modifying mpif.h.in.
 */
  enum {
    SPL_IDENT 		= MPI_IDENT,
    SPL_CONGRUENT 	= MPI_CONGRUENT,
    SPL_SIMILAR 	= MPI_SIMILAR,
    SPL_UNEQUAL 	= MPI_UNEQUAL
  };

/*
 * MPI_Init_thread constants
 * Do not change the order of these without also modifying mpif.h.in.
 */
  enum {
    SPL_THREAD_SINGLE 		= MPI_THREAD_SINGLE,
    SPL_THREAD_FUNNELED 	= MPI_THREAD_FUNNELED,
    SPL_THREAD_SERIALIZED 	= MPI_THREAD_SERIALIZED,
    SPL_THREAD_MULTIPLE 	= MPI_THREAD_MULTIPLE
  };

/*
 * Datatype combiners.
 * Do not change the order of these without also modifying mpif.h.in.
 */
  enum {
    SPL_COMBINER_NAMED 			= MPI_COMBINER_NAMED,
    SPL_COMBINER_DUP 			= MPI_COMBINER_DUP,
    SPL_COMBINER_CONTIGUOUS 		= MPI_COMBINER_CONTIGUOUS,
    SPL_COMBINER_VECTOR 		= MPI_COMBINER_VECTOR,
    SPL_COMBINER_HVECTOR_INTEGER 	= MPI_COMBINER_HVECTOR_INTEGER,
    SPL_COMBINER_HVECTOR 		= MPI_COMBINER_HVECTOR,
    SPL_COMBINER_INDEXED 		= MPI_COMBINER_INDEXED,
    SPL_COMBINER_HINDEXED_INTEGER 	= MPI_COMBINER_HINDEXED_INTEGER,
    SPL_COMBINER_HINDEXED 		= MPI_COMBINER_HINDEXED,
    SPL_COMBINER_INDEXED_BLOCK 		= MPI_COMBINER_INDEXED_BLOCK,
    SPL_COMBINER_STRUCT_INTEGER		= MPI_COMBINER_STRUCT_INTEGER,
    SPL_COMBINER_STRUCT 		= MPI_COMBINER_STRUCT,
    SPL_COMBINER_SUBARRAY 		= MPI_COMBINER_SUBARRAY,
    SPL_COMBINER_DARRAY 		= MPI_COMBINER_DARRAY,
    SPL_COMBINER_F90_REAL 		= MPI_COMBINER_F90_REAL,
    SPL_COMBINER_F90_COMPLEX 		= MPI_COMBINER_F90_COMPLEX,
    SPL_COMBINER_F90_INTEGER 		= MPI_COMBINER_F90_INTEGER,
    SPL_COMBINER_RESIZED 		= MPI_COMBINER_RESIZED
  };

/*
 * NULL handles
 */
  #define MPI_GROUP_NULL OMPI_PREDEFINED_GLOBAL(MPI_Group, ompi_mpi_group_null)
  #define MPI_COMM_NULL OMPI_PREDEFINED_GLOBAL(MPI_Comm, ompi_mpi_comm_null)
  #define MPI_REQUEST_NULL OMPI_PREDEFINED_GLOBAL(MPI_Request, ompi_request_null)
  #define MPI_OP_NULL OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_null)
  #define MPI_ERRHANDLER_NULL OMPI_PREDEFINED_GLOBAL(MPI_Errhandler, ompi_mpi_errhandler_null)
  #define MPI_INFO_NULL OMPI_PREDEFINED_GLOBAL(MPI_Info, ompi_mpi_info_null)
  #define MPI_WIN_NULL OMPI_PREDEFINED_GLOBAL(MPI_Win, ompi_mpi_win_null)
  #if OMPI_PROVIDE_MPI_FILE_INTERFACE
    #define MPI_FILE_NULL OMPI_PREDEFINED_GLOBAL(MPI_File, ompi_mpi_file_null)
  #endif

  #define MPI_STATUS_IGNORE ((MPI_Status *) 0)
  #define MPI_STATUSES_IGNORE ((MPI_Status *) 0)

/* MPI-2 specifies that the name "MPI_TYPE_NULL_DELETE_FN" (and all
   related friends) must be accessible in C, C++, and Fortran. This is
   unworkable if the back-end Fortran compiler uses all caps for its
   linker symbol convention -- it results in two functions with
   different signatures that have the same name (i.e., both C and
   Fortran use the symbol MPI_TYPE_NULL_DELETE_FN).  So we have to
   #define the C names to be something else, so that they names are
   *accessed* through MPI_TYPE_NULL_DELETE_FN, but their actual symbol
   name is different.

   However, this file is included when the fortran wrapper functions
   are compiled in Open MPI, so we do *not* want these #defines in
   this case (i.e., we need the Fortran wrapper function to be
   compiled as MPI_TYPE_NULL_DELETE_FN).  So add some #if kinds of
   protection for this case. */

  #if !defined(OMPI_COMPILING_F77_WRAPPERS)
    #define MPI_NULL_DELETE_FN OMPI_C_MPI_NULL_DELETE_FN
    #define MPI_NULL_COPY_FN OMPI_C_MPI_NULL_COPY_FN
    #define MPI_DUP_FN OMPI_C_MPI_DUP_FN

    #define MPI_TYPE_NULL_DELETE_FN OMPI_C_MPI_TYPE_NULL_DELETE_FN
    #define MPI_TYPE_NULL_COPY_FN OMPI_C_MPI_TYPE_NULL_COPY_FN
    #define MPI_TYPE_DUP_FN OMPI_C_MPI_TYPE_DUP_FN

    #define MPI_COMM_NULL_DELETE_FN OMPI_C_MPI_COMM_NULL_DELETE_FN
    #define MPI_COMM_NULL_COPY_FN OMPI_C_MPI_COMM_NULL_COPY_FN
    #define MPI_COMM_DUP_FN OMPI_C_MPI_COMM_DUP_FN

    #define MPI_WIN_NULL_DELETE_FN OMPI_C_MPI_WIN_NULL_DELETE_FN
    #define MPI_WIN_NULL_COPY_FN OMPI_C_MPI_WIN_NULL_COPY_FN
    #define MPI_WIN_DUP_FN OMPI_C_MPI_WIN_DUP_FN

/* MPI_CONVERSION_FN_NULL is a sentinel value, but it has to be large
   enough to be the same size as a valid function pointer.  It
   therefore shares many characteristics between Fortran constants and
   Fortran sentinel functions.  For example, it shares the problem of
   having Fortran compilers have all-caps versions of the symbols that
   must be able to be present, and therefore has to be in this
   conditional block in mpi.h. */
    #define MPI_CONVERSION_FN_NULL ((MPI_Datarep_conversion_function*) 0)
  #endif

/*
 * MPI predefined handles
 */
  #define MPI_COMM_WORLD OMPI_PREDEFINED_GLOBAL( MPI_Comm, ompi_mpi_comm_world)
  #define MPI_COMM_SELF OMPI_PREDEFINED_GLOBAL(MPI_Comm, ompi_mpi_comm_self)

  #define MPI_GROUP_EMPTY OMPI_PREDEFINED_GLOBAL(MPI_Group, ompi_mpi_group_empty)

  #define MPI_MAX OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_max)
  #define MPI_MIN OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_min)
  #define MPI_SUM OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_sum)
  #define MPI_PROD OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_prod)
  #define MPI_LAND OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_land)
  #define MPI_BAND OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_band)
  #define MPI_LOR OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_lor)
  #define MPI_BOR OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_bor)
  #define MPI_LXOR OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_lxor)
  #define MPI_BXOR OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_bxor)
  #define MPI_MAXLOC OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_maxloc)
  #define MPI_MINLOC OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_minloc)
  #define MPI_REPLACE OMPI_PREDEFINED_GLOBAL(MPI_Op, ompi_mpi_op_replace)

/* C datatypes */
  #define MPI_DATATYPE_NULL OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_datatype_null)
  #define MPI_BYTE OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_byte)
  #define MPI_PACKED OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_packed)
  #define MPI_CHAR OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_char)
  #define MPI_SHORT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_short)
  #define MPI_INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_int)
  #define MPI_LONG OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_long)
  #define MPI_FLOAT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_float)
  #define MPI_DOUBLE OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_double)
  #define MPI_LONG_DOUBLE OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_long_double)
  #define MPI_UNSIGNED_CHAR OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_unsigned_char)
  #define MPI_SIGNED_CHAR OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_signed_char)
  #define MPI_UNSIGNED_SHORT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_unsigned_short)
  #define MPI_UNSIGNED_LONG OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_unsigned_long)
  #define MPI_UNSIGNED OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_unsigned)
  #define MPI_FLOAT_INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_float_int)
  #define MPI_DOUBLE_INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_double_int)
  #define MPI_LONG_DOUBLE_INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_longdbl_int)
  #define MPI_LONG_INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_long_int)
  #define MPI_SHORT_INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_short_int)
  #define MPI_2INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_2int)
  #define MPI_UB OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_ub)
  #define MPI_LB OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_lb)
  #define MPI_WCHAR OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_wchar)
  #if OMPI_HAVE_LONG_LONG
    #define MPI_LONG_LONG_INT OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_long_long_int)
    #define MPI_LONG_LONG OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_long_long_int)
    #define MPI_UNSIGNED_LONG_LONG OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_unsigned_long_long)
  #endif  /* OMPI_HAVE_LONG_LONG */
  #define MPI_2COMPLEX OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_2cplex)
  #define MPI_2DOUBLE_COMPLEX OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_2dblcplex)

/* Fortran datatype bindings */
  #define MPI_CHARACTER OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_character)
  #define MPI_LOGICAL OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_logic)
  #if OMPI_HAVE_FORTRAN_LOGICAL1
    #define MPI_LOGICAL1 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_logical1)
  #endif
  #if OMPI_HAVE_FORTRAN_LOGICAL2
    #define MPI_LOGICAL2 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_logical2)
  #endif
  #if OMPI_HAVE_FORTRAN_LOGICAL4
    #define MPI_LOGICAL4 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_logical4)
  #endif
  #if OMPI_HAVE_FORTRAN_LOGICAL8
    #define MPI_LOGICAL8 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_logical8)
  #endif
  #define MPI_INTEGER OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_integer)
  #if OMPI_HAVE_FORTRAN_INTEGER1
    #define MPI_INTEGER1 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_integer1)
  #endif
  #if OMPI_HAVE_FORTRAN_INTEGER2
    #define MPI_INTEGER2 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_integer2)
  #endif
  #if OMPI_HAVE_FORTRAN_INTEGER4
    #define MPI_INTEGER4 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_integer4)
  #endif
  #if OMPI_HAVE_FORTRAN_INTEGER8
    #define MPI_INTEGER8 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_integer8)
  #endif
  #if OMPI_HAVE_FORTRAN_INTEGER16
    #define MPI_INTEGER16 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_integer16)
  #endif
  #define MPI_REAL OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_real)
  #if OMPI_HAVE_FORTRAN_REAL4
    #define MPI_REAL4 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_real4)
  #endif
  #if OMPI_HAVE_FORTRAN_REAL8
    #define MPI_REAL8 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_real8)
  #endif
  #if OMPI_HAVE_FORTRAN_REAL16
    #define MPI_REAL16 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_real16)
  #endif
  #define MPI_DOUBLE_PRECISION OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_dblprec)
  #define MPI_COMPLEX OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_cplex)
  #if OMPI_HAVE_FORTRAN_REAL4
    #define MPI_COMPLEX8 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_complex8)
  #endif
  #if OMPI_HAVE_FORTRAN_REAL8
    #define MPI_COMPLEX16 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_complex16)
  #endif
  #if OMPI_HAVE_FORTRAN_REAL16
    #define MPI_COMPLEX32 OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_complex32)
  #endif
  #define MPI_DOUBLE_COMPLEX OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_dblcplex)
  #define MPI_2REAL OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_2real)
  #define MPI_2DOUBLE_PRECISION OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_2dblprec)
  #define MPI_2INTEGER OMPI_PREDEFINED_GLOBAL(MPI_Datatype, ompi_mpi_2integer)

  #define MPI_ERRORS_ARE_FATAL OMPI_PREDEFINED_GLOBAL(MPI_Errhandler, ompi_mpi_errors_are_fatal)
  #define MPI_ERRORS_RETURN OMPI_PREDEFINED_GLOBAL(MPI_Errhandler, ompi_mpi_errors_return)

/* Typeclass definition for MPI_Type_match_size */
  #define MPI_TYPECLASS_INTEGER    1
  #define MPI_TYPECLASS_REAL       2
  #define MPI_TYPECLASS_COMPLEX    3


/*
 * MPI API
 */

  OMPI_DECLSPEC  int MPI_Abort(MPI_Comm comm, int errorcode);
  OMPI_DECLSPEC  int MPI_Accumulate(void *origin_addr, int origin_count, MPI_Datatype origin_datatype,
                                  int target_rank, MPI_Aint target_disp, int target_count,
                                  MPI_Datatype target_datatype, MPI_Op op, MPI_Win win); 
  OMPI_DECLSPEC  int MPI_Add_error_class(int *errorclass);
  OMPI_DECLSPEC  int MPI_Add_error_code(int errorclass, int *errorcode);
  OMPI_DECLSPEC  int MPI_Add_error_string(int errorcode, char *string);
  OMPI_DECLSPEC  int MPI_Address(void *location, MPI_Aint *address);
  OMPI_DECLSPEC  int MPI_Allgather(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
                                 void *recvbuf, int recvcount, 
                                 MPI_Datatype recvtype, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Allgatherv(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
                                  void *recvbuf, int *recvcounts, 
                                  int *displs, MPI_Datatype recvtype, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Alloc_mem(MPI_Aint size, MPI_Info info, 
                                 void *baseptr);
  OMPI_DECLSPEC  int MPI_Allreduce(void *sendbuf, void *recvbuf, int count, 
                                 MPI_Datatype datatype, MPI_Op op, MPI_Comm comm); 
  OMPI_DECLSPEC  int MPI_Alltoall(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
                                void *recvbuf, int recvcount, 
                                MPI_Datatype recvtype, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Alltoallv(void *sendbuf, int *sendcounts, int *sdispls, 
                                 MPI_Datatype sendtype, void *recvbuf, int *recvcounts,
                                 int *rdispls, MPI_Datatype recvtype, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Alltoallw(void *sendbuf, int *sendcounts, int *sdispls, MPI_Datatype *sendtypes, 
                                 void *recvbuf, int *recvcounts, int *rdispls, MPI_Datatype *recvtypes,
                                 MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Attr_delete(MPI_Comm comm, int keyval);
  OMPI_DECLSPEC  int MPI_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag);
  OMPI_DECLSPEC  int MPI_Attr_put(MPI_Comm comm, int keyval, void *attribute_val);
  OMPI_DECLSPEC  int MPI_Barrier(MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Bcast(void *buffer, int count, MPI_Datatype datatype, 
                             int root, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Bsend(void *buf, int count, MPI_Datatype datatype, 
                             int dest, int tag, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Bsend_init(void *buf, int count, MPI_Datatype datatype, 
                                  int dest, int tag, MPI_Comm comm, MPI_Request *request); 
  OMPI_DECLSPEC  int MPI_Buffer_attach(void *buffer, int size);
  OMPI_DECLSPEC  int MPI_Buffer_detach(void *buffer, int *size);
  OMPI_DECLSPEC  int MPI_Cancel(MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Cart_coords(MPI_Comm comm, int rank, int maxdims, int *coords);
  OMPI_DECLSPEC  int MPI_Cart_create(MPI_Comm old_comm, int ndims, int *dims, 
                                   int *periods, int reorder, MPI_Comm *comm_cart);
  OMPI_DECLSPEC  int MPI_Cart_get(MPI_Comm comm, int maxdims, int *dims, 
                                int *periods, int *coords);
  OMPI_DECLSPEC  int MPI_Cart_map(MPI_Comm comm, int ndims, int *dims, 
                                int *periods, int *newrank);
  OMPI_DECLSPEC  int MPI_Cart_rank(MPI_Comm comm, int *coords, int *rank);
  OMPI_DECLSPEC  int MPI_Cart_shift(MPI_Comm comm, int direction, int disp, 
                                  int *rank_source, int *rank_dest);
  OMPI_DECLSPEC  int MPI_Cart_sub(MPI_Comm comm, int *remain_dims, MPI_Comm *new_comm);
  OMPI_DECLSPEC  int MPI_Cartdim_get(MPI_Comm comm, int *ndims);
  OMPI_DECLSPEC  int MPI_Close_port(char *port_name);
  OMPI_DECLSPEC  int MPI_Comm_accept(char *port_name, MPI_Info info, int root, 
                                   MPI_Comm comm, MPI_Comm *newcomm);
  OMPI_DECLSPEC  MPI_Fint MPI_Comm_c2f(MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Comm_call_errhandler(MPI_Comm comm, int errorcode);
  OMPI_DECLSPEC  int MPI_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result);
  OMPI_DECLSPEC  int MPI_Comm_connect(char *port_name, MPI_Info info, int root, 
                                    MPI_Comm comm, MPI_Comm *newcomm);
  OMPI_DECLSPEC  int MPI_Comm_create_errhandler(MPI_Comm_errhandler_fn *function, 
                                              MPI_Errhandler *errhandler);
  OMPI_DECLSPEC  int MPI_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, 
                                          MPI_Comm_delete_attr_function *comm_delete_attr_fn, 
                                          int *comm_keyval, void *extra_state);
  OMPI_DECLSPEC  int MPI_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);
  OMPI_DECLSPEC  int MPI_Comm_delete_attr(MPI_Comm comm, int comm_keyval);
  OMPI_DECLSPEC  int MPI_Comm_disconnect(MPI_Comm *comm);
  OMPI_DECLSPEC  int MPI_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm);
  OMPI_DECLSPEC  MPI_Comm MPI_Comm_f2c(MPI_Fint comm);
  OMPI_DECLSPEC  int MPI_Comm_free_keyval(int *comm_keyval);
  OMPI_DECLSPEC  int MPI_Comm_free(MPI_Comm *comm);
  OMPI_DECLSPEC  int MPI_Comm_get_attr(MPI_Comm comm, int comm_keyval, 
                                     void *attribute_val, int *flag);
  OMPI_DECLSPEC  int MPI_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler);
  OMPI_DECLSPEC  int MPI_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen);
  OMPI_DECLSPEC  int MPI_Comm_get_parent(MPI_Comm *parent);
  OMPI_DECLSPEC  int MPI_Comm_group(MPI_Comm comm, MPI_Group *group);
  OMPI_DECLSPEC  int MPI_Comm_join(int fd, MPI_Comm *intercomm);
  OMPI_DECLSPEC  int MPI_Comm_rank(MPI_Comm comm, int *rank);
  OMPI_DECLSPEC  int MPI_Comm_remote_group(MPI_Comm comm, MPI_Group *group);
  OMPI_DECLSPEC  int MPI_Comm_remote_size(MPI_Comm comm, int *size);
  OMPI_DECLSPEC  int MPI_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val);
  OMPI_DECLSPEC  int MPI_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler);
  OMPI_DECLSPEC  int MPI_Comm_set_name(MPI_Comm comm, char *comm_name);
  OMPI_DECLSPEC  int MPI_Comm_size(MPI_Comm comm, int *size);
  OMPI_DECLSPEC  int MPI_Comm_spawn(char *command, char **argv, int maxprocs, MPI_Info info, 
                                  int root, MPI_Comm comm, MPI_Comm *intercomm, 
                                  int *array_of_errcodes);
  OMPI_DECLSPEC  int MPI_Comm_spawn_multiple(int count, char **array_of_commands, char ***array_of_argv, 
                                           int *array_of_maxprocs, MPI_Info *array_of_info, 
                                           int root, MPI_Comm comm, MPI_Comm *intercomm, 
                                           int *array_of_errcodes);
  OMPI_DECLSPEC  int MPI_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm);
  OMPI_DECLSPEC  int MPI_Comm_test_inter(MPI_Comm comm, int *flag);
  OMPI_DECLSPEC  int MPI_Dims_create(int nnodes, int ndims, int *dims);
  OMPI_DECLSPEC  MPI_Fint MPI_Errhandler_c2f(MPI_Errhandler errhandler);
  OMPI_DECLSPEC  int MPI_Errhandler_create(MPI_Handler_function *function, 
                                         MPI_Errhandler *errhandler);
  OMPI_DECLSPEC  MPI_Errhandler MPI_Errhandler_f2c(MPI_Fint errhandler);
  OMPI_DECLSPEC  int MPI_Errhandler_free(MPI_Errhandler *errhandler);
  OMPI_DECLSPEC  int MPI_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler);
  OMPI_DECLSPEC  int MPI_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler);
  OMPI_DECLSPEC  int MPI_Error_class(int errorcode, int *errorclass);
  OMPI_DECLSPEC  int MPI_Error_string(int errorcode, char *string, int *resultlen);
  OMPI_DECLSPEC  int MPI_Exscan(void *sendbuf, void *recvbuf, int count, 
                              MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
  #if OMPI_PROVIDE_MPI_FILE_INTERFACE
    OMPI_DECLSPEC  MPI_Fint MPI_File_c2f(MPI_File file);
    OMPI_DECLSPEC  MPI_File MPI_File_f2c(MPI_Fint file);
    OMPI_DECLSPEC  int MPI_File_call_errhandler(MPI_File fh, int errorcode);
    OMPI_DECLSPEC  int MPI_File_create_errhandler(MPI_File_errhandler_fn *function,
                                              MPI_Errhandler *errhandler);
    OMPI_DECLSPEC  int MPI_File_set_errhandler( MPI_File file, MPI_Errhandler errhandler);
    OMPI_DECLSPEC  int MPI_File_get_errhandler( MPI_File file, MPI_Errhandler *errhandler);
    OMPI_DECLSPEC  int MPI_File_open(MPI_Comm comm, char *filename, int amode,
                                 MPI_Info info, MPI_File *fh);
    OMPI_DECLSPEC  int MPI_File_close(MPI_File *fh);
    OMPI_DECLSPEC  int MPI_File_delete(char *filename, MPI_Info info);
    OMPI_DECLSPEC  int MPI_File_set_size(MPI_File fh, MPI_Offset size);
    OMPI_DECLSPEC  int MPI_File_preallocate(MPI_File fh, MPI_Offset size);
    OMPI_DECLSPEC  int MPI_File_get_size(MPI_File fh, MPI_Offset *size);
    OMPI_DECLSPEC  int MPI_File_get_group(MPI_File fh, MPI_Group *group);
    OMPI_DECLSPEC  int MPI_File_get_amode(MPI_File fh, int *amode);
    OMPI_DECLSPEC  int MPI_File_set_info(MPI_File fh, MPI_Info info);
    OMPI_DECLSPEC  int MPI_File_get_info(MPI_File fh, MPI_Info *info_used);
    OMPI_DECLSPEC  int MPI_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype,
                                     MPI_Datatype filetype, char *datarep, MPI_Info info);
    OMPI_DECLSPEC  int MPI_File_get_view(MPI_File fh, MPI_Offset *disp,
                                     MPI_Datatype *etype, 
                                     MPI_Datatype *filetype, char *datarep);
    OMPI_DECLSPEC  int MPI_File_read_at(MPI_File fh, MPI_Offset offset, void *buf,
                                    int count, MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf,
                                        int count, MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_at(MPI_File fh, MPI_Offset offset, void *buf,
                                     int count, MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_at_all(MPI_File fh, MPI_Offset offset, void *buf,
                                         int count, MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf,
                                     int count, MPI_Datatype datatype, MPI_Request *request);
    OMPI_DECLSPEC  int MPI_File_iwrite_at(MPI_File fh, MPI_Offset offset, void *buf,
                                      int count, MPI_Datatype datatype, MPI_Request *request);
    OMPI_DECLSPEC  int MPI_File_read(MPI_File fh, void *buf, int count,
                                 MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_read_all(MPI_File fh, void *buf, int count,
                                     MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write(MPI_File fh, void *buf, int count,
                                  MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_all(MPI_File fh, void *buf, int count,
                                      MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_iread(MPI_File fh, void *buf, int count,
                                  MPI_Datatype datatype, MPI_Request *request);
    OMPI_DECLSPEC  int MPI_File_iwrite(MPI_File fh, void *buf, int count,
                                   MPI_Datatype datatype, MPI_Request *request);
    OMPI_DECLSPEC  int MPI_File_seek(MPI_File fh, MPI_Offset offset, int whence);
    OMPI_DECLSPEC  int MPI_File_get_position(MPI_File fh, MPI_Offset *offset);
    OMPI_DECLSPEC  int MPI_File_get_byte_offset(MPI_File fh, MPI_Offset offset,
                                            MPI_Offset *disp);
    OMPI_DECLSPEC  int MPI_File_read_shared(MPI_File fh, void *buf, int count,
                                        MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_shared(MPI_File fh, void *buf, int count,
					 MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_iread_shared(MPI_File fh, void *buf, int count,
                                         MPI_Datatype datatype, MPI_Request *request);
    OMPI_DECLSPEC  int MPI_File_iwrite_shared(MPI_File fh, void *buf, int count,
                                          MPI_Datatype datatype, MPI_Request *request);
    OMPI_DECLSPEC  int MPI_File_read_ordered(MPI_File fh, void *buf, int count,
                                         MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_ordered(MPI_File fh, void *buf, int count,
                                          MPI_Datatype datatype, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence);
    OMPI_DECLSPEC  int MPI_File_get_position_shared(MPI_File fh, MPI_Offset *offset);
    OMPI_DECLSPEC  int MPI_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf,
                                              int count, MPI_Datatype datatype);
    OMPI_DECLSPEC  int MPI_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf,
                                               int count, MPI_Datatype datatype);
    OMPI_DECLSPEC  int MPI_File_write_at_all_end(MPI_File fh, void *buf, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_read_all_begin(MPI_File fh, void *buf, int count,
                                           MPI_Datatype datatype);
    OMPI_DECLSPEC  int MPI_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_all_begin(MPI_File fh, void *buf, int count,
                                            MPI_Datatype datatype);
    OMPI_DECLSPEC  int MPI_File_write_all_end(MPI_File fh, void *buf, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_read_ordered_begin(MPI_File fh, void *buf, int count,
                                               MPI_Datatype datatype);
    OMPI_DECLSPEC  int MPI_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_write_ordered_begin(MPI_File fh, void *buf, int count,
                                                MPI_Datatype datatype);
    OMPI_DECLSPEC  int MPI_File_write_ordered_end(MPI_File fh, void *buf, MPI_Status *status);
    OMPI_DECLSPEC  int MPI_File_get_type_extent(MPI_File fh, MPI_Datatype datatype,
                                            MPI_Aint *extent);
    OMPI_DECLSPEC  int MPI_File_set_atomicity(MPI_File fh, int flag);
    OMPI_DECLSPEC  int MPI_File_get_atomicity(MPI_File fh, int *flag);
    OMPI_DECLSPEC  int MPI_File_sync(MPI_File fh);
  #endif /* #if OMPI_PROVIDE_MPI_FILE_INTERFACE */
  OMPI_DECLSPEC  int MPI_Finalize(void);
  OMPI_DECLSPEC  int MPI_Finalized(int *flag);
  OMPI_DECLSPEC  int MPI_Free_mem(void *base);
  OMPI_DECLSPEC  int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
                              void *recvbuf, int recvcount, MPI_Datatype recvtype, 
                              int root, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Gatherv(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
                               void *recvbuf, int *recvcounts, int *displs, 
                               MPI_Datatype recvtype, int root, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Get_address(void *location, MPI_Aint *address);
  OMPI_DECLSPEC  int MPI_Get_count(MPI_Status *status, MPI_Datatype datatype, int *count);
  OMPI_DECLSPEC  int MPI_Get_elements(MPI_Status *status, MPI_Datatype datatype, int *count);
  OMPI_DECLSPEC  int MPI_Get(void *origin_addr, int origin_count, 
                           MPI_Datatype origin_datatype, int target_rank, 
                           MPI_Aint target_disp, int target_count, 
                           MPI_Datatype target_datatype, MPI_Win win);
  OMPI_DECLSPEC  int MPI_Get_processor_name(char *name, int *resultlen);
  OMPI_DECLSPEC  int MPI_Get_version(int *version, int *subversion);
  OMPI_DECLSPEC  int MPI_Graph_create(MPI_Comm comm_old, int nnodes, int *index, 
                                    int *edges, int reorder, MPI_Comm *comm_graph);
  OMPI_DECLSPEC  int MPI_Graph_get(MPI_Comm comm, int maxindex, int maxedges, 
                                 int *index, int *edges);
  OMPI_DECLSPEC  int MPI_Graph_map(MPI_Comm comm, int nnodes, int *index, int *edges, 
                                 int *newrank);
  OMPI_DECLSPEC  int MPI_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors);
  OMPI_DECLSPEC  int MPI_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, 
                                       int *neighbors);
  OMPI_DECLSPEC  int MPI_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges);
  OMPI_DECLSPEC  int MPI_Grequest_complete(MPI_Request request);
  OMPI_DECLSPEC  int MPI_Grequest_start(MPI_Grequest_query_function *query_fn,
                                      MPI_Grequest_free_function *free_fn,
                                      MPI_Grequest_cancel_function *cancel_fn,
                                      void *extra_state, MPI_Request *request);
  OMPI_DECLSPEC  MPI_Fint MPI_Group_c2f(MPI_Group group);
  OMPI_DECLSPEC  int MPI_Group_compare(MPI_Group group1, MPI_Group group2, int *result);
  OMPI_DECLSPEC  int MPI_Group_difference(MPI_Group group1, MPI_Group group2, 
                                        MPI_Group *newgroup);
  OMPI_DECLSPEC  int MPI_Group_excl(MPI_Group group, int n, int *ranks, 
                                  MPI_Group *newgroup);
  OMPI_DECLSPEC  MPI_Group MPI_Group_f2c(MPI_Fint group);
  OMPI_DECLSPEC  int MPI_Group_free(MPI_Group *group);
  OMPI_DECLSPEC  int MPI_Group_incl(MPI_Group group, int n, int *ranks, 
                                  MPI_Group *newgroup);
  OMPI_DECLSPEC  int MPI_Group_intersection(MPI_Group group1, MPI_Group group2, 
                                          MPI_Group *newgroup);
  OMPI_DECLSPEC  int MPI_Group_range_excl(MPI_Group group, int n, int ranges[][3], 
                                        MPI_Group *newgroup);
  OMPI_DECLSPEC  int MPI_Group_range_incl(MPI_Group group, int n, int ranges[][3], 
                                        MPI_Group *newgroup);
  OMPI_DECLSPEC  int MPI_Group_rank(MPI_Group group, int *rank);
  OMPI_DECLSPEC  int MPI_Group_size(MPI_Group group, int *size);
  OMPI_DECLSPEC  int MPI_Group_translate_ranks(MPI_Group group1, int n, int *ranks1, 
                                             MPI_Group group2, int *ranks2);
  OMPI_DECLSPEC  int MPI_Group_union(MPI_Group group1, MPI_Group group2, 
                                   MPI_Group *newgroup);
  OMPI_DECLSPEC  int MPI_Ibsend(void *buf, int count, MPI_Datatype datatype, int dest, 
                              int tag, MPI_Comm comm, MPI_Request *request);
  OMPI_DECLSPEC  MPI_Fint MPI_Info_c2f(MPI_Info info);
  OMPI_DECLSPEC  int MPI_Info_create(MPI_Info *info);
  OMPI_DECLSPEC  int MPI_Info_delete(MPI_Info info, char *key);
  OMPI_DECLSPEC  int MPI_Info_dup(MPI_Info info, MPI_Info *newinfo);
  OMPI_DECLSPEC  MPI_Info MPI_Info_f2c(MPI_Fint info);
  OMPI_DECLSPEC  int MPI_Info_free(MPI_Info *info);
  OMPI_DECLSPEC  int MPI_Info_get(MPI_Info info, char *key, int valuelen, 
                                char *value, int *flag);
  OMPI_DECLSPEC  int MPI_Info_get_nkeys(MPI_Info info, int *nkeys);
  OMPI_DECLSPEC  int MPI_Info_get_nthkey(MPI_Info info, int n, char *key);
  OMPI_DECLSPEC  int MPI_Info_get_valuelen(MPI_Info info, char *key, int *valuelen, 
                                         int *flag);
  OMPI_DECLSPEC  int MPI_Info_set(MPI_Info info, char *key, char *value);
  OMPI_DECLSPEC  int MPI_Init(int *argc, char ***argv);
  OMPI_DECLSPEC  int MPI_Initialized(int *flag);
  OMPI_DECLSPEC  int MPI_Init_thread(int *argc, char ***argv, int required, 
                                   int *provided);
  OMPI_DECLSPEC  int MPI_Intercomm_create(MPI_Comm local_comm, int local_leader, 
                                        MPI_Comm bridge_comm, int remote_leader, 
                                        int tag, MPI_Comm *newintercomm);
  OMPI_DECLSPEC  int MPI_Intercomm_merge(MPI_Comm intercomm, int high, 
                                       MPI_Comm *newintercomm);
  OMPI_DECLSPEC  int MPI_Iprobe(int source, int tag, MPI_Comm comm, int *flag, 
                              MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Irecv(void *buf, int count, MPI_Datatype datatype, int source, 
                             int tag, MPI_Comm comm, MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Irsend(void *buf, int count, MPI_Datatype datatype, int dest, 
                              int tag, MPI_Comm comm, MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Isend(void *buf, int count, MPI_Datatype datatype, int dest, 
                             int tag, MPI_Comm comm, MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Issend(void *buf, int count, MPI_Datatype datatype, int dest, 
                              int tag, MPI_Comm comm, MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Is_thread_main(int *flag);
  OMPI_DECLSPEC  int MPI_Keyval_create(MPI_Copy_function *copy_fn, 
                                     MPI_Delete_function *delete_fn, 
                                     int *keyval, void *extra_state);
  OMPI_DECLSPEC  int MPI_Keyval_free(int *keyval);
  OMPI_DECLSPEC  int MPI_Lookup_name(char *service_name, MPI_Info info, char *port_name);
  OMPI_DECLSPEC  MPI_Fint MPI_Op_c2f(MPI_Op op); 
  OMPI_DECLSPEC  int MPI_Op_create(MPI_User_function *function, int commute, MPI_Op *op);
  OMPI_DECLSPEC  int MPI_Open_port(MPI_Info info, char *port_name);
  OMPI_DECLSPEC  MPI_Op MPI_Op_f2c(MPI_Fint op);
  OMPI_DECLSPEC  int MPI_Op_free(MPI_Op *op);
  OMPI_DECLSPEC  int MPI_Pack_external(char *datarep, void *inbuf, int incount,
                                     MPI_Datatype datatype, void *outbuf,
                                     MPI_Aint outsize, MPI_Aint *position);
  OMPI_DECLSPEC  int MPI_Pack_external_size(char *datarep, int incount, 
                                          MPI_Datatype datatype, MPI_Aint *size);
  OMPI_DECLSPEC  int MPI_Pack(void *inbuf, int incount, MPI_Datatype datatype, 
                            void *outbuf, int outsize, int *position, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, 
                                 int *size);
  OMPI_DECLSPEC  int MPI_Pcontrol(const int level, ...);
  OMPI_DECLSPEC  int MPI_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Publish_name(char *service_name, MPI_Info info, 
                                    char *port_name);
  OMPI_DECLSPEC  int MPI_Put(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
                           int target_rank, MPI_Aint target_disp, int target_count, 
                           MPI_Datatype target_datatype, MPI_Win win);
  OMPI_DECLSPEC  int MPI_Query_thread(int *provided);
  OMPI_DECLSPEC  int MPI_Recv_init(void *buf, int count, MPI_Datatype datatype, int source,
                                 int tag, MPI_Comm comm, MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Recv(void *buf, int count, MPI_Datatype datatype, int source, 
                            int tag, MPI_Comm comm, MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Reduce(void *sendbuf, void *recvbuf, int count, 
                              MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Reduce_scatter(void *sendbuf, void *recvbuf, int *recvcounts, 
                                      MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Register_datarep(char *datarep, 
                                        MPI_Datarep_conversion_function *read_conversion_fn,
                                        MPI_Datarep_conversion_function *write_conversion_fn,
                                        MPI_Datarep_extent_function *dtype_file_extent_fn,
                                        void *extra_state);
  OMPI_DECLSPEC  MPI_Fint MPI_Request_c2f(MPI_Request request);
  OMPI_DECLSPEC  MPI_Request MPI_Request_f2c(MPI_Fint request);
  OMPI_DECLSPEC  int MPI_Request_free(MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Request_get_status(MPI_Request request, int *flag, 
                                          MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Rsend(void *ibuf, int count, MPI_Datatype datatype, int dest, 
                             int tag, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Rsend_init(void *buf, int count, MPI_Datatype datatype, 
                                  int dest, int tag, MPI_Comm comm, 
                                  MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Scan(void *sendbuf, void *recvbuf, int count, 
                            MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Scatter(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
                               void *recvbuf, int recvcount, MPI_Datatype recvtype, 
                               int root, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Scatterv(void *sendbuf, int *sendcounts, int *displs, 
                                MPI_Datatype sendtype, void *recvbuf, int recvcount, 
                                MPI_Datatype recvtype, int root, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Send_init(void *buf, int count, MPI_Datatype datatype, 
                                 int dest, int tag, MPI_Comm comm, 
                                 MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Send(void *buf, int count, MPI_Datatype datatype, int dest, 
                            int tag, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Sendrecv(void *sendbuf, int sendcount, MPI_Datatype sendtype, 
                                int dest, int sendtag, void *recvbuf, int recvcount,
                                MPI_Datatype recvtype, int source, int recvtag, 
                                MPI_Comm comm,  MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Sendrecv_replace(void * buf, int count, MPI_Datatype datatype, 
                                        int dest, int sendtag, int source, int recvtag,
                                        MPI_Comm comm, MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Ssend_init(void *buf, int count, MPI_Datatype datatype, 
                                  int dest, int tag, MPI_Comm comm, 
                                  MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Ssend(void *buf, int count, MPI_Datatype datatype, int dest, 
                             int tag, MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Start(MPI_Request *request);
  OMPI_DECLSPEC  int MPI_Startall(int count, MPI_Request *array_of_requests);
  OMPI_DECLSPEC  int MPI_Status_c2f(MPI_Status *c_status, MPI_Fint *f_status);
  OMPI_DECLSPEC  int MPI_Status_f2c(MPI_Fint *f_status, MPI_Status *c_status);
  OMPI_DECLSPEC  int MPI_Status_set_cancelled(MPI_Status *status, int flag);
  OMPI_DECLSPEC  int MPI_Status_set_elements(MPI_Status *status, MPI_Datatype datatype,
                                           int count);
  OMPI_DECLSPEC  int MPI_Testall(int count, MPI_Request array_of_requests[], int *flag, 
                               MPI_Status array_of_statuses[]);
  OMPI_DECLSPEC  int MPI_Testany(int count, MPI_Request array_of_requests[], int *index, 
                               int *flag, MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Test(MPI_Request *request, int *flag, MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Test_cancelled(MPI_Status *status, int *flag);
  OMPI_DECLSPEC  int MPI_Testsome(int incount, MPI_Request array_of_requests[], 
                                int *outcount, int array_of_indices[], 
                                MPI_Status array_of_statuses[]);
  OMPI_DECLSPEC  int MPI_Topo_test(MPI_Comm comm, int *status);
  OMPI_DECLSPEC  MPI_Fint MPI_Type_c2f(MPI_Datatype datatype);
  OMPI_DECLSPEC  int MPI_Type_commit(MPI_Datatype *type);
  OMPI_DECLSPEC  int MPI_Type_contiguous(int count, MPI_Datatype oldtype, 
                                       MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_darray(int size, int rank, int ndims, 
                                          int gsize_array[], int distrib_array[], 
                                          int darg_array[], int psize_array[],
                                          int order, MPI_Datatype oldtype, 
                                          MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_f90_integer(int r, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_f90_real(int p, int r, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_hindexed(int count, int array_of_blocklengths[], 
                                            MPI_Aint array_of_displacements[], 
                                            MPI_Datatype oldtype, 
                                            MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_hvector(int count, int blocklength, MPI_Aint stride, 
                                           MPI_Datatype oldtype, 
                                           MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, 
                                          MPI_Type_delete_attr_function *type_delete_attr_fn, 
                                          int *type_keyval, void *extra_state);
  OMPI_DECLSPEC  int MPI_Type_create_indexed_block(int count, int blocklength,
                                                 int array_of_displacements[],
                                                 MPI_Datatype oldtype,
                                                 MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_struct(int count, int array_of_block_lengths[], 
                                          MPI_Aint array_of_displacements[], 
                                          MPI_Datatype array_of_types[], 
                                          MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_subarray(int ndims, int size_array[], int subsize_array[], 
                                            int start_array[], int order, 
                                            MPI_Datatype oldtype, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, 
                                           MPI_Aint extent, MPI_Datatype *newtype); 
  OMPI_DECLSPEC  int MPI_Type_delete_attr(MPI_Datatype type, int type_keyval);
  OMPI_DECLSPEC  int MPI_Type_dup(MPI_Datatype type, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_extent(MPI_Datatype type, MPI_Aint *extent);
  OMPI_DECLSPEC  int MPI_Type_free(MPI_Datatype *type);
  OMPI_DECLSPEC  int MPI_Type_free_keyval(int *type_keyval);
  OMPI_DECLSPEC  MPI_Datatype MPI_Type_f2c(MPI_Fint datatype);
  OMPI_DECLSPEC  int MPI_Type_get_attr(MPI_Datatype type, int type_keyval, 
                                     void *attribute_val, int *flag);
  OMPI_DECLSPEC  int MPI_Type_get_contents(MPI_Datatype mtype, int max_integers, 
                                         int max_addresses, int max_datatypes, 
                                         int array_of_integers[], 
                                         MPI_Aint array_of_addresses[], 
                                         MPI_Datatype array_of_datatypes[]);
  OMPI_DECLSPEC  int MPI_Type_get_envelope(MPI_Datatype type, int *num_integers, 
                                         int *num_addresses, int *num_datatypes, 
                                         int *combiner);
  OMPI_DECLSPEC  int MPI_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, 
                                       MPI_Aint *extent);
  OMPI_DECLSPEC  int MPI_Type_get_name(MPI_Datatype type, char *type_name, 
                                     int *resultlen);
  OMPI_DECLSPEC  int MPI_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, 
                                            MPI_Aint *true_extent);
  OMPI_DECLSPEC  int MPI_Type_hindexed(int count, int array_of_blocklengths[], 
                                     MPI_Aint array_of_displacements[], 
                                     MPI_Datatype oldtype, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_hvector(int count, int blocklength, MPI_Aint stride, 
                                    MPI_Datatype oldtype, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_indexed(int count, int array_of_blocklengths[], 
                                    int array_of_displacements[], 
                                    MPI_Datatype oldtype, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_lb(MPI_Datatype type, MPI_Aint *lb);
  OMPI_DECLSPEC  int MPI_Type_match_size(int typeclass, int size, MPI_Datatype *type);
  OMPI_DECLSPEC  int MPI_Type_set_attr(MPI_Datatype type, int type_keyval, 
                                     void *attr_val);
  OMPI_DECLSPEC  int MPI_Type_set_name(MPI_Datatype type, char *type_name);
  OMPI_DECLSPEC  int MPI_Type_size(MPI_Datatype type, int *size);
  OMPI_DECLSPEC  int MPI_Type_struct(int count, int array_of_blocklengths[], 
                                   MPI_Aint array_of_displacements[], 
                                   MPI_Datatype array_of_types[], 
                                   MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Type_ub(MPI_Datatype mtype, MPI_Aint *ub);
  OMPI_DECLSPEC  int MPI_Type_vector(int count, int blocklength, int stride, 
                                   MPI_Datatype oldtype, MPI_Datatype *newtype);
  OMPI_DECLSPEC  int MPI_Unpack(void *inbuf, int insize, int *position, 
                              void *outbuf, int outcount, MPI_Datatype datatype, 
                              MPI_Comm comm);
  OMPI_DECLSPEC  int MPI_Unpublish_name(char *service_name, MPI_Info info, char *port_name);
  OMPI_DECLSPEC  int MPI_Unpack_external (char *datarep, void *inbuf, MPI_Aint insize,
                                        MPI_Aint *position, void *outbuf, int outcount,
                                        MPI_Datatype datatype);
  OMPI_DECLSPEC  int MPI_Waitall(int count, MPI_Request *array_of_requests, 
                               MPI_Status *array_of_statuses);
  OMPI_DECLSPEC  int MPI_Waitany(int count, MPI_Request *array_of_requests, 
                               int *index, MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Wait(MPI_Request *request, MPI_Status *status);
  OMPI_DECLSPEC  int MPI_Waitsome(int incount, MPI_Request *array_of_requests, 
                                int *outcount, int *array_of_indices, 
                                MPI_Status *array_of_statuses);
  OMPI_DECLSPEC  MPI_Fint MPI_Win_c2f(MPI_Win win);
  OMPI_DECLSPEC  int MPI_Win_call_errhandler(MPI_Win win, int errorcode);
  OMPI_DECLSPEC  int MPI_Win_complete(MPI_Win win);
  OMPI_DECLSPEC  int MPI_Win_create(void *base, MPI_Aint size, int disp_unit, 
                                  MPI_Info info, MPI_Comm comm, MPI_Win *win);
  OMPI_DECLSPEC  int MPI_Win_create_errhandler(MPI_Win_errhandler_fn *function, 
                                             MPI_Errhandler *errhandler);
  OMPI_DECLSPEC  int MPI_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, 
                                         MPI_Win_delete_attr_function *win_delete_attr_fn, 
                                         int *win_keyval, void *extra_state);
  OMPI_DECLSPEC  int MPI_Win_delete_attr(MPI_Win win, int win_keyval);
  OMPI_DECLSPEC  MPI_Win MPI_Win_f2c(MPI_Fint win);
  OMPI_DECLSPEC  int MPI_Win_fence(int assert, MPI_Win win);
  OMPI_DECLSPEC  int MPI_Win_free(MPI_Win *win);
  OMPI_DECLSPEC  int MPI_Win_free_keyval(int *win_keyval);
  OMPI_DECLSPEC  int MPI_Win_get_attr(MPI_Win win, int win_keyval, 
                                    void *attribute_val, int *flag);
  OMPI_DECLSPEC  int MPI_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler);
  OMPI_DECLSPEC  int MPI_Win_get_group(MPI_Win win, MPI_Group *group);
  OMPI_DECLSPEC  int MPI_Win_get_name(MPI_Win win, char *win_name, int *resultlen);
  OMPI_DECLSPEC  int MPI_Win_lock(int lock_type, int rank, int assert, MPI_Win win);
  OMPI_DECLSPEC  int MPI_Win_post(MPI_Group group, int assert, MPI_Win win);
  OMPI_DECLSPEC  int MPI_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val);
  OMPI_DECLSPEC  int MPI_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler);
  OMPI_DECLSPEC  int MPI_Win_set_name(MPI_Win win, char *win_name);
  OMPI_DECLSPEC  int MPI_Win_start(MPI_Group group, int assert, MPI_Win win);
  OMPI_DECLSPEC  int MPI_Win_test(MPI_Win win, int *flag);
  OMPI_DECLSPEC  int MPI_Win_unlock(int rank, MPI_Win win);
  OMPI_DECLSPEC  int MPI_Win_wait(MPI_Win win);
  OMPI_DECLSPEC  double MPI_Wtick(void);
  OMPI_DECLSPEC  double MPI_Wtime(void);


#if defined(c_plusplus) || defined(__cplusplus)
}
#endif

#endif /* _SPL_MPI_OMPI_H_ */
