/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file util_endian.h
 * Implementation of endian convert utility
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file util_endian.h
 * エンディアン変換ユーティリティファイル
 * @author     hira
 * @date       2009/02/20
 */


#ifndef _SPL_UTIL_ENDIAN_H_
#define _SPL_UTIL_ENDIAN_H_

#include <string.h>

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum {UnKnown =0, Match, UnMatch} EMatchType;

//inline void
static inline void
BSwap16(void* a) {
  unsigned short* x = (unsigned short*)a;
  *x = (unsigned short)( ((((*x) & 0xff00) >> 8) | (((*x) & 0x00ff) << 8)) );
}

//inline void
static inline void
SBSwapVec(void* a, size_t l) {
  unsigned short* b = (unsigned short*)a;
  register size_t i;
  for(i=0; i<l; i++) { BSwap16((void*)&b[i]); }
}

//inline void
static inline void
BSwap32(void* a) {
  unsigned int* x = (unsigned int*)a;
  *x = ((((*x) & 0xff000000) >> 24) | (((*x) & 0x00ff0000) >>  8) |
        (((*x) & 0x0000ff00) <<  8) | (((*x) & 0x000000ff) << 24));
}

//inline void
static inline void
BSwapVec(void* a, size_t l) {
  unsigned int* b = (unsigned int*)a;
  register size_t i;
  for(i=0; i<l; i++) { BSwap32((void*)&b[i]); }
}

//inline void
static inline void
BSwap64(void* a) {
  unsigned long long* x = (unsigned long long*)a;
  *x=((((*x)&0xff00000000000000ull)>>56) | (((*x)&0x00ff000000000000ull)>>40) |
      (((*x)&0x0000ff0000000000ull)>>24) | (((*x)&0x000000ff00000000ull)>> 8) |
      (((*x)&0x00000000ff000000ull)<< 8) | (((*x)&0x0000000000ff0000ull)<<24) |
      (((*x)&0x000000000000ff00ull)<<40) | (((*x)&0x00000000000000ffull)<<56));
}

//inline void
static inline void
DBSwapVec(void* a, size_t l) {
  unsigned long long* b = (unsigned long long*)a;
  register size_t i;
  for(i=0; i<l; i++) { BSwap64((void*)&b[i]); }
}

#if defined(__cplusplus)
}
#endif

#endif // _SPL_UTIL_ENDIAN_H_

