/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
 /**
 * \~english
 * @file main.C
 * Sphere main File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file main.C
 * Sphereメインファイル
 * @author     hira
 * @date       2009/10/01
 */

#include "base/sphBase.h"
#include "app/sphApp.h"

// using namespace std;


/**
 * バージョンを表示する。
 * @param       cmd     起動コマンド
 * @return      EXIT_SUCCESS/EXIT_FAILURE
 */
int sphere_version(char* cmd)
{
  if( !cmd ) return EXIT_FAILURE;

  std::cout << "\tSPHERE - Skeleton for PHysical and Engineering REsearch" << std::endl;
  std::cout << "\t=======================================================" << std::endl;
  std::cout << "\t version \t" << SPHERE_VERSION_NO << std::endl;
  std::cout << "\t build id \t" << SPHERE_BUILD_ID << std::endl;
  std::cout << "\t=======================================================" << std::endl;
  std::cout << std::endl;

  return EXIT_SUCCESS;
}


/**
 * コマンドヘルプを表示する。
 * @param       cmd     起動コマンド
 * @return      EXIT_SUCCESS/EXIT_FAILURE
 */
int usage(char* cmd)
{
  if( !cmd ) return EXIT_FAILURE;

  char version[32];
  sprintf(version, "%s", SPHERE_VERSION_NO);

  std::cout << std::endl;
  // バージョン表示
  sphere_version(cmd);

  std::cout << std::endl;
  std::cout << " usage : ";
  std::cout << cmd
       << " [options]"
       << " config_file" << std::endl;
  std::cout << std::endl;
  std::cout << " \tconfig_file is solver configuration file." << std::endl;
  std::cout << std::endl;
  std::cout << " \tOption List" << std::endl;
  std::cout << " \t\t-h [Solver_Keyword]\t print help message." << std::endl;
  std::cout << " \t\t--help [Solver_Keyword]\t print help message." << std::endl;
  std::cout << " \t\t If the Solver_Keyword is specified, displayed help message of" << std::endl;
  std::cout << " \t\t the solver corresponding to the Solver_Keyword." << std::endl;
  std::cout << std::endl;
  std::cout << " \t\t-v, --version\t print SPHERE version No." << std::endl;
  std::cout << std::endl;
  return EXIT_SUCCESS;
}


/**
 * Sphere Main プログラム。
 * @param       argc     起動引数の数
 * @param       argc     起動引数
 * @return      EXIT_SUCCESS/EXIT_FAILURE
 */
int main(int argc, char** argv)
{

  bool usage_flag = false;
  bool version_flag = false;
  bool p4_flag = false;
  char cmd_name[MAXPATHLEN];
  const char* config_fname = NULL;
  register int i;
  int app_result = EXIT_FAILURE;
  sphApp app;                   // sphere application class

//  std::cout << "Sphere Main Program Start!!!" << std::endl;

  for(i=1; i<argc; i++){
    if( !strcmp(argv[i], "--help") || !strcmp(argv[i], "-h") ){
      usage_flag = true;
    }
    else if( !strcmp(argv[i], "--version") || !strcmp(argv[i], "-v") ){
      version_flag = true;
    }
    else if( !strncmp(argv[i], "-p4", 3) ){
      p4_flag = true;
    }
    else {
      if (config_fname == NULL) {
        config_fname = argv[i];
      }
    }
  }

  if (config_fname == NULL) {
    // Sphereのusage, version出力
    if (version_flag) {
      sphere_version(basename(argv[0]));
    }
    else {
      usage(basename(argv[0]));
    }
  }
  else {
    // Sphere 実行
    app.setUsageFlag(usage_flag);
    app.setP4Flag(p4_flag);
    app.setConfigFile(config_fname);
    if (app.run(argc, argv)) {
      app_result = EXIT_SUCCESS;
    }
    else {
      app_result = EXIT_FAILURE;
    }
  }

#ifdef _WIN32
  WSACleanup(void)
#endif

//  std::cout << "Sphere Main Program End!!!" << std::endl;
  return EXIT_SUCCESS;
}
