/*
 * sphSphereObject.C
 *
 *  Created on: 2009/10/29
 *      Author: hira
 */

#include "app/sphSphereObject.h"

sphSphereObject::sphSphereObject()
        : m_sphereId(0), m_sphereLabel(""), m_configElem(NULL)
{
}

/**
 * \~english
 * Constructor
 * @param     label     sphere label
 * @param     id     sphere id
 *
 * \~japanese
 * コンストラクタ
 * @param     label     sphereラベル
 * @param     id        sphere id
 */
sphSphereObject::sphSphereObject(const char* label, int id)
        : m_sphereId(id), m_sphereLabel(label), m_configElem(NULL)
{

}


sphSphereObject::~sphSphereObject()
{
}


void sphSphereObject::setSphere(sphSphere* object)
{
  m_sphere = object;
}


const sphSphere* sphSphereObject::getSphere()
{
  return m_sphere;
}


/**
 * \~english
 * get sphere object id
 * @return    sphere object id
 *
 * \~japanese
 * SphereIDを取得する。
 * @return    sphere object ID
 */
int sphSphereObject::getSphereId() const
{
  return m_sphereId;
}


/**
 * \~english
 * get sphere label
 * @return    sphere label
 *
 * \~japanese
 * sphereラベルを取得する。
 * @return    sphereラベル
 */
std::string sphSphereObject::getSphereLabel() const
{
  return m_sphereLabel;
}


/**
 * \~english
 * get solver manager
 * @param       void
 * @return      solver manager
 *
 * \~japanese
 * ソルバーマネージャーを取得する。
 * @param       void
 * @return    ソルバーマネージャー
 */
sphSolverManager* sphSphereObject::getSolverManager()
{
  return m_sphere->getSolverManager();
}

/**
 * \~english
 * get coupler manager
 * @param       void
 * @return      coupler manager
 *
 * \~japanese
 * カプラーマネージャーを取得する。
 * @param       void
 * @return    カプラーマネージャー
 */
sphCouplerManager* sphSphereObject::getCouplerManager()
{
  return m_sphere->getCouplerManager();
}


/**
 * \~english
 * get parallel manager
 * @param       void
 * @return      parallel manager
 *
 * \~japanese
 * パラレルマネージャーを取得する。
 * @param       void
 * @return      パラレルマネージャー
 */
sphParaManager* sphSphereObject::getParaManager()
{

  return m_sphere->getParaManager();
}


/**
 * コンフィグレーション要素を設定する。
 * @param       config_elem       コンフィグレーション要素
 * @return      void
 */
bool sphSphereObject::setConfigElem(const sphElem* config_elem)
{
  m_configElem = config_elem;
  return true;
}


/**
 * コンフィグレーション要素を取得する。
 * @param       void
 * @return      コンフィグレーション要素
 */
const sphElem* sphSphereObject::getConfigElem() const
{
  return m_configElem;
}



/**
 * フローコントローラーを取得する。(const)
 * @param       void
 * @return      フローコントローラー
 */
const sphFlowBase* sphSphereObject::getFlow() const
{
  return m_sphere->getFlow();
}


/**
 * フローコントローラーを取得する。
 * @param       void
 * @return      フローコントローラー
 */
sphFlowBase* sphSphereObject::getFlow()
{
  return m_sphere->getFlow();
}


