/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphCfgBase.C
 * Configuration super Class source File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file sphCfgBase.C
 * コンフィギュレーション基底クラスソースファイル
 * @author     hira
 * @date       2009/10/01
 */


#include "config/sphCfgBase.h"

/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 */
sphCfgBase::sphCfgBase()
{

}


/**
 * \~english
 * Destructor
 *
 * \~japanese
 * デストラクタ
 */
sphCfgBase::~sphCfgBase()
{
}


/**
 * \~english
 * get attribute value, ignoring the case of the characters.
 * @param     node         current node pointer
 * @param     src_name       attribute name
 * @return    attribute value
 *
 * \~japanese
 * 要素から大文字・小文字を区別せずに属性を取得する。
 * @param     node             検索対象ノードポインタ
 * @param     src_name         検索属性名
 * @return    検索結果属性値
 */
xmlChar* sphCfgBase::getXmlAttribute(xmlNodePtr node, const xmlChar* src_name) const
{
  if (node == NULL) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_PARAMETER, "xmlNodePtr is null.");
    return NULL;
  }
  if (src_name == NULL) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_PARAMETER, "src_name is null.");
    return NULL;
  }

  xmlChar* attr_value = NULL;
  xmlAttrPtr prop = node->properties;
  while( prop ) {
    const xmlChar* attr_name = prop->name;
    attr_value = xmlGetProp(node, attr_name);
    if (strcasecmp((const char*)attr_name, (const char*)src_name) == 0) {
      break;
    }
    xmlFree(attr_value);
    attr_value = NULL;
    prop = prop->next;
  }

  return attr_value;
}


