/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphDataUtils.C
 * Data Utilities Class Source File
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file sphDataUtils.C
 * データユーティリティクラスソースファイル
 * @author     hira
 * @date       2009/02/20
 */

// ----- add 20100414 @tamaki begin -----
#include "data/sphVoxArray.h"
// ----- add 20100414 @tamaki end   -----
#include "data/sphDataUtils.h"

/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 */
sphDataUtils::sphDataUtils()
{

}


/**
 * \~english
 * Destoructor
 *
 * \~japanese
 * デストラクタ
 */
sphDataUtils::~sphDataUtils()
{

}


/**
 * データクラスタイプから次元数を取得する。
 *
 * @param        dctype         データクラスタイプ
 * @return      次元数
 */
int sphDataUtils::getDimsOfSphDCType(SphDCType dctype)
{

  switch(dctype) {
  case SPH_DC_ARRAY_1D:    /* (i) */
  case SPH_DC_ARRAY_1DN:  /* (i, n) */
  case SPH_DC_ARRAY_1DNRV:  /* (n, i) */
    return 1;
    break;
  case SPH_DC_ARRAY_2D:    /* (i, j) */
  case SPH_DC_ARRAY_2DN:  /* (i, j, n) */
  case SPH_DC_ARRAY_2DNRV:  /* (n, i, j) */
    return 2;
    break;
  case SPH_DC_ARRAY_3D:    /* (i, j, k) */
  case SPH_DC_ARRAY_3DN:  /* (i, j, k, n) */
  case SPH_DC_ARRAY_3DNRV:  /* (n, i, j, k) */
    return 3;
    break;
  }

  return 0;

}


/**
 * sphDataの２つのデータをコピーする。(sphData).
 * 重なる領域のすべてのデータをコピーする。
 * @param[out]    dst        コピー先sphDataデータ
 * @param[in]    src        コピー元sphDataデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::copyArray(sphData* dst, const sphData* src) {
  if( !dst || !src ) return false;
  SPL_Datatype dataType = dst->getDataType();

  // char型
  if (dataType == SPL_CHAR) {
    sphVoxArray<char>* dstArray = dynamic_cast<sphVoxArray<char>*>(dst);
    const sphVoxArray<char>* srcArray = dynamic_cast<const sphVoxArray<char>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // unsigned char型
  else if (dataType == SPL_UNSIGNED_CHAR) {
    sphVoxArray<unsigned char>* dstArray = dynamic_cast<sphVoxArray<unsigned char>*>(dst);
    const sphVoxArray<unsigned char>* srcArray = dynamic_cast<const sphVoxArray<unsigned char>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // short型
  else if (dataType == SPL_SHORT) {
    sphVoxArray<short>* dstArray = dynamic_cast<sphVoxArray<short>*>(dst);
    const sphVoxArray<short>* srcArray = dynamic_cast<const sphVoxArray<short>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // unsigned short型
  else if (dataType == SPL_UNSIGNED_SHORT) {
    sphVoxArray<unsigned short>* dstArray = dynamic_cast<sphVoxArray<unsigned short>*>(dst);
    const sphVoxArray<unsigned short>* srcArray = dynamic_cast<const sphVoxArray<unsigned short>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // int型
  else if (dataType == SPL_INT) {
    sphVoxArray<int>* dstArray = dynamic_cast<sphVoxArray<int>*>(dst);
    const sphVoxArray<int>* srcArray = dynamic_cast<const sphVoxArray<int>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // unsigned int型
  else if (dataType == SPL_UNSIGNED) {
    sphVoxArray<unsigned int>* dstArray = dynamic_cast<sphVoxArray<unsigned int>*>(dst);
    const sphVoxArray<unsigned int>* srcArray = dynamic_cast<const sphVoxArray<unsigned int>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // long型
  else if (dataType == SPL_LONG) {
    sphVoxArray<long>* dstArray = dynamic_cast<sphVoxArray<long>*>(dst);
    const sphVoxArray<long>* srcArray = dynamic_cast<const sphVoxArray<long>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // unsigned long型
  else if (dataType == SPL_UNSIGNED_LONG) {
    sphVoxArray<unsigned long>* dstArray = dynamic_cast<sphVoxArray<unsigned long>*>(dst);
    const sphVoxArray<unsigned long>* srcArray = dynamic_cast<const sphVoxArray<unsigned long>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // long long型(SPL_LONG_LONG=SPL_LONG_LONG_INT)
  else if (dataType == SPL_LONG_LONG) {
    sphVoxArray<long long>* dstArray = dynamic_cast<sphVoxArray<long long>*>(dst);
    const sphVoxArray<long long>* srcArray = dynamic_cast<const sphVoxArray<long long>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // float型
  else if (dataType == SPL_FLOAT) {
    sphVoxArray<float>* dstArray = dynamic_cast<sphVoxArray<float>*>(dst);
    const sphVoxArray<float>* srcArray = dynamic_cast<const sphVoxArray<float>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }
  // double型
  else if (dataType == SPL_DOUBLE) {
    sphVoxArray<double>* dstArray = dynamic_cast<sphVoxArray<double>*>(dst);
    const sphVoxArray<double>* srcArray = dynamic_cast<const sphVoxArray<double>*>(src);
    if (dstArray == NULL || srcArray == NULL) return false;
    return dstArray->copyArray(srcArray);
  }

  return false;
}




/**
 * sphDataの２つのデータをコピーする。(sphData).
 * 重なる領域のすべてのデータをコピーする。
 * @param[out]    dst        コピー先sphDataデータ
 * @param[in]    src        コピー元sphDataデータ
 * @param[in]    copy_size         コピーサイズ
 * @param[in]    dst_start_idx    コピー先始点インデックス（ガイドセルを含まない。ローカルインデックス）
 * @param[in]    src_start_idx    コピー元始点インデックス（ガイドセルを含まない。ローカルインデックス）
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::copyArray(sphData* dst,
                        const sphData* src,
                        const size_t copy_size[3],
                        const size_t dst_start_idx[3],
                        const size_t src_start_idx[3]) {
  if( !dst || !src ) return false;
  if( !copy_size || !dst_start_idx || !src_start_idx ) return false;
  if (dst->getDataType() != src->getDataType()) return false;
  if (dst->getDcType() != src->getDcType()) return false;
  if (dst->getDims() != src->getDims()) return false;
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;

  int dims = dst->getDims();
  SPL_Datatype dataType = dst->getDataType();
  int sizeofdata = spl_get_datasize(dataType);
  SphDCType dctype= dst->getDcType();
  size_t data_num = dst->getNumOfDtElem();
  const size_t* dst_size = dst->_getArraySize();
  const size_t* src_size = src->_getArraySize();
  size_t dst_len = dst->getArrayLength();
  size_t src_len = src->getArrayLength();
  dst_len = dst_len/data_num;
  src_len = src_len/data_num;
  void* dst_data = dst->getDataPointer();
  const void* src_data = src->getDataPointer();

  register size_t i, j, k, l;
  size_t sta_idx[3] = {0, 0, 0};
  size_t end_idx[3] = {0, 0, 0};
  size_t array_len = 1;
  for (i=0; i<dims; i++) {
    sta_idx[i] = src_start_idx[i];
    end_idx[i] = src_start_idx[i] + copy_size[i] - 1;
    array_len *= copy_size[i];
  }
  if (array_len == 0) return false;

  if (dctype == SPH_DC_ARRAY_1DNRV || dctype == SPH_DC_ARRAY_2DNRV || dctype == SPH_DC_ARRAY_3DNRV) {
    sizeofdata *= data_num;
  }

  size_t dst_i, dst_j, dst_k;
  for (k=sta_idx[2]; k<=end_idx[2]; k++) {
    if (dims >= 3) {
      dst_k = dst_start_idx[2]+(k-sta_idx[2]);
      if (dst_k > dst_size[2]-1) dst_k = dst_size[2]-1;
    }
    else dst_k = 0;

    for (j=sta_idx[1]; j<=end_idx[1]; j++) {
      if (dims >= 2) {
        dst_j = dst_start_idx[1]+(j-sta_idx[1]);
        if (dst_j > dst_size[1]-1) dst_j = dst_size[1]-1;
      }
      else dst_j = 0;

      for (i=sta_idx[0]; i<=end_idx[0]; i++) {
        dst_i = dst_start_idx[0]+(i-sta_idx[0]);
        if (dst_i > dst_size[0]-1) dst_i = dst_size[0]-1;

        size_t src_idx = src_size[0]*src_size[1]*k
                        + src_size[0]*j
                        +i;
        size_t dst_idx = dst_size[0]*dst_size[1]*dst_k
                        + dst_size[0]*dst_j
                        +dst_i;

        if (dctype == SPH_DC_ARRAY_1DN || dctype == SPH_DC_ARRAY_2DN || dctype == SPH_DC_ARRAY_3DN) {
          for (l=0; l<data_num; l++) {
            memcpy((char*)dst_data+dst_idx*sizeofdata, (const char*)(src_data)+src_idx*sizeofdata, sizeofdata);
            src_idx += src_len;
            dst_idx += dst_len;
          }
        }
        else {
          memcpy((char*)dst_data+dst_idx*sizeofdata, (const char*)(src_data)+src_idx*sizeofdata, sizeofdata);
        }
      }
    }
  }

  return true;
}




/**
 * 2Dスカラデータのコピー元データをコピー先データに加算する。(float)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先2Dデータ
 * @param[in]    src        コピー元2Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrS2D(sphVoxArray<float>* dst, const sphVoxArray<float>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}


/**
 * 2Dベクトルデータのコピー元データをコピー先データに加算する。(float)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先3Dデータ
 * @param[in]    src        コピー元3Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrV2D(sphVoxArray<float>* dst, const sphVoxArray<float>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}


/**
 * 2DスカラデータにstepAvr値の乗算/商算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    src        元2Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrS2D(sphVoxArray<float>* src, unsigned stepAvr, bool mode) {

  if( !src ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return src->scaleAvr(mode_num, NULL, stepAvr, NULL);
}


/**
 * 2Dスカラデータのコピー元データをコピー先データに加算する。(double)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先2Dデータ
 * @param[in]    src        コピー元2Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrS2D(sphVoxArray<double>* dst, const sphVoxArray<double>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}


/**
 * 2Dベクトルデータのコピー元データをコピー先データに加算する。(double)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先3Dデータ
 * @param[in]    src        コピー元3Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrV2D(sphVoxArray<double>* dst, const sphVoxArray<double>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}


/**
 * 2DスカラデータにstepAvr値の乗算/商算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    src        元2Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrS2D(sphVoxArray<double>* src, unsigned stepAvr, bool mode) {

  if( !src ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return src->scaleAvr(mode_num, NULL, stepAvr, NULL);
}



/**
 * 3DベクトルデータにstepAvr値の乗算/商算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    vec       計算3Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrV3D(sphVoxArray<float>* vec, unsigned stepAvr, bool mode) {
  if( !vec ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return vec->scaleAvr(mode_num, NULL, stepAvr, NULL);
}


/**
 * 2DベクトルデータにstepAvr値の乗算,v00の加算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        加算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVin2D(sphVoxArray<float>* dst,
                    const sphVoxArray<float>* src,
                    float v00[2], unsigned stepAvr) {
  if( !dst || !src || !v00 ) return false;
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 2) return false;

  register size_t i;
  size_t data_num = dst->getNumOfDtElem();
  double*  va = new double[data_num];
  double  ra = (double)stepAvr;
  for (i=0; i<data_num; i++) va[i] = (double)v00[i]*ra;

  if (!dst->scaleAvr(4, src, stepAvr, va)) {
    delete [] va;
    return false;
  }
  delete [] va;
  return true;
}

/**
 * 2DベクトルデータにstepAvr値の乗算,v00の減算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        減算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVout2D(sphVoxArray<float>* dst,
                     const sphVoxArray<float>* src,
                     float v00[2], unsigned stepAvr) {

  if( !dst || !src || !v00 ) return false;

  if (dst->isVoxContainer() && src->isVoxContainer()) {
    // 部分領域データ管理の場合、サブブロック毎に行う。
    int sub_count = dst->getNumberOfVoxArray();
    for (unsigned n=0; n<sub_count; n++) {
      // 部分領域データクラスの取得
      sphVoxArray<float> *sub_dst = dst->getVoxArray(n);
      const sphVoxArray<float> *sub_src = src->getVoxArray(n);
      if (!sphDataUtils::shiftVout3D(sub_dst, sub_src, v00, stepAvr)) {
        return false;
      }
    }
    return true;
  }

  // ガイドセルを含まないボクセルサイズの取得 (I,J,K方向サイズ)
  const size_t* dst_sz = dst->getArraySize();
  const size_t* src_sz = src->getArraySize();
  // ボクセルサイズの一致チェック
  if(     (src_sz[0] != dst_sz[0])
       || (src_sz[1] != dst_sz[1]) ) return false;

  // ベクトルデータ長のチェック
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 2) return false;

  // データクラス種別のチェック
  if (dst->getDcType() != SPH_DC_ARRAY_3DNRV) return false;
  if (src->getDcType() != SPH_DC_ARRAY_3DN) return false;

  float* dst_data = dst->getData();
  const float* src_data = src->getData();
  if( !dst_data || !src_data ) return false;

  // ガイドセルを含むボクセルサイズを取得する。 (I,J,K方向サイズ)
  dst_sz = dst->_getArraySize();
  src_sz = src->_getArraySize();
  size_t dst_gc = dst->getGc();
  size_t src_gc = src->getGc();
  size_t sta, ix, jx;
  int diff;

  calcIndex(dst_sz[0], dst_sz[1],
            dst_gc, src_gc, ix, jx, diff, sta);

  float  ra = 1.0/(float)stepAvr; // output

  size_t src_idx, dst_idx;
  size_t s1 = src_sz[0]*src_sz[1];
  register size_t i, j;

  for(j=sta; j<jx; j++){
    unsigned jj = j+diff;
    for(i=sta; i<ix; i++){
      unsigned src_idx = src_sz[0]*jj + (i+diff); //indecies for SklVector3D
      unsigned dst_idx = 2*(dst_sz[0]*j + i);     //indecies for SklVector3DEx
      dst_data[dst_idx  ] = src_data[src_idx   ]*ra - v00[0];
      dst_data[dst_idx+1] = src_data[src_idx+s1]*ra - v00[1];
    }
  }

  return true;

}


/**
 * 2DベクトルデータにstepAvr値の乗算/商算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    src        元2Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrV2D(sphVoxArray<float>* vec, unsigned stepAvr, bool mode) {
  if( !vec ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return vec->scaleAvr(mode_num, NULL, stepAvr, NULL);
}


/**
 * 3DベクトルデータにstepAvr値の乗算/商算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    vec       計算3Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrV3D(sphVoxArray<double>* vec, unsigned stepAvr, bool mode) {
  if( !vec ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return vec->scaleAvr(mode_num, NULL, stepAvr, NULL);
}


/**
 * 2DベクトルデータにstepAvr値の乗算,v00の加算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        加算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVin2D(sphVoxArray<double>* dst,
                    const sphVoxArray<double>* src,
                    double v00[2], unsigned stepAvr) {
  if( !dst || !src || !v00 ) return false;
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 2) return false;

  register size_t i;
  size_t data_num = dst->getNumOfDtElem();
  double*  va = new double[data_num];
  double  ra = (double)stepAvr;
  for (i=0; i<data_num; i++) va[i] = v00[i]*ra;

  if (!dst->scaleAvr(4, src, stepAvr, va)) {
    delete [] va;
    return false;
  }
  delete [] va;
  return true;
}

/**
 * 2DベクトルデータにstepAvr値の乗算,v00の減算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        減算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVout2D(sphVoxArray<double>* dst,
                     const sphVoxArray<double>* src,
                     double v00[2], unsigned stepAvr) {

  if( !dst || !src || !v00 ) return false;

  if (dst->isVoxContainer() && src->isVoxContainer()) {
    // 部分領域データ管理の場合、サブブロック毎に行う。
    int sub_count = dst->getNumberOfVoxArray();
    for (unsigned n=0; n<sub_count; n++) {
      // 部分領域データクラスの取得
      sphVoxArray<double> *sub_dst = dst->getVoxArray(n);
      const sphVoxArray<double> *sub_src = src->getVoxArray(n);
      if (!sphDataUtils::shiftVout3D(sub_dst, sub_src, v00, stepAvr)) {
        return false;
      }
    }
    return true;
  }

  // ガイドセルを含まないボクセルサイズの取得 (I,J,K方向サイズ)
  const size_t* dst_sz = dst->getArraySize();
  const size_t* src_sz = src->getArraySize();
  // ボクセルサイズの一致チェック
  if(     (src_sz[0] != dst_sz[0])
       || (src_sz[1] != dst_sz[1]) ) return false;

  // ベクトルデータ長のチェック
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 2) return false;

  // データクラス種別のチェック
  if (dst->getDcType() != SPH_DC_ARRAY_3DNRV) return false;
  if (src->getDcType() != SPH_DC_ARRAY_3DN) return false;

  double* dst_data = dst->getData();
  const double* src_data = src->getData();
  if( !dst_data || !src_data ) return false;

  // ガイドセルを含むボクセルサイズを取得する。 (I,J,K方向サイズ)
  dst_sz = dst->_getArraySize();
  src_sz = src->_getArraySize();
  size_t dst_gc = dst->getGc();
  size_t src_gc = src->getGc();
  size_t sta, ix, jx;
  int diff;

  calcIndex(dst_sz[0], dst_sz[1],
            dst_gc, src_gc, ix, jx, diff, sta);

  double  ra = 1.0/(double)stepAvr; // output

  size_t src_idx, dst_idx;
  size_t s1 = src_sz[0]*src_sz[1];
  register size_t i, j;

  for(j=sta; j<jx; j++){
    size_t jj = j+diff;
    for(i=sta; i<ix; i++){
      size_t src_idx = src_sz[0]*jj + (i+diff); //indecies for SklVector3D
      size_t dst_idx = 2*(dst_sz[0]*j + i);     //indecies for SklVector3DEx
      dst_data[dst_idx  ] = src_data[src_idx   ]*ra - v00[0];
      dst_data[dst_idx+1] = src_data[src_idx+s1]*ra - v00[1];
    }
  }

  return true;

}


/**
 * 2DベクトルデータにstepAvr値の乗算/商算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    src        元2Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrV2D(sphVoxArray<double>* vec, unsigned stepAvr, bool mode) {
  if( !vec ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return vec->scaleAvr(mode_num, NULL, stepAvr, NULL);
}



//-------------------
// 3D
//-------------------

/**
 * 3Dスカラデータのコピー元データをコピー先データに加算する。(float)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先2Dデータ
 * @param[in]    src        コピー元2Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrS3D(sphVoxArray<float>* dst, const sphVoxArray<float>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}

/**
 * 3Dベクトルデータのコピー元データをコピー先データに加算する。(float)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先2Dデータ
 * @param[in]    src        コピー元2Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrV3D(sphVoxArray<float>* dst, const sphVoxArray<float>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}

/**
 * 3DスカラデータにstepAvr値の乗算/商算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    src        元2Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrS3D(sphVoxArray<float>* src, unsigned stepAvr, bool mode) {
  if( !src ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return src->scaleAvr(mode_num, NULL, stepAvr, NULL);

  return true;
}


/**
 * 3Dスカラデータのコピー元データをコピー先データに加算する。(double)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先2Dデータ
 * @param[in]    src        コピー元2Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrS3D(sphVoxArray<double>* dst, const sphVoxArray<double>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}

/**
 * 3Dベクトルデータのコピー元データをコピー先データに加算する。(double)
 * データ領域のサイズは同じであること。（ガイドセルは異なってもよい）
 * ガイドセル領域の加算も行う。
 * @param[out]    dst        コピー先2Dデータ
 * @param[in]    src        コピー元2Dデータ
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::avrV3D(sphVoxArray<double>* dst, const sphVoxArray<double>* src) {
  if( !dst || !src ) return false;
  return dst->scaleAvr(1, src, 0, NULL);
}

/**
 * 3DスカラデータにstepAvr値の乗算/商算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[in/out]    src        元2Dデータ
 * @param[in]    stepAvr    乗算/商算を行う値
 * @param[in]    mode       true:乗算/false:商算
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::scaleAvrS3D(sphVoxArray<double>* src, unsigned stepAvr, bool mode) {
  if( !src ) return false;
  int mode_num = 0;
  if ( mode==true ) mode_num = 2;
  else              mode_num = 3;
  return src->scaleAvr(mode_num, NULL, stepAvr, NULL);

  return true;
}


/**
 * 3DベクトルデータにstepAvr値の乗算,v00の加算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        加算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVin3D(sphVoxArray<float>* dst,
                    const sphVoxArray<float>* src,
                    float v00[3], unsigned stepAvr) {
  if( !dst || !src || !v00 ) return false;
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 3) return false;

  register size_t i;
  size_t data_num = dst->getNumOfDtElem();
  double*  va = new double[data_num];
  double  ra = (double)stepAvr;
  for (i=0; i<data_num; i++) va[i] = (double)v00[i]*ra;

  if (!dst->scaleAvr(4, src, stepAvr, va)) {
    delete [] va;
    return false;
  }
  delete [] va;
  return true;
}

/**
 * 3DベクトルデータにstepAvr値の乗算,v00の減算を行う。(float)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        減算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVout3D(sphVoxArray<float>* dst,
                     const sphVoxArray<float>* src,
                     float v00[3], unsigned stepAvr) {

  if( !dst || !src || !v00 ) return false;

  if (dst->isVoxContainer() && src->isVoxContainer()) {
    // 部分領域データ管理の場合、サブブロック毎に行う。
    int sub_count = dst->getNumberOfVoxArray();
    for (unsigned n=0; n<sub_count; n++) {
      // 部分領域データクラスの取得
      sphVoxArray<float> *sub_dst = dst->getVoxArray(n);
      const sphVoxArray<float> *sub_src = src->getVoxArray(n);
      if (!sphDataUtils::shiftVout3D(sub_dst, sub_src, v00, stepAvr)) {
        return false;
      }
    }
    return true;
  }

  // ガイドセルを含まないボクセルサイズの取得 (I,J,K方向サイズ)
  const size_t* dst_sz = dst->getArraySize();
  const size_t* src_sz = src->getArraySize();
  // ボクセルサイズの一致チェック
  if(     (src_sz[0] != dst_sz[0])
       || (src_sz[1] != dst_sz[1])
       || (src_sz[2] != dst_sz[2]) ) return false;

  // ベクトルデータ長のチェック
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 3) return false;

  // データクラス種別のチェック
  if (dst->getDcType() != SPH_DC_ARRAY_3DNRV) return false;
  if (src->getDcType() != SPH_DC_ARRAY_3DN) return false;

  float* dst_data = dst->getData();
  const float* src_data = src->getData();
  if( !dst_data || !src_data ) return false;

  // ガイドセルを含むボクセルサイズを取得する。 (I,J,K方向サイズ)
  dst_sz = dst->_getArraySize();
  src_sz = src->_getArraySize();
  size_t dst_gc = dst->getGc();
  size_t src_gc = src->getGc();
  size_t sta, ix, jx, kx;
  int diff;

  calcIndex(dst_sz[0], dst_sz[1], dst_sz[2],
            dst_gc, src_gc, ix, jx, kx, diff, sta);

  float  ra = 1.0/(float)stepAvr; // output

  size_t src_idx, dst_idx;
  size_t s1 = src_sz[0]*src_sz[1]*src_sz[2];
  size_t s2 = s1*2;
  register size_t i, j, k;

  for(k=sta; k<kx; k++){
    size_t kk = k+diff;
    for(j=sta; j<jx; j++){
      size_t jj = j+diff;
      for(i=sta; i<ix; i++){
        // src_idx : indecies for SklVector3D
        src_idx = src_sz[0]*src_sz[1]*kk + src_sz[0]*jj + (i+diff);
        // dst_idx : indecies for SklVector3DEx
        dst_idx = 3*(dst_sz[0]*dst_sz[1]*k + dst_sz[0]*j + i);
        dst_data[dst_idx  ] = src_data[src_idx   ]*ra - v00[0];
        dst_data[dst_idx+1] = src_data[src_idx+s1]*ra - v00[1];
        dst_data[dst_idx+2] = src_data[src_idx+s2]*ra - v00[2];
      }
    }
  }

  return true;
}


/**
 * 3DベクトルデータにstepAvr値の乗算,v00の加算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        加算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVin3D(sphVoxArray<double>* dst,
                    const sphVoxArray<double>* src,
                    double v00[3], unsigned stepAvr) {
  if( !dst || !src || !v00 ) return false;
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 3) return false;

  register size_t i;
  size_t data_num = dst->getNumOfDtElem();
  double*  va = new double[data_num];
  double  ra = (double)stepAvr;
  for (i=0; i<data_num; i++) va[i] = v00[i]*ra;

  if (!dst->scaleAvr(4, src, stepAvr, va)) {
    delete [] va;
    return false;
  }
  delete [] va;
  return true;
}

/**
 * 3DベクトルデータにstepAvr値の乗算,v00の減算を行う。(double)
 * ガイドセル領域の乗算/商算も行う。
 * @param[out]    dst       計算結果3Dデータ
 * @param[in]    src        計算3D元データ
 * @param[in]    v00        減算を行う値
 * @param[in]    stepAvr    乗算を行う値
 * @return    success(=true)/fault(=false)
 */
bool
sphDataUtils::shiftVout3D(sphVoxArray<double>* dst,
                     const sphVoxArray<double>* src,
                     double v00[3], unsigned stepAvr) {

  if( !dst || !src || !v00 ) return false;

  if (dst->isVoxContainer() && src->isVoxContainer()) {
    // 部分領域データ管理の場合、サブブロック毎に行う。
    int sub_count = dst->getNumberOfVoxArray();
    for (unsigned n=0; n<sub_count; n++) {
      // 部分領域データクラスの取得
      sphVoxArray<double> *sub_dst = dst->getVoxArray(n);
      const sphVoxArray<double> *sub_src = src->getVoxArray(n);
      if (!sphDataUtils::shiftVout3D(sub_dst, sub_src, v00, stepAvr)) {
        return false;
      }
    }
    return true;
  }

  // ガイドセルを含まないボクセルサイズの取得 (I,J,K方向サイズ)
  const size_t* dst_sz = dst->getArraySize();
  const size_t* src_sz = src->getArraySize();
  // ボクセルサイズの一致チェック
  if(     (src_sz[0] != dst_sz[0])
       || (src_sz[1] != dst_sz[1])
       || (src_sz[2] != dst_sz[2]) ) return false;

  // ベクトルデータ長のチェック
  if (dst->getNumOfDtElem() != src->getNumOfDtElem()) return false;
  if (dst->getNumOfDtElem() != 3) return false;

  // データクラス種別のチェック
  if (dst->getDcType() != SPH_DC_ARRAY_3DNRV) return false;
  if (src->getDcType() != SPH_DC_ARRAY_3DN) return false;

  double* dst_data = dst->getData();
  const double* src_data = src->getData();
  if( !dst_data || !src_data ) return false;

  // ガイドセルを含むボクセルサイズを取得する。 (I,J,K方向サイズ)
  dst_sz = dst->_getArraySize();
  src_sz = src->_getArraySize();
  size_t dst_gc = dst->getGc();
  size_t src_gc = src->getGc();
  size_t sta, ix, jx, kx;
  int diff;

  calcIndex(dst_sz[0], dst_sz[1], dst_sz[2],
            dst_gc, src_gc, ix, jx, kx, diff, sta);

  double  ra = 1.0/(double)stepAvr; // output

  size_t src_idx, dst_idx;
  size_t s1 = src_sz[0]*src_sz[1]*src_sz[2];
  size_t s2 = s1*2;
  register size_t i, j, k;

  for(k=sta; k<kx; k++){
    size_t kk = k+diff;
    for(j=sta; j<jx; j++){
      size_t jj = j+diff;
      for(i=sta; i<ix; i++){
        // src_idx : indecies for SklVector3D
        src_idx = src_sz[0]*src_sz[1]*kk + src_sz[0]*jj + (i+diff);
        // dst_idx : indecies for SklVector3DEx
        dst_idx = 3*(dst_sz[0]*dst_sz[1]*k + dst_sz[0]*j + i);
        dst_data[dst_idx  ] = src_data[src_idx   ]*ra - v00[0];
        dst_data[dst_idx+1] = src_data[src_idx+s1]*ra - v00[1];
        dst_data[dst_idx+2] = src_data[src_idx+s2]*ra - v00[2];
      }
    }
  }

  return true;
}





