/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphFile.C
 * SBX File Input/Output Class
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphFile.C
 * SBXファイル入出力クラス
 * @author     hira
 * @date       2009/09/07
 */

#include "fileio/sphFile.h"

/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 */
sphFile::sphFile()
{
  // 初期化を行う。
  initialize();
}



/**
 * \~english
 * Destructor
 *
 * \~japanese
 * デストラクタ
 */
sphFile::~sphFile()
{
  // 破棄処理
  finalize();
}


/**
 * \~english
 *
 * \~japanese
 *  初期化を行う.
 *  メンバーの初期化（デフォルト値の代入）を行う.
 * @param      なし
 * @return    success(=true)/fault(=false)
 */
bool sphFile::initialize()
{
  /// メンバーの初期化
  m_rankId = 0;          // 読込/書込ランク番号
  m_procGrp = SPL_COMM_WORLD;      // 読込/書込プロセスグループ

  return true;
}


/**
 * \~english
 *
 * \~japanese
 *  破棄処理を行う.
 *  splSbx, splSvFileList構造体の破棄処理を行う。
 * @param     mem_free_flag    splSbx->data(splData)の開放フラグ
 *                  true:開放を行う。, false:開放を行わない。
 * @return    success(=true)/fault(=false)
 */
bool sphFile::finalize()
{
  return true;
}




/**
 * \~english
 * set mask on Upper bits, get lower bits data
 * @param      void
 * @return    sphData pointer
 *
 * \~japanese
 * 上位ビットデータをマスクし(0とする）、下位ビットのみのデータとする.
 * @param      void
 * @return    sphData pointer
 */
sphData* sphFile::maskUpperBits()
{
  // error
  m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "not support maskUpperBits");
  return NULL;
}

/**
 * \~english
 * shift lower bits, get upper bits data
 * @param      void
 * @return    sphData pointer
 *
 * \~japanese
 *  下位２ビットをシフトし、上位のみのデータとする。.
 * @param      void
 * @return    sphData pointer
 */
sphData* sphFile::shiftLowBits()
{
  // error
  m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "not support shiftLowBits");
  return NULL;
}


/**
 * \~english
 * set read/write process group
 * @param      procGrp			read/write process group
 * @return     void
 *
 * \~japanese
 *  読込/書込プロセスグループを設定する
 * @param      procGrp			読込/書込プロセスグループ
 * @return    void
 */
void sphFile::setProcGrp(SPL_Comm procGrp)
{
  m_procGrp = procGrp;
}

/**
 * \~english
 * set read/write rank id
 * @param      rankId			read/write rank id
 * @return     void
 *
 * \~japanese
 *  読込/書込ランク番号を設定する
 * @param      rankId			読込/書込ランク番号
 * @return    void
 */
/// 読込/書込ランク番号を設定する
void sphFile::setRankId(int rankId)
{
  m_rankId = rankId;
}


