/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphSbxHeader.C
 * SBX File Header Class
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphSbxHeader.C
 * SBXファイルヘッダークラス
 * @author     hira
 * @date       2009/09/07
 */

#include "fileio/sphSbxHeader.h"

/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 */
sphSbxHeader::sphSbxHeader()
{
  // メンバの初期化
  initialize();
}

/**
 * \~english
 * Destructor
 *
 * \~japanese
 * デストラクタ
 */
sphSbxHeader::~sphSbxHeader()
{
  finalize();
}


/**
 * \~english
 * Copy Constructor
 *
 * \~japanese
 * コピーコンストラクタ
 */
sphSbxHeader::sphSbxHeader(const sphSbxHeader& sbxHeader)
{
  register int i;
  for(i=0; i<3; i++) {
    m_wsize[i] = sbxHeader.m_wsize[i];
    m_start_idx[i] = sbxHeader.m_start_idx[i];
    m_size[i] = sbxHeader.m_size[i];
    m_origin[i] = sbxHeader.m_origin[i];
    m_pitch[i] = sbxHeader.m_pitch[i];
  }
  m_dims = sbxHeader.m_dims;
  m_vlen = sbxHeader.m_vlen;
  m_dtype = sbxHeader.m_dtype;
  m_gc = sbxHeader.m_gc;
  m_rlen = sbxHeader.m_rlen;
  m_crddef = sbxHeader.m_crddef;
  m_aux = sbxHeader.m_aux;
  m_block_size = sbxHeader.m_block_size;

  return;
}


/**
 * \~english
 * operator (=)
 *
 * \~japanese
 * 代入演算子 (=)
 */
sphSbxHeader& sphSbxHeader::operator = (const sphSbxHeader& sbxHeaer)
{

  register int i;
  for(i=0; i<3; i++) {
    m_wsize[i] = sbxHeaer.m_wsize[i];
    m_start_idx[i] = sbxHeaer.m_start_idx[i];
    m_size[i] = sbxHeaer.m_size[i];
    m_origin[i] = sbxHeaer.m_origin[i];
    m_pitch[i] = sbxHeaer.m_pitch[i];
  }
  m_dims = sbxHeaer.m_dims;
  m_vlen = sbxHeaer.m_vlen;
  m_dtype = sbxHeaer.m_dtype;
  m_gc = sbxHeaer.m_gc;
  m_rlen = sbxHeaer.m_rlen;
  m_crddef = sbxHeaer.m_crddef;
  m_aux = sbxHeaer.m_aux;
  m_block_size = sbxHeaer.m_block_size;

  return *this;
}

/**
 * \~english
 * initialize member
 * @param      void
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  初期化を行う。
 *  メンバの初期化を行う。
 * @param      なし
 * @return    success(=true)/fault(=false)
 */
bool sphSbxHeader::initialize()
{
    register int i;
    for(i=0; i<3; i++) {
      m_wsize[i] = 0;
      m_start_idx[i] = 0;
      m_size[i] = 0;
      m_origin[i] = 0.0;
      m_pitch[i] = 0.0;
    }
    m_dims = 3;
    m_vlen = 1;
    m_dtype = 0;
    m_gc = 0;
    m_rlen = 4;
    m_crddef = 1;
    m_aux = 0;
    m_block_size = 0;

    return true;
}


/**
 * \~english
 * finalize
 * @param      void
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  破棄処理を行う。
 *  メンバの破棄を行う。
 * @param      void
 * @return    success(=true)/fault(=false)
 */
bool sphSbxHeader::finalize()
{
  return true;
}


/**
 * \~english
 * get Sbx Header member by splSbx
 * @param      sbx    splSbx Object
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  SBXファイルのヘッダ情報をsplSbxオブジェクトに設定して取得する。
 * @param      sbx    splSbxオブジェクト
 * @return    success(=true)/fault(=false)
 */
void sphSbxHeader::getSplSbxHeader(splSbx* sbx_header)
{
  register int i;
  for(i=0; i<3; i++) {
    sbx_header->wsize[i] = m_wsize[i];
    sbx_header->start_idx[i] = m_start_idx[i];
    sbx_header->size[i] = m_size[i];
    sbx_header->origin[i] = m_origin[i];
    sbx_header->pitch[i] = m_pitch[i];
  }
  sbx_header->dims = m_dims;
  sbx_header->vlen = m_vlen;
  sbx_header->dtype = m_dtype;
  sbx_header->gc = m_gc;
  sbx_header->rlen = m_rlen;
  sbx_header->crddef = m_crddef;
  sbx_header->aux = m_aux;
  sbx_header->block_size = m_block_size;

  return;
}


/**
 * \~english
 * set Sbx Header member from splSbx object
 * @param      sbx    splSbx Object
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  SBXファイルのヘッダ情報をsplSbxオブジェクトから設定する。
 * @param      sbx    splSbxオブジェクト
 * @return    success(=true)/fault(=false)
 */
void sphSbxHeader::setSplSbxHeader(const splSbx* sbx_header)
{
  register int i;
  for(i=0; i<3; i++) {
    m_wsize[i] = sbx_header->wsize[i];
    m_start_idx[i] = sbx_header->start_idx[i];
    m_size[i] = sbx_header->size[i];
    m_origin[i] = sbx_header->origin[i];
    m_pitch[i] = sbx_header->pitch[i];
  }
  m_dims = sbx_header->dims;
  m_vlen = sbx_header->vlen;
  m_dtype = sbx_header->dtype;
  m_gc = sbx_header->gc;
  m_rlen = sbx_header->rlen;
  m_crddef = sbx_header->crddef;
  m_aux = sbx_header->aux;
  m_block_size = sbx_header->block_size;

  return;

}

/**
 * \~english
 *
 * \~japanese
 * AUXレコード(拡張仕様)の占有ビット数、データ存在情報を取得する。
 * ビットデータ情報は、４バイトレコードの内、下位１バイトにて表現する。 (FF FF FF YX)
 *     占有ビット数 : 下位４ビット     0000XXXX
 *     データ存在情報 : 上位４ビット   YYYY0000
 * @param     bits    占有ビット数（上位ビット数）
 * @param    dataflag  データ存在情報
 *               0:上位、下位ビット共に有効な情報が記述されている場合
 *               1:下位のみに有効な情報が記述されている場合
 *               2:上位のみに有効な情報が記述されている場合
 * @return  void
 */
void sphSbxHeader::getDataExistFlags(unsigned int& bits, unsigned int& dataflag)
{

  bits = m_aux & 0x0000000F;
  dataflag = (m_aux & 0x000000F0) >> 4;

  return;
}


/**
 * \~english
 *
 * \~japanese
 * AUXレコード(拡張仕様)の占有ビット数、データ存在情報を設定する。
 * ビットデータ情報は、４バイトレコードの内、下位１バイトにて表現する。 (FF FF FF YX)
 *     占有ビット数 : 下位４ビット     0000XXXX
 *     データ存在情報 : 上位４ビット   YYYY0000
 * @param     bits    占有ビット数（上位ビット数）
 * @param    dataflag  データ存在情報
 *               0:上位、下位ビット共に有効な情報が記述されている場合
 *               1:下位のみに有効な情報が記述されている場合
 *               2:上位のみに有効な情報が記述されている場合
 * @return  void
 */
void sphSbxHeader::setDataExistFlags(unsigned int bits, unsigned int dataflag)
{
  m_aux = m_aux & 0xFFFFFF00;
  m_aux = m_aux | (bits & (dataflag << 4));

  return;
}


/**
 * \~english
 *
 * \~japanese
 * ヘッダーデータのチェックを行う。
 * @param     void
 * @return  success(=true)/fault(=false)
 */
bool sphSbxHeader::checkHeader() const
{
  register int i;

  // 次元数 (１次元=１, ２次元=2, ３次元=3)
  if (m_dims != 1 && m_dims != 2 && m_dims != 3) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_DIMS, "invalid dims[=%d], have to be 1, 2 or 3.", m_dims);
    return false;
  }
  // ベクトル長 (=1)
  if (m_vlen != 1) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_PARAMETER, "invalid vlen[=%d], have to be 1.", m_vlen);
    return false;
  }
  // データ種別 (1バイト（unsigned char）=1, 2バイト（unsigned short）=2, 4バイト（unsigned int）=4)
  if (m_dtype != 1 && m_dtype != 2 && m_dtype != 4) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_DTYPE, "invalid dtype[=%d], have to be 1, 2 or 4.", m_dtype);
    return false;
  }
  // 実数データのサイズ (単精度浮動小数点数（float：4byte）=4, 倍精度浮動小数点数（double：8byte）=8)
  if (m_rlen != 4 && m_rlen != 8) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_RLEN, "invalid rlen[=%d], have to be 4 or 8.", m_rlen);
    return false;
  }
  // 原点座標の定義位置 (Regular=1, Collocate=2, Staggered（１）=3, Staggered（２）=4)
  if (m_crddef != 1 && m_crddef != 2 && m_crddef != 3 && m_crddef != 4) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_CRDDEF, "invalid crddef[=%d], have to be 1,2,3 or 4.", m_crddef);
    return false;
  }
  // サイズチェック
  for (i=0;i<m_dims;i++) {
    if (m_size[i] <= m_gc*2) {
      // error
      m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_ARRAY_SIZE, "size have to be more than gc*2, size[%d]:%ld <= gc[%d]:%ld * 2", i, m_size[i], i, m_gc);
      return false;
    }
  }

  return true;
}


/**
 * \~english
 *
 * \~japanese
 * 圧縮ブロックサイズをサイズから設定する.
 * １次元の場合：０
 * ２次元の場合：I方向サイズ
 * ３次元の場合：I方向サイズ x J方向サイズ
 * @param     void
 * @return  void
 */
void sphSbxHeader::setBlockSizeBySize()
{
  size_t size = m_size[0];
  if (m_dims == 1) size = m_size[0];
  else if (m_dims == 2) size = m_size[0];
  else if (m_dims == 3) size = m_size[0]*m_size[1];

  m_block_size = size;

  return;
}


/**
 * \~english
 * set block size
 * @param     block_size               block size
 * @return  void

 * \~japanese
 * 圧縮ブロックサイズを設定する.
 * @param     block_size                圧縮ブロックサイズ
 * @return  void
 */
void sphSbxHeader::setBlockSize(size_t block_size)
{
  m_block_size = block_size;
  return;
}


/**
 * \~english
 *
 * \~japanese
 * ヘッダー情報を出力する。(for debug)
 * @param     void
 * @return  void
 */
void sphSbxHeader::printHeaderInfo()
{
  printf("dimension\t\t\t: %d\n", m_dims);
  printf("vector length\t\t\t: %d\n", m_vlen);
  printf("data type\t\t\t: %d\n", m_dtype);
  printf("guide cell size\t\t\t: %ld\n", m_gc);
  printf("real data type\t\t\t: %d\n", m_rlen);
  printf("origin definision\t\t: %d\n", m_crddef);
  printf("aux\t\t\t\t: 0x%02X\n", m_aux);
  printf("size[i,j,k]\t\t\t: %ld, %ld, %ld\n", m_size[0], m_size[1], m_size[2]);
  printf("origin[i,j,k]\t\t\t: %f, %f, %f\n", m_origin[0], m_origin[1], m_origin[2]);
  printf("pitch[i,j,k]\t\t\t: %f, %f, %f\n", m_pitch[0], m_pitch[1], m_pitch[2]);
  printf("block size\t\t\t: %ld\n", m_block_size);
  printf("whole voxel size[i,j,k]\t\t: %ld, %ld, %ld\n", m_wsize[0], m_wsize[1], m_wsize[2]);
  printf("global start index[i,j,k]\t: %ld, %ld, %ld\n", m_start_idx[0], m_start_idx[1], m_start_idx[2]);

  return;
}


/**
 * \~english
 * get whole voxel size
 * @param     void
 * @return  whole voxel size : size_t[3]
 *
 * \~japanese
 * 全体ボクセルサイズを取得する。
 * @param     void
 * @return  全体ボクセルサイズ : size_t[3]
 */
const size_t* sphSbxHeader::getWholeSize()  const
{
  return m_wsize;
}

/**
 * \~english
 * get start index
 * @param     void
 * @return  start index : size_t[3]
 *
 * \~japanese
 * 始点インデックスを取得する。
 * @param     void
 * @return  始点インデックス : size_t[3]
 */
const size_t* sphSbxHeader::getStartIdx()  const
{
  return m_start_idx;
}


/**
 * \~english
 * get dims
 * @param     void
 * @return  dims
 *
 * \~japanese
 * 次元数を取得する。
 * @param     void
 * @return  次元数
 */
unsigned int sphSbxHeader::getDims()  const
{
  return m_dims;
}


/**
 * \~english
 * get cell data length in cell.
 * @param     void
 * @return    data length
 *
 * \~japanese
 * １セルのデータサイズを取得する.
 * SBXファイルの場合、vlen(=1) * dtype
 * SPXファイルの場合、vlen * rlen
 * @param     void
 * @return  データサイズ
 */
unsigned int sphSbxHeader::getCellDataLength() const
{
  return m_vlen*m_dtype;
}


/**
 * \~english
 * get total voxel data length.
 * @param     void
 * @return    data length in voxel
 *
 * \~japanese
 * ボクセルのデータサイズを取得する.
 * SBXファイルの場合、vlen(=1) * dtype * size[]
 * SPXファイルの場合、vlen * rlen * size[]
 * @param     void
 * @return    ボクセルデータサイズ
 */
size_t sphSbxHeader::getVoxelDataLength() const
{
  register int i;
  size_t len = 1;
  for (i=0; i<m_dims; i++) {
    len *= m_size[i];
  }
  len *= getCellDataLength();

  return len;
}


/**
 * \~english
 * get gc
 * @param     void
 * @return    gc
 *
 * \~japanese
 * Guide Cell数を取得する.
 * @param     void
 * @return    gc
 */
size_t sphSbxHeader::getGc() const
{
  return m_gc;
}



/**
 * \~english
 * get coords definition position
 * @param     void
 * @return    coords definition
 *
 * \~japanese
 * 定義位置を取得する。
 * @param     void
 * @return    定義位置(1,2,3,4)
 */
unsigned int sphSbxHeader::getCrddef() const
{
  return m_crddef;
}



/**
 * \~english
 * get aux
 * @param     void
 * @return    aux
 *
 * \~japanese
 * AUXを取得する。
 * @param     void
 * @return    aux
 */
unsigned int sphSbxHeader::getAux() const
{
  return m_aux;
}


/**
 * \~english
 * get voxel size
 * @param     void
 * @return  whole voxel size : size_t[3]
 *
 * \~japanese
 * ボクセルサイズを取得する。
 * @param     void
 * @return  ボクセルサイズ : size_t[3]
 */
const size_t* sphSbxHeader::getSize() const
{
  return m_size;
}


/**
 * \~english
 * get origin coords
 * @param     void
 * @return    origin coords : double[3]
 *
 * \~japanese
 * 原点座標を取得する。
 * @param     void
 * @return    原点座標 : double[3]
 */
const double* sphSbxHeader::getOrigin() const
{
  return m_origin;
}


/**
 * \~english
 * get pitch
 * @param     void
 * @return    pitch : double[3]
 *
 * \~japanese
 * ピッチを取得する。
 * @param     void
 * @return   ピッチ : double[3]
 */
const double* sphSbxHeader::getPitch() const
{
  return m_pitch;
}


/**
 * \~english
 * get block size
 * @param     void
 * @return    block size
 *
 * \~japanese
 * ブロックサイズを取得する。
 * @param     void
 * @return    ブロックサイズ
 */
size_t sphSbxHeader::getBlockSize() const
{
  return m_block_size;
}


/**
 * \~english
 * set size
 * @param     size      size
 * @return    void
 *
 * \~japanese
 * サイズを設定する。
 * @param     size             サイズ
 * @return    void
 */
void sphSbxHeader::setSize(const size_t size[3])
{
  m_size[0] = size[0];
  m_size[1] = size[1];
  m_size[2] = size[2];
}



/**
 * \~english
 * get header format
 * @param     void
 * @return    header format
 *
 * \~japanese
 * ヘッダーフォーマットを取得する。
 * @param     void
 * @return    ヘッダーフォーマット文字列
 */
std::string sphSbxHeader::getFormat()
{
  return std::string(HEADER_FORMAT);
}


/**
 * \~english
 * get vector length
 * @param     void
 * @return    vector length
 *
 * \~japanese
 * ヘッダーフォーマットを取得する。
 * @param     void
 * @return    ヘッダーフォーマット文字列
 */
unsigned int sphSbxHeader::getVlen()  const
{
  return m_vlen;
}


/**
 * \~english
 * get data type.
 * data type ( 1:unsigned char, 2:unsigned short, 4:unsigned int )
 * @param     void
 * @return    data type ( 1:unsigned char, 2:unsigned short, 4:unsigned int )
 *
 * \~japanese
 * dtypeを取得する.
 * data type ( 1:unsigned char, 2:unsigned short, 4:unsigned int )
 * @param     void
 * @return    data type ( 1:unsigned char, 2:unsigned short, 4:unsigned int )
 */
unsigned int sphSbxHeader::getDtype()  const
{
  return m_dtype;
}


/**
 * \~english
 * get rlen.
 * real type ( 4:float, 8:double )
 * @param     void
 * @return    data type ( 1:unsigned char, 2:unsigned short, 4:unsigned int )
 *
 * \~japanese
 * rlenを取得する.
 * real type ( 4:float, 8:double )
 * @param     void
 * @return    real type ( 4:float, 8:double )
 */
unsigned int sphSbxHeader::getRlen()  const
{
  return m_rlen;
}



/**
 * \~english
 * get start_idx.
 * @param     start_idx                 start_idx
 * @return    void
 *
 * \~japanese
 * start_idxを取得する.
 * @param     start_idx                 start_idx
 * @return    void
 */
void sphSbxHeader::setStartIdx(const size_t start_idx[3])
{
  m_start_idx[0] = start_idx[0];
  m_start_idx[1] = start_idx[1];
  m_start_idx[2] = start_idx[2];

}


/**
 * Guide Cell数を設定する.
 * @param     gc        ガイドセル
 * @return    void
 */
void sphSbxHeader::setGc(size_t gc)
{
  m_gc = gc;
}


/**
 * 次元数を設定する。
 * @param     dims      次元数
 * @return  void
 */
void sphSbxHeader::setDims(int dims)
{
  m_dims = dims;
}



/**
 * 全体ボクセルサイズを設定する。
 * @param     wsize             全体ボクセルサイズ
 * @return    void
 */
void sphSbxHeader::setWholeSize(const size_t wsize[3])
{
  m_wsize[0] = wsize[0];
  m_wsize[1] = wsize[1];
  m_wsize[2] = wsize[2];
  return;
}


/**
 * ベクトル長を設定する。
 * @param     vlen             ベクトル長
 * @return    void
 */
void sphSbxHeader::setVlen(unsigned int vlen)
{
  m_vlen = vlen;
}


/**
 * データタイプを設定する。
 * @param     dtype             データタイプ
 * @return    void
 */
void sphSbxHeader::setDtype(unsigned int dtype)
{
  m_dtype = dtype;
}


/**
 * 実数タイプを設定する。
 * @param     rlen             実数タイプ
 * @return    void
 */
void sphSbxHeader::setRlen(unsigned int rlen)
{
  m_rlen = rlen;
}


/**
 * 実行ステップを取得する。
 * @param     void
 * @return    実行ステップ数
 */
size_t sphSbxHeader::getStep() const
{
  return 0;
}


/**
 * 実行時間を取得する。
 * @param     void
 * @return    実行時間
 */
double sphSbxHeader::getTime() const
{
  return 0.0;
}


/**
 * 実行ステップ、時間を設定する。.
 * SBXファイルでは実行ステップ、時間はなし。
 * @param     step      実行ステップ
 * @param     time      実行時間
 * @return    void
 */
void sphSbxHeader::setStepTime(size_t step, double time)
{
  return;
}


/**
 * 原点座標を設定する。
 * @param     org[3]      原点座標
 * @return    void
 */
void sphSbxHeader::setOrigin(const double org[3])
{
  m_origin[0] = org[0];
  m_origin[1] = org[1];
  m_origin[2] = org[2];
}


/**
 * ピッチを設定する。
 * @param     pitch[3]      ピッチ
 * @return    void
 */
void sphSbxHeader::setPitch(const double pitch[3])
{

  m_pitch[0] = pitch[0];
  m_pitch[1] = pitch[1];
  m_pitch[2] = pitch[2];
}


/**
 * 定義位置を設定する。
 * @param     crddef          定義位置
 * @return    void
 */
void sphSbxHeader::setCrddef(unsigned int crddef)
{
  m_crddef = crddef;
}


/**
 * AUXを設定する。
 * @param     aux          AUX
 * @return    void
 */
void sphSbxHeader::setAux(unsigned int aux)
{
  m_aux = aux;
}


