/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphSphHeader.C
 * SPH File Header Class
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphSphHeader.C
 * SPHファイルヘッダークラス
 * @author     hira
 * @date       2009/09/07
 */


#include "fileio/sphSphHeader.h"

/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 */
sphSphHeader::sphSphHeader()
{
  // メンバの初期化
  initialize();
}


/**
 * \~english
 * Destructor
 *
 * \~japanese
 * デストラクタ
 */
sphSphHeader::~sphSphHeader()
{
  finalize();
}


/**
 * \~english
 * Copy Constructor
 *
 * \~japanese
 * コピーコンストラクタ
 */
sphSphHeader::sphSphHeader(const sphSphHeader& sph_header)
{
  register int i;
  for(i=0; i<3; i++) {
    m_wsize[i] = sph_header.m_wsize[i];
    m_start_idx[i] = sph_header.m_start_idx[i];
    m_size[i] = sph_header.m_size[i];
    m_origin[i] = sph_header.m_origin[i];
    m_pitch[i] = sph_header.m_pitch[i];
  }
  m_dims = sph_header.m_dims;
  m_vlen = sph_header.m_vlen;
  m_dtype = sph_header.m_dtype;
  m_gc = sph_header.m_gc;
  m_rlen = sph_header.m_rlen;
  m_crddef = sph_header.m_crddef;
  m_block_size = sph_header.m_block_size;
  m_step = sph_header.m_step;
  m_time = sph_header.m_time;
  m_gc_insph = sph_header.m_gc_insph;

  return;
}


/**
 * \~english
 * operator (=)
 *
 * \~japanese
 * 代入演算子 (=)
 * splData(m_data)オブジェクトはコピーしない。
 */
sphSphHeader& sphSphHeader::operator = (const sphSphHeader& sph_header)
{

  register int i;
  for(i=0; i<3; i++) {
    m_wsize[i] = sph_header.m_wsize[i];
    m_start_idx[i] = sph_header.m_start_idx[i];
    m_size[i] = sph_header.m_size[i];
    m_origin[i] = sph_header.m_origin[i];
    m_pitch[i] = sph_header.m_pitch[i];
  }
  m_dims = sph_header.m_dims;
  m_vlen = sph_header.m_vlen;
  m_dtype = sph_header.m_dtype;
  m_gc = sph_header.m_gc;
  m_rlen = sph_header.m_rlen;
  m_crddef = sph_header.m_crddef;
  m_block_size = sph_header.m_block_size;
  m_step = sph_header.m_step;
  m_time = sph_header.m_time;
  m_gc_insph = sph_header.m_gc_insph;

  return *this;
}

/**
 * \~english
 * initialize member
 * @param      void
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  初期化を行う。
 *  メンバの初期化を行う。
 * @param      なし
 * @return    success(=true)/fault(=false)
 */
bool sphSphHeader::initialize()
{
  register int i;
  for(i=0; i<3; i++) {
    m_wsize[i] = 0;
    m_start_idx[i] = 0;
    m_size[i] = 0;
    m_origin[i] = 0.0;
    m_pitch[i] = 0.0;
  }
  m_dims = 3;
  m_vlen = 1;
  m_dtype = 1;
  m_gc = 0;
  m_rlen = 4;
  m_crddef = 1;
  m_step = 0;
  m_time = 0.0;
  m_block_size = 0;
  m_gc_insph = 0;

  return true;
}


/**
 * \~english
 * finalize
 * @param      void
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  破棄処理を行う。
 *  メンバの破棄を行う。
 * @param      void
 * @return    success(=true)/fault(=false)
 */
bool sphSphHeader::finalize()
{
  return true;
}


/**
 * \~english
 * get SPH Header member by splSpx
 * @param      sph_header    splSpx Object
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  SPHファイルのヘッダ情報をsplSpxオブジェクトに設定して取得する。
 * @param      spx_header    splSpxオブジェクト
 * @return    success(=true)/fault(=false)
 */
void sphSphHeader::getSplSpxHeader(splSpx* spx_header)
{
  register int i;

  for(i=0; i<3; i++) {
    spx_header->wsize[i] = m_wsize[i];
    spx_header->start_idx[i] = m_start_idx[i];
    spx_header->size[i] = m_size[i];

    // SPHファイル出力の場合、GCを出力する場合はGC分原点座標を移動する
    spx_header->origin[i] = m_origin[i] - m_gc*m_pitch[i];
    // データ定義点がコロケートの時に原点座標を変更する。
    if (m_crddef == (int)CRDDEF_COLLOCATE) {
      // 1/2ピッチずらす
      spx_header->origin[i] += m_pitch[i]/2.0;
    }
    spx_header->pitch[i] = m_pitch[i];
  }
  spx_header->dims = m_dims;
  spx_header->vlen = m_vlen;
  spx_header->dtype = m_dtype;
  spx_header->gc = m_gc;
  spx_header->rlen = m_rlen;
  spx_header->crddef = m_crddef;
  spx_header->aux = 0;
  spx_header->step = m_step;
  spx_header->time = m_time;
  spx_header->block_size = m_block_size;
  spx_header->gc_insph = m_gc_insph;

  return;
}


/**
 * \~english
 * set SPH Header member from splSpx object
 * @param      sph_header    splSpx Object
 * @return    success(=true)/fault(=false)
 *
 * \~japanese
 *  SPHファイルのヘッダ情報をsplSpxオブジェクトから設定する。
 * @param      spx_header    splSpxオブジェクト
 * @return    success(=true)/fault(=false)
 */
void sphSphHeader::setSplSpxHeader(const splSpx* spx_header)
{
  register int i;
  for(i=0; i<3; i++) {
    m_wsize[i] = spx_header->wsize[i];
    m_start_idx[i] = spx_header->start_idx[i];
    m_size[i] = spx_header->size[i];
    m_origin[i] = spx_header->origin[i];
    m_pitch[i] = spx_header->pitch[i];
  }
  m_dims = spx_header->dims;
  m_vlen = spx_header->vlen;
  m_dtype = spx_header->dtype;
  m_gc = spx_header->gc;
  m_rlen = spx_header->rlen;
  m_step = spx_header->step;
  m_time = spx_header->time;
  m_block_size = spx_header->block_size;
  m_gc_insph = spx_header->gc_insph;

  return;

}


/**
 * \~english
 *
 * \~japanese
 * ヘッダーデータのチェックを行う。
 * @param     void
 * @return      success(=true)/fault(=false)
 */
bool sphSphHeader::checkHeader() const
{
  register int i;

  // 次元数 (１次元=１, ２次元=2, ３次元=3)
  if (m_dims != 1 && m_dims != 2 && m_dims != 3) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_DIMS, "invalid dims[=%d], have to be 1, 2 or 3.", m_dims);
    return false;
  }
  // ベクトル長 (>=1)
  if (m_vlen < 1) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_PARAMETER, "invalid vlen[=%d], have to be over than 1.", m_vlen);
    return false;
  }
  // データ種別 (スカラーデータ=1, ベクトルデータ=3)
  if (m_dtype != 1 && m_dtype != 3) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_DTYPE, "invalid dtype[=%d], have to be 1 or 3.", m_dtype);
    return false;
  }
  // 実数データのサイズ (単精度浮動小数点数（float：4byte）=4, 倍精度浮動小数点数（double：8byte）=8)
  if (m_rlen != 4 && m_rlen != 8) {
    // error
    m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_RLEN, "invalid rlen[=%d], have to be 4 or 8.", m_rlen);    return false;
  }
  // サイズチェック
  for (i=0;i<m_dims;i++) {
    if (m_size[i] <= m_gc*2) {
      // error
      m_errHandler->SPH_ERROR_INFO(SPH_ERR_INVALID_ARRAY_SIZE, "size have to be more than gc*2, size[%d]:%ld <= gc[%d]:%ld * 2", i, m_size[i], i, m_gc);
      return false;
    }
  }

  return true;
}


/**
 * \~english
 *
 * \~japanese
 * 圧縮ブロックサイズをサイズから設定する.
 * １次元の場合：０
 * ２次元の場合：I方向サイズ
 * ３次元の場合：I方向サイズ x J方向サイズ
 * @param     void
 * @return  void
 */
void sphSphHeader::setBlockSizeBySize()
{
  size_t size = m_size[0];
  if (m_dims == 1) size = m_size[0];
  else if (m_dims == 2) size = m_size[0];
  else if (m_dims == 3) size = m_size[0]*m_size[1];

  m_block_size = size;

  return;
}


/**
 * \~english
 * set block size
 * @param     block_size               block size
 * @return  void

 * \~japanese
 * 圧縮ブロックサイズを設定する.
 * @param     block_size                圧縮ブロックサイズ
 * @return  void
 */
void sphSphHeader::setBlockSize(size_t block_size)
{
  m_block_size = block_size;
  return;
}


/**
 * \~english
 *
 * \~japanese
 * ヘッダー情報を出力する。
 * @param     void
 * @return  void
 */
void sphSphHeader::printHeaderInfo()
{
  printf("dimension\t\t\t: %d\n", m_dims);
  printf("vector length\t\t\t: %d\n", m_vlen);
  printf("data type\t\t\t: %d\n", m_dtype);
  printf("guide cell size\t\t\t: %ld\n", m_gc);
  printf("real data type\t\t\t: %d\n", m_rlen);
  printf("size[i,j,k]\t\t\t: %ld, %ld, %ld\n", m_size[0], m_size[1], m_size[2]);
  printf("origin[i,j,k]\t\t\t: %f, %f, %f\n", m_origin[0], m_origin[1], m_origin[2]);
  printf("pitch[i,j,k]\t\t\t: %f, %f, %f\n", m_pitch[0], m_pitch[1], m_pitch[2]);
  printf("step\t\t\t\t: %ld\n", m_step);
  printf("time\t\t\t\t: %f\n", m_time);
  printf("block size\t\t\t: %ld\n", m_block_size);
  printf("whole voxel size[i,j,k]\t\t: %ld, %ld, %ld\n", m_wsize[0], m_wsize[1], m_wsize[2]);
  printf("global start index[i,j,k]\t: %ld, %ld, %ld\n", m_start_idx[0], m_start_idx[1], m_start_idx[2]);
  printf("gc in sph\t\t\t: %ld\n", m_gc_insph);

  return;
}



/**
 * \~english
 * get whole voxel size
 * @param     void
 * @return  whole voxel size : size_t[3]
 *
 * \~japanese
 * 全体ボクセルサイズを取得する。
 * @param     void
 * @return  全体ボクセルサイズ : size_t[3]
 */
const size_t* sphSphHeader::getWholeSize()  const
{
  return m_wsize;
}

/**
 * \~english
 * get start index
 * @param     void
 * @return  start index : size_t[3]
 *
 * \~japanese
 * 始点インデックスを取得する。
 * @param     void
 * @return  始点インデックス : size_t[3]
 */
const size_t* sphSphHeader::getStartIdx()  const
{
  return m_start_idx;
}


/**
 * \~english
 * get dims
 * @param     void
 * @return  dims
 *
 * \~japanese
 * 次元数を取得する。
 * @param     void
 * @return  次元数
 */
unsigned int sphSphHeader::getDims()  const
{
  return m_dims;
}


/**
 * \~english
 * get cell data length in cell.
 * @param     void
 * @return    data length
 *
 * \~japanese
 * １セルのデータサイズを取得する.
 * SBXファイルの場合、vlen(=1) * dtype
 * SPXファイルの場合、vlen * rlen
 * SPHファイルの場合、vlen(1 or 3) * rlen
 * @param     void
 * @return  データサイズ
 */
unsigned int sphSphHeader::getCellDataLength() const
{
  return m_vlen*m_rlen;
}


/**
 * \~english
 * get total voxel data length.
 * @param     void
 * @return    data length in voxel
 *
 * \~japanese
 * ボクセルのデータサイズを取得する.
 * SBXファイルの場合、vlen(=1) * dtype * size[]
 * SPXファイルの場合、vlen * rlen * size[]
 * SPHファイルの場合、vlen(1 or 3) * rlen * size[]
 * @param     void
 * @return    ボクセルデータサイズ
 */
size_t sphSphHeader::getVoxelDataLength() const
{
  register int i;
  size_t len = 1;
  for (i=0; i<m_dims; i++) {
    len *= m_size[i];
  }
  len *= getCellDataLength();

  return len;
}


/**
 * \~english
 * get gc
 * @param     void
 * @return    gc
 *
 * \~japanese
 * Guide Cell数を取得する.
 * @param     void
 * @return    gc
 */
size_t sphSphHeader::getGc() const
{
  return m_gc;
}


/**
 * \~english
 * get voxel size
 * @param     void
 * @return  whole voxel size : size_t[3]
 *
 * \~japanese
 * ボクセルサイズを取得する。
 * @param     void
 * @return  ボクセルサイズ : size_t[3]
 */
const size_t* sphSphHeader::getSize() const
{
  return m_size;
}


/**
 * \~english
 * get origin coords
 * @param     void
 * @return    origin coords : double[3]
 *
 * \~japanese
 * 原点座標を取得する。
 * @param     void
 * @return    原点座標 : double[3]
 */
const double* sphSphHeader::getOrigin() const
{
  return m_origin;
}


/**
 * \~english
 * get pitch
 * @param     void
 * @return    pitch : double[3]
 *
 * \~japanese
 * ピッチを取得する。
 * @param     void
 * @return   ピッチ : double[3]
 */
const double* sphSphHeader::getPitch() const
{
  return m_pitch;
}


/**
 * \~english
 * get block size
 * @param     void
 * @return    block size
 *
 * \~japanese
 * ブロックサイズを取得する。
 * @param     void
 * @return    ブロックサイズ
 */
size_t sphSphHeader::getBlockSize() const
{
  return m_block_size;
}


/**
 * \~english
 * set size
 * @param     size      size
 * @return    void
 *
 * \~japanese
 * サイズを設定する。
 * @param     size             サイズ
 * @return    void
 */
void sphSphHeader::setSize(const size_t size[3])
{
  m_size[0] = size[0];
  m_size[1] = size[1];
  m_size[2] = size[2];
}


/**
 * \~english
 * get header format
 * @param     void
 * @return    header format
 *
 * \~japanese
 * ヘッダーフォーマットを取得する。
 * @param     void
 * @return    ヘッダーフォーマット文字列
 */
std::string sphSphHeader::getFormat()
{
  return std::string(HEADER_FORMAT);
}


/**
 * \~english
 * get vector length
 * @param     void
 * @return    vector length
 *
 * \~japanese
 * ベクトル長を取得する。
 * @param     void
 * @return    ベクトル長
 */
unsigned int sphSphHeader::getVlen()  const
{
  return m_vlen;
}


/**
 * \~english
 * get data type.
 * data type ( 1:scaler data 3:vector data )
 * @param     void
 * @return    data type ( 1:scaler data 3:vector data )
 *
 * \~japanese
 * dtypeを取得する.
 * data type ( 1:scaler data 3:vector data )
 * @param     void
 * @return    data type ( 1:scaler data 3:vector data )
 */
unsigned int sphSphHeader::getDtype()  const
{
  return m_dtype;
}


/**
 * \~english
 * get rlen.
 * real length ( 4:float, 8:double )
 * @param     void
 * @return    data type ( 1:unsigned char, 2:unsigned short, 4:unsigned int )
 *
 * \~japanese
 * rlenを取得する.
 * real length ( 4:float, 8:double )
 * @param     void
 * @return    real length ( 4:float, 8:double )
 */
unsigned int sphSphHeader::getRlen()  const
{
  return m_rlen;
}



/**
 * \~english
 * get start_idx.
 * @param     start_idx                 start_idx
 * @return    void
 *
 * \~japanese
 * start_idxを取得する.
 * @param     start_idx                 start_idx
 * @return    void
 */
void sphSphHeader::setStartIdx(const size_t start_idx[3])
{
  m_start_idx[0] = start_idx[0];
  m_start_idx[1] = start_idx[1];
  m_start_idx[2] = start_idx[2];

}


/**
 * Guide Cell数を設定する.
 * @param     gc        ガイドセル
 * @return    void
 */
void sphSphHeader::setGc(size_t gc)
{
  m_gc = gc;
}


/**
 * \~english
 * get coords definition position
 * @param     void
 * @return    coords definition
 *
 * \~japanese
 * 定義位置を取得する。
 * @param     void
 * @return    定義位置(1,2,3,4)
 */
unsigned int sphSphHeader::getCrddef() const
{
  return m_crddef;
}


/**
 * 次元数を設定する。
 * @param     dims      次元数
 * @return  void
 */
void sphSphHeader::setDims(int dims)
{
  m_dims = dims;
}


/**
 * 全体ボクセルサイズを設定する。
 * @param     wsize             全体ボクセルサイズ
 * @return    void
 */
void sphSphHeader::setWholeSize(const size_t wsize[3])
{
  m_wsize[0] = wsize[0];
  m_wsize[1] = wsize[1];
  m_wsize[2] = wsize[2];
}


/**
 * ベクトル長を設定する。
 * @param     vlen             ベクトル長
 * @return    void
 */
void sphSphHeader::setVlen(unsigned int vlen)
{
  m_vlen = vlen;
}


/**
 * データタイプを設定する。
 * @param     dtype             データタイプ
 * @return    void
 */
void sphSphHeader::setDtype(unsigned int dtype)
{
  m_dtype = dtype;
}


/**
 * 実数タイプを設定する。
 * @param     rlen             実数タイプ
 * @return    void
 */
void sphSphHeader::setRlen(unsigned int rlen)
{
  m_rlen = rlen;
}


/**
 * 実行ステップを取得する。
 * @param     void
 * @return    実行ステップ数
 */
size_t sphSphHeader::getStep() const
{
  return m_step;
}


/**
 * 実行時間を取得する。
 * @param     void
 * @return    実行時間
 */
double sphSphHeader::getTime() const
{
  return m_time;
}


/**
 * 実行ステップ、時間を設定する。
 * @param     step      実行ステップ
 * @param     time      実行時間
 * @return    void
 */
void sphSphHeader::setStepTime(size_t step, double time)
{
  m_step = step;
  m_time = time;
}


/**
 * 原点座標を設定する。
 * @param     org[3]      原点座標
 * @return    void
 */
void sphSphHeader::setOrigin(const double org[3])
{
  m_origin[0] = org[0];
  m_origin[1] = org[1];
  m_origin[2] = org[2];
}


/**
 * ピッチを設定する。
 * @param     pitch[3]      ピッチ
 * @return    void
 */
void sphSphHeader::setPitch(const double pitch[3])
{

  m_pitch[0] = pitch[0];
  m_pitch[1] = pitch[1];
  m_pitch[2] = pitch[2];
}


/**
 * 定義位置を設定する。
 * @param     crddef          定義位置
 * @return    void
 */
void sphSphHeader::setCrddef(unsigned int crddef)
{
  m_crddef = crddef;
}




