/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphSvHeader.C
 * Sphere File DataSet SuperClass
 * @author     hira
 * @date       2009/09/07
 *
 * \~japanese
 * @file sphSvHeader.C
 * Sphereファイルデータセット基底クラス
 * @author     hira
 * @date       2009/09/07
 */

#include "fileio/sphSvHeader.h"


/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 */
sphSvHeader::sphSvHeader()
{
}


/**
 * \~english
 * Destructor
 *
 * \~japanese
 * デストラクタ
 */
sphSvHeader::~sphSvHeader()
{
}


/**
 * 全体ボクセルサイズを取得する。(not support)
 * @param     void
 * @return  全体ボクセルサイズ : size_t[3]
 */
const size_t* sphSvHeader::getWholeSize()  const
{
  return NULL;
}


/**
 * 始点インデックスを取得する。(not support)
 * @param     void
 * @return  始点インデックス : size_t[3]
 */
const size_t* sphSvHeader::getStartIdx()  const
{
  return NULL;
}

/**
 * 全体ボクセルサイズを設定する。(SPHファイルの場合、wsizeは存在しない。)
 * @param     wsize             全体ボクセルサイズ
 * @return    void
 */
void sphSvHeader::setWholeSize(const size_t wsize[3])
{
  return;
}


/**
 * start_idxを取得する.(SPHファイルの場合、start_idxは存在しない。)
 * @param     start_idx                 start_idx
 * @return    void
 */
void sphSvHeader::setStartIdx(const size_t start_idx[3])
{
  return;
}


/**
 * 定義位置を取得する。(SPHファイルの場合、crddefは存在しない。)
 * @param     void
 * @return    定義位置(1,2,3,4)
 */
unsigned int sphSvHeader::getCrddef() const
{
  // error
//  m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "sphSvHeader::getCrddef();");
  return 0;
}


/**
 * AUXを取得する。(SPHファイルの場合、auxは存在しない。)
 * @param     void
 * @return    aux
 */
unsigned int sphSvHeader::getAux() const
{
  // error
  //  m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "sphSvHeader::getAux();");
  return 0;
}

/**
 * 定義位置を設定する。(SPHファイルの場合、crddefは存在しない。)
 * @param     crddef          定義位置
 * @return    void
 */
void sphSvHeader::setCrddef(unsigned int crddef)
{
  // error
  // m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "sphSvHeader::setCrddef();");
}


/**
 * AUXを設定する。(SPHファイルの場合、crddefは存在しない。)
 * @param     aux          AUX
 * @return    void
 */
void sphSvHeader::setAux(unsigned int aux)
{
  // error
  // m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "sphSvHeader::setAux();");
}


/**
 * AUXレコード(拡張仕様)の占有ビット数、データ存在情報を取得する。(SPHファイルの場合、auxは存在しない。)
 * @param     bits    占有ビット数（上位ビット数）
 * @param    dataflag  データ存在情報
 * @return  void
 */
void sphSvHeader::getDataExistFlags(unsigned int& bits, unsigned int& dataflag)
{

  bits = 0;
  dataflag = 0;

  // error
  // m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "sphSvHeader::getDataExistFlags();");

  return;
}


/**
 * AUXレコード(拡張仕様)の占有ビット数、データ存在情報を設定する。(SPHファイルの場合、auxは存在しない。)
 * @param     bits    占有ビット数（上位ビット数）
 * @param    dataflag  データ存在情報
 * @return  void
 */
void sphSvHeader::setDataExistFlags(unsigned int bits, unsigned int dataflag)
{

  // error
  // m_errHandler->SPH_ERROR_INFO(SPH_ERR_NOT_SUPPORT_METHOD, "sphSvHeader::setDataExistFlags();");

  return;
}





