/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphUnstMesh.C
 * Unstructured Mesh Information Class Source File
 * @author     tamaki
 * @date       2010/04/22
 *
 * \~japanese
 * @file sphUnstMesh.C
 * 非構造格子情報クラスソースファイル
 * @author     tamaki
 * @date       2010/04/22
 */

#include "parallel/sphUnstMesh.h"

/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 */
sphUnstMesh::sphUnstMesh() {
  m_meshDtType = INVALID_MESH_DATA_TYPE;
  m_nPE = 0;
  m_myRank = -1;
  m_comm = SPL_COMM_NULL;
  m_sRequest = NULL;
  m_rRequest = NULL;
  m_sStatus = NULL;
  m_rStatus = NULL;
  m_PElist = NULL;
  m_commTable = NULL;
  m_commList = NULL;
  nNode = 0;
  nnNode = 0;
  node = NULL;
  nInner = 0;
  nOverlapped = 0;
  nElem = 0;
  elem = NULL;
  nop = NULL;
  nNgrp = 0;
  NgrpName = NULL;
  NgrpIndex = NULL;
  Ngrp = NULL;
  nEgrp = 0;
  EgrpName = NULL;
  EgrpIndex = NULL;
  Egrp = NULL;
  nSgrp = 0;
  SgrpName = NULL;
  SgrpIndex = NULL;
  Sgrp = NULL;
}

/**
 * \~english
 * Destoructor
 *
 * \~japanese
 * デストラクタ
 */
sphUnstMesh::~sphUnstMesh() {
  if( m_sRequest ) delete [] m_sRequest;
  if( m_rRequest ) delete [] m_rRequest;
  if( m_sStatus ) delete [] m_sStatus;
  if( m_rStatus ) delete [] m_rStatus;
  if( m_PElist ) delete [] m_PElist;
  if( m_commTable ) delete [] m_commTable;
  if( m_commList ) {
    for(int i=0; i<m_nPE; i++) if( m_commList[i] ) delete [] m_commList[i];
    delete [] m_commList;
  }
  if( node )      delete [] node;
  if( elem )      delete [] elem;
  if( nop )       delete [] nop;
  if( NgrpIndex ) delete [] NgrpIndex;
  if( Ngrp )      delete [] Ngrp;
  if( EgrpIndex ) delete [] EgrpIndex;
  if( Egrp )      delete [] Egrp;
  if( SgrpIndex ) delete [] SgrpIndex;
  if( Sgrp )      delete [] Sgrp;
  if( NgrpName ) {
    for(int i=0; i<nNgrp; i++) delete [] NgrpName[i];
    delete [] NgrpName;
  }
  if( EgrpName ) {
    for(int i=0; i<nEgrp; i++) delete [] EgrpName[i];
    delete [] EgrpName;
  }
  if( SgrpName ) {
    for(int i=0; i<nSgrp; i++) delete [] SgrpName[i];
    delete [] SgrpName;
  }
}


bool
sphUnstMesh::readElement(const char* fn) {
  FILE *fp;
  int i;
  /* ******** Open node file ******** */
  if( !fn ) return false;
  if( (fp = fopen(fn, "r")) == NULL ) return false;

  /* ******** Input variables ******** */
  fscanf(fp,"%d",&nInner);
  nElem = nInner;
  /* ----------- Memory allocation ------------- */
  if( !(nop = new int[nElem*8]) ) return false;
  memset(nop, 0, sizeof(int)*nElem*8);
  for ( i = 0; i < nInner; i++ ){
    fscanf(fp,"%d %d %d %d %d %d %d %d",
           &nop[8*i  ],&nop[8*i+1],&nop[8*i+2],&nop[8*i+3],
           &nop[8*i+4],&nop[8*i+5],&nop[8*i+6],&nop[8*i+7]);
  }
  fclose(fp);
  return true;
}

#define NAME_LEN 256
bool
sphUnstMesh::readGroup(const char* fn) {
  FILE *fp;
  int   i,j;
  int   n, index;
  int **list;

  /* ******** Open node file ******** */
  if( !fn ) return false;
  if( (fp = fopen(fn, "r")) == NULL ) return false;

  /* ******** Input variables ******** */
  /* <<<<<<<< Node groups >>>>>>>> */
  fscanf(fp,"%d",&nNgrp);

  /* ------------ For node groups ------------ */
  NgrpName = new char*[nNgrp];
  memset(NgrpName, 0, sizeof(char*)*nNgrp);
  for(i=0; i<nNgrp; i++) {
    NgrpName[i] = new char[NAME_LEN];
    NgrpName[i][0] = '\0';
  }
  NgrpIndex = new int[nNgrp+1];
  memset(NgrpIndex, 0, sizeof(int)*nNgrp+1);

  list = new int*[nNgrp];
  memset(list, 0, sizeof(int*)*nNgrp);
  if (!list){
    fprintf(stderr,"Error: Fail to allocate temporary array.\n");
    abort();
  }

  /* ******** Input group data into temp array ******** */
  for ( i = 0; i < nNgrp; i++ ){ /* Group loop */
    fscanf(fp,"%s %d",NgrpName[i],&NgrpIndex[i+1]);
    list[i] = new int[NgrpIndex[i+1]];
    memset(list[i], 0, sizeof(int)*NgrpIndex[i+1]);
    for ( j = 0; j < NgrpIndex[i+1]; j++ ){ /* Node loop in the group */
      fscanf(fp,"%d",&list[i][j]);
    }
  }

  /* ******** Reconstruct group data ******** */
  for ( i = 0; i < nNgrp; i++ ) NgrpIndex[i+1] += NgrpIndex[i];
  
  Ngrp = new int[NgrpIndex[nNgrp]];
  memset(Ngrp, 0, sizeof(int)*NgrpIndex[nNgrp]);
  for ( i = 0; i < nNgrp; i++ ){
    n     = NgrpIndex[i+1] - NgrpIndex[i];
    index = NgrpIndex[i];
    memcpy( (Ngrp+index),list[i],n*sizeof(int) );
    free(list[i]);
  }

  delete [] list;
  fclose(fp);
  return true;
}
#undef NAME_LEN

bool
sphUnstMesh::readBoundary(const char* fn) {
  return true;
}

/**
 * \~english
 * Create unstructured-Mesh from file
 * @param      fn_node      Node file
 * @param      fn_elem      Element file
 * @param      fn_grp       Group file
 * @return     success(=true)/fault(=false)
 *
 * \~japanese
 * ファイルから非構造格子情報を構築する。
 * @param      fn_node      節点情報ファイル
 * @param      fn_elem      要素情報ファイル
 * @param      fn_grp       グループ情報ファイル
 * @return     success(=true)/fault(=false)
 */
bool
sphUnstMesh::readMesh(const char* fn_node,
                          const char* fn_elem,
                          const char* fn_grp,
                          const char* fn_bnd  ) {
  if( !readNode(fn_node) ) return false;
  if( !readElement(fn_elem) ) return false;
  if( !readGroup(fn_grp) ) return false;
  if( !readBoundary(fn_bnd) ) return false;
  return true;
}

