/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgBase.C
 * コンフィグレーション基底クラスソースファイル
 * @author     riken
 * @date       2010/10/01
 */

#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "skl_config/SklCfgBase.h"

namespace SklCfg {

/**
 * コンストラクタ
 */
SklCfgBase::SklCfgBase()
{
}

/**
 * デストラクタ
 */
SklCfgBase::~SklCfgBase()
{
}

/**
 * 文字列比較を行う。
 * @param s1    比較文字列１
 * @param s2    比較文字列２
 * @param n     比較文字数
 * @return      s1 > s2 で正の値、s1 < s2 で負の値、s1 = s2で 0 を返す。
 */
int SklCfgBase::StrCompare(const char *s1, const char *s2, size_t n) const
{
  if( !s1 || !s2 ) return -1;
  if( n == 0 ) return strcmp(s1, s2);
  else return strncmp(s1, s2, n);
}

/**
 * 大文字小文字を無視した文字列を比較する
 * @param s1    比較文字列１
 * @param s2    比較文字列２
 * @param n     比較文字数
 * @return      s1 > s2 で正の値、s1 < s2 で負の値、s1 = s2で 0 を返す。
 */
int SklCfgBase::StrCaseCompare(const char *s1, const char *s2, size_t n) const
{
  if( !s1 || !s2 ) return -1;
  if( n == 0 ) return strcasecmp(s1, s2);
  else return strncasecmp(s1, s2, n);
}

/**
 * メッセージを標準出力に出力する。
 * @param format    出力フォーマット
 * @param ...       出力メッセージ
 */
void SklCfgBase::CfgMsg(char* format, ...) const
{
  if( !format ) return;
  char buf[1024];
  va_list vl;
  va_start(vl, format);
  vsprintf(buf, format, vl);
  va_end(vl);
  fprintf(stdout, "%s\n", buf);
  return;
}

/**
 * エラーメッセージを標準エラー出力に出力する。
 * @param format    出力フォーマット
 * @param ...       出力メッセージ
 */
void SklCfgBase::CfgErrMsg(char* format, ...) const
{
  if( !format ) return;
  char buf[1024];
  va_list vl;
  va_start(vl, format);
  vsprintf(buf, format, vl);
  va_end(vl);
  fprintf(stderr, "ERROR :: %s\n", buf);
  return;
}

/**
 * 警告メッセージを標準エラー出力に出力する。
 * @param format    出力フォーマット
 * @param ...       出力メッセージ
 */
void SklCfgBase::CfgWarnMsg(char* format, ...) const
{
  if( !format ) return;
  char buf[1024];
  va_list vl;
  va_start(vl, format);
  vsprintf(buf, format, vl);
  va_end(vl);
  fprintf(stderr, "WARNING :: %s\n", buf);
  return;
}

/**
 * デバッグメッセージを標準出力に出力する。
 * @param format    出力フォーマット
 * @param ...       出力メッセージ
 */
void SklCfgBase::CfgDbgMsg(char* format, ...) const
{
#ifdef DEBUG
  if( !format ) return;
  char buf[1024];
  va_list vl;
  va_start(vl, format);
  vsprintf(buf, format, vl);
  va_end(vl);
  fprintf(stderr, "DEBUG :: %s\n", buf);
#endif //DEBUG
  return;
}

} // end of namespace SklCfg

