/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgOutFile.C
 * コンフィグレーションOutFile要素クラスソースファイル
 * @author     riken
 * @date       2010/10/01
 */

#include "skl_config/SklCfgDefName.h"
#include "skl_config/SklCfgOutFile.h"
#include "skl_config/vfvPathUtil.h"

namespace SklCfg {
/**
 * コンストラクタ
 */
SklCfgOutFile::SklCfgOutFile()
{
  m_interval = 0;
  m_attr = NULL;
  m_format = NULL;
  m_baseName = NULL;
  // Sphere用 delete at 2010/12/07 by @hira
//  m_multiOut = false;
  // Sphere用 add at 2010/12/07 by @hira
  m_multimode = CFG_FILE_DEFAULT;   // デフォルト
}

/**
 * デストラクタ
 */
SklCfgOutFile::~SklCfgOutFile()
{
  if( m_attr ) delete [] m_attr;
  if( m_format ) delete [] m_format;
  if( m_baseName ) delete [] m_baseName;

}

bool SklCfgOutFile::CheckNodeName(const char* nname)
{
  if( !nname ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0001_);
    return false;
  }

  if( StrCaseCompare(nname,
                     _SKL_CFG_NNAME_STEER_OUT_FILE_,
                     strlen(_SKL_CFG_NNAME_STEER_OUT_FILE_)) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0002_, nname);
    return false;
  }

  return true;
}

bool SklCfgOutFile::ParseNode(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  if( !CheckNodeName((const char*)node->name) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  if( !ParseSklDefNode(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  if( !Verification() ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  return true;
}

bool SklCfgOutFile::ParseSklDefNode(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  if( !CheckNodeName((const char*)node->name) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  if( !ParseInterval(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }
  if( !ParseAttr(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }
  if( !ParseFormat(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }
  if( !ParseBaseName(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }
  if( !ParseMultiOutput(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  return true;
}

/**
 * XML要素から出力インターバルをパースする。
 * @param node    XML要素
 * @return      true=success, false=fault
 */
bool SklCfgOutFile::ParseInterval(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  xmlChar* tmpStr;
  tmpStr = xmlGetProp(node, (const xmlChar*)_SKL_CFG_ANAME_STEER_INTERVAL_);
  if( !tmpStr ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0004_,
              _SKL_CFG_NNAME_STEER_OUT_FILE_,
              _SKL_CFG_ANAME_STEER_INTERVAL_);
    return false;
  }
  m_interval = (unsigned int)atoi((const char*)tmpStr);
  xmlFree(tmpStr);

  return true;
}

/**
 * XML要素からファイル属性をパースする。
 * @param node    XML要素
 * @return      true=success, false=fault
 */
bool SklCfgOutFile::ParseAttr(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  xmlChar* tmpStr;
  tmpStr = xmlGetProp(node, (const xmlChar*)_SKL_CFG_ANAME_STEER_ATTR_);
  if( !tmpStr ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0004_,
              _SKL_CFG_NNAME_STEER_OUT_FILE_,
              _SKL_CFG_ANAME_STEER_ATTR_);
    return false;
  }
  if( m_attr ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0013_);
    return false;
  }
  size_t len = strlen((const char*)tmpStr);
  if( !(m_attr = new char[len + 1]) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0005_);
    return false;
  }
  memset(m_attr, 0, sizeof(char)*(len + 1));
  strncpy(m_attr, (const char*)tmpStr, len+1);
  xmlFree(tmpStr);

  return true;
}


/**
 * XML要素からファイルフォーマットをパースする。
 * @param node    XML要素
 * @return      true=success, false=fault
 */
bool SklCfgOutFile::ParseFormat(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  xmlChar* tmpStr;
  tmpStr = xmlGetProp(node, (const xmlChar*)_SKL_CFG_ANAME_STEER_FORMAT_);
  if( !tmpStr ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0004_,
              _SKL_CFG_NNAME_STEER_OUT_FILE_,
              _SKL_CFG_ANAME_STEER_FORMAT_);
    return false;
  }
  if( m_format ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0014_);
    return false;
  }
  size_t len = strlen((const char*)tmpStr);
  if( !(m_format = new char[len + 1]) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0005_);
    return false;
  }
  memset(m_format, 0, sizeof(char)*(len + 1));
  strncpy(m_format, (const char*)tmpStr, len+1);
  xmlFree(tmpStr);

  return true;
}

/**
 * XML要素からファイルベース名をパースする。
 * @param node    XML要素
 * @return      true=success, false=fault
 */
bool SklCfgOutFile::ParseBaseName(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  xmlChar* tmpStr;
  tmpStr = xmlGetProp(node, (const xmlChar*)_SKL_CFG_ANAME_STEER_BASE_NAME);
  if( !tmpStr ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0004_,
              _SKL_CFG_NNAME_STEER_OUT_FILE_,
              _SKL_CFG_ANAME_STEER_BASE_NAME);
    return false;
  }
  if( m_baseName ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0015_);
    return false;
  }
  size_t len = strlen((const char*)tmpStr);
  if( !(m_baseName = new char[len + 1]) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0005_);
    return false;
  }
  memset(m_baseName, 0, sizeof(char)*(len + 1));
  strncpy(m_baseName, (const char*)tmpStr, len+1);
  xmlFree(tmpStr);

  return true;
}

/**
 * XML要素から分散出力フラグをパースする。
 * @param node    XML要素
 * @return      true=success, false=fault
 */
bool SklCfgOutFile::ParseMultiOutput(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  xmlChar* tmpStr;
  tmpStr = xmlGetProp(node, (const xmlChar*)_SKL_CFG_ANAME_STEER_MULTI_IO_);
  if( tmpStr ){
    // Sphere用 modyfy at 2010/12/07 by @hira
    // if( !strcasecmp((char*)tmpStr, "yes") ) m_multiOut = true;
    if( !strcasecmp((char*)tmpStr, "yes") ) m_multimode = CFG_FILE_MULTI;
    else if( !strcasecmp((char*)tmpStr, "no") ) m_multimode = CFG_FILE_WHOLE;
    else if( !strcasecmp((char*)tmpStr, "process") ) m_multimode = CFG_FILE_PROCESS;
    else m_multimode = CFG_FILE_UNKNOWN;
  }

  xmlFree(tmpStr);

  return true;
}

/**
 * ファイル名にディレクトリパスを付加する。
 * @param basedir    ディレクトリパス
 * @return      true=success, false=fault
 */
bool SklCfgOutFile::CreateFileName(const char* basedir)
{
  if( !m_baseName ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0033_);
    return false;
  }
  if( !basedir ){
    string strbasename(m_baseName);
    strbasename = VFV::vfvPath_normalize(strbasename);
    char* tmpbasename = new char[strbasename.size() + 1];
    if( !tmpbasename ){
      CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
      CfgErrMsg(_SKL_CFG_ERR_0005_);
      return false;
    }
    strcpy(tmpbasename, strbasename.c_str());
    delete [] m_baseName;
    m_baseName = tmpbasename;
    return true;
  }

  string strbasename(m_baseName);
  string strbasedir(basedir);
  string path;
#ifndef WIN32
  path.clear();
#endif //WIN32

  if( !VFV::vfvPath_isAbsolute(strbasename) ){
    path = strbasedir;
    path += VFV::vfvPath_getDelimChar();
    path += strbasename;
  }
  else path = strbasename;
  path = VFV::vfvPath_normalize(path);

  char* tmpbasename = new char[path.size() + 1];
  if( !tmpbasename ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0005_);
    return false;
  }
  strcpy(tmpbasename, path.c_str());
  delete [] m_baseName;
  m_baseName = tmpbasename;

  return true;
}

bool SklCfgOutFile::Verification()
{
  if( !m_baseName ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0006_, "OutFile Base Name");
    return false;
  }
  if( m_interval == 0 ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0006_, "OutFile Interval is zero");
    return false;
  }
  if( !m_attr ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0006_, "OutFile Attribute");
    return false;
  }
  if( !m_format ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0006_, "OutFile Format");
    return false;
  }

  return true;
}

// Sphere用 delete start at 2010/12/07 by @hira
/**
 * 分散入力フラグを"true"に設定する。

void SklCfgOutFile::SetMultiOutput()
{
  m_multiOut = true;
}*/

/**
 * 分散入力フラグを"false"に設定する。

void SklCfgOutFile::UnsetMultiOutput()
{
  m_multiOut = false;
}*/
// Sphere用 delete end at 2010/12/07 by @hira


/**
 * 出力インターバルを設定する。
 * @param val   出力インターバル
 */
void SklCfgOutFile::SetInterval(unsigned val)
{
  m_interval = val;
}

/**
 * 出力インターバルを取得する。
 * @return    出力インターバル
 */
unsigned int SklCfgOutFile::GetInterval() const
{
  return m_interval;
}

/**
 * ファイル属性を取得する。
 * @return    ファイル属性
 */
const char* SklCfgOutFile::GetAttr() const
{
  return (const char*)m_attr;
}

/**
 * ファイルフォーマットを取得する。
 * @return    ファイルフォーマット
 */
const char* SklCfgOutFile::GetFormat() const
{
  return (const char*)m_format;
}

/**
 * ファイルベース名を取得する。
 * @return    ファイルベース名
 */
const char* SklCfgOutFile::GetBaseName() const
{
  return (const char*)m_baseName;
}


// Sphere用 delete at 2010/12/07 by @hira
/**
 * 分散出力フラグを取得する。
 *
 * V-Sphere互換の為に定義する。
 * （推奨) GetMultiMode()を使用すること。
 * @return      分散出力フラグ
 */
bool SklCfgOutFile::IsMultiOutput() const
{
  return (m_multimode == CFG_FILE_MULTI);
}


/**
 * multi属性を取得する。
 * @return    multi属性モード
 *             CFG_FILE_MULTI = 1       : 分割(サブブロック)ファイル入出力
 *             CFG_FILE_PROCESS = 2     : プロセスファイル入出力
 *             CFG_FILE_WHOLE = 3       : 一括ファイル入出力
 */
SphCfgFileIOMode SklCfgOutFile::GetMultiMode() const
{
  return m_multimode;
}

/**
 * multi属性を設定する。
 *
 * @param     multi    multi属性モード
 *             CFG_FILE_MULTI = 1       : 分割(サブブロック)ファイル入出力
 *             CFG_FILE_PROCESS = 2     : プロセスファイル入出力
 *             CFG_FILE_WHOLE = 3       : 一括ファイル入出力
 */
void SklCfgOutFile::SetMultiMode(SphCfgFileIOMode multi)
{
  m_multimode = multi;
}


/**
 * multi属性をCFG_FILE_MULTI(分散入出力)に設定する。
 *
 * V-Sphere互換の為に定義する。
 * （推奨) SetMultiMode(SphCfgFileIOMode multi)を使用すること。
 */
void SklCfgOutFile::SetMultiOutput()
{
  SetMultiMode(CFG_FILE_MULTI);
}

/**
 * multi属性をCFG_FILE_WHOLE(一括ファイル入出力)に設定する。
 *
 * V-Sphere互換の為に定義する。
 * （推奨) SetMultiMode(SphCfgFileIOMode multi)を使用すること。
 */
void SklCfgOutFile::UnsetMultiOutput()
{
  SetMultiMode(CFG_FILE_WHOLE);
}


#ifdef DEBUG
void SklCfgOutFile::Print()
{
  cout << "   <-- OutFile -- : " << flush;

  if( m_interval != 0 ){
    cout << "(interval)" << m_interval << ", " << flush;
  }else{
    cout << "(interval) not set, " << flush;
  }
  if( m_attr ){
    cout << "(attr)" << m_attr << ", " << flush;
  }else{
    cout << "(attr) not set, " << flush;
  }
  if( m_format ){
    cout << "(format)" << m_format << ", " << flush;
  }else{
    cout << "(format) not set, " << flush;
  }
  if( m_baseName ){
    cout << "(basename)" << m_baseName << ", " << flush;
  }else{
    cout << "(basename) not set, " << flush;
  }
  cout << "-->" << endl << flush;
}
#endif //DEBUG

} // end of namespace SklCfg

