/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgSteerOutputData.C
 * コンフィグレーションOutputData要素クラスソースファイル
 * @author     riken
 * @date       2010/10/01
 */

#include <string.h> // add 20090629
#include "skl_config/SklCfgDefName.h"
#include "skl_config/SklCfgSteerOutputData.h"

namespace SklCfg {
/**
 * コンストラクタ
 */
SklCfgSteerOutputData::SklCfgSteerOutputData()
{
  m_baseDir = NULL;
  m_monitorInterval = 1;
  m_monIntFlg = false;
  m_historyList.clear();
  m_outFileList.clear();
}

/**
 * デストラクタ
 */
SklCfgSteerOutputData::~SklCfgSteerOutputData()
{
  if( m_baseDir ) delete [] m_baseDir;

  if( !m_historyList.empty() ){
    list<SklCfgHistory*>::iterator itr = m_historyList.begin();
    list<SklCfgHistory*>::iterator itrEnd = m_historyList.end();
    for( ; itr != itrEnd ; itr++ ) delete *itr;
  }
  m_historyList.clear();

  if( !m_outFileList.empty() ){
    list<SklCfgOutFile*>::iterator itr = m_outFileList.begin();
    list<SklCfgOutFile*>::iterator itrEnd = m_outFileList.end();
    for( ; itr != itrEnd ; itr++ ) delete *itr;
  }
  m_outFileList.clear();

}

bool SklCfgSteerOutputData::CheckNodeName(const char* nname)
{
  if( !nname ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0001_);
    return false;
  }

  if( StrCaseCompare(nname,
                     _SKL_CFG_NNAME_STEER_OUTPUT_DATA_,
                     strlen(_SKL_CFG_NNAME_STEER_OUTPUT_DATA_)) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0002_, nname);
    return false;
  }

  return true;
}

bool SklCfgSteerOutputData::ParseNode(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  if( !CheckNodeName((const char*)node->name) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  if( !ParseSklDefNode(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  if( !Verification() ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  return true;
}

bool SklCfgSteerOutputData::ParseSklDefNode(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  if( !CheckNodeName((const char*)node->name) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  xmlChar* tmpStr;
  tmpStr = xmlGetProp(node, (const xmlChar*)_SKL_CFG_ANAME_STEER_BASE_DIR_);
  if( tmpStr ){
    if( m_baseDir ){
      CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
      CfgErrMsg(_SKL_CFG_ERR_0016_);
      return false;
    }
    size_t len = strlen((const char*)tmpStr);
    if( !(m_baseDir = new char[len + 1]) ){
      CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
      CfgErrMsg(_SKL_CFG_ERR_0005_);
      return false;
    }
    memset(m_baseDir, 0, sizeof(char)*(len + 1));
    strncpy(m_baseDir, (const char*)tmpStr, len+1);
    xmlFree(tmpStr);
  }
/*
  else{
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0004_,
              _SKL_CFG_NNAME_STEER_OUTPUT_DATA_,
              _SKL_CFG_ANAME_STEER_BASE_DIR_);
    return false;
  }
*/

  xmlNodePtr cur = node->xmlChildrenNode;
  while ( cur ) {
    if ( cur->type == XML_COMMENT_NODE ) {
      cur = cur->next;
      continue;
    }

    if( !StrCaseCompare((const char*)cur->name,
                        _SKL_CFG_NNAME_STEER_OUT_FILE_,
                        strlen(_SKL_CFG_NNAME_STEER_OUT_FILE_)) ){
      SklCfgOutFile* outfile = new SklCfgOutFile();
      if( !outfile ){
        CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
        CfgErrMsg(_SKL_CFG_ERR_0005_);
        return false;
      }
      if( !outfile->ParseNode(cur) ){
        CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
        delete outfile;
        return false;
      }
      if( !outfile->CreateFileName(m_baseDir) ){
        CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
        return false;
      }
      m_outFileList.push_back(outfile);
    }
    else if( !StrCaseCompare((const char*)cur->name,
                             _SKL_CFG_NNAME_STEER_MONITOR_,
                             strlen(_SKL_CFG_NNAME_STEER_MONITOR_)) ){
      if( !ParseMonitor(cur) ){
        CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
        return false;
      }
    }
    else if( !StrCaseCompare((const char*)cur->name,
                             _SKL_CFG_NNAME_STEER_HISTORY_,
                             strlen(_SKL_CFG_NNAME_STEER_HISTORY_)) ){
      SklCfgHistory* history = new SklCfgHistory();
      if( !history ){
        CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
        CfgErrMsg(_SKL_CFG_ERR_0005_);
        return false;
      }
      if( !history->ParseNode(cur) ){
        CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
        delete history;
        return false;
      }
      if( !history->CreateFileName(m_baseDir) ){
        CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
        return false;
      }
      m_historyList.push_back(history);
    }

    cur = cur->next;
  }

  return true;
}

/**
 * XML要素から出力インターバルをパースする。
 * @param node    XML要素
 * @return      true=success, false=fault
 */
bool SklCfgSteerOutputData::ParseMonitor(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  if( StrCaseCompare((const char*)node->name,
                     _SKL_CFG_NNAME_STEER_MONITOR_,
                     strlen(_SKL_CFG_NNAME_STEER_MONITOR_)) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0002_, (const char*)node->name);
    return false;
  }

  xmlChar* tmpStr = NULL;
  if( !(tmpStr = xmlGetProp(node,
                    (const xmlChar*)_SKL_CFG_ANAME_STEER_INTERVAL_)) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0004_,
              _SKL_CFG_NNAME_STEER_OUTPUT_DATA_,
              _SKL_CFG_ANAME_STEER_INTERVAL_);
    return false;
  }
  if( m_monIntFlg ){
    CfgWarnMsg("element \"%s\" is redefined. \"%s=%d\" is disregarded.",
               _SKL_CFG_NNAME_STEER_MONITOR_,
               _SKL_CFG_NNAME_STEER_MONITOR_,
               atoi((const char*)tmpStr));
    xmlFree(tmpStr);
    return true;
  }

  m_monitorInterval = (unsigned int)atoi((const char*)tmpStr);
  xmlFree(tmpStr);

  m_monIntFlg = true;
  return true;
}

bool SklCfgSteerOutputData::Verification()
{
  if( m_monitorInterval == 0 ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0006_, "Monitor Interval is zero.");
    return false;
  }
  return true;
}

/**
 * ディレクトリパスを取得する。
 * @return    ディレクトリパス
 */
const char* SklCfgSteerOutputData::GetBaseDir() const
{
  return (const char*)m_baseDir;
}

/**
 * 出力インターバルを取得する。
 * @return    出力インターバル
 */
unsigned int SklCfgSteerOutputData::GetMonitorInterval() const
{
  return m_monitorInterval;
}

/**
 * OutFile要素数を取得する。
 * @return    OutFile要素数
 */
unsigned int SklCfgSteerOutputData::GetOutFileSize() const
{
  return (unsigned int)(m_outFileList.size());
}

/**
 * 最初のOutFile要素を取得する。
 * @return    OutFile要素クラス
 */
const SklCfgOutFile* SklCfgSteerOutputData::GetOutFileFirst() const
{
  if( m_outFileList.size() == 0 ) return NULL;
  return m_outFileList.front();
}

/**
 * 次のOutFile要素を取得する
 * @param file    前のOutFile要素クラス
 * @return    OutFile要素クラス
 */
const SklCfgOutFile*
      SklCfgSteerOutputData::GetOutFileNext(const SklCfgOutFile* file) const
{
  if( !file ) return NULL;
  list<SklCfgOutFile*>::const_iterator itr = m_outFileList.begin();
  list<SklCfgOutFile*>::const_iterator itrEnd = m_outFileList.end();
  for( ; itr != itrEnd; itr++ ){
    if( file == (*itr) ){
      list<SklCfgOutFile*>::const_iterator itr2 = itr;
      itr2++;
      if( itr2 == itrEnd ) return NULL;
      return *itr2;
    }
  }
  return NULL;
}


/**
 * History要素数を取得する。
 * @return    History要素数
 */
unsigned int SklCfgSteerOutputData::GetHistorySize() const
{
  return (unsigned int)(m_historyList.size());
}

/**
 * 最初のHistory要素を取得する。
 * @return    History要素クラス
 */
const SklCfgHistory* SklCfgSteerOutputData::GetHistoryFirst() const
{
  if( m_historyList.size() == 0 ) return NULL;
  return m_historyList.front();
}


/**
 * 次のHistory要素を取得する
 * @param history    前のHistory要素クラス
 * @return    History要素クラス
 */
const SklCfgHistory*
      SklCfgSteerOutputData::GetHistoryNext(const SklCfgHistory* history) const
{
  if( !history ) return NULL;
  list<SklCfgHistory*>::const_iterator itr = m_historyList.begin();
  list<SklCfgHistory*>::const_iterator itrEnd = m_historyList.end();
  for( ; itr != itrEnd; itr++ ){
    if( history == (*itr) ){
      list<SklCfgHistory*>::const_iterator itr2 = itr;
      itr2++;
      if( itr2 == itrEnd ) return NULL;
      return *itr2;
    }
  }
  return NULL;
}

#ifdef DEBUG
void SklCfgSteerOutputData::Print()
{
  cout << "  <-- SteerOutputData --" << endl << flush;

  if( m_baseDir ){
    cout << "   base directory   : " << m_baseDir << endl << flush;
  }else{
    cout << "   base directory not set" << endl << flush;
  }
  cout << "   monitor interval : " << m_monitorInterval << endl << flush;

  if( !m_historyList.empty() ){
    list<SklCfgHistory*>::iterator itr = m_historyList.begin();
    list<SklCfgHistory*>::iterator itrEnd = m_historyList.end();
    for( ; itr != itrEnd ; itr++ ) (*itr)->Print();
  }

  if( !m_outFileList.empty() ){
    list<SklCfgOutFile*>::iterator itr = m_outFileList.begin();
    list<SklCfgOutFile*>::iterator itrEnd = m_outFileList.end();
    for( ; itr != itrEnd ; itr++ ) (*itr)->Print();
  }

  cout << "   -- SteerOutputData -->" << endl << flush;
}
#endif //DEBUG

} // end of namespace SklCfg

