/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */
/**
 * @file SklCfgUsrDefTopNode.C
 * コンフィグレーションユーザ定義要素クラスソースファイル
 * @author     riken
 * @date       2010/10/01
 */

#include <string.h> // add 20090629
#include "skl_config/SklCfgDefName.h"
#include "skl_config/SklCfgUsrDefTopNode.h"

namespace SklCfg {
/**
 * コンストラクタ
 */
SklCfgUsrDefTopNode::SklCfgUsrDefTopNode()
{
}

/**
 * デストラクタ
 */
SklCfgUsrDefTopNode::~SklCfgUsrDefTopNode()
{
}

bool SklCfgUsrDefTopNode::Parse(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  if( !CheckNodeName((const char*)node->name) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    return false;
  }

  CfgElem* topElem = new CfgElem;
  if( !topElem ) {
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0005_);
    return false;
  }
  if( !topElem->SetAttr(node) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    delete topElem;
    return false;
  }
  if( !m_root.AddElem(topElem) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    delete topElem;
    return false;
  }

  return true;
}

bool SklCfgUsrDefTopNode::CheckNodeName(const char* nname)
{
  if( !nname ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0001_);
    return false;
  }

  if( StrCaseCompare(nname,
                     _SKL_CFG_NNAME_USRDEF_ELEM_,
                     strlen(_SKL_CFG_NNAME_USRDEF_ELEM_)) ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0002_, nname);
    return false;
  }

  return true;
}

bool SklCfgUsrDefTopNode::ParseSklDefNode(xmlNodePtr node)
{
  if( !node ){
    CfgDbgMsg("%s (%d): ",__FILE__, __LINE__);
    CfgErrMsg(_SKL_CFG_ERR_0003_);
    return false;
  }

  return true;
}

bool SklCfgUsrDefTopNode::Verification()
{
  return true;
}

#ifdef DEBUG
void SklCfgUsrDefTopNode::Print()
{
  cout << " <-- UserDefineTopNode --" << endl << flush;
  cout << "  -- UserDefineTopNode -->" << endl << flush;
}
#endif //DEBUG

} // end of namespace SklCfg

