/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2009
 *
 */


#include "skl_config/SklParseUtils.h"


int SklParseUtils::getParamID(const CfgParam* p) {
  int id=-1;
  if ( !p->isSetID() )         return -1;
  if ( -1 == (id=p->GetID()) ) return -1;
  return id;
}

bool SklParseUtils::getVecParams(const CfgElem* elmL,
                         const char* key1,
                         const char* key2,
                         const char* key3,
                         int* values) {
  if( !elmL || !key1 || !key2 || !key3 || !values ) return false;
  const CfgParam* param = NULL;
  if( !(param = elmL->GetParamFirst( key1 )) ) return false;
  else if ( !(param->GetData( (values+0) )) )  return false;
  if( !(param = elmL->GetParamFirst( key2 )) ) return false;
  else if ( !(param->GetData( (values+1) )) )  return false;
  if( !(param = elmL->GetParamFirst( key3 )) ) return false;
  else if ( !(param->GetData( (values+2) )) )  return false;
  return true;
}

bool SklParseUtils::getVecParams(const CfgElem* elmL,
                         const char* key1,
                         const char* key2,
                         const char* key3,
                         unsigned* values) {
  if( !elmL || !key1 || !key2 || !key3 || !values ) return false;
  const CfgParam* param = NULL;
  int v[3];
  if( !(param = elmL->GetParamFirst( key1 )) ) return false;
  else if ( !(param->GetData( (v+0) )) )       return false;
  if( !(param = elmL->GetParamFirst( key2 )) ) return false;
  else if ( !(param->GetData( (v+1) )) )       return false;
  if( !(param = elmL->GetParamFirst( key3 )) ) return false;
  else if ( !(param->GetData( (v+2) )) )       return false;
  for (int i=0; i<3; i++) {
    if( v[i] < 0 ) {
      fprintf(stderr, "\tWarning : value is less than zero. (v[%d]=%d)\n", i, v[i]);
    }
    values[i]=(unsigned)v[i];
  }
  return true;
}

bool SklParseUtils::getVecParams(const CfgElem* elmL,
                         const char* key1,
                         const char* key2,
                         const char* key3,
                         float* vec) {
  if( !elmL || !key1 || !key2 || !key3 || !vec ) return false;
  const CfgParam* param = NULL;
  if( !(param = elmL->GetParamFirst( key1 )) ) return false;
  else if ( !(param->GetData( (vec+0) )) )     return false;
  if( !(param = elmL->GetParamFirst( key2 )) ) return false;
  else if ( !(param->GetData( (vec+1) )) )     return false;
  if( !(param = elmL->GetParamFirst( key3 )) ) return false;
  else if ( !(param->GetData( (vec+2) )) )     return false;
  return true;
}

bool SklParseUtils::getVecParams(const CfgElem* elmL,
                         const char* key1,
                         const char* key2,
                         const char* key3,
                         double* vec) {
  if( !elmL || !key1 || !key2 || !key3 || !vec ) return false;
  const CfgParam* param = NULL;
  float v[3];
  if( !(param = elmL->GetParamFirst( key1 )) ) return false;
  else if ( !(param->GetData( (v+0) )) )       return false;
  if( !(param = elmL->GetParamFirst( key2 )) ) return false;
  else if ( !(param->GetData( (v+1) )) )       return false;
  if( !(param = elmL->GetParamFirst( key3 )) ) return false;
  else if ( !(param->GetData( (v+2) )) )       return false;
  for (int i=0; i<3; i++) vec[i]=(double)v[i];
  return true;
}



bool
SklParseUtils::chkXMLTopTag(int label, const char* name) {
  const SklSolverConfig* solvCfg = sphSphere::getInstance()->getSklSolverConfig();
  if( solvCfg == NULL ) {
    fprintf(stderr, "\tCan't get solver config object.\n");
    return false;
  }

  const CfgElem *elemTop=NULL;
  if( label == USERDEF ) {
    if( !name ) {
      fprintf(stderr, "\tParsing error : Missing USERDEF tree. name = NULL\n");
      return false;
    }
    if( !(elemTop = solvCfg->GetTop(USERDEF, name)) ) {
      fprintf(stderr,
              "\tParsing error : Missing USERDEF(name=\"%s\") tree.\n", name);
      return false;
    }
  }
  else {
    if( !(elemTop = solvCfg->GetTop((SKLNODE)label)) ) {
      switch( label ) {
        case STEER:
          fprintf(stderr, "\tParsing error : Missing  STEER tree\n");
          break;
        case PARAMETER:
          fprintf(stderr, "\tParsing error : Missing  PARAMETER tree\n");
          break;
        case DOMAININFO:
          fprintf(stderr, "\tParsing error : Missing  DOMAININFO tree\n");
          break;
        case INNERBND:
          fprintf(stderr, "\tParsing error : Missing  INNERBOUNDARY tree\n");
          break;
        case OUTERBND:
          fprintf(stderr, "\tParsing error : Missing  OUTERBOUNDARY tree\n");
          break;
        case MDMTBL:
          fprintf(stderr, "\tParsing error : Missing  MDMTBL tree\n");
          break;
        default:
          fprintf(stderr, "\tParsing error : Label unknown\n");
          break;
      }
      return false;
    }
  }
        return true;
}

bool
SklParseUtils::getGridInfo(unsigned Dims,
                     unsigned* size,
                     float* origin,
                     float* pitch,
                     float* width) {
  if ( (Dims != 1) && (Dims != 2) && (Dims != 3) ) {
    fprintf(stderr, "\tError Dims = %d\n", Dims);
    return false;
  }
  if( !size ) {
    fprintf(stderr, "\tInvalid parameter(unsigned* size).\n");
    return false;
  }

  const SklSolverConfig* solvCfg = sphSphere::getInstance()->getSklSolverConfig();
  if( solvCfg == NULL ) {
    fprintf(stderr, "\tCan't get solver config object.\n");
    return false;
  }

  unsigned  sz[3], n;
  float pit[3], wid[3], org[3];
  bool sz_flg, pit_flg, wid_flg, org_flg;
  sz_flg = pit_flg = wid_flg = org_flg = true;
  // Get Size
  if( (sz_flg = solvCfg->GetVoxelSize(sz[0], sz[1], sz[2])) ) {
    for(n=0; n<Dims; n++) size[n] = sz[n];
  }
  // Get Pitch
  if( (pit_flg = solvCfg->GetVoxelPitch(pit[0], pit[1], pit[2])) ) {
    if( pitch ) {
      for(n=0; n<Dims; n++) pitch[n] = pit[n];
    }
  }
  // Get Width
  if( (wid_flg = solvCfg->GetVoxelWidth(wid[0], wid[1], wid[2])) ) {
    if( width ) {
      for(n=0; n<Dims; n++) width[n] = wid[n];
    }
  }
  // Get Origin
  if( (org_flg = solvCfg->GetVoxelOrigin(org[0], org[1], org[2])) ) {
    if( origin ) {
      for(n=0; n<Dims; n++) origin[n] = org[n];
    }
  }

  if( pit_flg && wid_flg && pitch && width ) {
    fprintf(stderr, "\tWarning : Both the Pitch and Width are set.\n");
  }
  else if( !pit_flg && wid_flg && pitch ) {
    if( pitch ) {
      for(n=0; n<Dims; n++)
        pitch[n] = wid[n] / (size[n] - 1);
    }
  }
  else if( pit_flg && !wid_flg && width ) {
    if( width ) {
      for(n=0; n<Dims; n++)
        width[n] = pit[n] * (size[n] - 1);
    }
  }
  else if( !pit_flg && !wid_flg && (pitch || width) ) {
    fprintf(stderr, "\tCan't get values and pitch or width from solver config.\n");
    return false;
  }
  if( !sz_flg ) {
    fprintf(stderr, "\tCan't get voxel size.\n");
    return false;
  }
  if( !org_flg && origin ) {
    fprintf(stderr, "\tCan't get voxel origin.\n");
    return false;
  }

  return true;
}

bool
SklParseUtils::getGridInfo(unsigned Dims,
                     unsigned* size,
                     double* origin,
                     double* pitch,
                     double* width) {
  if ( (Dims != 1) && (Dims != 2) && (Dims != 3) ) {
    fprintf(stderr, "\tError Dims = %d\n", Dims);
    return false;
  }
  if( !size ) {
    fprintf(stderr, "\tInvalid parameter(unsigned* size).\n");
    return false;
  }

  const SklSolverConfig* solvCfg = sphSphere::getInstance()->getSklSolverConfig();
  if( solvCfg == NULL ) {
    fprintf(stderr, "\tCan't get solver config object.\n");
    return false;
  }

  unsigned  sz[3], n;
  float pit[3], wid[3], org[3];
  bool sz_flg, pit_flg, wid_flg, org_flg;
  sz_flg = pit_flg = wid_flg = org_flg = true;
  // Get Size
  if( (sz_flg = solvCfg->GetVoxelSize(sz[0], sz[1], sz[2])) ) {
    for(n=0; n<Dims; n++) size[n] = sz[n];
  }
  // Get Pitch
  if( (pit_flg = solvCfg->GetVoxelPitch(pit[0], pit[1], pit[2])) ) {
    if( pitch ) {
      for(n=0; n<Dims; n++) pitch[n] = (double)pit[n];
    }
  }
  // Get Width
  if( (wid_flg = solvCfg->GetVoxelWidth(wid[0], wid[1], wid[2])) ) {
    if( width ) {
      for(n=0; n<Dims; n++) width[n] = (double)wid[n];
    }
  }
  // Get Origin
  if( (org_flg = solvCfg->GetVoxelOrigin(org[0], org[1], org[2])) ) {
    if( origin ) {
      for(n=0; n<Dims; n++) origin[n] = (double)org[n];
    }
  }

  if( pit_flg && wid_flg && pitch && width ) {
    fprintf(stderr, "\tWarning : Both the Pitch and Width are set.\n");
  }
  else if( !pit_flg && wid_flg && pitch ) {
    if( pitch ) {
      for(n=0; n<Dims; n++)
        pitch[n] = (double)((double)wid[n] / (double)(size[n] - 1));
    }
  }
  else if( pit_flg && !wid_flg && width ) {
    if( width ) {
      for(n=0; n<Dims; n++)
        width[n] = (double)((double)pit[n] * (double)(size[n] - 1));
    }
  }
  else if( !pit_flg && !wid_flg && (pitch || width) ) {
    fprintf(stderr, "\tCan't get values and pitch or width from solver config.\n");
    return false;
  }
  if( !sz_flg ) {
    fprintf(stderr, "\tCan't get voxel size.\n");
    return false;
  }
  if( !org_flg && origin ) {
    fprintf(stderr, "\tCan't get voxel origin.\n");
    return false;
  }

  return true;
}

bool
SklParseUtils::getCellInfo(unsigned Dims,
                     unsigned* size,
                     float* origin,
                     float* pitch,
                     float* width) {
  if ( (Dims != 1) && (Dims != 2) && (Dims != 3) ) {
    fprintf(stderr, "\tError Dims = %d\n", Dims);
    return false;
  }
  if( !size ) {
    fprintf(stderr, "\tInvalid parameter(unsigned* size).\n");
    return false;
  }

  const SklSolverConfig* solvCfg = sphSphere::getInstance()->getSklSolverConfig();
  if( solvCfg == NULL ) {
    fprintf(stderr, "\tCan't get solver config object.\n");
    return false;
  }

  unsigned  sz[3], n;
  float pit[3], wid[3], org[3];
  bool sz_flg, pit_flg, wid_flg, org_flg;
  sz_flg = pit_flg = wid_flg = org_flg = true;
  // Get Size
  if( (sz_flg = solvCfg->GetVoxelSize(sz[0], sz[1], sz[2])) ) {
    for(n=0; n<Dims; n++) size[n] = sz[n];
  }
  // Get Pitch
  if( (pit_flg = solvCfg->GetVoxelPitch(pit[0], pit[1], pit[2])) ) {
    if( pitch ) {
      for(n=0; n<Dims; n++) pitch[n] = pit[n];
    }
  }
  // Get Width
  if( (wid_flg = solvCfg->GetVoxelWidth(wid[0], wid[1], wid[2])) ) {
    if( width ) {
      for(n=0; n<Dims; n++) width[n] = wid[n];
    }
  }
  // Get Origin
  if( (org_flg = solvCfg->GetVoxelOrigin(org[0], org[1], org[2])) ) {
    if( origin ) {
      for(n=0; n<Dims; n++) origin[n] = org[n];
    }
  }

  if( pit_flg && wid_flg && pitch && width ) {
    fprintf(stderr, "\tWarning : Both the Pitch and Width are set.\n");
  }
  else if( !pit_flg && wid_flg && pitch ) {
    if( pitch ) {
      for(n=0; n<Dims; n++)
        pitch[n] = wid[n] / size[n];
    }
  }
  else if( pit_flg && !wid_flg && width ) {
    if( width ) {
      for(n=0; n<Dims; n++)
        width[n] = pit[n] * size[n];
    }
  }
  else if( !pit_flg && !wid_flg && (pitch || width) ) {
    fprintf(stderr, "\tCan't get values of pitch and width from solver config.\n");
    return false;
  }
  if( !sz_flg ) {
    fprintf(stderr, "\tCan't get voxel size.\n");
    return false;
  }
  if( !org_flg && origin ) {
    fprintf(stderr, "\tCan't get voxel origin.\n");
    return false;
  }

  return true;
}

bool
SklParseUtils::getCellInfo(unsigned Dims,
                     unsigned* size,
                     double* origin,
                     double* pitch,
                     double* width) {
  if ( (Dims != 1) && (Dims != 2) && (Dims != 3) ) {
    fprintf(stderr, "\tError Dims = %d\n", Dims);
    return false;
  }
  if( !size ) {
    fprintf(stderr, "\tInvalid parameter(unsigned* size).\n");
    return false;
  }

  const SklSolverConfig* solvCfg = sphSphere::getInstance()->getSklSolverConfig();
  if( solvCfg == NULL ) {
    fprintf(stderr, "\tCan't get solver config object.\n");
    return false;
  }

  unsigned  sz[3], n;
  float pit[3], wid[3], org[3];
  bool sz_flg, pit_flg, wid_flg, org_flg;
  sz_flg = pit_flg = wid_flg = org_flg = true;
  // Get Size
  if( (sz_flg = solvCfg->GetVoxelSize(sz[0], sz[1], sz[2])) ) {
    for(n=0; n<Dims; n++) size[n] = sz[n];
  }
  // Get Pitch
  if( (pit_flg = solvCfg->GetVoxelPitch(pit[0], pit[1], pit[2])) ) {
    if( pitch ) {
      for(n=0; n<Dims; n++) pitch[n] = (double)pit[n];
    }
  }
  // Get Width
  if( (wid_flg = solvCfg->GetVoxelWidth(wid[0], wid[1], wid[2])) ) {
    if( width ) {
      for(n=0; n<Dims; n++) width[n] = (double)wid[n];
    }
  }
  // Get Origin
  if( (org_flg = solvCfg->GetVoxelOrigin(org[0], org[1], org[2])) ) {
    if( origin ) {
      for(n=0; n<Dims; n++) origin[n] = (double)org[n];
    }
  }

  if( pit_flg && wid_flg && pitch && width ) {
    fprintf(stderr, "\tWarning : Both the Pitch and Width are set.\n");
  }
  else if( !pit_flg && wid_flg && pitch ) {
    if( pitch ) {
      for(n=0; n<Dims; n++)
        pitch[n] = (double)((double)wid[n] / (double)size[n]);
    }
  }
  else if( pit_flg && !wid_flg && width ) {
    if( width ) {
      for(n=0; n<Dims; n++)
        width[n] = (double)((double)pit[n] * (double)size[n]);
    }
  }
  else if( !pit_flg && !wid_flg && (pitch || width) ) {
    fprintf(stderr, "\tCan't get values of pitch and width from solver config.\n");
    return false;
  }
  if( !sz_flg ) {
    fprintf(stderr, "\tCan't get voxel size.\n");
    return false;
  }
  if( !org_flg && origin ) {
    fprintf(stderr, "\tCan't get voxel origin.\n");
    return false;
  }

  return true;
}

char*
SklParseUtils::searchTextFname(const char* key) {
  if( !key ) return NULL;

  const SklSolverConfig* solvCfg = sphSphere::getInstance()->getSklSolverConfig();
  if( solvCfg == NULL ) {
    fprintf(stderr, "\tCan't get solver config object.\n");
    return NULL;
  }
  const SklCfgOutFile* outFile = solvCfg->GetOutFileFirst();
  char *str=NULL;

  while( outFile ) {
    const char *format, *attr;
    if ( !(format = outFile->GetFormat()) ) {
      fprintf(stderr, "\tParsing error : OutFile description\n");
      return false;
    }
    if ( !(attr = outFile->GetAttr()) ) {
      fprintf(stderr, "\tParsing error : OutFile description\n");
      return false;
    }

    if ( !strcasecmp(format, "sph") ) {
      ;
    }
    else if ( !strcasecmp(format, "svx") ) {
      ;
    }
    else if ( !strcasecmp(format, "otv") ) {
      ;
    }
    else if ( !strcasecmp(format, "ovx") ) {
      ;
    }
    else if ( !strcasecmp(format, "fdv") ) {
      ;
    }
    else if ( !strcasecmp(format, "vcar") ) {
      ;
    }
    else if ( !strcasecmp(format, "p3dxyz") ) {
      ;
    }
    else if ( !strcasecmp(format, "p3dq") ) {
      ;
    }
    else if ( !strcasecmp(format, "p3dfunc") ) {
      ;
    }
    else if( !strcasecmp(format, "ascii") ){
      const char* basename = NULL;
      if( !strcasecmp(attr, key) ) {
        if( !(basename = outFile->GetBaseName()) ) return false;
        if( !(str = new char[strlen(basename)+10]) ) return false;
        sprintf(str, "%s.txt", basename);
      }
    }
    else {
      fprintf(stderr, "\tFile format '%s' is not supported.\n", format);
      return false;
    }

    outFile = solvCfg->GetOutFileNext(outFile);
  }

  return str;
}

