/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file sphFlowFactoryBase.C
 * Sphere Flow Factory super Class Source File
 * @author     hira
 * @date       2009/10/01
 *
 * \~japanese
 * @file sphFlowFactoryBase.C
 * Sphereフロー生成基底クラスソースファイル
 * @author     hira
 * @date       2009/10/01
 */

#include "solverbase/sphFlowFactoryBase.h"
#include "solverbase/sphSimpleFlow.h"

/**
 * \~english
 * Constructor
 *
 * \~japanese
 * コンストラクタ
 *
 */
sphFlowFactoryBase::sphFlowFactoryBase()
{
  // TODO 自動生成されたコンストラクター・スタブ

}


/**
 * \~english
 * Destructor
 *
 * \~japanese
 * デストラクタ.
 */
sphFlowFactoryBase::~sphFlowFactoryBase()
{
  // TODO Auto-generated destructor stub
}



/**
 * \~english
 * create Flow Object
 * @param       class_name   class name
 * @param       label    flow label
 * @param       id       flow id
 * @return    Flow Object
 *
 * \~japanese
 * フローオブジェクトを生成する。
 * @param       class_name   クラス識別文字列
 * @param       label   フローラベル
 * @param       id      フローID
 * @return    フローオブジェクト
 */
sphFlowBase* sphFlowFactoryBase::factoryFlow(const char* class_name, const char* label, int id) const
{
  sphFlowBase* flowObj = NULL;

  if (class_name != NULL) {
    /// "sph_simple_voxel"ラベルであれば、sphFlowSvを生成する。
    if (strcasecmp(class_name, SPH_CFG_LABEL_SIMPLEVOXEL) == 0) flowObj = new sphSimpleFlow(label, id);
  }

  /// デフォルトとしてsphFlowSvを生成する。
  if (flowObj == NULL) {
    flowObj = new sphSimpleFlow(label, id);
    if (label != NULL) {
      // warning
      m_errHandler->SPH_WARNING_INFO(SPH_WARNING_NOTDEFINE_LABEL, "label = %s, therefore create sphFlowBase.", label);
    }
  }

  return flowObj;
}




