/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/

#include"solverbase/sphSolverUnst.h"

/* Gauss integration point */
#define ALPHA  0.577350269189626
bool
sphSolverUnst::ShapeFunctionF(int Enum, float *detJ, float N[8][8],
                              float NX[8][8], float NY[8][8], float NZ[8][8]) {
  /* ----- On the original(XYZ) coordinate ----- */
  float x[8],y[8],z[8];
  float J[3][3];                        /* Jacobian matrix          */
  float Jinv[3][3];                     /* Inverse Jacobian matrix  */
  /* ----- On the original(PQR) coordinate ----- */
  float HP[8],HQ[8],HR[8];              /* Derivative of shape func.*/  
  const float xg[2] = {-ALPHA, ALPHA};

  int    i, j, index;
  int    ip,iq,ir;

  float pp[2],qq[2],rr[2];
  float temp;
  char   str[256];

  if( !detJ || !N || !NX || !NY || !NZ ) {
    fprintf(stderr, "ERROR(sphSolverUnst::ShapeFunctionF) : invalid parameter.\n");
    return false;
  }

  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::ShapeFunctionF) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::ShapeFunctionF) : UnstMesh is NULL.\n");
    return false;
  }

  float* xyz;
  if( !mesh->getCoord(&xyz) ) {
    printf("ERROR :: can't get coord.\n");
    return false;
  }

  for ( i = 0; i < 8; i++ ){
    index = mesh->nop[ Enum*8 + i ];
    x[i]  = xyz[ index*3    ];
    y[i]  = xyz[ index*3 +1 ];
    z[i]  = xyz[ index*3 +2 ];
  }

  index = 0;
  for     ( ip = 0; ip < 2; ip++ ){
    for   ( iq = 0; iq < 2; iq++ ){
      for ( ir = 0; ir < 2; ir++ ){
        pp[0] = 1.0 - xg[ip];
        qq[0] = 1.0 - xg[iq];
        rr[0] = 1.0 - xg[ir];
        pp[1] = 1.0 + xg[ip];
        qq[1] = 1.0 + xg[iq];
        rr[1] = 1.0 + xg[ir];

        /* ******** Derivative of  shape function ******** */
        /* ------------ For P-coordinate ------------ */
        HP[0] = -0.125 * qq[0] * rr[0];
        HP[1] =  0.125 * qq[0] * rr[0];
        HP[2] =  0.125 * qq[1] * rr[0];
        HP[3] = -0.125 * qq[1] * rr[0];
        HP[4] = -0.125 * qq[0] * rr[1];
        HP[5] =  0.125 * qq[0] * rr[1];
        HP[6] =  0.125 * qq[1] * rr[1];
        HP[7] = -0.125 * qq[1] * rr[1];

        /* ------------ For Q-coordinate ------------ */
        HQ[0] = -0.125 * pp[0] * rr[0];
        HQ[1] = -0.125 * pp[1] * rr[0];
        HQ[2] =  0.125 * pp[1] * rr[0];
        HQ[3] =  0.125 * pp[0] * rr[0];
        HQ[4] = -0.125 * pp[0] * rr[1];
        HQ[5] = -0.125 * pp[1] * rr[1];
        HQ[6] =  0.125 * pp[1] * rr[1];
        HQ[7] =  0.125 * pp[0] * rr[1];

        /* ------------ For R-coordinate ------------ */
        HR[0] = -0.125 * pp[0] * qq[0];
        HR[1] = -0.125 * pp[1] * qq[0];
        HR[2] = -0.125 * pp[1] * qq[1];
        HR[3] = -0.125 * pp[0] * qq[1];
        HR[4] =  0.125 * pp[0] * qq[0];
        HR[5] =  0.125 * pp[1] * qq[0];
        HR[6] =  0.125 * pp[1] * qq[1];
        HR[7] =  0.125 * pp[0] * qq[1];

        /* ******** Jacobi matrix ******** */
        J[0][0] = HP[0]*x[0] + HP[1]*x[1] + HP[2]*x[2] + HP[3]*x[3]
                 +HP[4]*x[4] + HP[5]*x[5] + HP[6]*x[6] + HP[7]*x[7];
        J[0][1] = HP[0]*y[0] + HP[1]*y[1] + HP[2]*y[2] + HP[3]*y[3]
                 +HP[4]*y[4] + HP[5]*y[5] + HP[6]*y[6] + HP[7]*y[7];
        J[0][2] = HP[0]*z[0] + HP[1]*z[1] + HP[2]*z[2] + HP[3]*z[3]
                 +HP[4]*z[4] + HP[5]*z[5] + HP[6]*z[6] + HP[7]*z[7];

        J[1][0] = HQ[0]*x[0] + HQ[1]*x[1] + HQ[2]*x[2] + HQ[3]*x[3]
                 +HQ[4]*x[4] + HQ[5]*x[5] + HQ[6]*x[6] + HQ[7]*x[7];
        J[1][1] = HQ[0]*y[0] + HQ[1]*y[1] + HQ[2]*y[2] + HQ[3]*y[3]
                 +HQ[4]*y[4] + HQ[5]*y[5] + HQ[6]*y[6] + HQ[7]*y[7];
        J[1][2] = HQ[0]*z[0] + HQ[1]*z[1] + HQ[2]*z[2] + HQ[3]*z[3]
                 +HQ[4]*z[4] + HQ[5]*z[5] + HQ[6]*z[6] + HQ[7]*z[7];

        J[2][0] = HR[0]*x[0] + HR[1]*x[1] + HR[2]*x[2] + HR[3]*x[3]
                 +HR[4]*x[4] + HR[5]*x[5] + HR[6]*x[6] + HR[7]*x[7];
        J[2][1] = HR[0]*y[0] + HR[1]*y[1] + HR[2]*y[2] + HR[3]*y[3]
                 +HR[4]*y[4] + HR[5]*y[5] + HR[6]*y[6] + HR[7]*y[7];
        J[2][2] = HR[0]*z[0] + HR[1]*z[1] + HR[2]*z[2] + HR[3]*z[3]
                 +HR[4]*z[4] + HR[5]*z[5] + HR[6]*z[6] + HR[7]*z[7];

        /* ******** Jacobian ******** */
        *detJ = J[0][0]*J[1][1]*J[2][2]
               +J[0][1]*J[1][2]*J[2][0]
               +J[0][2]*J[1][0]*J[2][1]
               -J[0][2]*J[1][1]*J[2][0]
               -J[0][1]*J[1][0]*J[2][2]
               -J[0][0]*J[1][2]*J[2][1];

        if ( *detJ <= 0.0 ){
          fprintf(stderr, "\nError: Determinant of Jacobian is not positive.");
          fprintf(stderr,"  ElementID,detJ = %d %e\n", Enum, *detJ);
          abort();
        }

        /* ******** Inverse of Jacobi matrix ******** */
        temp = 1.0 / *detJ;

        Jinv[0][0] = (J[1][1]*J[2][2] - J[1][2]*J[2][1]) * temp;
        Jinv[0][1] = (J[0][2]*J[2][1] - J[0][1]*J[2][2]) * temp;
        Jinv[0][2] = (J[0][1]*J[1][2] - J[0][2]*J[1][1]) * temp;

        Jinv[1][0] = (J[1][2]*J[2][0] - J[1][0]*J[2][2]) * temp;
        Jinv[1][1] = (J[0][0]*J[2][2] - J[0][2]*J[2][0]) * temp;
        Jinv[1][2] = (J[0][2]*J[1][0] - J[0][0]*J[1][2]) * temp;

        Jinv[2][0] = (J[1][0]*J[2][1] - J[1][1]*J[2][0]) * temp;
        Jinv[2][1] = (J[0][1]*J[2][0] - J[0][0]*J[2][1]) * temp;
        Jinv[2][2] = (J[0][0]*J[1][1] - J[0][1]*J[1][0]) * temp;

        /* ******** Shape function ******** */
        N[0][index] = 0.125 * pp[0] * qq[0] * rr[0];
        N[1][index] = 0.125 * pp[1] * qq[0] * rr[0];
        N[2][index] = 0.125 * pp[1] * qq[1] * rr[0];
        N[3][index] = 0.125 * pp[0] * qq[1] * rr[0];
        N[4][index] = 0.125 * pp[0] * qq[0] * rr[1];
        N[5][index] = 0.125 * pp[1] * qq[0] * rr[1];
        N[6][index] = 0.125 * pp[1] * qq[1] * rr[1];
        N[7][index] = 0.125 * pp[0] * qq[1] * rr[1];

        /* ******** Derivative od shape function on XYZ-coord. ******** */
        for ( j = 0; j < 8; j++ ){
          NX[j][index] = Jinv[0][0]*HP[j] + Jinv[0][1]*HQ[j] + Jinv[0][2]*HR[j];
          NY[j][index] = Jinv[1][0]*HP[j] + Jinv[1][1]*HQ[j] + Jinv[1][2]*HR[j];
          NZ[j][index] = Jinv[2][0]*HP[j] + Jinv[2][1]*HQ[j] + Jinv[2][2]*HR[j];
        }
        index++;
      }
    }
  }

  return true;
}
#undef ALPHA

void
sphSolverUnst::ZeroClearF(int n, float *v) {
  int i;
  for ( i = 0; i < n; i++ ) v[i] = 0.0;
}

bool
sphSolverUnst::MatVecProduct(int type, MatrixF *A, float *x, float *b ) {
  int    n[20];
  float s[20];
  int i,j,iElem,index;

  if( !A || !x || !b ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MatVecProduct) : Invalid parameter.\n");
    return false;
  }

  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MatVecProduct) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MatVecProduct) : UnstMesh is NULL.\n");
    return false;
  }

  ZeroClearF( mesh->nnNode, b);

  for ( iElem = mesh->nInner; iElem < mesh->nInner+mesh->nOverlapped; iElem++ ){
    for ( i = 0; i < type; i++ ){
      n[i] = mesh->nop[ iElem*type +i ];
    }
    for ( i = 0; i < type; i++ ){
      s[i]  = 0.0;
      index = iElem*type*type + type*i;
      for ( j = 0; j < type; j++ ){
        s[i] += A->mat[index+j]*x[n[j]];
      }
    }
    for ( i = 0; i < type; i++ ){
      b[n[i]] += s[i];
    }
  }

// ---------------------------------------------------------
// 20100430 tamaki
//  sphUnst::SendRecv( b );
// ---------------------------------------------------------
  if( !mesh->comm(b) ) {
    fprintf(stderr, "ERROR : communication failed.\n");
    return false;
  }

  for ( iElem = 0; iElem < mesh->nInner; iElem++ ){
    for ( i = 0; i < type; i++ ){
      n[i] = mesh->nop[ iElem*type +i ];
    }
    for ( i = 0; i < type; i++ ){
      s[i]  = 0.0;
      index = iElem*type*type + type*i;
      for ( j = 0; j < type; j++ ){
        s[i] += A->mat[index+j]*x[n[j]];
      }
    }
    for ( i = 0; i < type; i++ ){
      b[n[i]] += s[i];
    }
  }
// ---------------------------------------------------------
// 20100430 tamaki
//  sphUnst::WaitData( b );
// ---------------------------------------------------------
  if( !mesh->wait(b) ) {
    fprintf(stderr, "ERROR : wait failed.\n");
    return false;
  }

  return true;
}

bool
sphSolverUnst::MakeAdvMatrix(MatrixF *advMAT,
                             float* dp_u, float* dp_v, float* dp_w) {
  int    ie, ig, in;
  int    index;
  int    n[8];  /* Node number */
  float detJ;
  float U, V, W;
  float N[8][8], NX[8][8], NY[8][8], NZ[8][8];

  if( !advMAT || !dp_u || !dp_v || !dp_w ) {
    fprintf(stderr, "ERROR : Invalid parameter.\n");
    return false;
  }

  /* ******** Initialize ******** */
  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeAdvMatrix) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeAdvMatrix) : UnstMesh is NULL.\n");
    return false;
  }
  ZeroClearF( (mesh->nInner+mesh->nOverlapped)*64, advMAT->mat );

  /* ******** Element loop ******** */
  for ( ie = 0; ie < mesh->nInner+mesh->nOverlapped; ie++ ){
    ShapeFunctionF(ie, &detJ, N, NX, NY, NZ);
    index = 8*ie;
    n[0] = mesh->nop[index   ];
    n[1] = mesh->nop[index +1];
    n[2] = mesh->nop[index +2];
    n[3] = mesh->nop[index +3];
    n[4] = mesh->nop[index +4];
    n[5] = mesh->nop[index +5];
    n[6] = mesh->nop[index +6];
    n[7] = mesh->nop[index +7];
      
    for   ( ig = 0; ig < 8; ig++ ){ /* Gauss point loop */
      U = 0.0;
      V = 0.0;
      W = 0.0;
      for ( in = 0; in < 8; in++ ){ /* Node loop */
	U += dp_u[n[in]] * N[in][ig];
	V += dp_v[n[in]] * N[in][ig];
	W += dp_w[n[in]] * N[in][ig];
      }

      for ( in = 0; in < 8; in++ ){
	index = ie*64 + in*8;
	advMAT->mat[index   ] += detJ * N[in][ig] 
	                        * (U*NX[0][ig] + V*NY[0][ig] + W*NZ[0][ig]);
	advMAT->mat[index +1] += detJ * N[in][ig] 
	                        * (U*NX[1][ig] + V*NY[1][ig] + W*NZ[1][ig]);
	advMAT->mat[index +2] += detJ * N[in][ig] 
	                        * (U*NX[2][ig] + V*NY[2][ig] + W*NZ[2][ig]);
	advMAT->mat[index +3] += detJ * N[in][ig] 
	                        * (U*NX[3][ig] + V*NY[3][ig] + W*NZ[3][ig]);
	advMAT->mat[index +4] += detJ * N[in][ig] 
	                        * (U*NX[4][ig] + V*NY[4][ig] + W*NZ[4][ig]);
	advMAT->mat[index +5] += detJ * N[in][ig] 
	                        * (U*NX[5][ig] + V*NY[5][ig] + W*NZ[5][ig]);
	advMAT->mat[index +6] += detJ * N[in][ig] 
	                        * (U*NX[6][ig] + V*NY[6][ig] + W*NZ[6][ig]);
	advMAT->mat[index +7] += detJ * N[in][ig] 
	                        * (U*NX[7][ig] + V*NY[7][ig] + W*NZ[7][ig]);
      }
    }
  }

  return true;
}

bool
sphSolverUnst::MakeDiffMatrix(float Reinv, MatrixF9 *diffMAT) {
  int    ie, in, im, ig;
  int    index;
  float detJ, K;
  float N[8][8], NX[8][8], NY[8][8], NZ[8][8];

  if( !diffMAT ) {
    fprintf(stderr, "ERROR : diffMAT is NULL.\n");
    return false;
  }

  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeDiffMatrix) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeDiffMatrix) : UnstMesh is NULL.\n");
    return false;
  }

  for ( ie = 0; ie < mesh->nInner+mesh->nOverlapped; ie++ ){
    ShapeFunctionF(ie, &detJ, N, NX, NY, NZ);
    detJ *= Reinv;
    for   ( in = 0; in < 8; in++ ){
      for ( im = 0; im < 8; im++ ){
	for ( ig = 0; ig < 8; ig++ ){
	  index = 64*ie + 8*in + im;
	  K = detJ * ( NX[in][ig]*NX[im][ig]
	              +NY[in][ig]*NY[im][ig] + NZ[in][ig]*NZ[im][ig]);
#define SDIVERGENCE
#ifdef SDIVERGENCE
	  diffMAT->xx->mat[index] += detJ * NX[in][ig]*NX[im][ig] +K;
	  diffMAT->xy->mat[index] += detJ * NY[in][ig]*NX[im][ig];
	  diffMAT->xz->mat[index] += detJ * NZ[in][ig]*NX[im][ig];

	  diffMAT->yx->mat[index] += detJ * NX[in][ig]*NY[im][ig];
	  diffMAT->yy->mat[index] += detJ * NY[in][ig]*NY[im][ig] +K;
	  diffMAT->yz->mat[index] += detJ * NZ[in][ig]*NY[im][ig];

	  diffMAT->zx->mat[index] += detJ * NX[in][ig]*NZ[im][ig];
	  diffMAT->zy->mat[index] += detJ * NY[in][ig]*NZ[im][ig];
	  diffMAT->zz->mat[index] += detJ * NZ[in][ig]*NZ[im][ig] +K;
#else
	  diffMAT->xx->mat[index] += K;
	  diffMAT->yy->mat[index] += K;
	  diffMAT->zz->mat[index] += K;
#endif	  
	}
      }
    }
  }

  return true;
}

bool
sphSolverUnst::MakeDivMatrix(MatrixF3 *divMAT) {
  int    ie, in, im, ig;
  int    index;
  float detJ;
  float N[8][8], NX[8][8], NY[8][8], NZ[8][8];

  if( !divMAT ) {
    fprintf(stderr, "ERROR : divMAT is NULL.\n");
    return false;
  }

  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeDivMatrix) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeDivMatrix) : UnstMesh is NULL.\n");
    return false;
  }

  for ( ie = 0; ie < mesh->nInner+mesh->nOverlapped; ie++ ){
    ShapeFunctionF(ie, &detJ, N, NX, NY, NZ);
    for   ( in = 0; in < 8; in++ ){
      for ( im = 0; im < 8; im++ ){
	index = 64*ie + 8*in + im;	
	for ( ig = 0; ig < 8; ig++ ){
	  divMAT->x->mat[index] += detJ * N[in][ig]*NX[im][ig];
	  divMAT->y->mat[index] += detJ * N[in][ig]*NY[im][ig];
	  divMAT->z->mat[index] += detJ * N[in][ig]*NZ[im][ig];
	}
      }
    }
  }

  return true;
}

bool
sphSolverUnst::MakeGradMatrix(MatrixF3 *gradMAT) {
  int    ie, in, im, ig;
  int    index;
  float detJ;
  float N[8][8], NX[8][8], NY[8][8], NZ[8][8];

  if( !gradMAT ) {
    fprintf(stderr, "ERROR : gradMAT is NULL.\n");
    return false;
  }

  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeGradMatrix) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeGradMatrix) : UnstMesh is NULL.\n");
    return false;
  }

  for ( ie = 0; ie < mesh->nInner+mesh->nOverlapped; ie++ ){
    ShapeFunctionF(ie, &detJ, N, NX, NY, NZ);
    for   ( in = 0; in < 8; in++ ){
      for ( im = 0; im < 8; im++ ){
	for ( ig = 0; ig < 8; ig++ ){
	  index = 64*ie + 8*in + im;
	  gradMAT->x->mat[index] += detJ * NX[in][ig]*N[im][ig];
	  gradMAT->y->mat[index] += detJ * NY[in][ig]*N[im][ig];
	  gradMAT->z->mat[index] += detJ * NZ[in][ig]*N[im][ig];
	}
      }
    }
  }

  return true;
}

bool
sphSolverUnst::MakeMassMatrix(MatrixF *massMAT) {
  int    ie, in, ig;
  int    index, temp;
  float detJ;
  float N[8][8], NX[8][8], NY[8][8], NZ[8][8], mat[8];
  char   str[256];
  int i,j;
  
  if( !massMAT ) {
    fprintf(stderr, "ERROR : massMAT is NULL.\n");
    return false;
  }


  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeMassMatrix) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakeMassMatrix) : UnstMesh is NULL.\n");
    return false;
  }

  /* ******** EbE matrix calculation ******** */
  for ( ie = 0; ie < mesh->nInner+mesh->nOverlapped; ie++ ){
    ShapeFunctionF(ie, &detJ, N, NX, NY, NZ);
    index = 8*ie;
    for ( in = 0; in < 8; in++ ) mat[in] = 0.0; /* Zero clear */
    /* <<<<<<<< Lumped element matrix >>>>>>>> */
    for   ( in = 0; in < 8; in++ ){
      for ( ig = 0; ig < 8; ig++ ){
	mat[0] += detJ * N[0][ig]*N[in][ig];
	mat[1] += detJ * N[1][ig]*N[in][ig];
	mat[2] += detJ * N[2][ig]*N[in][ig];
	mat[3] += detJ * N[3][ig]*N[in][ig];
	mat[4] += detJ * N[4][ig]*N[in][ig];
	mat[5] += detJ * N[5][ig]*N[in][ig];
	mat[6] += detJ * N[6][ig]*N[in][ig];
	mat[7] += detJ * N[7][ig]*N[in][ig];
      }
    }

    /* <<<<<<<< Lumping >>>>>>>> */
    for ( in = 0; in < 8; in++ ){
      temp = mesh->nop[index +in];
      massMAT->mat[temp] += mat[in];
    }
  }

// -----------------------------------------------------------------
//  20100430 tamaki
//  sphUnst::SendRecv( massMAT->mat );
// -----------------------------------------------------------------
  if( !mesh->comm(massMAT->mat) ) {
    fprintf(stderr, "ERROR : communication failed.\n");
    return false;
  }

  /* ******** Inverse mass matrix ******** */
  for ( in = 0; in < mesh->nnNode; in++ ){
    if ( massMAT->mat[in] > 0.0 ){
      massMAT->mat[in] = 1.0/massMAT->mat[in];
    }
    else {
      fprintf(stderr, "\nError: Negative mass node is detected.\n");
      fprintf(stderr, "  mass[%d] = %e\n", in, massMAT->mat[in]);
      abort();
    }
  }

  return true;
}

bool
sphSolverUnst::MakePoisson(MatrixF *Poisson) {
  int    ie, in, im, ig;
  int    index;
  float detJ;
  float N[8][8], NX[8][8], NY[8][8], NZ[8][8];

  if( !Poisson ) {
    fprintf(stderr, "ERROR Invalid parameter.\n");
    return false;
  }

  if( !m_unstParaCtrl ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakePoisson) : UnstParaController is NULL.\n");
    return false;
  }
  sphUnstMesh* mesh = m_unstParaCtrl->getMesh();
  if( !mesh ) {
    fprintf(stderr, "ERROR(sphSolverUnst::MakePoisson) : UnstMesh is NULL.\n");
    return false;
  }

  for ( ie = 0; ie < mesh->nInner+mesh->nOverlapped; ie++ ){
    ShapeFunctionF(ie, &detJ, N, NX, NY, NZ);
    for   ( in = 0; in < 8; in++ ){
      for ( im = 0; im < 8; im++ ){
        for ( ig = 0; ig < 8; ig++ ){
          index = 64*ie + 8*in + im;
          Poisson->mat[index] += detJ * ( NX[im][ig]*NX[in][ig]
                                +NY[im][ig]*NY[in][ig] + NZ[im][ig]*NZ[in][ig] );
        }
      }
    }
  }
  return true;
}

