/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splfio_filetype_check.h
 * Implementation of endian utility procedure.
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_filetype_check.h
 * SBX/SPXファイルのエンディアンユーティリティヘッダーファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPL_FIO_FILE_TYPE_CHECK_H_
#define _SPL_FIO_FILE_TYPE_CHECK_H_

#include "spl/splmpi.h"
#include <stdio.h>
//#include "spl/spl.h"
#include "spl/util_endian.h"
//#include "spl/spl_errno.h"

/*********** SBX file magick string **********/
#define SPL_SBX_MAGICK_STR     "SBX\0"
#define SPL_SBX_MAGICK_STR_REV "\0XBS"

/*********** SPX file magick string **********/
#define SPL_SPX_MAGICK_STR     "SPX\0"
#define SPL_SPX_MAGICK_STR_REV "\0XPS"

#if defined(__cplusplus)
extern "C" {
#endif

typedef enum {  SPH_FMT_SBX
               ,SPH_FMT_SPX  } SplFileType;

EMatchType
isMatchEndianSbxMagick(int magick);

EMatchType
isMatchEndianSbxFile(const char* fname, SPL_Comm grp);

EMatchType
isMatchEndianSpxMagick(int magick);

EMatchType
isMatchEndianSpxFile(const char* fname, SPL_Comm grp);

EMatchType
isMatchEndianSvMagick(int magick, SplFileType ftype);

SplBool
isSbxFile(const char* fname, SPL_Comm grp);

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_FIO_FILE_TYPE_CHECK_H_ */

