/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splfio_sbx_io.h
 * Implementation of SBX utility subroutine.
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_sbx_io.h
 * SBXファイルの低レベル入出力関数ヘッダーファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPL_FIO_SBX_IO_H_
#define _SPL_FIO_SBX_IO_H_

#include <stdio.h>

#if defined(__cplusplus)
extern "C" {
#endif

SplBool
copySbx(splSbx* dst, const splSbx* src);

SplBool
getSbxInfo(int mode, int readRank, SPL_Comm grp, const char* fname,
           splSbx* obj, EMatchType* etype, unsigned long long* blk_sz,
           unsigned long long* cmp_flg, SplBool* has_ext_record);

SplBool
readSbxDataBlkSz(int mode,
                 int readRank,
                 SPL_Comm grp,
                 FILE* fp,
                 EMatchType eType,
                 size_t* comp_blk_sz);

SplBool
readSbxDataBlock(int mode,
                 int readRank,
                 SPL_Comm grp,
                 FILE* fp,
                 EMatchType eType,
                 unsigned int dtype,
                 unsigned int vlen,
                 size_t cmp_blk_sz,
                 size_t vox_num_per_blk,
                 size_t cmp_buf_sz,
                 size_t exp_buf_sz,
                 unsigned char* cmp_buf,
                 unsigned char* exp_buf,
                 size_t* readed_exp_buf_sz );

SplBool
readSbxDataRecord(int mode,
                  int readRank,
                  SPL_Comm grp,
                  FILE* fp,
                  EMatchType eType,
                  unsigned long long cmp_flg,
                  unsigned int dims,
                  unsigned int file_vlen,
                  unsigned int dest_vlen,
                  unsigned int dtype,
                  size_t file_gc,
                  size_t dest_gc,
                  unsigned int rlen,
                  size_t file_vsz[3],
                  size_t file_vsta[3],
                  size_t dest_vsz[3],
                  size_t dest_vsta[3],
                  size_t vox_num_per_blk,
                  unsigned char* dst);

SplBool
writeSbxHeaderRecord(FILE* fp,
                     int mode,
                     int writeRank,
                     SPL_Comm grp,
                     unsigned int dims,
                     unsigned int vlen,
                     unsigned int dtype,
                     size_t gc,
                     unsigned int rlen,
                     unsigned int crddef,
                     unsigned int aux,
                     unsigned long long block_size,
                     const size_t* size,
                     const double* org,
                     const double* pit);

SplBool
writeSbxExtRecord(FILE* fp,
                  int mode,
                  int writeRank,
                  SPL_Comm grp,
                  unsigned int dims,
                  const size_t* wsize,
                  const size_t* sta_idx );

SplBool
writeSbxDataRecord(int mode,
                   int writeRank,
                   SPL_Comm grp,
                   FILE* fp,
                   unsigned long long cmp_flg,
                   size_t vox_num_per_blk,
                   unsigned int dims,
                   unsigned int vlen,
                   unsigned int rlen,
                   const size_t* vsz,
                   const unsigned char* data,
                   size_t* wsize,
                   const size_t* sta_idx);

SplBool
checkSbx(const splSbx* fobj, SplBool writeFlag);

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_FIO_SBX_IO_H_ */

