/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splfio_sv_read.h
 * Prototype declaration of file read subroutine related to Sv(Simple Voxel).
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_sv_read.h
 * Sv(Simple Voxel)の低レベルファイル読込関数群ヘッダーファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPL_FIO_SV_READ_H_
#define _SPL_FIO_SV_READ_H_

#include "spl/spl.h"
#include "spl/util_endian.h"
#include "spl/spl_errno.h"
#include "splfio_filetype_check.h"

#include <stdio.h>

#if defined(__cplusplus)
extern "C" {
#endif

SplBool
readSvAttrRecord(FILE* fp, SplFileType fmt, EMatchType* endian_type,
                 unsigned int *dims, unsigned int *vlen, unsigned int *dtype,
                 size_t *gc, unsigned int *rlen, unsigned int *crddef,
                 unsigned int *aux, unsigned long long* block_size);

SplBool
readSvSizeRecord(FILE* fp, long offset, EMatchType eType, size_t* size);

SplBool
readSvOriginRecord(FILE* fp, long offset, EMatchType eType, int rlen, double* org);

SplBool
readSvPitchRecord(FILE* fp, long offset, EMatchType eType, int rlen, double* pit);

SplBool
readSvTimeRecord(FILE* fp, long offset, EMatchType eType, int rlen,
                 long long* step, double* dtime);

SplBool
checkCompDtBlk(FILE* fp, long file_offset, EMatchType eType, unsigned long long* cmp_flg);

SplBool
readSvBlkSz(FILE* fp, EMatchType eType, size_t* comp_blk_sz);

SplBool
readSvDataBlkSz(int mode, int readRank, SPL_Comm grp,
                FILE* fp, EMatchType eType, size_t* comp_blk_sz);

SplBool
readSvDtBlk(FILE* fp, size_t dt_blk_sz, size_t buf_sz, unsigned char* buf);

SplBool
readSvDataBlock(int mode, int readRank, SPL_Comm grp,
                FILE* fp, EMatchType eType,
                size_t dt_blk_sz, size_t buf_sz, unsigned char* buf);

SplBool
readSvExtRecord(FILE* fp, long offset, EMatchType eType, int rlen,
                size_t* wsz, size_t* widx, SplBool* has_ext_record);

int
readSvCmpDtBlks(int mode, int readRank, SPL_Comm grp, FILE* fp,
                EMatchType eType, SplBool decomp_flag, unsigned int dtype,
                size_t* cmp_blk_buf_sz, size_t ext_blk_buf_sz,
                size_t dest_buf_sz, size_t surplus_buf_sz,
                unsigned char** cmp_blk_buf, unsigned char* ext_blk_buf,
                unsigned char* dest_buf, unsigned char* surplus_buf,
                size_t* dest_effective_len, size_t* surplus_effective_len);

SplBool
readSvDataRecord(int mode,
                  int readRank,
                  SPL_Comm grp,
                  FILE* fp,
                  EMatchType eType,
                  unsigned long long cmp_flg,
                  unsigned int dims,
                  unsigned int file_vlen,
                  unsigned int dest_vlen,
                  unsigned int data_len,
                  size_t file_gc,
                  size_t dest_gc,
                  size_t file_vsz[3],
                  size_t file_vsta[3],
                  size_t dest_vsz[3],
                  size_t dest_vsta[3],
                  size_t vox_num_per_blk,
                  unsigned char* dst);



#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_FIO_SV_READ_H_ */

