/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/

/*************************************************************
  -- DO NOT INCLUDE THIS FILE DIRECTLY --
  splfio_sv_util.h  -- Private header file.
    Prototype declaration of subroutine related to Sv(Simple Voxel).
**************************************************************/
/**
 * \~english
 * @file splfio_sv_util.h
 * Prototype declaration of utility subroutine related to Sv(Simple Voxel).
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_sv_util.h
 * Sv(Simple Voxel)のユーティリティヘッダーファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPL_FIO_SV_UTILITY_H_
#define _SPL_FIO_SV_UTILITY_H_

#include <stdio.h>
#include "spl/spl.h"

#if defined(__cplusplus)
extern "C" {
#endif

extern SplBool
getIndexFromPos(unsigned int dims,
                size_t position,
                const size_t* vsz,
                size_t* idx );

extern SplBool
getReadBufLength(unsigned int dims,
                 const size_t* vsize,
                 unsigned int dsize,
                 unsigned int vec_len,
                 size_t* buf_len);

extern SplBool
getOverlapArea(const size_t  src_gloval_sidx,
               const size_t  src_gloval_eidx,
               const size_t  dst_gloval_sidx,
               const size_t  dst_gloval_eidx,
               size_t* ov_gloval_sidx,
               size_t* ov_gloval_size);

extern SplBool
getOverlapAreas(unsigned int dims,
                const size_t* src_gloval_sidx,
                const size_t* src_gloval_eidx,
                const size_t* dst_gloval_sidx,
                const size_t* dst_gloval_eidx,
                size_t* ov_gloval_sidx,
                size_t* ov_gloval_size );

extern SplBool
copyBuffer(size_t src_buf_size,
           size_t dst_buf_size,
           size_t surplus_buf_size,
           const unsigned char* src_buf,
           unsigned char* dst_buf,
           unsigned char* surplus_buf,
           size_t* surplus_effective_len);
#if 0
extern SplBool
copySvBuffer(unsigned int dtype,
             unsigned int dims,
             unsigned int src_vlen,
             unsigned int dst_vlen,
             int dst_gc_offset,
             const size_t* src_size,
             const size_t* src_gloval_sidx,
             const size_t* dst_size,
             const size_t* dst_gloval_sidx,
             const size_t* cp_gloval_sidx,
             const size_t* cp_gloval_size,
             const unsigned char* src,
             unsigned char* dst );
#endif

extern SplBool
copySvBuffer(
             unsigned int dims,
             unsigned int src_dlen,
             unsigned int dst_dlen,
             const size_t* src_gloval_sidx,
             const size_t* src_size,
             unsigned int src_gc,
             const size_t* dst_gloval_sidx,
             const size_t* dst_size,
             unsigned int dst_gc,
             const size_t* cp_gloval_sidx,
             const size_t* cp_gloval_size,
             unsigned int cp_gc,
             const unsigned char* src,
             unsigned char* dst );


SplBool
compSvDataBlock(size_t ext_buf_sz, size_t cmp_buf_sz,
                unsigned char* ext_buf, unsigned char* cmp_buf,
                size_t* compressed_data_sz);

SplBool
decompSvDataBlock(EMatchType eType, unsigned int dtype,
                  size_t cmp_buf_sz, size_t ext_buf_sz,
                  unsigned char* cmp_buf, unsigned char* ext_buf,
                  size_t* extended_data_sz);

SplBool
getOverlapLength(unsigned int dims,
                const size_t* gloval_size,
                const size_t* gloval_sidx,
                const size_t gloval_len,
                const size_t* bnd_sidx,
                const size_t* bnd_eidx,
                size_t* ov_sidx,
                size_t* ov_len );

SplBool
getPosFromIndex(unsigned int dims,
                const size_t* vsz,
                const size_t* idx,
                size_t* position);


SplBool
getLocalIndexFromPos(unsigned int dims,
                const size_t* bnd_size,
                const size_t* bnd_sta,
                const size_t* add_sta,
                const size_t add_len,
                size_t* idx);


SplBool
copyLengthBuffer(
             unsigned int dims,
             const size_t src_dtype,
             const size_t* src_size,
             const size_t* src_sidx,
             const size_t* src_eidx,
             const size_t* cp_sidx,
             const size_t cp_len,
             const size_t dst_len,
             const unsigned char* src_buf,
             unsigned char* dst_buf );

#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_FIO_SV_UTILITY_H_ */

