/*************************************************************
   SPHERE - Skeleton for PHysical and Engineering REsearch
   Copyright (c) RIKEN, Japan. All right reserved. 2004-2010
 *************************************************************/
/**
 * \~english
 * @file splfio_sv_write.h
 * Prototype declaration of file write subroutine related to Sv(Simple Voxel).
 *
 * DO NOT INCLUDE THIS FILE DIRECTLY
 * @author     hira
 * @date       2009/02/20
 *
 * \~japanese
 * @file splfio_sv_write.h
 * Sv(Simple Voxel)の低レベルファイル書込関数群ヘッダーファイル
 *
 * このヘッダーファイルの直接インクルード禁止
 * @author     hira
 * @date       2009/02/20
 */

#ifndef _SPL_FIO_SV_WRITE_H_
#define _SPL_FIO_SV_WRITE_H_

#include "spl/spl.h"
#include "spl/util_endian.h"
#include "spl/spl_errno.h"
#include "splfio_filetype_check.h"
#include <stdio.h>

#if defined(__cplusplus)
extern "C" {
#endif

SplBool
writeSvAttrRecord(FILE* fp, SplFileType fmt, unsigned int dims,
                  unsigned int vlen, unsigned int dtype, size_t gc,
                  unsigned int rlen, unsigned int crddef,
                  unsigned int aux, unsigned long long block_size);

SplBool
writeSvSizeRecord(FILE* fp, long file_offset, unsigned int dims, const size_t* size);

SplBool
writeSvOriginRecord(FILE* fp, long file_offset, unsigned int dims,
                    int rlen, const double* org);

SplBool
writeSvPitchRecord(FILE* fp, long file_offset, unsigned int dims,
                   int rlen, const double* pit);

SplBool
writeSvTimeRecord(FILE* fp, long file_offset, int rlen, long long step, double time);

SplBool
writeSvBlkSz(FILE* fp, size_t comp_blk_sz);

SplBool
writeSvDataBlkSz(int mode, int writeRank, SPL_Comm grp, FILE* fp, size_t comp_blk_sz);

SplBool
writeSvDtBlk(FILE* fp, size_t buf_sz, const unsigned char* buf);

SplBool
writeSvDataBlock(int mode, int writeRank, SPL_Comm grp, FILE* fp,
                 size_t buf_sz, const unsigned char* buf);

SplBool
writeSvExtRecord(FILE* fp, unsigned int dims, const size_t* wsz, const size_t* widx);


SplBool
writeSvDataRecord(int mode,
                   int writeRank,
                   SPL_Comm grp,
                   FILE* fp,
                   unsigned long long cmp_flg,
                   size_t vox_num_per_blk,
                   unsigned int dims,
                   unsigned int vlen,
                   unsigned int data_len,
                   const size_t* vsz,
                   const unsigned char* data,
                   const size_t* wsize,
                   const size_t* sta_idx);

SplBool
getSvWholeSize(int mode,
              unsigned int dims,
              const size_t* my_size,
              const size_t* my_sidx,
              size_t* wsize,
              size_t* wsidx,
              size_t* write_size,
              size_t* write_sidx,
              SPL_Comm grp);

SplBool
getVoxelCoords(
              unsigned int dims,
              const size_t* my_size,
              const size_t* my_sidx,
              const size_t* wsidx,
              double* worg,
              double* pit,
              SPL_Comm grp);


#if defined(__cplusplus)
}
#endif /* extern "C" */

#endif /* _SPL_FIO_SV_WRITE_H_ */

