/*
 * SPHERE - Skeleton for PHysical and Engineering REsearch
 *
 * Copyright (c) RIKEN, Japan. All right reserved. 2004-2008
 *
 */

#include "SklSphDataSet.h"


bool SklSphDataSet::Read(const char* fname)
{
  if( !fname ) return false;

  EMatchType eType;
  eType = isMatchEndian(fname, 8);
  if( eType == UnKnown ){ return false; }

  FILE* fp;
  if( !(fp = fopen(fname, "rb")) ) {
    fprintf(stderr, "Can't open file.(%s)\n", fname);
    return false;
  }
  int vc = 0;
  unsigned int size[3];
  int procGrp = 0;
  if( !ReadAttrRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadVoxSizeRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadVoxOriginRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadVoxPitchRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadTimeRecord(fp, eType) ) { fclose(fp); return false; }

  for (int i=0; i<3; i++) {
    if( m_dType == TYPE_SINGLE ) {
      size[i] = m_sz[i];
    }
    else if( m_dType == TYPE_DOUBLE ) {
      size[i] = (unsigned int)m_lsz[i];
    }
  }
  if( !ReadDataRecord(fp, size, vc, eType, procGrp) ) { fclose(fp); return false; }

  fclose(fp);

  if( m_dType == TYPE_SINGLE ) {
    int igc = ((int)m_sz[0] - (int)size[0]) / 2;
    int jgc = ((int)m_sz[1] - (int)size[1]) / 2;
    int kgc = ((int)m_sz[2] - (int)size[2]) / 2;
    if( (igc < 0) || (igc!=jgc) || (jgc!=kgc) || (kgc!=igc) ) return false;
    m_sz[0] = size[0];
    m_sz[1] = size[1];
    m_sz[2] = size[2];
    m_org[0] = m_org[0] + m_pit[0]*igc;
    m_org[1] = m_org[1] + m_pit[1]*jgc;
    m_org[2] = m_org[2] + m_pit[2]*kgc;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    int igc = ((int)m_lsz[0] - (int)size[0]) / 2;
    int jgc = ((int)m_lsz[1] - (int)size[1]) / 2;
    int kgc = ((int)m_lsz[2] - (int)size[2]) / 2;
    if( (igc < 0) || (igc!=jgc) || (jgc!=kgc) || (kgc!=igc) ) return false;
    m_lsz[0] = size[0];
    m_lsz[1] = size[1];
    m_lsz[2] = size[2];
    m_dorg[0] = m_dorg[0] + m_dpit[0]*igc;
    m_dorg[1] = m_dorg[1] + m_dpit[1]*jgc;
    m_dorg[2] = m_dorg[2] + m_dpit[2]*kgc;
  }

  SetFileName(fname);

  return true;
}

bool SklSphDataSet::Read(const char* fname,
                         const unsigned* size,
                         unsigned vc,
                         bool multi_io_flag,
                         int procGrp)
{
  if( !fname ) return false;

  EMatchType eType;
  eType = isMatchEndian(fname, 8);
  if( eType == UnKnown ){ return false; }

  FILE* fp;
  if( !(fp = fopen(fname, "rb")) ) {
    fprintf(stderr, "Can't open file.(%s)\n", fname);
    return false;
  }

  if( !ReadAttrRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadVoxSizeRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadVoxOriginRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadVoxPitchRecord(fp, eType) ) { fclose(fp); return false; }
  if( !ReadTimeRecord(fp, eType) ) { fclose(fp); return false; }
// modify start for vsphere_cmp at 2008/08/15 by @hira
//  if( !ReadDataRecord(fp, size, vc, eType) ) { fclose(fp); return false; }
  if( !ReadDataRecord(fp, size, vc, eType, procGrp) ) { fclose(fp); return false; }
// modify end for vsphere_cmp at 2008/08/15 by @hira

  fclose(fp);

  if( m_dType == TYPE_SINGLE ) {
    int igc = ((int)m_sz[0] - (int)size[0]) / 2;
    int jgc = ((int)m_sz[1] - (int)size[1]) / 2;
    int kgc = ((int)m_sz[2] - (int)size[2]) / 2;
    if( (igc < 0) || (igc!=jgc) || (jgc!=kgc) || (kgc!=igc) ) return false;
    m_sz[0] = size[0];
    m_sz[1] = size[1];
    m_sz[2] = size[2];
    m_org[0] = m_org[0] + m_pit[0]*igc;
    m_org[1] = m_org[1] + m_pit[1]*jgc;
    m_org[2] = m_org[2] + m_pit[2]*kgc;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    int igc = ((int)m_lsz[0] - (int)size[0]) / 2;
    int jgc = ((int)m_lsz[1] - (int)size[1]) / 2;
    int kgc = ((int)m_lsz[2] - (int)size[2]) / 2;
    if( (igc < 0) || (igc!=jgc) || (jgc!=kgc) || (kgc!=igc) ) return false;
    m_lsz[0] = size[0];
    m_lsz[1] = size[1];
    m_lsz[2] = size[2];
    m_dorg[0] = m_dorg[0] + m_dpit[0]*igc;
    m_dorg[1] = m_dorg[1] + m_dpit[1]*jgc;
    m_dorg[2] = m_dorg[2] + m_dpit[2]*kgc;
  }

  return true;
}

bool SklSphDataSet::Write(const char* fname,
                          unsigned gc,
                          bool multi_io_flag,
                          int procGrp)
{
  if( !fname ) return false;

  // add start for vsphere_cmp at 2008/08/15 by @hira
  // check voxel size
  if (m_sz[0] == 0 || m_sz[1] == 0 || m_sz[2] == 0) {
    return true;
  }
  // add end for vsphere_cmp at 2008/08/15 by @hira

  FILE* fp;
  if( !(fp = fopen(fname, "wb")) ) {
    fprintf(stderr, "Can't open file.(%s)\n", fname);
    return false;
  }

  if( !WriteAttrRecord(fp) ) { fclose(fp); return false; }
  if( !WriteVoxSizeRecord(fp, gc) ) { fclose(fp); return false; }
  if( !WriteVoxOriginRecord(fp, gc) ) { fclose(fp); return false; }
  if( !WriteVoxPitchRecord(fp) ) { fclose(fp); return false; }
  if( !WriteTimeRecord(fp) ) { fclose(fp); return false; }
  if( !WriteDataRecord(fp, gc) ) { fclose(fp); return false; }

  fclose(fp);
  return true;
}

bool SklSphDataSet::ReadAttrRecord(FILE* fp, EMatchType eType)
{
  if( !fp ) return false;
  int sv_type, data_type, dmy;

  if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  if( eType == UnMatch ) BSWAP32(dmy);
  if( dmy != 8 ) return false;

  if( fread(&sv_type, sizeof(int), 1, fp) != 1 ) return false;
  if( eType == UnMatch ) BSWAP32(sv_type);
  if( sv_type == 1 )      m_svType = TYPE_SCALAR;
  else if( sv_type == 2 ) m_svType = TYPE_VECTOR;
  else { m_svType = UNKNOWN_SVTYPE; return false; }

  if( fread(&data_type, sizeof(int), 1, fp) != 1 ) return false;
  if( eType == UnMatch ) BSWAP32(data_type);
  if( data_type == 1 )      m_dType = TYPE_SINGLE;
  else if( data_type == 2 ) m_dType = TYPE_DOUBLE;
  else { m_dType = UNKNOWN_DATATYPE; return false; }

  if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  if( eType == UnMatch ) BSWAP32(dmy);
  if( dmy != 8 ) return false;

  return true;
}

bool SklSphDataSet::ReadVoxSizeRecord(FILE* fp, EMatchType eType)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 12 ) return false;

    if( fread(m_sz, sizeof(int), 3, fp) != 3 ) return false;
    if( eType == UnMatch ) BSWAPVEC(m_sz, 3);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 12 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 24 ) return false;

    if( fread(m_lsz, sizeof(long long), 3, fp) != 3 ) return false;
    if( eType == UnMatch ) DBSWAPVEC(m_sz, 3);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 24 ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::ReadVoxOriginRecord(FILE* fp, EMatchType eType)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 12 ) return false;

    if( fread(m_org, sizeof(float), 3, fp) != 3 ) return false;
    if( eType == UnMatch ) BSWAPVEC(m_org, 3);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 12 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 24 ) return false;

    if( fread(m_dorg, sizeof(double), 3, fp) != 3 ) return false;
    if( eType == UnMatch ) DBSWAPVEC(m_dorg, 3);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 24 ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::ReadVoxPitchRecord(FILE* fp, EMatchType eType)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 12 ) return false;

    if( fread(m_pit, sizeof(float), 3, fp) != 3 ) return false;
    if( eType == UnMatch ) BSWAPVEC(m_pit, 3);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 12 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 24 ) return false;

    if( fread(m_dpit, sizeof(double), 3, fp) != 3 ) return false;
    if( eType == UnMatch ) DBSWAPVEC(m_dpit, 3);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 24 ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::ReadTimeRecord(FILE* fp, EMatchType eType)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 8 ) return false;

    if( fread(&m_step, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(m_step);

    if( fread(&m_time, sizeof(float), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(m_time);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 8 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 16 ) return false;

    if( fread(&m_lstep, sizeof(long long), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP64(m_lstep);

    if( fread(&m_dtime, sizeof(double), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP64(m_dtime);

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != 16 ) return false;
  }
  else return false;

  return true;
}

// modify start for vsphere_cmp at 2008/08/15 by @hira
//      to add para_mng, procGrp
bool SklSphDataSet::ReadDataRecord(FILE* fp,
                                   const unsigned* size,
                                   unsigned vc,
                                   EMatchType eType,
                                   int procGrp)
{
  if( !fp || !size ) return false;
  unsigned dmy;

  if( m_dType == TYPE_SINGLE ) {
    int igc = ((int)m_sz[0] - (int)size[0]) / 2;
    int jgc = ((int)m_sz[1] - (int)size[1]) / 2;
    int kgc = ((int)m_sz[2] - (int)size[2]) / 2;
    if( (igc < 0) || (igc!=jgc) || (jgc!=kgc) || (kgc!=igc) ) return false;

    float* data = NULL;
    size_t len = 0;
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);

    if( m_svType == TYPE_SCALAR ) len = m_sz[0]*m_sz[1]*m_sz[2];
    else if( m_svType == TYPE_VECTOR ) len = m_sz[0]*m_sz[1]*m_sz[2]*3;
    else return false;

    if( dmy != len*sizeof(float) ) return false;
    if( !(data = new float[len]) ) return false;
    if( fread(data, sizeof(float), len, fp) != len ) return false;
    if( eType == UnMatch ) BSWAPVEC(data, len);

    if( m_svType == TYPE_SCALAR ) {
      int fsize = (size[0]+vc*2)*(size[1]+vc*2)*(size[2]+vc*2);
      m_float_data = (float*)new float[fsize];

      if( (vc == 0)
          && (m_sz[0]==size[0]) && (m_sz[1]==size[1]) && (m_sz[2]==size[2]) ){
        memcpy(m_float_data, data, sizeof(float)*m_sz[0]*m_sz[1]*m_sz[2]);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<m_sz[2]; k++){
          unsigned kk = k+vc-kgc;
          for(j=0; j<m_sz[1]; j++){
            unsigned jj = j+vc-jgc;
            for(i=0; i<m_sz[0]; i++){
              unsigned ii = i+vc-igc;
              size_t idx1 = m_sz[0]*m_sz[1]*k + m_sz[0]*j + i;
              size_t idx2 = (size[0]+vc*2)*(size[1]+vc*2)*kk
                           +(size[0]+vc*2)*jj + ii;
              m_float_data[idx2] = data[idx1];
            }
          }
        }
      }
    }
    else if( m_svType == TYPE_VECTOR ) {

      int fsize = (size[0]+vc*2)*(size[1]+vc*2)*(size[2]+vc*2)*3;
      m_float_data = (float*)new float[fsize];

      // data copy
      if( (vc == 0)
          && (m_sz[0]==size[0]) && (m_sz[1]==size[1]) && (m_sz[2]==size[2]) ){
        memcpy(m_float_data, data, sizeof(float)*m_sz[0]*m_sz[1]*m_sz[2]*3);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<m_sz[2]; k++){
          unsigned kk = k+vc-kgc;
          for(j=0; j<m_sz[1]; j++){
            unsigned jj = j+vc-jgc;
            for(i=0; i<m_sz[0]; i++){
              unsigned ii = i+vc-igc;
              size_t idx1 = 3*( m_sz[0]*m_sz[1]*k + m_sz[0]*j + i );
              size_t idx2 = 3*( (size[0]+vc*2)*(size[1]+vc*2)*kk
                               +(size[0]+vc*2)*jj + ii );
              m_float_data[idx2  ] = data[idx1  ];
              m_float_data[idx2+1] = data[idx1+1];
              m_float_data[idx2+2] = data[idx1+2];
            }
          }
        }
      }
    }
    else { delete [] data; return false; }
    delete [] data;

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != len*sizeof(float) ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    int igc = ((int)m_lsz[0] - (int)size[0]) / 2;
    int jgc = ((int)m_lsz[1] - (int)size[1]) / 2;
    int kgc = ((int)m_lsz[2] - (int)size[2]) / 2;
    if( (igc < 0) || (igc!=jgc) || (jgc!=kgc) || (kgc!=igc) ) return false;

    double* data = NULL;
    size_t len = 0;
    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);

    if( m_svType == TYPE_SCALAR ) len = m_lsz[0]*m_lsz[1]*m_lsz[2];
    else if( m_svType == TYPE_VECTOR ) len = m_lsz[0]*m_lsz[1]*m_lsz[2]*3;
    else return false;

    if( dmy != len*sizeof(double) ) return false;
    if( !(data = new double[len]) ) return false;
    if( fread(data, sizeof(double), len, fp) != len ) return false;
    if( eType == UnMatch ) DBSWAPVEC(data, len);

    if( m_svType == TYPE_SCALAR ) {

      int dsize = (size[0]+vc*2)*(size[1]+vc*2)*(size[2]+vc*2);
      m_double_data = (double*)new double[dsize];

      if( (vc == 0)
          && (m_lsz[0]==size[0])&&(m_lsz[1]==size[1])&&(m_lsz[2]==size[2]) ){
        memcpy(m_double_data, data, sizeof(double)*m_lsz[0]*m_lsz[1]*m_lsz[2]);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<m_lsz[2]; k++){
          unsigned kk = k+vc-kgc;
          for(j=0; j<m_lsz[1]; j++){
            unsigned jj = j+vc-jgc;
            for(i=0; i<m_lsz[0]; i++){
              unsigned ii = i+vc-igc;
              size_t idx1 = m_lsz[0]*m_lsz[1]*k + m_lsz[0]*j + i;
              size_t idx2 = (size[0]+vc*2)*(size[1]+vc*2)*kk
                           +(size[0]+vc*2)*jj + ii;
              m_double_data[idx2] = data[idx1];
            }
          }
        }
      }
    }
    else if( m_svType == TYPE_VECTOR ) {
      int dsize = (size[0]+vc*2)*(size[1]+vc*2)*(size[2]+vc*2)*3;
      m_double_data = (double*)new double[dsize];

      if( (vc == 0)
          && (m_lsz[0]==size[0])&&(m_lsz[1]==size[1])&&(m_lsz[2]==size[2]) ){
        memcpy(m_double_data, data, sizeof(double)*m_lsz[0]*m_lsz[1]*m_lsz[2]*3);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<m_lsz[2]; k++){
          unsigned kk = k+vc-kgc;
          for(j=0; j<m_lsz[1]; j++){
            unsigned jj = j+vc-jgc;
            for(i=0; i<m_lsz[0]; i++){
              unsigned ii = i+vc-igc;
              size_t idx1 = 3*( m_lsz[0]*m_lsz[1]*k + m_lsz[0]*j + i );
              size_t idx2 = 3*( (size[0]+vc*2)*(size[1]+vc*2)*kk
                               +(size[0]+vc*2)*jj + ii );
              m_double_data[idx2  ] = data[idx1  ];
              m_double_data[idx2+1] = data[idx1+1];
              m_double_data[idx2+2] = data[idx1+2];
            }
          }
        }
      }
    }
    else { delete [] data; return false; }
    delete [] data;

    if( fread(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( eType == UnMatch ) BSWAP32(dmy);
    if( dmy != len*sizeof(double) ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::WriteAttrRecord(FILE* fp)
{
  if( !fp ) return false;
  int sv_type, data_type, dmy;

  dmy = 8;
  if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;

  if( m_svType == TYPE_SCALAR )      sv_type = 1;
  else if( m_svType == TYPE_VECTOR ) sv_type = 2;
  else return false;
  if( fwrite(&sv_type, sizeof(int), 1, fp) != 1 ) return false;

  if( m_dType == TYPE_SINGLE )      data_type = 1;
  else if( m_dType == TYPE_DOUBLE ) data_type = 2;
  else return false;
  if( fwrite(&data_type, sizeof(int), 1, fp) != 1 ) return false;

  if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;

  return true;
}

bool SklSphDataSet::WriteVoxSizeRecord(FILE* fp, unsigned gc)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    unsigned sz[3];
    sz[0] = m_sz[0]+gc*2;
    sz[1] = m_sz[1]+gc*2;
    sz[2] = m_sz[2]+gc*2;
    dmy = 12;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(sz, sizeof(int), 3, fp) != 3 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    unsigned long long lsz[3];
    lsz[0] = m_lsz[0]+gc*2;
    lsz[1] = m_lsz[1]+gc*2;
    lsz[2] = m_lsz[2]+gc*2;
    dmy = 24;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(lsz, sizeof(long long), 3, fp) != 3 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::WriteVoxOriginRecord(FILE* fp, unsigned gc)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    float org[3];
    org[0] = m_org[0] - m_pit[0]*gc;
    org[1] = m_org[1] - m_pit[1]*gc;
    org[2] = m_org[2] - m_pit[2]*gc;
    dmy = 12;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(org, sizeof(float), 3, fp) != 3 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    double dorg[3];
    dorg[0] = m_dorg[0] - m_dpit[0]*gc;
    dorg[1] = m_dorg[1] - m_dpit[1]*gc;
    dorg[2] = m_dorg[2] - m_dpit[2]*gc;
    dmy = 24;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(dorg, sizeof(double), 3, fp) != 3 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::WriteVoxPitchRecord(FILE* fp)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    dmy = 12;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(m_pit, sizeof(float), 3, fp) != 3 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    dmy = 24;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(m_dpit, sizeof(double), 3, fp) != 3 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::WriteTimeRecord(FILE* fp)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ) {
    dmy = 8;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(&m_step, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(&m_time, sizeof(float), 1, fp) != 1 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else if( m_dType == TYPE_DOUBLE ) {
    dmy = 16;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
    if( fwrite(&m_lstep, sizeof(long long), 1, fp) != 1 ) return false;
    if( fwrite(&m_dtime, sizeof(double), 1, fp) != 1 ) return false;
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ) return false;
  }
  else return false;

  return true;
}

bool SklSphDataSet::WriteDataRecord(FILE* fp, unsigned gc)
{
  if( !fp ) return false;
  int dmy;

  if( m_dType == TYPE_SINGLE ){
    unsigned sz[3];
    sz[0] = m_sz[0]+gc*2;
    sz[1] = m_sz[1]+gc*2;
    sz[2] = m_sz[2]+gc*2;

    float* data = NULL;
    size_t len = 0;

    if( m_svType == TYPE_SCALAR ) len = sz[0]*sz[1]*sz[2];
    else if( m_svType == TYPE_VECTOR ) len = sz[0]*sz[1]*sz[2]*3;
    else return false;

    if( !(data = new float[len]) ) return false;
    // data copy
    if( m_svType == TYPE_SCALAR ){
      unsigned vc = 0;
      float* src = m_float_data;
      if( vc == 0 ){
        if( gc != 0 ) return false;
        memcpy(data, src, sizeof(float)*m_sz[0]*m_sz[1]*m_sz[2]);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<sz[2]; k++){
          unsigned kk = k+vc-gc;
          for(j=0; j<sz[1]; j++){
            unsigned jj = j+vc-gc;
            for(i=0; i<sz[0]; i++){
              unsigned ii = i+vc-gc;
              size_t idx1 = sz[0]*sz[1]*k + sz[0]*j + i;
              size_t idx2 =  (m_sz[0]+vc*2)*(m_sz[1]+vc*2)*kk
                           + (m_sz[0]+vc*2)*jj
                           + ii;
              data[idx1] = src[idx2];
            }
          }
        }
      }
    }
    else if( m_svType == TYPE_VECTOR ){
      unsigned vc = 0;
      float* src = m_float_data;
      if( vc == 0 ){
        if( gc != 0 ) return false;
        memcpy(data, src, sizeof(float)*m_sz[0]*m_sz[1]*m_sz[2]*3);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<sz[2]; k++){
          unsigned kk = k+vc-gc;
          for(j=0; j<sz[1]; j++){
            unsigned jj = j+vc-gc;
            for(i=0; i<sz[0]; i++){
              unsigned ii = i+vc-gc;
              size_t idx1 = 3*( sz[0]*sz[1]*k + sz[0]*j + i );
              size_t idx2 = 3*(  (m_sz[0]+vc*2)*(m_sz[1]+vc*2)*kk
                               + (m_sz[0]+vc*2)*jj
                               + ii );
              data[idx1  ] = src[idx2  ];
              data[idx1+1] = src[idx2+1];
              data[idx1+2] = src[idx2+2];
            }
          }
        }
      }
    }

    dmy = len*sizeof(float);
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ){delete [] data; return false;}
    if( fwrite(data, sizeof(float), len, fp) != len ){
      delete [] data; return false;
    }
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ){delete [] data; return false;}
    delete [] data;
  }
  else if( m_dType == TYPE_DOUBLE ){
    unsigned long long lsz[3];
    lsz[0] = m_lsz[0]+gc*2;
    lsz[1] = m_lsz[1]+gc*2;
    lsz[2] = m_lsz[2]+gc*2;

    double* data = NULL;
    size_t len = 0;

    if( m_svType == TYPE_SCALAR ) len = lsz[0]*lsz[1]*lsz[2];
    else if( m_svType == TYPE_VECTOR ) len = lsz[0]*lsz[1]*lsz[2]*3;
    else return false;

    if( !(data = new double[len]) ) return false;
    // data copy
    if( m_svType == TYPE_SCALAR ){
      unsigned vc = 0;
      double* src = m_double_data;
      if( vc == 0 ){
        if( gc != 0 ) return false;
        memcpy(data, src, sizeof(double)*m_lsz[0]*m_lsz[1]*m_lsz[2]);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<lsz[2]; k++){
          unsigned kk = k+vc-gc;
          for(j=0; j<lsz[1]; j++){
            unsigned jj = j+vc-gc;
            for(i=0; i<lsz[0]; i++){
              unsigned ii = i+vc-gc;
              size_t idx1 = lsz[0]*lsz[1]*k + lsz[0]*j + i;
              size_t idx2 =  (m_lsz[0]+vc*2)*(m_lsz[1]+vc*2)*kk
                           + (m_lsz[0]+vc*2)*jj
                           + ii;
              data[idx1] = src[idx2];
            }
          }
        }
      }
    }
    else if( m_svType == TYPE_VECTOR ){
      unsigned vc = 0;
      double* src = m_double_data;
      if( vc == 0 ){
        if( gc != 0 ) return false;
        memcpy(data, src, sizeof(double)*m_lsz[0]*m_lsz[1]*m_lsz[2]*3);
      }
      else{
        register unsigned i, j, k;
        for(k=0; k<lsz[2]; k++){
          unsigned kk = k+vc-gc;
          for(j=0; j<lsz[1]; j++){
            unsigned jj = j+vc-gc;
            for(i=0; i<lsz[0]; i++){
              unsigned ii = i+vc-gc;
              size_t idx1 = 3*( lsz[0]*lsz[1]*k + lsz[0]*j + i );
              size_t idx2 = 3*(  (m_lsz[0]+vc*2)*(m_lsz[1]+vc*2)*kk
                               + (m_lsz[0]+vc*2)*jj
                               + ii);
              data[idx1  ] = src[idx2  ];
              data[idx1+1] = src[idx2+1];
              data[idx1+2] = src[idx2+2];
            }
          }
        }
      }
    }

    dmy = len*sizeof(double);
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ){delete [] data; return false;}
    if( fwrite(data, sizeof(double), len, fp) != len ){
      delete [] data; return false;
    }
    if( fwrite(&dmy, sizeof(int), 1, fp) != 1 ){delete [] data; return false;}
    delete [] data;
  }
  else return false;

  return true;
}


std::string SklSphDataSet::printInformation()
{
  std::string msg;
  char value[1024];

  msg += "File Name            : " + m_fname;
  msg += "\n";

  // データ種別フラグ
  if( m_svType == TYPE_SCALAR ) {
    msg += "Scalar/Vector Type   : Scalar Type";
  }
  else if( m_svType == TYPE_VECTOR ) {
    msg += "Scalar/Vector Type   : Vector Type";
  }
  else {
    msg += "Scalar/Vector Type   : Unknown Type";
  }
  msg += "\n";

  // dtype
  if( m_dType == TYPE_SINGLE ) {
    msg += "Real Data Size       : float";
  }
  else if( m_dType == TYPE_DOUBLE ) {
    msg += "Real Data Size       : double";
  }
  else {
    msg += "Real Data Size       : Unknown";
  }
  msg += "\n";

  // size
  unsigned int sz[3] = {0, 0, 0};
  this->GetSize(sz[0], sz[1], sz[2]);
  sprintf(value, "Voxel Size           : %d, %d, %d", sz[0], sz[1], sz[2]);
  msg += value;
  msg += "\n";

  // 原点座標レコード
  if( m_dType == TYPE_SINGLE ) {
    sprintf(value, "Origin Coords        : %f, %f, %f", m_org[0], m_org[1], m_org[2]);
    msg += value;
  }
  if( m_dType == TYPE_DOUBLE ) {
    sprintf(value, "Origin Coords        : %lf, %lf, %lf", m_dorg[0], m_dorg[1], m_dorg[2]);
    msg += value;
  }
  msg += "\n";

  // ボクセルピッチレコード
  if( m_dType == TYPE_SINGLE ) {
    sprintf(value, "Voxel Pitch          : %f, %f, %f", m_pit[0], m_pit[1], m_pit[2]);
    msg += value;
  }
  if( m_dType == TYPE_DOUBLE ) {
    sprintf(value, "Voxel Pitch          : %lf, %lf, %lf", m_dpit[0], m_dpit[1], m_dpit[2]);
    msg += value;
  }
  msg += "\n";

  // 時刻レコード
  if( m_dType == TYPE_SINGLE ) {
    sprintf(value, "Time Step Count      : %d\n", m_step);
    msg += value;
    sprintf(value, "Time                 : %f\n", m_time);
    msg += value;
  }
  if( m_dType == TYPE_DOUBLE ) {
    sprintf(value, "Time Step Count      : %lld\n", m_lstep);
    msg += value;
    sprintf(value, "Time                 : %lf\n", m_dtime);
    msg += value;
  }

  return msg;
}


